=== modified file 'Makefile.am'
--- Makefile.am	2010-07-26 01:50:52 +0000
+++ Makefile.am	2010-08-12 20:27:32 +0000
@@ -74,7 +74,9 @@
 
 
 doxygen:
-	doxygen Doxyfile
+	${DOXYGEN} docs/Doxyfile.api
+	${DOXYGEN} docs/Doxyfile.dev
+	${DOXYGEN} docs/Doxyfile.drizzled
 
 indent:
 	for f in `find ${top_srcdir} -type f | grep -v innobase |\
@@ -128,6 +130,7 @@
 
 clean-local:
 	find . -name '*.gcno' | xargs rm -f 
+	-rm -rf docs/api docs/dev
 
 distclean-local:
 	rm -r -f unittests/.libs
@@ -144,6 +147,7 @@
 include support-files/include.am
 include tests/include.am
 include unittests/include.am
+include libdrizzle/include.am
 
 TESTS += ${check_PROGRAMS}
 

=== modified file 'client/include.am'
--- client/include.am	2010-06-21 11:08:00 +0000
+++ client/include.am	2010-08-12 20:27:32 +0000
@@ -4,7 +4,7 @@
 
 CLIENT_LDADD= \
 	      drizzled/internal/libinternal.la \
-	      $(LIBDRIZZLE) \
+	      libdrizzle/libdrizzle.la \
 	      $(LIBINTL) \
 	      $(LIBZ) \
 	      gnulib/libgnu.la \
@@ -41,7 +41,7 @@
 
 client_drizzle_SOURCES=		client/drizzle.cc client/linebuffer.cc
 client_drizzle_LDADD=		${READLINE_LIBS} ${BOOST_LIBS} \
-				$(LIBDRIZZLE) \
+				libdrizzle/libdrizzle.la \
 				$(LIBINTL) \
 				$(LIBZ) \
 				gnulib/libgnu.la \

=== modified file 'configure.ac'
--- configure.ac	2010-07-06 21:50:49 +0000
+++ configure.ac	2010-08-12 20:27:32 +0000
@@ -24,7 +24,30 @@
 AC_CONFIG_SRCDIR([drizzled/drizzled.cc])
 AC_CONFIG_AUX_DIR([config])
 
-PANDORA_CANONICAL_TARGET(less-warnings, warnings-always-on, require-cxx, force-gcc42, skip-visibility, version-from-vc)
+PANDORA_CANONICAL_TARGET(less-warnings, warnings-always-on, require-cxx, force-gcc42, version-from-vc, skip-visibility)
+
+LIBDRIZZLE_LIBRARY_VERSION=0:8:0
+#                          | | |
+#                   +------+ | +---+
+#                   |        |     |
+#                current:revision:age
+#                   |        |     |
+#                   |        |     +- increment if interfaces have been
+added
+#                   |        |        set to zero if interfaces have been
+#                   |        |        removed or changed
+#                   |        +- increment if source code has changed
+#                   |           set to zero if current is incremented
+#                   +- increment if interfaces have been added, removed or
+#                      changed
+AC_SUBST(LIBDRIZZLE_LIBRARY_VERSION)
+
+# libdrizzle versioning when linked with GNU ld.
+AS_IF([test "$lt_cv_prog_gnu_ld" = "yes"],[
+  LD_VERSION_SCRIPT="-Wl,--version-script=\$(top_srcdir)/libdrizzle/libdrizzle.ver"
+  ])
+AC_SUBST(LD_VERSION_SCRIPT)
+
 
 dnl Set the default datadir location to /usr/local/var/drizzle. If a
 dnl localstatedir is explicitly given, just use that.
@@ -50,7 +73,6 @@
 AC_LANG_PUSH(C++)
 PANDORA_REQUIRE_PTHREAD
 PANDORA_REQUIRE_LIBUUID
-PANDORA_REQUIRE_LIBDRIZZLE
 PANDORA_REQUIRE_LIBZ
 PANDORA_REQUIRE_LIBPCRE
 PANDORA_REQUIRE_LIBREADLINE
@@ -64,6 +86,10 @@
           [Define indicating we are building and not consuming drizzle.])
 
 PANDORA_HAVE_LIBGTEST
+PANDORA_HAVE_LIBSQLITE3
+
+AC_CHECK_FUNC(inet_ntoa, [], [AC_CHECK_LIB(nsl, inet_ntoa)])
+
 
 AC_ARG_WITH([server-suffix],
     [AS_HELP_STRING([--with-server-suffix],
@@ -115,6 +141,7 @@
  po/Makefile.in dnl
  tests/Makefile dnl
  drizzled/plugin/version.h dnl
+ support-files/libdrizzle.pc dnl
  support-files/drizzle.server support-files/drizzle-log-rotate
  support-files/smf/install.sh dnl
  support-files/drizzle.spec dnl

=== added directory 'docs'
=== added file 'docs/Doxyfile.api'
--- docs/Doxyfile.api	1970-01-01 00:00:00 +0000
+++ docs/Doxyfile.api	2010-08-12 20:27:32 +0000
@@ -0,0 +1,1433 @@
+# Doxyfile 1.5.6
+
+# This file describes the settings to be used by the documentation system
+# doxygen (www.doxygen.org) for a project
+#
+# All text after a hash (#) is considered a comment and will be ignored
+# The format is:
+#       TAG = value [value, ...]
+# For lists items can also be appended using:
+#       TAG += value [value, ...]
+# Values that contain spaces should be placed between quotes (" ")
+
+#---------------------------------------------------------------------------
+# Project related configuration options
+#---------------------------------------------------------------------------
+
+# This tag specifies the encoding used for all characters in the config file 
+# that follow. The default is UTF-8 which is also the encoding used for all 
+# text before the first occurrence of this tag. Doxygen uses libiconv (or the 
+# iconv built into libc) for the transcoding. See 
+# http://www.gnu.org/software/libiconv for the list of possible encodings.
+
+DOXYFILE_ENCODING      = UTF-8
+
+# The PROJECT_NAME tag is a single word (or a sequence of words surrounded 
+# by quotes) that should identify the project.
+
+PROJECT_NAME           = 
+
+# The PROJECT_NUMBER tag can be used to enter a project or revision number. 
+# This could be handy for archiving the generated documentation or 
+# if some version control system is used.
+
+PROJECT_NUMBER         = 
+
+# The OUTPUT_DIRECTORY tag is used to specify the (relative or absolute) 
+# base path where the generated documentation will be put. 
+# If a relative path is entered, it will be relative to the location 
+# where doxygen was started. If left blank the current directory will be used.
+
+OUTPUT_DIRECTORY       = docs/api
+
+# If the CREATE_SUBDIRS tag is set to YES, then doxygen will create 
+# 4096 sub-directories (in 2 levels) under the output directory of each output 
+# format and will distribute the generated files over these directories. 
+# Enabling this option can be useful when feeding doxygen a huge amount of 
+# source files, where putting all generated files in the same directory would 
+# otherwise cause performance problems for the file system.
+
+CREATE_SUBDIRS         = NO
+
+# The OUTPUT_LANGUAGE tag is used to specify the language in which all 
+# documentation generated by doxygen is written. Doxygen will use this 
+# information to generate all constant output in the proper language. 
+# The default language is English, other supported languages are: 
+# Afrikaans, Arabic, Brazilian, Catalan, Chinese, Chinese-Traditional, 
+# Croatian, Czech, Danish, Dutch, Farsi, Finnish, French, German, Greek, 
+# Hungarian, Italian, Japanese, Japanese-en (Japanese with English messages), 
+# Korean, Korean-en, Lithuanian, Norwegian, Macedonian, Persian, Polish, 
+# Portuguese, Romanian, Russian, Serbian, Slovak, Slovene, Spanish, Swedish, 
+# and Ukrainian.
+
+OUTPUT_LANGUAGE        = English
+
+# If the BRIEF_MEMBER_DESC tag is set to YES (the default) Doxygen will 
+# include brief member descriptions after the members that are listed in 
+# the file and class documentation (similar to JavaDoc). 
+# Set to NO to disable this.
+
+BRIEF_MEMBER_DESC      = YES
+
+# If the REPEAT_BRIEF tag is set to YES (the default) Doxygen will prepend 
+# the brief description of a member or function before the detailed description. 
+# Note: if both HIDE_UNDOC_MEMBERS and BRIEF_MEMBER_DESC are set to NO, the 
+# brief descriptions will be completely suppressed.
+
+REPEAT_BRIEF           = YES
+
+# This tag implements a quasi-intelligent brief description abbreviator 
+# that is used to form the text in various listings. Each string 
+# in this list, if found as the leading text of the brief description, will be 
+# stripped from the text and the result after processing the whole list, is 
+# used as the annotated text. Otherwise, the brief description is used as-is. 
+# If left blank, the following values are used ("$name" is automatically 
+# replaced with the name of the entity): "The $name class" "The $name widget" 
+# "The $name file" "is" "provides" "specifies" "contains" 
+# "represents" "a" "an" "the"
+
+ABBREVIATE_BRIEF       = 
+
+# If the ALWAYS_DETAILED_SEC and REPEAT_BRIEF tags are both set to YES then 
+# Doxygen will generate a detailed section even if there is only a brief 
+# description.
+
+ALWAYS_DETAILED_SEC    = YES
+
+# If the INLINE_INHERITED_MEMB tag is set to YES, doxygen will show all 
+# inherited members of a class in the documentation of that class as if those 
+# members were ordinary class members. Constructors, destructors and assignment 
+# operators of the base classes will not be shown.
+
+INLINE_INHERITED_MEMB  = NO
+
+# If the FULL_PATH_NAMES tag is set to YES then Doxygen will prepend the full 
+# path before files name in the file list and in the header files. If set 
+# to NO the shortest path that makes the file name unique will be used.
+
+FULL_PATH_NAMES        = YES
+
+# If the FULL_PATH_NAMES tag is set to YES then the STRIP_FROM_PATH tag 
+# can be used to strip a user-defined part of the path. Stripping is 
+# only done if one of the specified strings matches the left-hand part of 
+# the path. The tag can be used to show relative paths in the file list. 
+# If left blank the directory from which doxygen is run is used as the 
+# path to strip.
+
+STRIP_FROM_PATH        = 
+
+# The STRIP_FROM_INC_PATH tag can be used to strip a user-defined part of 
+# the path mentioned in the documentation of a class, which tells 
+# the reader which header file to include in order to use a class. 
+# If left blank only the name of the header file containing the class 
+# definition is used. Otherwise one should specify the include paths that 
+# are normally passed to the compiler using the -I flag.
+
+STRIP_FROM_INC_PATH    = 
+
+# If the SHORT_NAMES tag is set to YES, doxygen will generate much shorter 
+# (but less readable) file names. This can be useful is your file systems 
+# doesn't support long names like on DOS, Mac, or CD-ROM.
+
+SHORT_NAMES            = NO
+
+# If the JAVADOC_AUTOBRIEF tag is set to YES then Doxygen 
+# will interpret the first line (until the first dot) of a JavaDoc-style 
+# comment as the brief description. If set to NO, the JavaDoc 
+# comments will behave just like regular Qt-style comments 
+# (thus requiring an explicit @brief command for a brief description.)
+
+JAVADOC_AUTOBRIEF      = NO
+
+# If the QT_AUTOBRIEF tag is set to YES then Doxygen will 
+# interpret the first line (until the first dot) of a Qt-style 
+# comment as the brief description. If set to NO, the comments 
+# will behave just like regular Qt-style comments (thus requiring 
+# an explicit \brief command for a brief description.)
+
+QT_AUTOBRIEF           = NO
+
+# The MULTILINE_CPP_IS_BRIEF tag can be set to YES to make Doxygen 
+# treat a multi-line C++ special comment block (i.e. a block of //! or /// 
+# comments) as a brief description. This used to be the default behaviour. 
+# The new default is to treat a multi-line C++ comment block as a detailed 
+# description. Set this tag to YES if you prefer the old behaviour instead.
+
+MULTILINE_CPP_IS_BRIEF = NO
+
+# If the INHERIT_DOCS tag is set to YES (the default) then an undocumented 
+# member inherits the documentation from any documented member that it 
+# re-implements.
+
+INHERIT_DOCS           = YES
+
+# If the SEPARATE_MEMBER_PAGES tag is set to YES, then doxygen will produce 
+# a new page for each member. If set to NO, the documentation of a member will 
+# be part of the file/class/namespace that contains it.
+
+SEPARATE_MEMBER_PAGES  = NO
+
+# The TAB_SIZE tag can be used to set the number of spaces in a tab. 
+# Doxygen uses this value to replace tabs by spaces in code fragments.
+
+TAB_SIZE               = 8
+
+# This tag can be used to specify a number of aliases that acts 
+# as commands in the documentation. An alias has the form "name=value". 
+# For example adding "sideeffect=\par Side Effects:\n" will allow you to 
+# put the command \sideeffect (or @sideeffect) in the documentation, which 
+# will result in a user-defined paragraph with heading "Side Effects:". 
+# You can put \n's in the value part of an alias to insert newlines.
+
+ALIASES                = 
+
+# Set the OPTIMIZE_OUTPUT_FOR_C tag to YES if your project consists of C 
+# sources only. Doxygen will then generate output that is more tailored for C. 
+# For instance, some of the names that are used will be different. The list 
+# of all members will be omitted, etc.
+
+OPTIMIZE_OUTPUT_FOR_C  = YES
+
+# Set the OPTIMIZE_OUTPUT_JAVA tag to YES if your project consists of Java 
+# sources only. Doxygen will then generate output that is more tailored for 
+# Java. For instance, namespaces will be presented as packages, qualified 
+# scopes will look different, etc.
+
+OPTIMIZE_OUTPUT_JAVA   = NO
+
+# Set the OPTIMIZE_FOR_FORTRAN tag to YES if your project consists of Fortran 
+# sources only. Doxygen will then generate output that is more tailored for 
+# Fortran.
+
+OPTIMIZE_FOR_FORTRAN   = NO
+
+# Set the OPTIMIZE_OUTPUT_VHDL tag to YES if your project consists of VHDL 
+# sources. Doxygen will then generate output that is tailored for 
+# VHDL.
+
+OPTIMIZE_OUTPUT_VHDL   = NO
+
+# If you use STL classes (i.e. std::string, std::vector, etc.) but do not want 
+# to include (a tag file for) the STL sources as input, then you should 
+# set this tag to YES in order to let doxygen match functions declarations and 
+# definitions whose arguments contain STL classes (e.g. func(std::string); v.s. 
+# func(std::string) {}). This also make the inheritance and collaboration 
+# diagrams that involve STL classes more complete and accurate.
+
+BUILTIN_STL_SUPPORT    = NO
+
+# If you use Microsoft's C++/CLI language, you should set this option to YES to
+# enable parsing support.
+
+CPP_CLI_SUPPORT        = NO
+
+# Set the SIP_SUPPORT tag to YES if your project consists of sip sources only. 
+# Doxygen will parse them like normal C++ but will assume all classes use public 
+# instead of private inheritance when no explicit protection keyword is present.
+
+SIP_SUPPORT            = NO
+
+# For Microsoft's IDL there are propget and propput attributes to indicate getter 
+# and setter methods for a property. Setting this option to YES (the default) 
+# will make doxygen to replace the get and set methods by a property in the 
+# documentation. This will only work if the methods are indeed getting or 
+# setting a simple type. If this is not the case, or you want to show the 
+# methods anyway, you should set this option to NO.
+
+IDL_PROPERTY_SUPPORT   = YES
+
+# If member grouping is used in the documentation and the DISTRIBUTE_GROUP_DOC 
+# tag is set to YES, then doxygen will reuse the documentation of the first 
+# member in the group (if any) for the other members of the group. By default 
+# all members of a group must be documented explicitly.
+
+DISTRIBUTE_GROUP_DOC   = NO
+
+# Set the SUBGROUPING tag to YES (the default) to allow class member groups of 
+# the same type (for instance a group of public functions) to be put as a 
+# subgroup of that type (e.g. under the Public Functions section). Set it to 
+# NO to prevent subgrouping. Alternatively, this can be done per class using 
+# the \nosubgrouping command.
+
+SUBGROUPING            = YES
+
+# When TYPEDEF_HIDES_STRUCT is enabled, a typedef of a struct, union, or enum 
+# is documented as struct, union, or enum with the name of the typedef. So 
+# typedef struct TypeS {} TypeT, will appear in the documentation as a struct 
+# with name TypeT. When disabled the typedef will appear as a member of a file, 
+# namespace, or class. And the struct will be named TypeS. This can typically 
+# be useful for C code in case the coding convention dictates that all compound 
+# types are typedef'ed and only the typedef is referenced, never the tag name.
+
+TYPEDEF_HIDES_STRUCT   = YES
+
+#---------------------------------------------------------------------------
+# Build related configuration options
+#---------------------------------------------------------------------------
+
+# If the EXTRACT_ALL tag is set to YES doxygen will assume all entities in 
+# documentation are documented, even if no documentation was available. 
+# Private class members and static file members will be hidden unless 
+# the EXTRACT_PRIVATE and EXTRACT_STATIC tags are set to YES
+
+EXTRACT_ALL            = YES
+
+# If the EXTRACT_PRIVATE tag is set to YES all private members of a class 
+# will be included in the documentation.
+
+EXTRACT_PRIVATE        = YES
+
+# If the EXTRACT_STATIC tag is set to YES all static members of a file 
+# will be included in the documentation.
+
+EXTRACT_STATIC         = YES
+
+# If the EXTRACT_LOCAL_CLASSES tag is set to YES classes (and structs) 
+# defined locally in source files will be included in the documentation. 
+# If set to NO only classes defined in header files are included.
+
+EXTRACT_LOCAL_CLASSES  = YES
+
+# This flag is only useful for Objective-C code. When set to YES local 
+# methods, which are defined in the implementation section but not in 
+# the interface are included in the documentation. 
+# If set to NO (the default) only methods in the interface are included.
+
+EXTRACT_LOCAL_METHODS  = NO
+
+# If this flag is set to YES, the members of anonymous namespaces will be 
+# extracted and appear in the documentation as a namespace called 
+# 'anonymous_namespace{file}', where file will be replaced with the base 
+# name of the file that contains the anonymous namespace. By default 
+# anonymous namespace are hidden.
+
+EXTRACT_ANON_NSPACES   = NO
+
+# If the HIDE_UNDOC_MEMBERS tag is set to YES, Doxygen will hide all 
+# undocumented members of documented classes, files or namespaces. 
+# If set to NO (the default) these members will be included in the 
+# various overviews, but no documentation section is generated. 
+# This option has no effect if EXTRACT_ALL is enabled.
+
+HIDE_UNDOC_MEMBERS     = NO
+
+# If the HIDE_UNDOC_CLASSES tag is set to YES, Doxygen will hide all 
+# undocumented classes that are normally visible in the class hierarchy. 
+# If set to NO (the default) these classes will be included in the various 
+# overviews. This option has no effect if EXTRACT_ALL is enabled.
+
+HIDE_UNDOC_CLASSES     = NO
+
+# If the HIDE_FRIEND_COMPOUNDS tag is set to YES, Doxygen will hide all 
+# friend (class|struct|union) declarations. 
+# If set to NO (the default) these declarations will be included in the 
+# documentation.
+
+HIDE_FRIEND_COMPOUNDS  = NO
+
+# If the HIDE_IN_BODY_DOCS tag is set to YES, Doxygen will hide any 
+# documentation blocks found inside the body of a function. 
+# If set to NO (the default) these blocks will be appended to the 
+# function's detailed documentation block.
+
+HIDE_IN_BODY_DOCS      = NO
+
+# The INTERNAL_DOCS tag determines if documentation 
+# that is typed after a \internal command is included. If the tag is set 
+# to NO (the default) then the documentation will be excluded. 
+# Set it to YES to include the internal documentation.
+
+INTERNAL_DOCS          = NO
+
+# If the CASE_SENSE_NAMES tag is set to NO then Doxygen will only generate 
+# file names in lower-case letters. If set to YES upper-case letters are also 
+# allowed. This is useful if you have classes or files whose names only differ 
+# in case and if your file system supports case sensitive file names. Windows 
+# and Mac users are advised to set this option to NO.
+
+CASE_SENSE_NAMES       = YES
+
+# If the HIDE_SCOPE_NAMES tag is set to NO (the default) then Doxygen 
+# will show members with their full class and namespace scopes in the 
+# documentation. If set to YES the scope will be hidden.
+
+HIDE_SCOPE_NAMES       = NO
+
+# If the SHOW_INCLUDE_FILES tag is set to YES (the default) then Doxygen 
+# will put a list of the files that are included by a file in the documentation 
+# of that file.
+
+SHOW_INCLUDE_FILES     = YES
+
+# If the INLINE_INFO tag is set to YES (the default) then a tag [inline] 
+# is inserted in the documentation for inline members.
+
+INLINE_INFO            = YES
+
+# If the SORT_MEMBER_DOCS tag is set to YES (the default) then doxygen 
+# will sort the (detailed) documentation of file and class members 
+# alphabetically by member name. If set to NO the members will appear in 
+# declaration order.
+
+SORT_MEMBER_DOCS       = NO
+
+# If the SORT_BRIEF_DOCS tag is set to YES then doxygen will sort the 
+# brief documentation of file, namespace and class members alphabetically 
+# by member name. If set to NO (the default) the members will appear in 
+# declaration order.
+
+SORT_BRIEF_DOCS        = NO
+
+# If the SORT_GROUP_NAMES tag is set to YES then doxygen will sort the 
+# hierarchy of group names into alphabetical order. If set to NO (the default) 
+# the group names will appear in their defined order.
+
+SORT_GROUP_NAMES       = NO
+
+# If the SORT_BY_SCOPE_NAME tag is set to YES, the class list will be 
+# sorted by fully-qualified names, including namespaces. If set to 
+# NO (the default), the class list will be sorted only by class name, 
+# not including the namespace part. 
+# Note: This option is not very useful if HIDE_SCOPE_NAMES is set to YES.
+# Note: This option applies only to the class list, not to the 
+# alphabetical list.
+
+SORT_BY_SCOPE_NAME     = NO
+
+# The GENERATE_TODOLIST tag can be used to enable (YES) or 
+# disable (NO) the todo list. This list is created by putting \todo 
+# commands in the documentation.
+
+GENERATE_TODOLIST      = YES
+
+# The GENERATE_TESTLIST tag can be used to enable (YES) or 
+# disable (NO) the test list. This list is created by putting \test 
+# commands in the documentation.
+
+GENERATE_TESTLIST      = YES
+
+# The GENERATE_BUGLIST tag can be used to enable (YES) or 
+# disable (NO) the bug list. This list is created by putting \bug 
+# commands in the documentation.
+
+GENERATE_BUGLIST       = YES
+
+# The GENERATE_DEPRECATEDLIST tag can be used to enable (YES) or 
+# disable (NO) the deprecated list. This list is created by putting 
+# \deprecated commands in the documentation.
+
+GENERATE_DEPRECATEDLIST= YES
+
+# The ENABLED_SECTIONS tag can be used to enable conditional 
+# documentation sections, marked by \if sectionname ... \endif.
+
+ENABLED_SECTIONS       = 
+
+# The MAX_INITIALIZER_LINES tag determines the maximum number of lines 
+# the initial value of a variable or define consists of for it to appear in 
+# the documentation. If the initializer consists of more lines than specified 
+# here it will be hidden. Use a value of 0 to hide initializers completely. 
+# The appearance of the initializer of individual variables and defines in the 
+# documentation can be controlled using \showinitializer or \hideinitializer 
+# command in the documentation regardless of this setting.
+
+MAX_INITIALIZER_LINES  = 30
+
+# Set the SHOW_USED_FILES tag to NO to disable the list of files generated 
+# at the bottom of the documentation of classes and structs. If set to YES the 
+# list will mention the files that were used to generate the documentation.
+
+SHOW_USED_FILES        = YES
+
+# If the sources in your project are distributed over multiple directories 
+# then setting the SHOW_DIRECTORIES tag to YES will show the directory hierarchy 
+# in the documentation. The default is NO.
+
+SHOW_DIRECTORIES       = NO
+
+# Set the SHOW_FILES tag to NO to disable the generation of the Files page.
+# This will remove the Files entry from the Quick Index and from the 
+# Folder Tree View (if specified). The default is YES.
+
+SHOW_FILES             = YES
+
+# Set the SHOW_NAMESPACES tag to NO to disable the generation of the 
+# Namespaces page.  This will remove the Namespaces entry from the Quick Index
+# and from the Folder Tree View (if specified). The default is YES.
+
+SHOW_NAMESPACES        = YES
+
+# The FILE_VERSION_FILTER tag can be used to specify a program or script that 
+# doxygen should invoke to get the current version for each file (typically from 
+# the version control system). Doxygen will invoke the program by executing (via 
+# popen()) the command <command> <input-file>, where <command> is the value of 
+# the FILE_VERSION_FILTER tag, and <input-file> is the name of an input file 
+# provided by doxygen. Whatever the program writes to standard output 
+# is used as the file version. See the manual for examples.
+
+FILE_VERSION_FILTER    = 
+
+#---------------------------------------------------------------------------
+# configuration options related to warning and progress messages
+#---------------------------------------------------------------------------
+
+# The QUIET tag can be used to turn on/off the messages that are generated 
+# by doxygen. Possible values are YES and NO. If left blank NO is used.
+
+QUIET                  = NO
+
+# The WARNINGS tag can be used to turn on/off the warning messages that are 
+# generated by doxygen. Possible values are YES and NO. If left blank 
+# NO is used.
+
+WARNINGS               = YES
+
+# If WARN_IF_UNDOCUMENTED is set to YES, then doxygen will generate warnings 
+# for undocumented members. If EXTRACT_ALL is set to YES then this flag will 
+# automatically be disabled.
+
+WARN_IF_UNDOCUMENTED   = NO
+
+# If WARN_IF_DOC_ERROR is set to YES, doxygen will generate warnings for 
+# potential errors in the documentation, such as not documenting some 
+# parameters in a documented function, or documenting parameters that 
+# don't exist or using markup commands wrongly.
+
+WARN_IF_DOC_ERROR      = YES
+
+# This WARN_NO_PARAMDOC option can be abled to get warnings for 
+# functions that are documented, but have no documentation for their parameters 
+# or return value. If set to NO (the default) doxygen will only warn about 
+# wrong or incomplete parameter documentation, but not about the absence of 
+# documentation.
+
+WARN_NO_PARAMDOC       = NO
+
+# The WARN_FORMAT tag determines the format of the warning messages that 
+# doxygen can produce. The string should contain the $file, $line, and $text 
+# tags, which will be replaced by the file and line number from which the 
+# warning originated and the warning text. Optionally the format may contain 
+# $version, which will be replaced by the version of the file (if it could 
+# be obtained via FILE_VERSION_FILTER)
+
+WARN_FORMAT            = "$file:$line: $text"
+
+# The WARN_LOGFILE tag can be used to specify a file to which warning 
+# and error messages should be written. If left blank the output is written 
+# to stderr.
+
+WARN_LOGFILE           = 
+
+#---------------------------------------------------------------------------
+# configuration options related to the input files
+#---------------------------------------------------------------------------
+
+# The INPUT tag can be used to specify the files and/or directories that contain 
+# documented source files. You may enter file names like "myfile.cpp" or 
+# directories like "/usr/src/myproject". Separate the files or directories 
+# with spaces.
+
+INPUT                  = docs/doxygen.h \
+                         libdrizzle/constants.h \
+                         libdrizzle/drizzle.h \
+                         libdrizzle/drizzle_client.h \
+                         libdrizzle/drizzle_server.h \
+                         libdrizzle/conn.h \
+                         libdrizzle/conn_client.h \
+                         libdrizzle/conn_server.h \
+                         libdrizzle/handshake_client.h \
+                         libdrizzle/handshake_server.h \
+                         libdrizzle/command_client.h \
+                         libdrizzle/command_server.h \
+                         libdrizzle/query.h \
+                         libdrizzle/result.h \
+                         libdrizzle/result_client.h \
+                         libdrizzle/result_server.h \
+                         libdrizzle/column.h \
+                         libdrizzle/column_client.h \
+                         libdrizzle/column_server.h \
+                         libdrizzle/row_client.h \
+                         libdrizzle/row_server.h \
+                         libdrizzle/field_client.h \
+                         libdrizzle/field_server.h \
+                         examples
+
+# This tag can be used to specify the character encoding of the source files 
+# that doxygen parses. Internally doxygen uses the UTF-8 encoding, which is 
+# also the default input encoding. Doxygen uses libiconv (or the iconv built 
+# into libc) for the transcoding. See http://www.gnu.org/software/libiconv for 
+# the list of possible encodings.
+
+INPUT_ENCODING         = UTF-8
+
+# If the value of the INPUT tag contains directories, you can use the 
+# FILE_PATTERNS tag to specify one or more wildcard pattern (like *.cpp 
+# and *.h) to filter out the source-files in the directories. If left 
+# blank the following patterns are tested: 
+# *.c *.cc *.cxx *.cpp *.c++ *.java *.ii *.ixx *.ipp *.i++ *.inl *.h *.hh *.hxx 
+# *.hpp *.h++ *.idl *.odl *.cs *.php *.php3 *.inc *.m *.mm *.py *.f90
+
+FILE_PATTERNS          = *.h *.c
+
+# The RECURSIVE tag can be used to turn specify whether or not subdirectories 
+# should be searched for input files as well. Possible values are YES and NO. 
+# If left blank NO is used.
+
+RECURSIVE              = YES
+
+# The EXCLUDE tag can be used to specify files and/or directories that should 
+# excluded from the INPUT source files. This way you can easily exclude a 
+# subdirectory from a directory tree whose root is specified with the INPUT tag.
+
+EXCLUDE                = 
+
+# The EXCLUDE_SYMLINKS tag can be used select whether or not files or 
+# directories that are symbolic links (a Unix filesystem feature) are excluded 
+# from the input.
+
+EXCLUDE_SYMLINKS       = NO
+
+# If the value of the INPUT tag contains directories, you can use the 
+# EXCLUDE_PATTERNS tag to specify one or more wildcard patterns to exclude 
+# certain files from those directories. Note that the wildcards are matched 
+# against the file with absolute path, so to exclude all test directories 
+# for example use the pattern */test/*
+
+EXCLUDE_PATTERNS       = 
+
+# The EXCLUDE_SYMBOLS tag can be used to specify one or more symbol names 
+# (namespaces, classes, functions, etc.) that should be excluded from the 
+# output. The symbol name can be a fully qualified name, a word, or if the 
+# wildcard * is used, a substring. Examples: ANamespace, AClass, 
+# AClass::ANamespace, ANamespace::*Test
+
+EXCLUDE_SYMBOLS        = 
+
+# The EXAMPLE_PATH tag can be used to specify one or more files or 
+# directories that contain example code fragments that are included (see 
+# the \include command).
+
+EXAMPLE_PATH           = 
+
+# If the value of the EXAMPLE_PATH tag contains directories, you can use the 
+# EXAMPLE_PATTERNS tag to specify one or more wildcard pattern (like *.cpp 
+# and *.h) to filter out the source-files in the directories. If left 
+# blank all files are included.
+
+EXAMPLE_PATTERNS       = 
+
+# If the EXAMPLE_RECURSIVE tag is set to YES then subdirectories will be 
+# searched for input files to be used with the \include or \dontinclude 
+# commands irrespective of the value of the RECURSIVE tag. 
+# Possible values are YES and NO. If left blank NO is used.
+
+EXAMPLE_RECURSIVE      = NO
+
+# The IMAGE_PATH tag can be used to specify one or more files or 
+# directories that contain image that are included in the documentation (see 
+# the \image command).
+
+IMAGE_PATH             = 
+
+# The INPUT_FILTER tag can be used to specify a program that doxygen should 
+# invoke to filter for each input file. Doxygen will invoke the filter program 
+# by executing (via popen()) the command <filter> <input-file>, where <filter> 
+# is the value of the INPUT_FILTER tag, and <input-file> is the name of an 
+# input file. Doxygen will then use the output that the filter program writes 
+# to standard output.  If FILTER_PATTERNS is specified, this tag will be 
+# ignored.
+
+INPUT_FILTER           = 
+
+# The FILTER_PATTERNS tag can be used to specify filters on a per file pattern 
+# basis.  Doxygen will compare the file name with each pattern and apply the 
+# filter if there is a match.  The filters are a list of the form: 
+# pattern=filter (like *.cpp=my_cpp_filter). See INPUT_FILTER for further 
+# info on how filters are used. If FILTER_PATTERNS is empty, INPUT_FILTER 
+# is applied to all files.
+
+FILTER_PATTERNS        = 
+
+# If the FILTER_SOURCE_FILES tag is set to YES, the input filter (if set using 
+# INPUT_FILTER) will be used to filter the input files when producing source 
+# files to browse (i.e. when SOURCE_BROWSER is set to YES).
+
+FILTER_SOURCE_FILES    = NO
+
+#---------------------------------------------------------------------------
+# configuration options related to source browsing
+#---------------------------------------------------------------------------
+
+# If the SOURCE_BROWSER tag is set to YES then a list of source files will 
+# be generated. Documented entities will be cross-referenced with these sources. 
+# Note: To get rid of all source code in the generated output, make sure also 
+# VERBATIM_HEADERS is set to NO.
+
+SOURCE_BROWSER         = YES
+
+# Setting the INLINE_SOURCES tag to YES will include the body 
+# of functions and classes directly in the documentation.
+
+INLINE_SOURCES         = NO
+
+# Setting the STRIP_CODE_COMMENTS tag to YES (the default) will instruct 
+# doxygen to hide any special comment blocks from generated source code 
+# fragments. Normal C and C++ comments will always remain visible.
+
+STRIP_CODE_COMMENTS    = YES
+
+# If the REFERENCED_BY_RELATION tag is set to YES 
+# then for each documented function all documented 
+# functions referencing it will be listed.
+
+REFERENCED_BY_RELATION = NO
+
+# If the REFERENCES_RELATION tag is set to YES 
+# then for each documented function all documented entities 
+# called/used by that function will be listed.
+
+REFERENCES_RELATION    = NO
+
+# If the REFERENCES_LINK_SOURCE tag is set to YES (the default)
+# and SOURCE_BROWSER tag is set to YES, then the hyperlinks from
+# functions in REFERENCES_RELATION and REFERENCED_BY_RELATION lists will
+# link to the source code.  Otherwise they will link to the documentstion.
+
+REFERENCES_LINK_SOURCE = YES
+
+# If the USE_HTAGS tag is set to YES then the references to source code 
+# will point to the HTML generated by the htags(1) tool instead of doxygen 
+# built-in source browser. The htags tool is part of GNU's global source 
+# tagging system (see http://www.gnu.org/software/global/global.html). You 
+# will need version 4.8.6 or higher.
+
+USE_HTAGS              = NO
+
+# If the VERBATIM_HEADERS tag is set to YES (the default) then Doxygen 
+# will generate a verbatim copy of the header file for each class for 
+# which an include is specified. Set to NO to disable this.
+
+VERBATIM_HEADERS       = YES
+
+#---------------------------------------------------------------------------
+# configuration options related to the alphabetical class index
+#---------------------------------------------------------------------------
+
+# If the ALPHABETICAL_INDEX tag is set to YES, an alphabetical index 
+# of all compounds will be generated. Enable this if the project 
+# contains a lot of classes, structs, unions or interfaces.
+
+ALPHABETICAL_INDEX     = NO
+
+# If the alphabetical index is enabled (see ALPHABETICAL_INDEX) then 
+# the COLS_IN_ALPHA_INDEX tag can be used to specify the number of columns 
+# in which this list will be split (can be a number in the range [1..20])
+
+COLS_IN_ALPHA_INDEX    = 5
+
+# In case all classes in a project start with a common prefix, all 
+# classes will be put under the same header in the alphabetical index. 
+# The IGNORE_PREFIX tag can be used to specify one or more prefixes that 
+# should be ignored while generating the index headers.
+
+IGNORE_PREFIX          = 
+
+#---------------------------------------------------------------------------
+# configuration options related to the HTML output
+#---------------------------------------------------------------------------
+
+# If the GENERATE_HTML tag is set to YES (the default) Doxygen will 
+# generate HTML output.
+
+GENERATE_HTML          = YES
+
+# The HTML_OUTPUT tag is used to specify where the HTML docs will be put. 
+# If a relative path is entered the value of OUTPUT_DIRECTORY will be 
+# put in front of it. If left blank `html' will be used as the default path.
+
+HTML_OUTPUT            = html
+
+# The HTML_FILE_EXTENSION tag can be used to specify the file extension for 
+# each generated HTML page (for example: .htm,.php,.asp). If it is left blank 
+# doxygen will generate files with .html extension.
+
+HTML_FILE_EXTENSION    = .html
+
+# The HTML_HEADER tag can be used to specify a personal HTML header for 
+# each generated HTML page. If it is left blank doxygen will generate a 
+# standard header.
+
+HTML_HEADER            = docs/api_header.html
+
+# The HTML_FOOTER tag can be used to specify a personal HTML footer for 
+# each generated HTML page. If it is left blank doxygen will generate a 
+# standard footer.
+
+HTML_FOOTER            =
+
+# The HTML_STYLESHEET tag can be used to specify a user-defined cascading 
+# style sheet that is used by each HTML page. It can be used to 
+# fine-tune the look of the HTML output. If the tag is left blank doxygen 
+# will generate a default style sheet. Note that doxygen will try to copy 
+# the style sheet file to the HTML output directory, so don't put your own 
+# stylesheet in the HTML output directory as well, or it will be erased!
+
+HTML_STYLESHEET        = 
+
+# If the HTML_ALIGN_MEMBERS tag is set to YES, the members of classes, 
+# files or namespaces will be aligned in HTML using tables. If set to 
+# NO a bullet list will be used.
+
+HTML_ALIGN_MEMBERS     = YES
+
+# If the GENERATE_HTMLHELP tag is set to YES, additional index files 
+# will be generated that can be used as input for tools like the 
+# Microsoft HTML help workshop to generate a compiled HTML help file (.chm) 
+# of the generated HTML documentation.
+
+GENERATE_HTMLHELP      = NO
+
+# If the GENERATE_DOCSET tag is set to YES, additional index files 
+# will be generated that can be used as input for Apple's Xcode 3 
+# integrated development environment, introduced with OSX 10.5 (Leopard). 
+# To create a documentation set, doxygen will generate a Makefile in the 
+# HTML output directory. Running make will produce the docset in that 
+# directory and running "make install" will install the docset in 
+# ~/Library/Developer/Shared/Documentation/DocSets so that Xcode will find 
+# it at startup.
+
+GENERATE_DOCSET        = NO
+
+# When GENERATE_DOCSET tag is set to YES, this tag determines the name of the 
+# feed. A documentation feed provides an umbrella under which multiple 
+# documentation sets from a single provider (such as a company or product suite) 
+# can be grouped.
+
+DOCSET_FEEDNAME        = "Doxygen generated docs"
+
+# When GENERATE_DOCSET tag is set to YES, this tag specifies a string that 
+# should uniquely identify the documentation set bundle. This should be a 
+# reverse domain-name style string, e.g. com.mycompany.MyDocSet. Doxygen 
+# will append .docset to the name.
+
+DOCSET_BUNDLE_ID       = org.doxygen.Project
+
+# If the HTML_DYNAMIC_SECTIONS tag is set to YES then the generated HTML 
+# documentation will contain sections that can be hidden and shown after the 
+# page has loaded. For this to work a browser that supports 
+# JavaScript and DHTML is required (for instance Mozilla 1.0+, Firefox 
+# Netscape 6.0+, Internet explorer 5.0+, Konqueror, or Safari).
+
+HTML_DYNAMIC_SECTIONS  = NO
+
+# If the GENERATE_HTMLHELP tag is set to YES, the CHM_FILE tag can 
+# be used to specify the file name of the resulting .chm file. You 
+# can add a path in front of the file if the result should not be 
+# written to the html output directory.
+
+CHM_FILE               = 
+
+# If the GENERATE_HTMLHELP tag is set to YES, the HHC_LOCATION tag can 
+# be used to specify the location (absolute path including file name) of 
+# the HTML help compiler (hhc.exe). If non-empty doxygen will try to run 
+# the HTML help compiler on the generated index.hhp.
+
+HHC_LOCATION           = 
+
+# If the GENERATE_HTMLHELP tag is set to YES, the GENERATE_CHI flag 
+# controls if a separate .chi index file is generated (YES) or that 
+# it should be included in the master .chm file (NO).
+
+GENERATE_CHI           = NO
+
+# If the GENERATE_HTMLHELP tag is set to YES, the CHM_INDEX_ENCODING
+# is used to encode HtmlHelp index (hhk), content (hhc) and project file
+# content.
+
+CHM_INDEX_ENCODING     = 
+
+# If the GENERATE_HTMLHELP tag is set to YES, the BINARY_TOC flag 
+# controls whether a binary table of contents is generated (YES) or a 
+# normal table of contents (NO) in the .chm file.
+
+BINARY_TOC             = NO
+
+# The TOC_EXPAND flag can be set to YES to add extra items for group members 
+# to the contents of the HTML help documentation and to the tree view.
+
+TOC_EXPAND             = NO
+
+# The DISABLE_INDEX tag can be used to turn on/off the condensed index at 
+# top of each HTML page. The value NO (the default) enables the index and 
+# the value YES disables it.
+
+DISABLE_INDEX          = NO
+
+# This tag can be used to set the number of enum values (range [1..20]) 
+# that doxygen will group on one line in the generated HTML documentation.
+
+ENUM_VALUES_PER_LINE   = 4
+
+# The GENERATE_TREEVIEW tag is used to specify whether a tree-like index
+# structure should be generated to display hierarchical information.
+# If the tag value is set to FRAME, a side panel will be generated
+# containing a tree-like index structure (just like the one that 
+# is generated for HTML Help). For this to work a browser that supports 
+# JavaScript, DHTML, CSS and frames is required (for instance Mozilla 1.0+, 
+# Netscape 6.0+, Internet explorer 5.0+, or Konqueror). Windows users are 
+# probably better off using the HTML help feature. Other possible values 
+# for this tag are: HIERARCHIES, which will generate the Groups, Directories,
+# and Class Hiererachy pages using a tree view instead of an ordered list;
+# ALL, which combines the behavior of FRAME and HIERARCHIES; and NONE, which
+# disables this behavior completely. For backwards compatibility with previous
+# releases of Doxygen, the values YES and NO are equivalent to FRAME and NONE
+# respectively.
+
+GENERATE_TREEVIEW      = NONE
+
+# If the treeview is enabled (see GENERATE_TREEVIEW) then this tag can be 
+# used to set the initial width (in pixels) of the frame in which the tree 
+# is shown.
+
+TREEVIEW_WIDTH         = 250
+
+# Use this tag to change the font size of Latex formulas included 
+# as images in the HTML documentation. The default is 10. Note that 
+# when you change the font size after a successful doxygen run you need 
+# to manually remove any form_*.png images from the HTML output directory 
+# to force them to be regenerated.
+
+FORMULA_FONTSIZE       = 10
+
+#---------------------------------------------------------------------------
+# configuration options related to the LaTeX output
+#---------------------------------------------------------------------------
+
+# If the GENERATE_LATEX tag is set to YES (the default) Doxygen will 
+# generate Latex output.
+
+GENERATE_LATEX         = NO
+
+# The LATEX_OUTPUT tag is used to specify where the LaTeX docs will be put. 
+# If a relative path is entered the value of OUTPUT_DIRECTORY will be 
+# put in front of it. If left blank `latex' will be used as the default path.
+
+LATEX_OUTPUT           = latex
+
+# The LATEX_CMD_NAME tag can be used to specify the LaTeX command name to be 
+# invoked. If left blank `latex' will be used as the default command name.
+
+LATEX_CMD_NAME         = latex
+
+# The MAKEINDEX_CMD_NAME tag can be used to specify the command name to 
+# generate index for LaTeX. If left blank `makeindex' will be used as the 
+# default command name.
+
+MAKEINDEX_CMD_NAME     = makeindex
+
+# If the COMPACT_LATEX tag is set to YES Doxygen generates more compact 
+# LaTeX documents. This may be useful for small projects and may help to 
+# save some trees in general.
+
+COMPACT_LATEX          = NO
+
+# The PAPER_TYPE tag can be used to set the paper type that is used 
+# by the printer. Possible values are: a4, a4wide, letter, legal and 
+# executive. If left blank a4wide will be used.
+
+PAPER_TYPE             = a4wide
+
+# The EXTRA_PACKAGES tag can be to specify one or more names of LaTeX 
+# packages that should be included in the LaTeX output.
+
+EXTRA_PACKAGES         = 
+
+# The LATEX_HEADER tag can be used to specify a personal LaTeX header for 
+# the generated latex document. The header should contain everything until 
+# the first chapter. If it is left blank doxygen will generate a 
+# standard header. Notice: only use this tag if you know what you are doing!
+
+LATEX_HEADER           = 
+
+# If the PDF_HYPERLINKS tag is set to YES, the LaTeX that is generated 
+# is prepared for conversion to pdf (using ps2pdf). The pdf file will 
+# contain links (just like the HTML output) instead of page references 
+# This makes the output suitable for online browsing using a pdf viewer.
+
+PDF_HYPERLINKS         = YES
+
+# If the USE_PDFLATEX tag is set to YES, pdflatex will be used instead of 
+# plain latex in the generated Makefile. Set this option to YES to get a 
+# higher quality PDF documentation.
+
+USE_PDFLATEX           = YES
+
+# If the LATEX_BATCHMODE tag is set to YES, doxygen will add the \\batchmode. 
+# command to the generated LaTeX files. This will instruct LaTeX to keep 
+# running if errors occur, instead of asking the user for help. 
+# This option is also used when generating formulas in HTML.
+
+LATEX_BATCHMODE        = NO
+
+# If LATEX_HIDE_INDICES is set to YES then doxygen will not 
+# include the index chapters (such as File Index, Compound Index, etc.) 
+# in the output.
+
+LATEX_HIDE_INDICES     = NO
+
+#---------------------------------------------------------------------------
+# configuration options related to the RTF output
+#---------------------------------------------------------------------------
+
+# If the GENERATE_RTF tag is set to YES Doxygen will generate RTF output 
+# The RTF output is optimized for Word 97 and may not look very pretty with 
+# other RTF readers or editors.
+
+GENERATE_RTF           = NO
+
+# The RTF_OUTPUT tag is used to specify where the RTF docs will be put. 
+# If a relative path is entered the value of OUTPUT_DIRECTORY will be 
+# put in front of it. If left blank `rtf' will be used as the default path.
+
+RTF_OUTPUT             = rtf
+
+# If the COMPACT_RTF tag is set to YES Doxygen generates more compact 
+# RTF documents. This may be useful for small projects and may help to 
+# save some trees in general.
+
+COMPACT_RTF            = NO
+
+# If the RTF_HYPERLINKS tag is set to YES, the RTF that is generated 
+# will contain hyperlink fields. The RTF file will 
+# contain links (just like the HTML output) instead of page references. 
+# This makes the output suitable for online browsing using WORD or other 
+# programs which support those fields. 
+# Note: wordpad (write) and others do not support links.
+
+RTF_HYPERLINKS         = NO
+
+# Load stylesheet definitions from file. Syntax is similar to doxygen's 
+# config file, i.e. a series of assignments. You only have to provide 
+# replacements, missing definitions are set to their default value.
+
+RTF_STYLESHEET_FILE    = 
+
+# Set optional variables used in the generation of an rtf document. 
+# Syntax is similar to doxygen's config file.
+
+RTF_EXTENSIONS_FILE    = 
+
+#---------------------------------------------------------------------------
+# configuration options related to the man page output
+#---------------------------------------------------------------------------
+
+# If the GENERATE_MAN tag is set to YES (the default) Doxygen will 
+# generate man pages
+
+GENERATE_MAN           = NO
+
+# The MAN_OUTPUT tag is used to specify where the man pages will be put. 
+# If a relative path is entered the value of OUTPUT_DIRECTORY will be 
+# put in front of it. If left blank `man' will be used as the default path.
+
+MAN_OUTPUT             = man
+
+# The MAN_EXTENSION tag determines the extension that is added to 
+# the generated man pages (default is the subroutine's section .3)
+
+MAN_EXTENSION          = .3
+
+# If the MAN_LINKS tag is set to YES and Doxygen generates man output, 
+# then it will generate one additional man file for each entity 
+# documented in the real man page(s). These additional files 
+# only source the real man page, but without them the man command 
+# would be unable to find the correct page. The default is NO.
+
+MAN_LINKS              = NO
+
+#---------------------------------------------------------------------------
+# configuration options related to the XML output
+#---------------------------------------------------------------------------
+
+# If the GENERATE_XML tag is set to YES Doxygen will 
+# generate an XML file that captures the structure of 
+# the code including all documentation.
+
+GENERATE_XML           = NO
+
+# The XML_OUTPUT tag is used to specify where the XML pages will be put. 
+# If a relative path is entered the value of OUTPUT_DIRECTORY will be 
+# put in front of it. If left blank `xml' will be used as the default path.
+
+XML_OUTPUT             = xml
+
+# The XML_SCHEMA tag can be used to specify an XML schema, 
+# which can be used by a validating XML parser to check the 
+# syntax of the XML files.
+
+XML_SCHEMA             = 
+
+# The XML_DTD tag can be used to specify an XML DTD, 
+# which can be used by a validating XML parser to check the 
+# syntax of the XML files.
+
+XML_DTD                = 
+
+# If the XML_PROGRAMLISTING tag is set to YES Doxygen will 
+# dump the program listings (including syntax highlighting 
+# and cross-referencing information) to the XML output. Note that 
+# enabling this will significantly increase the size of the XML output.
+
+XML_PROGRAMLISTING     = YES
+
+#---------------------------------------------------------------------------
+# configuration options for the AutoGen Definitions output
+#---------------------------------------------------------------------------
+
+# If the GENERATE_AUTOGEN_DEF tag is set to YES Doxygen will 
+# generate an AutoGen Definitions (see autogen.sf.net) file 
+# that captures the structure of the code including all 
+# documentation. Note that this feature is still experimental 
+# and incomplete at the moment.
+
+GENERATE_AUTOGEN_DEF   = NO
+
+#---------------------------------------------------------------------------
+# configuration options related to the Perl module output
+#---------------------------------------------------------------------------
+
+# If the GENERATE_PERLMOD tag is set to YES Doxygen will 
+# generate a Perl module file that captures the structure of 
+# the code including all documentation. Note that this 
+# feature is still experimental and incomplete at the 
+# moment.
+
+GENERATE_PERLMOD       = NO
+
+# If the PERLMOD_LATEX tag is set to YES Doxygen will generate 
+# the necessary Makefile rules, Perl scripts and LaTeX code to be able 
+# to generate PDF and DVI output from the Perl module output.
+
+PERLMOD_LATEX          = NO
+
+# If the PERLMOD_PRETTY tag is set to YES the Perl module output will be 
+# nicely formatted so it can be parsed by a human reader.  This is useful 
+# if you want to understand what is going on.  On the other hand, if this 
+# tag is set to NO the size of the Perl module output will be much smaller 
+# and Perl will parse it just the same.
+
+PERLMOD_PRETTY         = YES
+
+# The names of the make variables in the generated doxyrules.make file 
+# are prefixed with the string contained in PERLMOD_MAKEVAR_PREFIX. 
+# This is useful so different doxyrules.make files included by the same 
+# Makefile don't overwrite each other's variables.
+
+PERLMOD_MAKEVAR_PREFIX = 
+
+#---------------------------------------------------------------------------
+# Configuration options related to the preprocessor   
+#---------------------------------------------------------------------------
+
+# If the ENABLE_PREPROCESSING tag is set to YES (the default) Doxygen will 
+# evaluate all C-preprocessor directives found in the sources and include 
+# files.
+
+ENABLE_PREPROCESSING   = YES
+
+# If the MACRO_EXPANSION tag is set to YES Doxygen will expand all macro 
+# names in the source code. If set to NO (the default) only conditional 
+# compilation will be performed. Macro expansion can be done in a controlled 
+# way by setting EXPAND_ONLY_PREDEF to YES.
+
+MACRO_EXPANSION        = YES
+
+# If the EXPAND_ONLY_PREDEF and MACRO_EXPANSION tags are both set to YES 
+# then the macro expansion is limited to the macros specified with the 
+# PREDEFINED and EXPAND_AS_DEFINED tags.
+
+EXPAND_ONLY_PREDEF     = YES
+
+# If the SEARCH_INCLUDES tag is set to YES (the default) the includes files 
+# in the INCLUDE_PATH (see below) will be search if a #include is found.
+
+SEARCH_INCLUDES        = YES
+
+# The INCLUDE_PATH tag can be used to specify one or more directories that 
+# contain include files that are not input files but should be processed by 
+# the preprocessor.
+
+INCLUDE_PATH           = 
+
+# You can use the INCLUDE_FILE_PATTERNS tag to specify one or more wildcard 
+# patterns (like *.h and *.hpp) to filter out the header-files in the 
+# directories. If left blank, the patterns specified with FILE_PATTERNS will 
+# be used.
+
+INCLUDE_FILE_PATTERNS  = 
+
+# The PREDEFINED tag can be used to specify one or more macro names that 
+# are defined before the preprocessor is started (similar to the -D option of 
+# gcc). The argument of the tag is a list of macros of the form: name 
+# or name=definition (no spaces). If the definition and the = are 
+# omitted =1 is assumed. To prevent a macro definition from being 
+# undefined via #undef or recursively expanded use the := operator 
+# instead of the = operator.
+
+PREDEFINED             = DRIZZLE_API=
+
+# If the MACRO_EXPANSION and EXPAND_ONLY_PREDEF tags are set to YES then 
+# this tag can be used to specify a list of macro names that should be expanded. 
+# The macro definition that is found in the sources will be used. 
+# Use the PREDEFINED tag if you want to use a different macro definition.
+
+EXPAND_AS_DEFINED      = 
+
+# If the SKIP_FUNCTION_MACROS tag is set to YES (the default) then 
+# doxygen's preprocessor will remove all function-like macros that are alone 
+# on a line, have an all uppercase name, and do not end with a semicolon. Such 
+# function macros are typically used for boiler-plate code, and will confuse 
+# the parser if not removed.
+
+SKIP_FUNCTION_MACROS   = YES
+
+#---------------------------------------------------------------------------
+# Configuration::additions related to external references   
+#---------------------------------------------------------------------------
+
+# The TAGFILES option can be used to specify one or more tagfiles. 
+# Optionally an initial location of the external documentation 
+# can be added for each tagfile. The format of a tag file without 
+# this location is as follows: 
+#   TAGFILES = file1 file2 ... 
+# Adding location for the tag files is done as follows: 
+#   TAGFILES = file1=loc1 "file2 = loc2" ... 
+# where "loc1" and "loc2" can be relative or absolute paths or 
+# URLs. If a location is present for each tag, the installdox tool 
+# does not have to be run to correct the links.
+# Note that each tag file must have a unique name
+# (where the name does NOT include the path)
+# If a tag file is not located in the directory in which doxygen 
+# is run, you must also specify the path to the tagfile here.
+
+TAGFILES               = 
+
+# When a file name is specified after GENERATE_TAGFILE, doxygen will create 
+# a tag file that is based on the input files it reads.
+
+GENERATE_TAGFILE       = 
+
+# If the ALLEXTERNALS tag is set to YES all external classes will be listed 
+# in the class index. If set to NO only the inherited external classes 
+# will be listed.
+
+ALLEXTERNALS           = NO
+
+# If the EXTERNAL_GROUPS tag is set to YES all external groups will be listed 
+# in the modules index. If set to NO, only the current project's groups will 
+# be listed.
+
+EXTERNAL_GROUPS        = YES
+
+# The PERL_PATH should be the absolute path and name of the perl script 
+# interpreter (i.e. the result of `which perl').
+
+PERL_PATH              = /usr/bin/perl
+
+#---------------------------------------------------------------------------
+# Configuration options related to the dot tool   
+#---------------------------------------------------------------------------
+
+# If the CLASS_DIAGRAMS tag is set to YES (the default) Doxygen will 
+# generate a inheritance diagram (in HTML, RTF and LaTeX) for classes with base 
+# or super classes. Setting the tag to NO turns the diagrams off. Note that 
+# this option is superseded by the HAVE_DOT option below. This is only a 
+# fallback. It is recommended to install and use dot, since it yields more 
+# powerful graphs.
+
+CLASS_DIAGRAMS         = YES
+
+# You can define message sequence charts within doxygen comments using the \msc 
+# command. Doxygen will then run the mscgen tool (see 
+# http://www.mcternan.me.uk/mscgen/) to produce the chart and insert it in the 
+# documentation. The MSCGEN_PATH tag allows you to specify the directory where 
+# the mscgen tool resides. If left empty the tool is assumed to be found in the 
+# default search path.
+
+MSCGEN_PATH            = 
+
+# If set to YES, the inheritance and collaboration graphs will hide 
+# inheritance and usage relations if the target is undocumented 
+# or is not a class.
+
+HIDE_UNDOC_RELATIONS   = YES
+
+# If you set the HAVE_DOT tag to YES then doxygen will assume the dot tool is 
+# available from the path. This tool is part of Graphviz, a graph visualization 
+# toolkit from AT&T and Lucent Bell Labs. The other options in this section 
+# have no effect if this option is set to NO (the default)
+
+HAVE_DOT               = NO
+
+# By default doxygen will write a font called FreeSans.ttf to the output 
+# directory and reference it in all dot files that doxygen generates. This 
+# font does not include all possible unicode characters however, so when you need 
+# these (or just want a differently looking font) you can specify the font name 
+# using DOT_FONTNAME. You need need to make sure dot is able to find the font, 
+# which can be done by putting it in a standard location or by setting the 
+# DOTFONTPATH environment variable or by setting DOT_FONTPATH to the directory 
+# containing the font.
+
+DOT_FONTNAME           = FreeSans
+
+# By default doxygen will tell dot to use the output directory to look for the 
+# FreeSans.ttf font (which doxygen will put there itself). If you specify a 
+# different font using DOT_FONTNAME you can set the path where dot 
+# can find it using this tag.
+
+DOT_FONTPATH           = 
+
+# If the CLASS_GRAPH and HAVE_DOT tags are set to YES then doxygen 
+# will generate a graph for each documented class showing the direct and 
+# indirect inheritance relations. Setting this tag to YES will force the 
+# the CLASS_DIAGRAMS tag to NO.
+
+CLASS_GRAPH            = YES
+
+# If the COLLABORATION_GRAPH and HAVE_DOT tags are set to YES then doxygen 
+# will generate a graph for each documented class showing the direct and 
+# indirect implementation dependencies (inheritance, containment, and 
+# class references variables) of the class with other documented classes.
+
+COLLABORATION_GRAPH    = YES
+
+# If the GROUP_GRAPHS and HAVE_DOT tags are set to YES then doxygen 
+# will generate a graph for groups, showing the direct groups dependencies
+
+GROUP_GRAPHS           = YES
+
+# If the UML_LOOK tag is set to YES doxygen will generate inheritance and 
+# collaboration diagrams in a style similar to the OMG's Unified Modeling 
+# Language.
+
+UML_LOOK               = NO
+
+# If set to YES, the inheritance and collaboration graphs will show the 
+# relations between templates and their instances.
+
+TEMPLATE_RELATIONS     = NO
+
+# If the ENABLE_PREPROCESSING, SEARCH_INCLUDES, INCLUDE_GRAPH, and HAVE_DOT 
+# tags are set to YES then doxygen will generate a graph for each documented 
+# file showing the direct and indirect include dependencies of the file with 
+# other documented files.
+
+INCLUDE_GRAPH          = YES
+
+# If the ENABLE_PREPROCESSING, SEARCH_INCLUDES, INCLUDED_BY_GRAPH, and 
+# HAVE_DOT tags are set to YES then doxygen will generate a graph for each 
+# documented header file showing the documented files that directly or 
+# indirectly include this file.
+
+INCLUDED_BY_GRAPH      = YES
+
+# If the CALL_GRAPH and HAVE_DOT options are set to YES then 
+# doxygen will generate a call dependency graph for every global function 
+# or class method. Note that enabling this option will significantly increase 
+# the time of a run. So in most cases it will be better to enable call graphs 
+# for selected functions only using the \callgraph command.
+
+CALL_GRAPH             = NO
+
+# If the CALLER_GRAPH and HAVE_DOT tags are set to YES then 
+# doxygen will generate a caller dependency graph for every global function 
+# or class method. Note that enabling this option will significantly increase 
+# the time of a run. So in most cases it will be better to enable caller 
+# graphs for selected functions only using the \callergraph command.
+
+CALLER_GRAPH           = NO
+
+# If the GRAPHICAL_HIERARCHY and HAVE_DOT tags are set to YES then doxygen 
+# will graphical hierarchy of all classes instead of a textual one.
+
+GRAPHICAL_HIERARCHY    = YES
+
+# If the DIRECTORY_GRAPH, SHOW_DIRECTORIES and HAVE_DOT tags are set to YES 
+# then doxygen will show the dependencies a directory has on other directories 
+# in a graphical way. The dependency relations are determined by the #include
+# relations between the files in the directories.
+
+DIRECTORY_GRAPH        = YES
+
+# The DOT_IMAGE_FORMAT tag can be used to set the image format of the images 
+# generated by dot. Possible values are png, jpg, or gif
+# If left blank png will be used.
+
+DOT_IMAGE_FORMAT       = png
+
+# The tag DOT_PATH can be used to specify the path where the dot tool can be 
+# found. If left blank, it is assumed the dot tool can be found in the path.
+
+DOT_PATH               = 
+
+# The DOTFILE_DIRS tag can be used to specify one or more directories that 
+# contain dot files that are included in the documentation (see the 
+# \dotfile command).
+
+DOTFILE_DIRS           = 
+
+# The DOT_GRAPH_MAX_NODES tag can be used to set the maximum number of 
+# nodes that will be shown in the graph. If the number of nodes in a graph 
+# becomes larger than this value, doxygen will truncate the graph, which is 
+# visualized by representing a node as a red box. Note that doxygen if the 
+# number of direct children of the root node in a graph is already larger than 
+# DOT_GRAPH_MAX_NODES then the graph will not be shown at all. Also note 
+# that the size of a graph can be further restricted by MAX_DOT_GRAPH_DEPTH.
+
+DOT_GRAPH_MAX_NODES    = 50
+
+# The MAX_DOT_GRAPH_DEPTH tag can be used to set the maximum depth of the 
+# graphs generated by dot. A depth value of 3 means that only nodes reachable 
+# from the root by following a path via at most 3 edges will be shown. Nodes 
+# that lay further from the root node will be omitted. Note that setting this 
+# option to 1 or 2 may greatly reduce the computation time needed for large 
+# code bases. Also note that the size of a graph can be further restricted by 
+# DOT_GRAPH_MAX_NODES. Using a depth of 0 means no depth restriction.
+
+MAX_DOT_GRAPH_DEPTH    = 0
+
+# Set the DOT_TRANSPARENT tag to YES to generate images with a transparent 
+# background. This is enabled by default, which results in a transparent 
+# background. Warning: Depending on the platform used, enabling this option 
+# may lead to badly anti-aliased labels on the edges of a graph (i.e. they 
+# become hard to read).
+
+DOT_TRANSPARENT        = YES
+
+# Set the DOT_MULTI_TARGETS tag to YES allow dot to generate multiple output 
+# files in one run (i.e. multiple -o and -T options on the command line). This 
+# makes dot run faster, but since only newer versions of dot (>1.8.10) 
+# support this, this feature is disabled by default.
+
+DOT_MULTI_TARGETS      = NO
+
+# If the GENERATE_LEGEND tag is set to YES (the default) Doxygen will 
+# generate a legend page explaining the meaning of the various boxes and 
+# arrows in the dot generated graphs.
+
+GENERATE_LEGEND        = YES
+
+# If the DOT_CLEANUP tag is set to YES (the default) Doxygen will 
+# remove the intermediate dot files that are used to generate 
+# the various graphs.
+
+DOT_CLEANUP            = YES
+
+#---------------------------------------------------------------------------
+# Configuration::additions related to the search engine   
+#---------------------------------------------------------------------------
+
+# The SEARCHENGINE tag specifies whether or not a search engine should be 
+# used. If set to NO the values of all tags below this one will be ignored.
+
+SEARCHENGINE           = NO

=== added file 'docs/Doxyfile.dev'
--- docs/Doxyfile.dev	1970-01-01 00:00:00 +0000
+++ docs/Doxyfile.dev	2010-08-12 20:27:32 +0000
@@ -0,0 +1,1434 @@
+# Doxyfile 1.5.6
+
+# This file describes the settings to be used by the documentation system
+# doxygen (www.doxygen.org) for a project
+#
+# All text after a hash (#) is considered a comment and will be ignored
+# The format is:
+#       TAG = value [value, ...]
+# For lists items can also be appended using:
+#       TAG += value [value, ...]
+# Values that contain spaces should be placed between quotes (" ")
+
+#---------------------------------------------------------------------------
+# Project related configuration options
+#---------------------------------------------------------------------------
+
+# This tag specifies the encoding used for all characters in the config file 
+# that follow. The default is UTF-8 which is also the encoding used for all 
+# text before the first occurrence of this tag. Doxygen uses libiconv (or the 
+# iconv built into libc) for the transcoding. See 
+# http://www.gnu.org/software/libiconv for the list of possible encodings.
+
+DOXYFILE_ENCODING      = UTF-8
+
+# The PROJECT_NAME tag is a single word (or a sequence of words surrounded 
+# by quotes) that should identify the project.
+
+PROJECT_NAME           = 
+
+# The PROJECT_NUMBER tag can be used to enter a project or revision number. 
+# This could be handy for archiving the generated documentation or 
+# if some version control system is used.
+
+PROJECT_NUMBER         = 
+
+# The OUTPUT_DIRECTORY tag is used to specify the (relative or absolute) 
+# base path where the generated documentation will be put. 
+# If a relative path is entered, it will be relative to the location 
+# where doxygen was started. If left blank the current directory will be used.
+
+OUTPUT_DIRECTORY       = docs/dev
+
+# If the CREATE_SUBDIRS tag is set to YES, then doxygen will create 
+# 4096 sub-directories (in 2 levels) under the output directory of each output 
+# format and will distribute the generated files over these directories. 
+# Enabling this option can be useful when feeding doxygen a huge amount of 
+# source files, where putting all generated files in the same directory would 
+# otherwise cause performance problems for the file system.
+
+CREATE_SUBDIRS         = NO
+
+# The OUTPUT_LANGUAGE tag is used to specify the language in which all 
+# documentation generated by doxygen is written. Doxygen will use this 
+# information to generate all constant output in the proper language. 
+# The default language is English, other supported languages are: 
+# Afrikaans, Arabic, Brazilian, Catalan, Chinese, Chinese-Traditional, 
+# Croatian, Czech, Danish, Dutch, Farsi, Finnish, French, German, Greek, 
+# Hungarian, Italian, Japanese, Japanese-en (Japanese with English messages), 
+# Korean, Korean-en, Lithuanian, Norwegian, Macedonian, Persian, Polish, 
+# Portuguese, Romanian, Russian, Serbian, Slovak, Slovene, Spanish, Swedish, 
+# and Ukrainian.
+
+OUTPUT_LANGUAGE        = English
+
+# If the BRIEF_MEMBER_DESC tag is set to YES (the default) Doxygen will 
+# include brief member descriptions after the members that are listed in 
+# the file and class documentation (similar to JavaDoc). 
+# Set to NO to disable this.
+
+BRIEF_MEMBER_DESC      = YES
+
+# If the REPEAT_BRIEF tag is set to YES (the default) Doxygen will prepend 
+# the brief description of a member or function before the detailed description. 
+# Note: if both HIDE_UNDOC_MEMBERS and BRIEF_MEMBER_DESC are set to NO, the 
+# brief descriptions will be completely suppressed.
+
+REPEAT_BRIEF           = YES
+
+# This tag implements a quasi-intelligent brief description abbreviator 
+# that is used to form the text in various listings. Each string 
+# in this list, if found as the leading text of the brief description, will be 
+# stripped from the text and the result after processing the whole list, is 
+# used as the annotated text. Otherwise, the brief description is used as-is. 
+# If left blank, the following values are used ("$name" is automatically 
+# replaced with the name of the entity): "The $name class" "The $name widget" 
+# "The $name file" "is" "provides" "specifies" "contains" 
+# "represents" "a" "an" "the"
+
+ABBREVIATE_BRIEF       = 
+
+# If the ALWAYS_DETAILED_SEC and REPEAT_BRIEF tags are both set to YES then 
+# Doxygen will generate a detailed section even if there is only a brief 
+# description.
+
+ALWAYS_DETAILED_SEC    = YES
+
+# If the INLINE_INHERITED_MEMB tag is set to YES, doxygen will show all 
+# inherited members of a class in the documentation of that class as if those 
+# members were ordinary class members. Constructors, destructors and assignment 
+# operators of the base classes will not be shown.
+
+INLINE_INHERITED_MEMB  = NO
+
+# If the FULL_PATH_NAMES tag is set to YES then Doxygen will prepend the full 
+# path before files name in the file list and in the header files. If set 
+# to NO the shortest path that makes the file name unique will be used.
+
+FULL_PATH_NAMES        = YES
+
+# If the FULL_PATH_NAMES tag is set to YES then the STRIP_FROM_PATH tag 
+# can be used to strip a user-defined part of the path. Stripping is 
+# only done if one of the specified strings matches the left-hand part of 
+# the path. The tag can be used to show relative paths in the file list. 
+# If left blank the directory from which doxygen is run is used as the 
+# path to strip.
+
+STRIP_FROM_PATH        = 
+
+# The STRIP_FROM_INC_PATH tag can be used to strip a user-defined part of 
+# the path mentioned in the documentation of a class, which tells 
+# the reader which header file to include in order to use a class. 
+# If left blank only the name of the header file containing the class 
+# definition is used. Otherwise one should specify the include paths that 
+# are normally passed to the compiler using the -I flag.
+
+STRIP_FROM_INC_PATH    = 
+
+# If the SHORT_NAMES tag is set to YES, doxygen will generate much shorter 
+# (but less readable) file names. This can be useful is your file systems 
+# doesn't support long names like on DOS, Mac, or CD-ROM.
+
+SHORT_NAMES            = NO
+
+# If the JAVADOC_AUTOBRIEF tag is set to YES then Doxygen 
+# will interpret the first line (until the first dot) of a JavaDoc-style 
+# comment as the brief description. If set to NO, the JavaDoc 
+# comments will behave just like regular Qt-style comments 
+# (thus requiring an explicit @brief command for a brief description.)
+
+JAVADOC_AUTOBRIEF      = NO
+
+# If the QT_AUTOBRIEF tag is set to YES then Doxygen will 
+# interpret the first line (until the first dot) of a Qt-style 
+# comment as the brief description. If set to NO, the comments 
+# will behave just like regular Qt-style comments (thus requiring 
+# an explicit \brief command for a brief description.)
+
+QT_AUTOBRIEF           = NO
+
+# The MULTILINE_CPP_IS_BRIEF tag can be set to YES to make Doxygen 
+# treat a multi-line C++ special comment block (i.e. a block of //! or /// 
+# comments) as a brief description. This used to be the default behaviour. 
+# The new default is to treat a multi-line C++ comment block as a detailed 
+# description. Set this tag to YES if you prefer the old behaviour instead.
+
+MULTILINE_CPP_IS_BRIEF = NO
+
+# If the INHERIT_DOCS tag is set to YES (the default) then an undocumented 
+# member inherits the documentation from any documented member that it 
+# re-implements.
+
+INHERIT_DOCS           = YES
+
+# If the SEPARATE_MEMBER_PAGES tag is set to YES, then doxygen will produce 
+# a new page for each member. If set to NO, the documentation of a member will 
+# be part of the file/class/namespace that contains it.
+
+SEPARATE_MEMBER_PAGES  = NO
+
+# The TAB_SIZE tag can be used to set the number of spaces in a tab. 
+# Doxygen uses this value to replace tabs by spaces in code fragments.
+
+TAB_SIZE               = 8
+
+# This tag can be used to specify a number of aliases that acts 
+# as commands in the documentation. An alias has the form "name=value". 
+# For example adding "sideeffect=\par Side Effects:\n" will allow you to 
+# put the command \sideeffect (or @sideeffect) in the documentation, which 
+# will result in a user-defined paragraph with heading "Side Effects:". 
+# You can put \n's in the value part of an alias to insert newlines.
+
+ALIASES                = 
+
+# Set the OPTIMIZE_OUTPUT_FOR_C tag to YES if your project consists of C 
+# sources only. Doxygen will then generate output that is more tailored for C. 
+# For instance, some of the names that are used will be different. The list 
+# of all members will be omitted, etc.
+
+OPTIMIZE_OUTPUT_FOR_C  = YES
+
+# Set the OPTIMIZE_OUTPUT_JAVA tag to YES if your project consists of Java 
+# sources only. Doxygen will then generate output that is more tailored for 
+# Java. For instance, namespaces will be presented as packages, qualified 
+# scopes will look different, etc.
+
+OPTIMIZE_OUTPUT_JAVA   = NO
+
+# Set the OPTIMIZE_FOR_FORTRAN tag to YES if your project consists of Fortran 
+# sources only. Doxygen will then generate output that is more tailored for 
+# Fortran.
+
+OPTIMIZE_FOR_FORTRAN   = NO
+
+# Set the OPTIMIZE_OUTPUT_VHDL tag to YES if your project consists of VHDL 
+# sources. Doxygen will then generate output that is tailored for 
+# VHDL.
+
+OPTIMIZE_OUTPUT_VHDL   = NO
+
+# If you use STL classes (i.e. std::string, std::vector, etc.) but do not want 
+# to include (a tag file for) the STL sources as input, then you should 
+# set this tag to YES in order to let doxygen match functions declarations and 
+# definitions whose arguments contain STL classes (e.g. func(std::string); v.s. 
+# func(std::string) {}). This also make the inheritance and collaboration 
+# diagrams that involve STL classes more complete and accurate.
+
+BUILTIN_STL_SUPPORT    = NO
+
+# If you use Microsoft's C++/CLI language, you should set this option to YES to
+# enable parsing support.
+
+CPP_CLI_SUPPORT        = NO
+
+# Set the SIP_SUPPORT tag to YES if your project consists of sip sources only. 
+# Doxygen will parse them like normal C++ but will assume all classes use public 
+# instead of private inheritance when no explicit protection keyword is present.
+
+SIP_SUPPORT            = NO
+
+# For Microsoft's IDL there are propget and propput attributes to indicate getter 
+# and setter methods for a property. Setting this option to YES (the default) 
+# will make doxygen to replace the get and set methods by a property in the 
+# documentation. This will only work if the methods are indeed getting or 
+# setting a simple type. If this is not the case, or you want to show the 
+# methods anyway, you should set this option to NO.
+
+IDL_PROPERTY_SUPPORT   = YES
+
+# If member grouping is used in the documentation and the DISTRIBUTE_GROUP_DOC 
+# tag is set to YES, then doxygen will reuse the documentation of the first 
+# member in the group (if any) for the other members of the group. By default 
+# all members of a group must be documented explicitly.
+
+DISTRIBUTE_GROUP_DOC   = NO
+
+# Set the SUBGROUPING tag to YES (the default) to allow class member groups of 
+# the same type (for instance a group of public functions) to be put as a 
+# subgroup of that type (e.g. under the Public Functions section). Set it to 
+# NO to prevent subgrouping. Alternatively, this can be done per class using 
+# the \nosubgrouping command.
+
+SUBGROUPING            = YES
+
+# When TYPEDEF_HIDES_STRUCT is enabled, a typedef of a struct, union, or enum 
+# is documented as struct, union, or enum with the name of the typedef. So 
+# typedef struct TypeS {} TypeT, will appear in the documentation as a struct 
+# with name TypeT. When disabled the typedef will appear as a member of a file, 
+# namespace, or class. And the struct will be named TypeS. This can typically 
+# be useful for C code in case the coding convention dictates that all compound 
+# types are typedef'ed and only the typedef is referenced, never the tag name.
+
+TYPEDEF_HIDES_STRUCT   = YES
+
+#---------------------------------------------------------------------------
+# Build related configuration options
+#---------------------------------------------------------------------------
+
+# If the EXTRACT_ALL tag is set to YES doxygen will assume all entities in 
+# documentation are documented, even if no documentation was available. 
+# Private class members and static file members will be hidden unless 
+# the EXTRACT_PRIVATE and EXTRACT_STATIC tags are set to YES
+
+EXTRACT_ALL            = YES
+
+# If the EXTRACT_PRIVATE tag is set to YES all private members of a class 
+# will be included in the documentation.
+
+EXTRACT_PRIVATE        = YES
+
+# If the EXTRACT_STATIC tag is set to YES all static members of a file 
+# will be included in the documentation.
+
+EXTRACT_STATIC         = YES
+
+# If the EXTRACT_LOCAL_CLASSES tag is set to YES classes (and structs) 
+# defined locally in source files will be included in the documentation. 
+# If set to NO only classes defined in header files are included.
+
+EXTRACT_LOCAL_CLASSES  = YES
+
+# This flag is only useful for Objective-C code. When set to YES local 
+# methods, which are defined in the implementation section but not in 
+# the interface are included in the documentation. 
+# If set to NO (the default) only methods in the interface are included.
+
+EXTRACT_LOCAL_METHODS  = NO
+
+# If this flag is set to YES, the members of anonymous namespaces will be 
+# extracted and appear in the documentation as a namespace called 
+# 'anonymous_namespace{file}', where file will be replaced with the base 
+# name of the file that contains the anonymous namespace. By default 
+# anonymous namespace are hidden.
+
+EXTRACT_ANON_NSPACES   = NO
+
+# If the HIDE_UNDOC_MEMBERS tag is set to YES, Doxygen will hide all 
+# undocumented members of documented classes, files or namespaces. 
+# If set to NO (the default) these members will be included in the 
+# various overviews, but no documentation section is generated. 
+# This option has no effect if EXTRACT_ALL is enabled.
+
+HIDE_UNDOC_MEMBERS     = NO
+
+# If the HIDE_UNDOC_CLASSES tag is set to YES, Doxygen will hide all 
+# undocumented classes that are normally visible in the class hierarchy. 
+# If set to NO (the default) these classes will be included in the various 
+# overviews. This option has no effect if EXTRACT_ALL is enabled.
+
+HIDE_UNDOC_CLASSES     = NO
+
+# If the HIDE_FRIEND_COMPOUNDS tag is set to YES, Doxygen will hide all 
+# friend (class|struct|union) declarations. 
+# If set to NO (the default) these declarations will be included in the 
+# documentation.
+
+HIDE_FRIEND_COMPOUNDS  = NO
+
+# If the HIDE_IN_BODY_DOCS tag is set to YES, Doxygen will hide any 
+# documentation blocks found inside the body of a function. 
+# If set to NO (the default) these blocks will be appended to the 
+# function's detailed documentation block.
+
+HIDE_IN_BODY_DOCS      = NO
+
+# The INTERNAL_DOCS tag determines if documentation 
+# that is typed after a \internal command is included. If the tag is set 
+# to NO (the default) then the documentation will be excluded. 
+# Set it to YES to include the internal documentation.
+
+INTERNAL_DOCS          = NO
+
+# If the CASE_SENSE_NAMES tag is set to NO then Doxygen will only generate 
+# file names in lower-case letters. If set to YES upper-case letters are also 
+# allowed. This is useful if you have classes or files whose names only differ 
+# in case and if your file system supports case sensitive file names. Windows 
+# and Mac users are advised to set this option to NO.
+
+CASE_SENSE_NAMES       = YES
+
+# If the HIDE_SCOPE_NAMES tag is set to NO (the default) then Doxygen 
+# will show members with their full class and namespace scopes in the 
+# documentation. If set to YES the scope will be hidden.
+
+HIDE_SCOPE_NAMES       = NO
+
+# If the SHOW_INCLUDE_FILES tag is set to YES (the default) then Doxygen 
+# will put a list of the files that are included by a file in the documentation 
+# of that file.
+
+SHOW_INCLUDE_FILES     = YES
+
+# If the INLINE_INFO tag is set to YES (the default) then a tag [inline] 
+# is inserted in the documentation for inline members.
+
+INLINE_INFO            = YES
+
+# If the SORT_MEMBER_DOCS tag is set to YES (the default) then doxygen 
+# will sort the (detailed) documentation of file and class members 
+# alphabetically by member name. If set to NO the members will appear in 
+# declaration order.
+
+SORT_MEMBER_DOCS       = NO
+
+# If the SORT_BRIEF_DOCS tag is set to YES then doxygen will sort the 
+# brief documentation of file, namespace and class members alphabetically 
+# by member name. If set to NO (the default) the members will appear in 
+# declaration order.
+
+SORT_BRIEF_DOCS        = NO
+
+# If the SORT_GROUP_NAMES tag is set to YES then doxygen will sort the 
+# hierarchy of group names into alphabetical order. If set to NO (the default) 
+# the group names will appear in their defined order.
+
+SORT_GROUP_NAMES       = NO
+
+# If the SORT_BY_SCOPE_NAME tag is set to YES, the class list will be 
+# sorted by fully-qualified names, including namespaces. If set to 
+# NO (the default), the class list will be sorted only by class name, 
+# not including the namespace part. 
+# Note: This option is not very useful if HIDE_SCOPE_NAMES is set to YES.
+# Note: This option applies only to the class list, not to the 
+# alphabetical list.
+
+SORT_BY_SCOPE_NAME     = NO
+
+# The GENERATE_TODOLIST tag can be used to enable (YES) or 
+# disable (NO) the todo list. This list is created by putting \todo 
+# commands in the documentation.
+
+GENERATE_TODOLIST      = YES
+
+# The GENERATE_TESTLIST tag can be used to enable (YES) or 
+# disable (NO) the test list. This list is created by putting \test 
+# commands in the documentation.
+
+GENERATE_TESTLIST      = YES
+
+# The GENERATE_BUGLIST tag can be used to enable (YES) or 
+# disable (NO) the bug list. This list is created by putting \bug 
+# commands in the documentation.
+
+GENERATE_BUGLIST       = YES
+
+# The GENERATE_DEPRECATEDLIST tag can be used to enable (YES) or 
+# disable (NO) the deprecated list. This list is created by putting 
+# \deprecated commands in the documentation.
+
+GENERATE_DEPRECATEDLIST= YES
+
+# The ENABLED_SECTIONS tag can be used to enable conditional 
+# documentation sections, marked by \if sectionname ... \endif.
+
+ENABLED_SECTIONS       = 
+
+# The MAX_INITIALIZER_LINES tag determines the maximum number of lines 
+# the initial value of a variable or define consists of for it to appear in 
+# the documentation. If the initializer consists of more lines than specified 
+# here it will be hidden. Use a value of 0 to hide initializers completely. 
+# The appearance of the initializer of individual variables and defines in the 
+# documentation can be controlled using \showinitializer or \hideinitializer 
+# command in the documentation regardless of this setting.
+
+MAX_INITIALIZER_LINES  = 30
+
+# Set the SHOW_USED_FILES tag to NO to disable the list of files generated 
+# at the bottom of the documentation of classes and structs. If set to YES the 
+# list will mention the files that were used to generate the documentation.
+
+SHOW_USED_FILES        = YES
+
+# If the sources in your project are distributed over multiple directories 
+# then setting the SHOW_DIRECTORIES tag to YES will show the directory hierarchy 
+# in the documentation. The default is NO.
+
+SHOW_DIRECTORIES       = NO
+
+# Set the SHOW_FILES tag to NO to disable the generation of the Files page.
+# This will remove the Files entry from the Quick Index and from the 
+# Folder Tree View (if specified). The default is YES.
+
+SHOW_FILES             = YES
+
+# Set the SHOW_NAMESPACES tag to NO to disable the generation of the 
+# Namespaces page.  This will remove the Namespaces entry from the Quick Index
+# and from the Folder Tree View (if specified). The default is YES.
+
+SHOW_NAMESPACES        = YES
+
+# The FILE_VERSION_FILTER tag can be used to specify a program or script that 
+# doxygen should invoke to get the current version for each file (typically from 
+# the version control system). Doxygen will invoke the program by executing (via 
+# popen()) the command <command> <input-file>, where <command> is the value of 
+# the FILE_VERSION_FILTER tag, and <input-file> is the name of an input file 
+# provided by doxygen. Whatever the program writes to standard output 
+# is used as the file version. See the manual for examples.
+
+FILE_VERSION_FILTER    = 
+
+#---------------------------------------------------------------------------
+# configuration options related to warning and progress messages
+#---------------------------------------------------------------------------
+
+# The QUIET tag can be used to turn on/off the messages that are generated 
+# by doxygen. Possible values are YES and NO. If left blank NO is used.
+
+QUIET                  = NO
+
+# The WARNINGS tag can be used to turn on/off the warning messages that are 
+# generated by doxygen. Possible values are YES and NO. If left blank 
+# NO is used.
+
+WARNINGS               = YES
+
+# If WARN_IF_UNDOCUMENTED is set to YES, then doxygen will generate warnings 
+# for undocumented members. If EXTRACT_ALL is set to YES then this flag will 
+# automatically be disabled.
+
+WARN_IF_UNDOCUMENTED   = NO
+
+# If WARN_IF_DOC_ERROR is set to YES, doxygen will generate warnings for 
+# potential errors in the documentation, such as not documenting some 
+# parameters in a documented function, or documenting parameters that 
+# don't exist or using markup commands wrongly.
+
+WARN_IF_DOC_ERROR      = YES
+
+# This WARN_NO_PARAMDOC option can be abled to get warnings for 
+# functions that are documented, but have no documentation for their parameters 
+# or return value. If set to NO (the default) doxygen will only warn about 
+# wrong or incomplete parameter documentation, but not about the absence of 
+# documentation.
+
+WARN_NO_PARAMDOC       = NO
+
+# The WARN_FORMAT tag determines the format of the warning messages that 
+# doxygen can produce. The string should contain the $file, $line, and $text 
+# tags, which will be replaced by the file and line number from which the 
+# warning originated and the warning text. Optionally the format may contain 
+# $version, which will be replaced by the version of the file (if it could 
+# be obtained via FILE_VERSION_FILTER)
+
+WARN_FORMAT            = "$file:$line: $text"
+
+# The WARN_LOGFILE tag can be used to specify a file to which warning 
+# and error messages should be written. If left blank the output is written 
+# to stderr.
+
+WARN_LOGFILE           = 
+
+#---------------------------------------------------------------------------
+# configuration options related to the input files
+#---------------------------------------------------------------------------
+
+# The INPUT tag can be used to specify the files and/or directories that contain 
+# documented source files. You may enter file names like "myfile.cpp" or 
+# directories like "/usr/src/myproject". Separate the files or directories 
+# with spaces.
+
+INPUT                  = docs/doxygen.h \
+                         libdrizzle/constants.h \
+                         libdrizzle/drizzle.h \
+                         libdrizzle/drizzle_client.h \
+                         libdrizzle/drizzle_server.h \
+                         libdrizzle/conn.h \
+                         libdrizzle/conn_client.h \
+                         libdrizzle/conn_server.h \
+                         libdrizzle/handshake_client.h \
+                         libdrizzle/handshake_server.h \
+                         libdrizzle/command_client.h \
+                         libdrizzle/command_server.h \
+                         libdrizzle/query.h \
+                         libdrizzle/result.h \
+                         libdrizzle/result_client.h \
+                         libdrizzle/result_server.h \
+                         libdrizzle/column.h \
+                         libdrizzle/column_client.h \
+                         libdrizzle/column_server.h \
+                         libdrizzle/row_client.h \
+                         libdrizzle/row_server.h \
+                         libdrizzle/field_client.h \
+                         libdrizzle/field_server.h \
+                         libdrizzle \
+                         examples
+
+# This tag can be used to specify the character encoding of the source files 
+# that doxygen parses. Internally doxygen uses the UTF-8 encoding, which is 
+# also the default input encoding. Doxygen uses libiconv (or the iconv built 
+# into libc) for the transcoding. See http://www.gnu.org/software/libiconv for 
+# the list of possible encodings.
+
+INPUT_ENCODING         = UTF-8
+
+# If the value of the INPUT tag contains directories, you can use the 
+# FILE_PATTERNS tag to specify one or more wildcard pattern (like *.cpp 
+# and *.h) to filter out the source-files in the directories. If left 
+# blank the following patterns are tested: 
+# *.c *.cc *.cxx *.cpp *.c++ *.java *.ii *.ixx *.ipp *.i++ *.inl *.h *.hh *.hxx 
+# *.hpp *.h++ *.idl *.odl *.cs *.php *.php3 *.inc *.m *.mm *.py *.f90
+
+FILE_PATTERNS          = *.h *.c
+
+# The RECURSIVE tag can be used to turn specify whether or not subdirectories 
+# should be searched for input files as well. Possible values are YES and NO. 
+# If left blank NO is used.
+
+RECURSIVE              = YES
+
+# The EXCLUDE tag can be used to specify files and/or directories that should 
+# excluded from the INPUT source files. This way you can easily exclude a 
+# subdirectory from a directory tree whose root is specified with the INPUT tag.
+
+EXCLUDE                = 
+
+# The EXCLUDE_SYMLINKS tag can be used select whether or not files or 
+# directories that are symbolic links (a Unix filesystem feature) are excluded 
+# from the input.
+
+EXCLUDE_SYMLINKS       = NO
+
+# If the value of the INPUT tag contains directories, you can use the 
+# EXCLUDE_PATTERNS tag to specify one or more wildcard patterns to exclude 
+# certain files from those directories. Note that the wildcards are matched 
+# against the file with absolute path, so to exclude all test directories 
+# for example use the pattern */test/*
+
+EXCLUDE_PATTERNS       = 
+
+# The EXCLUDE_SYMBOLS tag can be used to specify one or more symbol names 
+# (namespaces, classes, functions, etc.) that should be excluded from the 
+# output. The symbol name can be a fully qualified name, a word, or if the 
+# wildcard * is used, a substring. Examples: ANamespace, AClass, 
+# AClass::ANamespace, ANamespace::*Test
+
+EXCLUDE_SYMBOLS        = 
+
+# The EXAMPLE_PATH tag can be used to specify one or more files or 
+# directories that contain example code fragments that are included (see 
+# the \include command).
+
+EXAMPLE_PATH           = 
+
+# If the value of the EXAMPLE_PATH tag contains directories, you can use the 
+# EXAMPLE_PATTERNS tag to specify one or more wildcard pattern (like *.cpp 
+# and *.h) to filter out the source-files in the directories. If left 
+# blank all files are included.
+
+EXAMPLE_PATTERNS       = 
+
+# If the EXAMPLE_RECURSIVE tag is set to YES then subdirectories will be 
+# searched for input files to be used with the \include or \dontinclude 
+# commands irrespective of the value of the RECURSIVE tag. 
+# Possible values are YES and NO. If left blank NO is used.
+
+EXAMPLE_RECURSIVE      = NO
+
+# The IMAGE_PATH tag can be used to specify one or more files or 
+# directories that contain image that are included in the documentation (see 
+# the \image command).
+
+IMAGE_PATH             = 
+
+# The INPUT_FILTER tag can be used to specify a program that doxygen should 
+# invoke to filter for each input file. Doxygen will invoke the filter program 
+# by executing (via popen()) the command <filter> <input-file>, where <filter> 
+# is the value of the INPUT_FILTER tag, and <input-file> is the name of an 
+# input file. Doxygen will then use the output that the filter program writes 
+# to standard output.  If FILTER_PATTERNS is specified, this tag will be 
+# ignored.
+
+INPUT_FILTER           = 
+
+# The FILTER_PATTERNS tag can be used to specify filters on a per file pattern 
+# basis.  Doxygen will compare the file name with each pattern and apply the 
+# filter if there is a match.  The filters are a list of the form: 
+# pattern=filter (like *.cpp=my_cpp_filter). See INPUT_FILTER for further 
+# info on how filters are used. If FILTER_PATTERNS is empty, INPUT_FILTER 
+# is applied to all files.
+
+FILTER_PATTERNS        = 
+
+# If the FILTER_SOURCE_FILES tag is set to YES, the input filter (if set using 
+# INPUT_FILTER) will be used to filter the input files when producing source 
+# files to browse (i.e. when SOURCE_BROWSER is set to YES).
+
+FILTER_SOURCE_FILES    = NO
+
+#---------------------------------------------------------------------------
+# configuration options related to source browsing
+#---------------------------------------------------------------------------
+
+# If the SOURCE_BROWSER tag is set to YES then a list of source files will 
+# be generated. Documented entities will be cross-referenced with these sources. 
+# Note: To get rid of all source code in the generated output, make sure also 
+# VERBATIM_HEADERS is set to NO.
+
+SOURCE_BROWSER         = YES
+
+# Setting the INLINE_SOURCES tag to YES will include the body 
+# of functions and classes directly in the documentation.
+
+INLINE_SOURCES         = NO
+
+# Setting the STRIP_CODE_COMMENTS tag to YES (the default) will instruct 
+# doxygen to hide any special comment blocks from generated source code 
+# fragments. Normal C and C++ comments will always remain visible.
+
+STRIP_CODE_COMMENTS    = YES
+
+# If the REFERENCED_BY_RELATION tag is set to YES 
+# then for each documented function all documented 
+# functions referencing it will be listed.
+
+REFERENCED_BY_RELATION = NO
+
+# If the REFERENCES_RELATION tag is set to YES 
+# then for each documented function all documented entities 
+# called/used by that function will be listed.
+
+REFERENCES_RELATION    = NO
+
+# If the REFERENCES_LINK_SOURCE tag is set to YES (the default)
+# and SOURCE_BROWSER tag is set to YES, then the hyperlinks from
+# functions in REFERENCES_RELATION and REFERENCED_BY_RELATION lists will
+# link to the source code.  Otherwise they will link to the documentstion.
+
+REFERENCES_LINK_SOURCE = YES
+
+# If the USE_HTAGS tag is set to YES then the references to source code 
+# will point to the HTML generated by the htags(1) tool instead of doxygen 
+# built-in source browser. The htags tool is part of GNU's global source 
+# tagging system (see http://www.gnu.org/software/global/global.html). You 
+# will need version 4.8.6 or higher.
+
+USE_HTAGS              = NO
+
+# If the VERBATIM_HEADERS tag is set to YES (the default) then Doxygen 
+# will generate a verbatim copy of the header file for each class for 
+# which an include is specified. Set to NO to disable this.
+
+VERBATIM_HEADERS       = YES
+
+#---------------------------------------------------------------------------
+# configuration options related to the alphabetical class index
+#---------------------------------------------------------------------------
+
+# If the ALPHABETICAL_INDEX tag is set to YES, an alphabetical index 
+# of all compounds will be generated. Enable this if the project 
+# contains a lot of classes, structs, unions or interfaces.
+
+ALPHABETICAL_INDEX     = NO
+
+# If the alphabetical index is enabled (see ALPHABETICAL_INDEX) then 
+# the COLS_IN_ALPHA_INDEX tag can be used to specify the number of columns 
+# in which this list will be split (can be a number in the range [1..20])
+
+COLS_IN_ALPHA_INDEX    = 5
+
+# In case all classes in a project start with a common prefix, all 
+# classes will be put under the same header in the alphabetical index. 
+# The IGNORE_PREFIX tag can be used to specify one or more prefixes that 
+# should be ignored while generating the index headers.
+
+IGNORE_PREFIX          = 
+
+#---------------------------------------------------------------------------
+# configuration options related to the HTML output
+#---------------------------------------------------------------------------
+
+# If the GENERATE_HTML tag is set to YES (the default) Doxygen will 
+# generate HTML output.
+
+GENERATE_HTML          = YES
+
+# The HTML_OUTPUT tag is used to specify where the HTML docs will be put. 
+# If a relative path is entered the value of OUTPUT_DIRECTORY will be 
+# put in front of it. If left blank `html' will be used as the default path.
+
+HTML_OUTPUT            = html
+
+# The HTML_FILE_EXTENSION tag can be used to specify the file extension for 
+# each generated HTML page (for example: .htm,.php,.asp). If it is left blank 
+# doxygen will generate files with .html extension.
+
+HTML_FILE_EXTENSION    = .html
+
+# The HTML_HEADER tag can be used to specify a personal HTML header for 
+# each generated HTML page. If it is left blank doxygen will generate a 
+# standard header.
+
+HTML_HEADER            = docs/dev_header.html
+
+# The HTML_FOOTER tag can be used to specify a personal HTML footer for 
+# each generated HTML page. If it is left blank doxygen will generate a 
+# standard footer.
+
+HTML_FOOTER            = 
+
+# The HTML_STYLESHEET tag can be used to specify a user-defined cascading 
+# style sheet that is used by each HTML page. It can be used to 
+# fine-tune the look of the HTML output. If the tag is left blank doxygen 
+# will generate a default style sheet. Note that doxygen will try to copy 
+# the style sheet file to the HTML output directory, so don't put your own 
+# stylesheet in the HTML output directory as well, or it will be erased!
+
+HTML_STYLESHEET        = 
+
+# If the HTML_ALIGN_MEMBERS tag is set to YES, the members of classes, 
+# files or namespaces will be aligned in HTML using tables. If set to 
+# NO a bullet list will be used.
+
+HTML_ALIGN_MEMBERS     = YES
+
+# If the GENERATE_HTMLHELP tag is set to YES, additional index files 
+# will be generated that can be used as input for tools like the 
+# Microsoft HTML help workshop to generate a compiled HTML help file (.chm) 
+# of the generated HTML documentation.
+
+GENERATE_HTMLHELP      = NO
+
+# If the GENERATE_DOCSET tag is set to YES, additional index files 
+# will be generated that can be used as input for Apple's Xcode 3 
+# integrated development environment, introduced with OSX 10.5 (Leopard). 
+# To create a documentation set, doxygen will generate a Makefile in the 
+# HTML output directory. Running make will produce the docset in that 
+# directory and running "make install" will install the docset in 
+# ~/Library/Developer/Shared/Documentation/DocSets so that Xcode will find 
+# it at startup.
+
+GENERATE_DOCSET        = NO
+
+# When GENERATE_DOCSET tag is set to YES, this tag determines the name of the 
+# feed. A documentation feed provides an umbrella under which multiple 
+# documentation sets from a single provider (such as a company or product suite) 
+# can be grouped.
+
+DOCSET_FEEDNAME        = "Doxygen generated docs"
+
+# When GENERATE_DOCSET tag is set to YES, this tag specifies a string that 
+# should uniquely identify the documentation set bundle. This should be a 
+# reverse domain-name style string, e.g. com.mycompany.MyDocSet. Doxygen 
+# will append .docset to the name.
+
+DOCSET_BUNDLE_ID       = org.doxygen.Project
+
+# If the HTML_DYNAMIC_SECTIONS tag is set to YES then the generated HTML 
+# documentation will contain sections that can be hidden and shown after the 
+# page has loaded. For this to work a browser that supports 
+# JavaScript and DHTML is required (for instance Mozilla 1.0+, Firefox 
+# Netscape 6.0+, Internet explorer 5.0+, Konqueror, or Safari).
+
+HTML_DYNAMIC_SECTIONS  = NO
+
+# If the GENERATE_HTMLHELP tag is set to YES, the CHM_FILE tag can 
+# be used to specify the file name of the resulting .chm file. You 
+# can add a path in front of the file if the result should not be 
+# written to the html output directory.
+
+CHM_FILE               = 
+
+# If the GENERATE_HTMLHELP tag is set to YES, the HHC_LOCATION tag can 
+# be used to specify the location (absolute path including file name) of 
+# the HTML help compiler (hhc.exe). If non-empty doxygen will try to run 
+# the HTML help compiler on the generated index.hhp.
+
+HHC_LOCATION           = 
+
+# If the GENERATE_HTMLHELP tag is set to YES, the GENERATE_CHI flag 
+# controls if a separate .chi index file is generated (YES) or that 
+# it should be included in the master .chm file (NO).
+
+GENERATE_CHI           = NO
+
+# If the GENERATE_HTMLHELP tag is set to YES, the CHM_INDEX_ENCODING
+# is used to encode HtmlHelp index (hhk), content (hhc) and project file
+# content.
+
+CHM_INDEX_ENCODING     = 
+
+# If the GENERATE_HTMLHELP tag is set to YES, the BINARY_TOC flag 
+# controls whether a binary table of contents is generated (YES) or a 
+# normal table of contents (NO) in the .chm file.
+
+BINARY_TOC             = NO
+
+# The TOC_EXPAND flag can be set to YES to add extra items for group members 
+# to the contents of the HTML help documentation and to the tree view.
+
+TOC_EXPAND             = NO
+
+# The DISABLE_INDEX tag can be used to turn on/off the condensed index at 
+# top of each HTML page. The value NO (the default) enables the index and 
+# the value YES disables it.
+
+DISABLE_INDEX          = NO
+
+# This tag can be used to set the number of enum values (range [1..20]) 
+# that doxygen will group on one line in the generated HTML documentation.
+
+ENUM_VALUES_PER_LINE   = 4
+
+# The GENERATE_TREEVIEW tag is used to specify whether a tree-like index
+# structure should be generated to display hierarchical information.
+# If the tag value is set to FRAME, a side panel will be generated
+# containing a tree-like index structure (just like the one that 
+# is generated for HTML Help). For this to work a browser that supports 
+# JavaScript, DHTML, CSS and frames is required (for instance Mozilla 1.0+, 
+# Netscape 6.0+, Internet explorer 5.0+, or Konqueror). Windows users are 
+# probably better off using the HTML help feature. Other possible values 
+# for this tag are: HIERARCHIES, which will generate the Groups, Directories,
+# and Class Hiererachy pages using a tree view instead of an ordered list;
+# ALL, which combines the behavior of FRAME and HIERARCHIES; and NONE, which
+# disables this behavior completely. For backwards compatibility with previous
+# releases of Doxygen, the values YES and NO are equivalent to FRAME and NONE
+# respectively.
+
+GENERATE_TREEVIEW      = NONE
+
+# If the treeview is enabled (see GENERATE_TREEVIEW) then this tag can be 
+# used to set the initial width (in pixels) of the frame in which the tree 
+# is shown.
+
+TREEVIEW_WIDTH         = 250
+
+# Use this tag to change the font size of Latex formulas included 
+# as images in the HTML documentation. The default is 10. Note that 
+# when you change the font size after a successful doxygen run you need 
+# to manually remove any form_*.png images from the HTML output directory 
+# to force them to be regenerated.
+
+FORMULA_FONTSIZE       = 10
+
+#---------------------------------------------------------------------------
+# configuration options related to the LaTeX output
+#---------------------------------------------------------------------------
+
+# If the GENERATE_LATEX tag is set to YES (the default) Doxygen will 
+# generate Latex output.
+
+GENERATE_LATEX         = NO
+
+# The LATEX_OUTPUT tag is used to specify where the LaTeX docs will be put. 
+# If a relative path is entered the value of OUTPUT_DIRECTORY will be 
+# put in front of it. If left blank `latex' will be used as the default path.
+
+LATEX_OUTPUT           = latex
+
+# The LATEX_CMD_NAME tag can be used to specify the LaTeX command name to be 
+# invoked. If left blank `latex' will be used as the default command name.
+
+LATEX_CMD_NAME         = latex
+
+# The MAKEINDEX_CMD_NAME tag can be used to specify the command name to 
+# generate index for LaTeX. If left blank `makeindex' will be used as the 
+# default command name.
+
+MAKEINDEX_CMD_NAME     = makeindex
+
+# If the COMPACT_LATEX tag is set to YES Doxygen generates more compact 
+# LaTeX documents. This may be useful for small projects and may help to 
+# save some trees in general.
+
+COMPACT_LATEX          = NO
+
+# The PAPER_TYPE tag can be used to set the paper type that is used 
+# by the printer. Possible values are: a4, a4wide, letter, legal and 
+# executive. If left blank a4wide will be used.
+
+PAPER_TYPE             = a4wide
+
+# The EXTRA_PACKAGES tag can be to specify one or more names of LaTeX 
+# packages that should be included in the LaTeX output.
+
+EXTRA_PACKAGES         = 
+
+# The LATEX_HEADER tag can be used to specify a personal LaTeX header for 
+# the generated latex document. The header should contain everything until 
+# the first chapter. If it is left blank doxygen will generate a 
+# standard header. Notice: only use this tag if you know what you are doing!
+
+LATEX_HEADER           = 
+
+# If the PDF_HYPERLINKS tag is set to YES, the LaTeX that is generated 
+# is prepared for conversion to pdf (using ps2pdf). The pdf file will 
+# contain links (just like the HTML output) instead of page references 
+# This makes the output suitable for online browsing using a pdf viewer.
+
+PDF_HYPERLINKS         = YES
+
+# If the USE_PDFLATEX tag is set to YES, pdflatex will be used instead of 
+# plain latex in the generated Makefile. Set this option to YES to get a 
+# higher quality PDF documentation.
+
+USE_PDFLATEX           = YES
+
+# If the LATEX_BATCHMODE tag is set to YES, doxygen will add the \\batchmode. 
+# command to the generated LaTeX files. This will instruct LaTeX to keep 
+# running if errors occur, instead of asking the user for help. 
+# This option is also used when generating formulas in HTML.
+
+LATEX_BATCHMODE        = NO
+
+# If LATEX_HIDE_INDICES is set to YES then doxygen will not 
+# include the index chapters (such as File Index, Compound Index, etc.) 
+# in the output.
+
+LATEX_HIDE_INDICES     = NO
+
+#---------------------------------------------------------------------------
+# configuration options related to the RTF output
+#---------------------------------------------------------------------------
+
+# If the GENERATE_RTF tag is set to YES Doxygen will generate RTF output 
+# The RTF output is optimized for Word 97 and may not look very pretty with 
+# other RTF readers or editors.
+
+GENERATE_RTF           = NO
+
+# The RTF_OUTPUT tag is used to specify where the RTF docs will be put. 
+# If a relative path is entered the value of OUTPUT_DIRECTORY will be 
+# put in front of it. If left blank `rtf' will be used as the default path.
+
+RTF_OUTPUT             = rtf
+
+# If the COMPACT_RTF tag is set to YES Doxygen generates more compact 
+# RTF documents. This may be useful for small projects and may help to 
+# save some trees in general.
+
+COMPACT_RTF            = NO
+
+# If the RTF_HYPERLINKS tag is set to YES, the RTF that is generated 
+# will contain hyperlink fields. The RTF file will 
+# contain links (just like the HTML output) instead of page references. 
+# This makes the output suitable for online browsing using WORD or other 
+# programs which support those fields. 
+# Note: wordpad (write) and others do not support links.
+
+RTF_HYPERLINKS         = NO
+
+# Load stylesheet definitions from file. Syntax is similar to doxygen's 
+# config file, i.e. a series of assignments. You only have to provide 
+# replacements, missing definitions are set to their default value.
+
+RTF_STYLESHEET_FILE    = 
+
+# Set optional variables used in the generation of an rtf document. 
+# Syntax is similar to doxygen's config file.
+
+RTF_EXTENSIONS_FILE    = 
+
+#---------------------------------------------------------------------------
+# configuration options related to the man page output
+#---------------------------------------------------------------------------
+
+# If the GENERATE_MAN tag is set to YES (the default) Doxygen will 
+# generate man pages
+
+GENERATE_MAN           = NO
+
+# The MAN_OUTPUT tag is used to specify where the man pages will be put. 
+# If a relative path is entered the value of OUTPUT_DIRECTORY will be 
+# put in front of it. If left blank `man' will be used as the default path.
+
+MAN_OUTPUT             = man
+
+# The MAN_EXTENSION tag determines the extension that is added to 
+# the generated man pages (default is the subroutine's section .3)
+
+MAN_EXTENSION          = .3
+
+# If the MAN_LINKS tag is set to YES and Doxygen generates man output, 
+# then it will generate one additional man file for each entity 
+# documented in the real man page(s). These additional files 
+# only source the real man page, but without them the man command 
+# would be unable to find the correct page. The default is NO.
+
+MAN_LINKS              = NO
+
+#---------------------------------------------------------------------------
+# configuration options related to the XML output
+#---------------------------------------------------------------------------
+
+# If the GENERATE_XML tag is set to YES Doxygen will 
+# generate an XML file that captures the structure of 
+# the code including all documentation.
+
+GENERATE_XML           = NO
+
+# The XML_OUTPUT tag is used to specify where the XML pages will be put. 
+# If a relative path is entered the value of OUTPUT_DIRECTORY will be 
+# put in front of it. If left blank `xml' will be used as the default path.
+
+XML_OUTPUT             = xml
+
+# The XML_SCHEMA tag can be used to specify an XML schema, 
+# which can be used by a validating XML parser to check the 
+# syntax of the XML files.
+
+XML_SCHEMA             = 
+
+# The XML_DTD tag can be used to specify an XML DTD, 
+# which can be used by a validating XML parser to check the 
+# syntax of the XML files.
+
+XML_DTD                = 
+
+# If the XML_PROGRAMLISTING tag is set to YES Doxygen will 
+# dump the program listings (including syntax highlighting 
+# and cross-referencing information) to the XML output. Note that 
+# enabling this will significantly increase the size of the XML output.
+
+XML_PROGRAMLISTING     = YES
+
+#---------------------------------------------------------------------------
+# configuration options for the AutoGen Definitions output
+#---------------------------------------------------------------------------
+
+# If the GENERATE_AUTOGEN_DEF tag is set to YES Doxygen will 
+# generate an AutoGen Definitions (see autogen.sf.net) file 
+# that captures the structure of the code including all 
+# documentation. Note that this feature is still experimental 
+# and incomplete at the moment.
+
+GENERATE_AUTOGEN_DEF   = NO
+
+#---------------------------------------------------------------------------
+# configuration options related to the Perl module output
+#---------------------------------------------------------------------------
+
+# If the GENERATE_PERLMOD tag is set to YES Doxygen will 
+# generate a Perl module file that captures the structure of 
+# the code including all documentation. Note that this 
+# feature is still experimental and incomplete at the 
+# moment.
+
+GENERATE_PERLMOD       = NO
+
+# If the PERLMOD_LATEX tag is set to YES Doxygen will generate 
+# the necessary Makefile rules, Perl scripts and LaTeX code to be able 
+# to generate PDF and DVI output from the Perl module output.
+
+PERLMOD_LATEX          = NO
+
+# If the PERLMOD_PRETTY tag is set to YES the Perl module output will be 
+# nicely formatted so it can be parsed by a human reader.  This is useful 
+# if you want to understand what is going on.  On the other hand, if this 
+# tag is set to NO the size of the Perl module output will be much smaller 
+# and Perl will parse it just the same.
+
+PERLMOD_PRETTY         = YES
+
+# The names of the make variables in the generated doxyrules.make file 
+# are prefixed with the string contained in PERLMOD_MAKEVAR_PREFIX. 
+# This is useful so different doxyrules.make files included by the same 
+# Makefile don't overwrite each other's variables.
+
+PERLMOD_MAKEVAR_PREFIX = 
+
+#---------------------------------------------------------------------------
+# Configuration options related to the preprocessor   
+#---------------------------------------------------------------------------
+
+# If the ENABLE_PREPROCESSING tag is set to YES (the default) Doxygen will 
+# evaluate all C-preprocessor directives found in the sources and include 
+# files.
+
+ENABLE_PREPROCESSING   = YES
+
+# If the MACRO_EXPANSION tag is set to YES Doxygen will expand all macro 
+# names in the source code. If set to NO (the default) only conditional 
+# compilation will be performed. Macro expansion can be done in a controlled 
+# way by setting EXPAND_ONLY_PREDEF to YES.
+
+MACRO_EXPANSION        = YES
+
+# If the EXPAND_ONLY_PREDEF and MACRO_EXPANSION tags are both set to YES 
+# then the macro expansion is limited to the macros specified with the 
+# PREDEFINED and EXPAND_AS_DEFINED tags.
+
+EXPAND_ONLY_PREDEF     = YES
+
+# If the SEARCH_INCLUDES tag is set to YES (the default) the includes files 
+# in the INCLUDE_PATH (see below) will be search if a #include is found.
+
+SEARCH_INCLUDES        = YES
+
+# The INCLUDE_PATH tag can be used to specify one or more directories that 
+# contain include files that are not input files but should be processed by 
+# the preprocessor.
+
+INCLUDE_PATH           = 
+
+# You can use the INCLUDE_FILE_PATTERNS tag to specify one or more wildcard 
+# patterns (like *.h and *.hpp) to filter out the header-files in the 
+# directories. If left blank, the patterns specified with FILE_PATTERNS will 
+# be used.
+
+INCLUDE_FILE_PATTERNS  = 
+
+# The PREDEFINED tag can be used to specify one or more macro names that 
+# are defined before the preprocessor is started (similar to the -D option of 
+# gcc). The argument of the tag is a list of macros of the form: name 
+# or name=definition (no spaces). If the definition and the = are 
+# omitted =1 is assumed. To prevent a macro definition from being 
+# undefined via #undef or recursively expanded use the := operator 
+# instead of the = operator.
+
+PREDEFINED             = DRIZZLE_API=
+
+# If the MACRO_EXPANSION and EXPAND_ONLY_PREDEF tags are set to YES then 
+# this tag can be used to specify a list of macro names that should be expanded. 
+# The macro definition that is found in the sources will be used. 
+# Use the PREDEFINED tag if you want to use a different macro definition.
+
+EXPAND_AS_DEFINED      = 
+
+# If the SKIP_FUNCTION_MACROS tag is set to YES (the default) then 
+# doxygen's preprocessor will remove all function-like macros that are alone 
+# on a line, have an all uppercase name, and do not end with a semicolon. Such 
+# function macros are typically used for boiler-plate code, and will confuse 
+# the parser if not removed.
+
+SKIP_FUNCTION_MACROS   = YES
+
+#---------------------------------------------------------------------------
+# Configuration::additions related to external references   
+#---------------------------------------------------------------------------
+
+# The TAGFILES option can be used to specify one or more tagfiles. 
+# Optionally an initial location of the external documentation 
+# can be added for each tagfile. The format of a tag file without 
+# this location is as follows: 
+#   TAGFILES = file1 file2 ... 
+# Adding location for the tag files is done as follows: 
+#   TAGFILES = file1=loc1 "file2 = loc2" ... 
+# where "loc1" and "loc2" can be relative or absolute paths or 
+# URLs. If a location is present for each tag, the installdox tool 
+# does not have to be run to correct the links.
+# Note that each tag file must have a unique name
+# (where the name does NOT include the path)
+# If a tag file is not located in the directory in which doxygen 
+# is run, you must also specify the path to the tagfile here.
+
+TAGFILES               = 
+
+# When a file name is specified after GENERATE_TAGFILE, doxygen will create 
+# a tag file that is based on the input files it reads.
+
+GENERATE_TAGFILE       = 
+
+# If the ALLEXTERNALS tag is set to YES all external classes will be listed 
+# in the class index. If set to NO only the inherited external classes 
+# will be listed.
+
+ALLEXTERNALS           = NO
+
+# If the EXTERNAL_GROUPS tag is set to YES all external groups will be listed 
+# in the modules index. If set to NO, only the current project's groups will 
+# be listed.
+
+EXTERNAL_GROUPS        = YES
+
+# The PERL_PATH should be the absolute path and name of the perl script 
+# interpreter (i.e. the result of `which perl').
+
+PERL_PATH              = /usr/bin/perl
+
+#---------------------------------------------------------------------------
+# Configuration options related to the dot tool   
+#---------------------------------------------------------------------------
+
+# If the CLASS_DIAGRAMS tag is set to YES (the default) Doxygen will 
+# generate a inheritance diagram (in HTML, RTF and LaTeX) for classes with base 
+# or super classes. Setting the tag to NO turns the diagrams off. Note that 
+# this option is superseded by the HAVE_DOT option below. This is only a 
+# fallback. It is recommended to install and use dot, since it yields more 
+# powerful graphs.
+
+CLASS_DIAGRAMS         = YES
+
+# You can define message sequence charts within doxygen comments using the \msc 
+# command. Doxygen will then run the mscgen tool (see 
+# http://www.mcternan.me.uk/mscgen/) to produce the chart and insert it in the 
+# documentation. The MSCGEN_PATH tag allows you to specify the directory where 
+# the mscgen tool resides. If left empty the tool is assumed to be found in the 
+# default search path.
+
+MSCGEN_PATH            = 
+
+# If set to YES, the inheritance and collaboration graphs will hide 
+# inheritance and usage relations if the target is undocumented 
+# or is not a class.
+
+HIDE_UNDOC_RELATIONS   = YES
+
+# If you set the HAVE_DOT tag to YES then doxygen will assume the dot tool is 
+# available from the path. This tool is part of Graphviz, a graph visualization 
+# toolkit from AT&T and Lucent Bell Labs. The other options in this section 
+# have no effect if this option is set to NO (the default)
+
+HAVE_DOT               = NO
+
+# By default doxygen will write a font called FreeSans.ttf to the output 
+# directory and reference it in all dot files that doxygen generates. This 
+# font does not include all possible unicode characters however, so when you need 
+# these (or just want a differently looking font) you can specify the font name 
+# using DOT_FONTNAME. You need need to make sure dot is able to find the font, 
+# which can be done by putting it in a standard location or by setting the 
+# DOTFONTPATH environment variable or by setting DOT_FONTPATH to the directory 
+# containing the font.
+
+DOT_FONTNAME           = FreeSans
+
+# By default doxygen will tell dot to use the output directory to look for the 
+# FreeSans.ttf font (which doxygen will put there itself). If you specify a 
+# different font using DOT_FONTNAME you can set the path where dot 
+# can find it using this tag.
+
+DOT_FONTPATH           = 
+
+# If the CLASS_GRAPH and HAVE_DOT tags are set to YES then doxygen 
+# will generate a graph for each documented class showing the direct and 
+# indirect inheritance relations. Setting this tag to YES will force the 
+# the CLASS_DIAGRAMS tag to NO.
+
+CLASS_GRAPH            = YES
+
+# If the COLLABORATION_GRAPH and HAVE_DOT tags are set to YES then doxygen 
+# will generate a graph for each documented class showing the direct and 
+# indirect implementation dependencies (inheritance, containment, and 
+# class references variables) of the class with other documented classes.
+
+COLLABORATION_GRAPH    = YES
+
+# If the GROUP_GRAPHS and HAVE_DOT tags are set to YES then doxygen 
+# will generate a graph for groups, showing the direct groups dependencies
+
+GROUP_GRAPHS           = YES
+
+# If the UML_LOOK tag is set to YES doxygen will generate inheritance and 
+# collaboration diagrams in a style similar to the OMG's Unified Modeling 
+# Language.
+
+UML_LOOK               = NO
+
+# If set to YES, the inheritance and collaboration graphs will show the 
+# relations between templates and their instances.
+
+TEMPLATE_RELATIONS     = NO
+
+# If the ENABLE_PREPROCESSING, SEARCH_INCLUDES, INCLUDE_GRAPH, and HAVE_DOT 
+# tags are set to YES then doxygen will generate a graph for each documented 
+# file showing the direct and indirect include dependencies of the file with 
+# other documented files.
+
+INCLUDE_GRAPH          = YES
+
+# If the ENABLE_PREPROCESSING, SEARCH_INCLUDES, INCLUDED_BY_GRAPH, and 
+# HAVE_DOT tags are set to YES then doxygen will generate a graph for each 
+# documented header file showing the documented files that directly or 
+# indirectly include this file.
+
+INCLUDED_BY_GRAPH      = YES
+
+# If the CALL_GRAPH and HAVE_DOT options are set to YES then 
+# doxygen will generate a call dependency graph for every global function 
+# or class method. Note that enabling this option will significantly increase 
+# the time of a run. So in most cases it will be better to enable call graphs 
+# for selected functions only using the \callgraph command.
+
+CALL_GRAPH             = NO
+
+# If the CALLER_GRAPH and HAVE_DOT tags are set to YES then 
+# doxygen will generate a caller dependency graph for every global function 
+# or class method. Note that enabling this option will significantly increase 
+# the time of a run. So in most cases it will be better to enable caller 
+# graphs for selected functions only using the \callergraph command.
+
+CALLER_GRAPH           = NO
+
+# If the GRAPHICAL_HIERARCHY and HAVE_DOT tags are set to YES then doxygen 
+# will graphical hierarchy of all classes instead of a textual one.
+
+GRAPHICAL_HIERARCHY    = YES
+
+# If the DIRECTORY_GRAPH, SHOW_DIRECTORIES and HAVE_DOT tags are set to YES 
+# then doxygen will show the dependencies a directory has on other directories 
+# in a graphical way. The dependency relations are determined by the #include
+# relations between the files in the directories.
+
+DIRECTORY_GRAPH        = YES
+
+# The DOT_IMAGE_FORMAT tag can be used to set the image format of the images 
+# generated by dot. Possible values are png, jpg, or gif
+# If left blank png will be used.
+
+DOT_IMAGE_FORMAT       = png
+
+# The tag DOT_PATH can be used to specify the path where the dot tool can be 
+# found. If left blank, it is assumed the dot tool can be found in the path.
+
+DOT_PATH               = 
+
+# The DOTFILE_DIRS tag can be used to specify one or more directories that 
+# contain dot files that are included in the documentation (see the 
+# \dotfile command).
+
+DOTFILE_DIRS           = 
+
+# The DOT_GRAPH_MAX_NODES tag can be used to set the maximum number of 
+# nodes that will be shown in the graph. If the number of nodes in a graph 
+# becomes larger than this value, doxygen will truncate the graph, which is 
+# visualized by representing a node as a red box. Note that doxygen if the 
+# number of direct children of the root node in a graph is already larger than 
+# DOT_GRAPH_MAX_NODES then the graph will not be shown at all. Also note 
+# that the size of a graph can be further restricted by MAX_DOT_GRAPH_DEPTH.
+
+DOT_GRAPH_MAX_NODES    = 50
+
+# The MAX_DOT_GRAPH_DEPTH tag can be used to set the maximum depth of the 
+# graphs generated by dot. A depth value of 3 means that only nodes reachable 
+# from the root by following a path via at most 3 edges will be shown. Nodes 
+# that lay further from the root node will be omitted. Note that setting this 
+# option to 1 or 2 may greatly reduce the computation time needed for large 
+# code bases. Also note that the size of a graph can be further restricted by 
+# DOT_GRAPH_MAX_NODES. Using a depth of 0 means no depth restriction.
+
+MAX_DOT_GRAPH_DEPTH    = 0
+
+# Set the DOT_TRANSPARENT tag to YES to generate images with a transparent 
+# background. This is enabled by default, which results in a transparent 
+# background. Warning: Depending on the platform used, enabling this option 
+# may lead to badly anti-aliased labels on the edges of a graph (i.e. they 
+# become hard to read).
+
+DOT_TRANSPARENT        = YES
+
+# Set the DOT_MULTI_TARGETS tag to YES allow dot to generate multiple output 
+# files in one run (i.e. multiple -o and -T options on the command line). This 
+# makes dot run faster, but since only newer versions of dot (>1.8.10) 
+# support this, this feature is disabled by default.
+
+DOT_MULTI_TARGETS      = NO
+
+# If the GENERATE_LEGEND tag is set to YES (the default) Doxygen will 
+# generate a legend page explaining the meaning of the various boxes and 
+# arrows in the dot generated graphs.
+
+GENERATE_LEGEND        = YES
+
+# If the DOT_CLEANUP tag is set to YES (the default) Doxygen will 
+# remove the intermediate dot files that are used to generate 
+# the various graphs.
+
+DOT_CLEANUP            = YES
+
+#---------------------------------------------------------------------------
+# Configuration::additions related to the search engine   
+#---------------------------------------------------------------------------
+
+# The SEARCHENGINE tag specifies whether or not a search engine should be 
+# used. If set to NO the values of all tags below this one will be ignored.
+
+SEARCHENGINE           = NO

=== renamed file 'Doxyfile' => 'docs/Doxyfile.drizzled'
--- Doxyfile	2008-08-06 20:57:54 +0000
+++ docs/Doxyfile.drizzled	2010-08-12 20:27:32 +0000
@@ -38,7 +38,7 @@
 # If a relative path is entered, it will be relative to the location 
 # where doxygen was started. If left blank the current directory will be used.
 
-OUTPUT_DIRECTORY       = docs/
+OUTPUT_DIRECTORY       = docs/drizzled
 
 # If the CREATE_SUBDIRS tag is set to YES, then doxygen will create 
 # 4096 sub-directories (in 2 levels) under the output directory of each output 
@@ -720,7 +720,7 @@
 # each generated HTML page. If it is left blank doxygen will generate a 
 # standard header.
 
-HTML_HEADER            = 
+HTML_HEADER            = docs/drizzled_header.html
 
 # The HTML_FOOTER tag can be used to specify a personal HTML footer for 
 # each generated HTML page. If it is left blank doxygen will generate a 

=== added file 'docs/api_header.html'
--- docs/api_header.html	1970-01-01 00:00:00 +0000
+++ docs/api_header.html	2010-08-12 20:27:32 +0000
@@ -0,0 +1,6 @@
+<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
+<html><head><meta http-equiv="Content-Type" content="text/html;charset=UTF-8">
+<title>$title</title>
+<link href="$relpath$doxygen.css" rel="stylesheet" type="text/css">
+<link href="$relpath$tabs.css" rel="stylesheet" type="text/css">
+</head><body><h1>libdrizzle Public API Documentation</h1>

=== added file 'docs/dev_header.html'
--- docs/dev_header.html	1970-01-01 00:00:00 +0000
+++ docs/dev_header.html	2010-08-12 20:27:32 +0000
@@ -0,0 +1,6 @@
+<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
+<html><head><meta http-equiv="Content-Type" content="text/html;charset=UTF-8">
+<title>$title</title>
+<link href="$relpath$doxygen.css" rel="stylesheet" type="text/css">
+<link href="$relpath$tabs.css" rel="stylesheet" type="text/css">
+</head><body><h1>libdrizzle Developer Documentation</h1>

=== added file 'docs/doxygen.h'
--- docs/doxygen.h	1970-01-01 00:00:00 +0000
+++ docs/doxygen.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,18 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Main page for Doxygen
+ */
+
+/**
+ * @mainpage Drizzle Client & Protocol Library
+ */

=== added file 'docs/drizzled_header.html'
--- docs/drizzled_header.html	1970-01-01 00:00:00 +0000
+++ docs/drizzled_header.html	2010-08-12 20:27:32 +0000
@@ -0,0 +1,6 @@
+<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
+<html><head><meta http-equiv="Content-Type" content="text/html;charset=UTF-8">
+<title>$title</title>
+<link href="$relpath$doxygen.css" rel="stylesheet" type="text/css">
+<link href="$relpath$tabs.css" rel="stylesheet" type="text/css">
+</head><body><h1>Drizzled Public API Documentation</h1>

=== added directory 'examples'
=== added file 'examples/client.c'
--- examples/client.c	1970-01-01 00:00:00 +0000
+++ examples/client.c	2010-08-12 20:27:32 +0000
@@ -0,0 +1,488 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+#include "config.h"
+
+#include <errno.h>
+#include <stdbool.h>
+#include <stdint.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <unistd.h>
+
+#include <libdrizzle/drizzle_client.h>
+
+typedef enum
+{
+  CLIENT_QUERY,
+  CLIENT_FIELDS,
+  CLIENT_ROWS
+} client_state;
+
+typedef enum
+{
+  BUFFER_NONE,
+  BUFFER_FIELD,
+  BUFFER_ROW,
+  BUFFER_ALL
+} buffer_level;
+
+typedef struct
+{
+  drizzle_con_st con;
+  drizzle_result_st result;
+  drizzle_column_st column;
+  client_state state;
+  uint64_t row;
+} client_con_st;
+
+typedef struct
+{
+  drizzle_st drizzle;
+  bool mysql_protocol;
+  client_con_st *client_con_list;
+  uint32_t client_con_count;
+  buffer_level level;
+  char *query;
+  size_t query_len;
+} client_st;
+
+char client_process(client_st *client, client_con_st *client_con);
+void con_info(drizzle_con_st *con);
+void result_info(drizzle_result_st *result);
+void column_info(drizzle_column_st *column);
+
+#define CLIENT_ERROR(__function, __ret, __client) { \
+    printf(__function ":%d:%s\n", __ret, \
+           drizzle_error(&((__client)->drizzle))); \
+    exit(1); }
+
+int main(int argc, char *argv[])
+{
+  client_st client;
+  int c;
+  char blocking= 0;
+  drizzle_return_t ret;
+  uint32_t x;
+  int wait_for_connections= 0;
+  drizzle_con_st *con;
+  client_con_st *client_con;
+  char *host= NULL;
+  in_port_t port= 0;
+  char *user= NULL;
+  char *password= NULL;
+  char *db= NULL;
+
+  memset(&client, 0, sizeof(client_st));
+
+  /* Use one connection by default. */
+  client.client_con_count= 1;
+
+  while ((c = getopt(argc, argv, "bB:c:d:h:Hmp:P:u:")) != -1)
+  {
+    switch(c)
+    {
+    case 'b':
+      blocking= 1;
+      break;
+
+    case 'B':
+      if (!strcasecmp(optarg, "none"))
+        client.level= BUFFER_NONE;
+      else if (!strcasecmp(optarg, "field"))
+        client.level= BUFFER_FIELD;
+      else if (!strcasecmp(optarg, "row"))
+        client.level= BUFFER_ROW;
+      else if (!strcasecmp(optarg, "all"))
+        client.level= BUFFER_ALL;
+      else
+      {
+        printf("Invalid buffer level: %s\n", optarg);
+        exit(0);
+      }
+      break;
+
+    case 'c':
+      client.client_con_count= (uint32_t)atoi(optarg);
+      break;
+
+    case 'd':
+      db= optarg;
+      break;
+
+    case 'h':
+      host= optarg;
+      break;
+
+    case 'm':
+      client.mysql_protocol= true;
+      break;
+
+    case 'p':
+      password= optarg;
+      break;
+
+    case 'P':
+      port= (in_port_t)atoi(optarg);
+      break;
+
+    case 'u':
+      user= optarg;
+      break;
+
+    case 'H':
+    default:
+      printf("\nUsage: %s [options] [query]\n", argv[0]);
+      printf("\t-b            - Use blocking sockets\n");
+      printf("\t-B <level>    - Use buffer <level>, options are:\n");
+      printf("\t                none - Don't buffer anything (default)\n");
+      printf("\t                field - Only buffer individual fields\n");
+      printf("\t                row - Only buffer individual rows\n");
+      printf("\t                all - Buffer entire result\n");
+      printf("\t-c <cons>     - Create <cons> connections\n");
+      printf("\t-d <db>       - Use <db> for the connection\n");
+      printf("\t-h <host>     - Connect to <host>\n");
+      printf("\t-H            - Print this help menu\n");
+      printf("\t-m            - Use MySQL protocol\n");
+      printf("\t-p <password> - Use <password> for authentication\n");
+      printf("\t-P <port>     - Connect to <port>\n");
+      printf("\t-u <user>     - Use <user> for authentication\n");
+      exit(0);
+    }
+  }
+
+  if (argc != optind)
+  {
+    client.query= argv[optind];
+    client.query_len= strlen(client.query);
+  }
+
+  if (client.client_con_count > 0)
+  {
+    client.client_con_list= calloc(client.client_con_count,
+                                   sizeof(client_con_st));
+    if (client.client_con_list == NULL)
+    {
+      printf("calloc:%d\n", errno);
+      exit(1);
+    }
+  }
+
+  /* This may fail if there is other initialization that fails. See docs. */
+  if (drizzle_create(&(client.drizzle)) == NULL)
+  {
+    printf("drizzle_create failed\n");
+    exit(1);
+  }
+
+  if (blocking == 0)
+    drizzle_add_options(&(client.drizzle), DRIZZLE_NON_BLOCKING);
+
+  /* Start all connections, and if in non-blocking mode, return as soon as the
+     connection would block. In blocking mode, this completes the entire
+     connection/query/result. */
+  for (x= 0; x < client.client_con_count; x++)
+  {
+    /* This may fail if there is other initialization that fails. See docs. */
+    con= drizzle_con_add_tcp(&(client.drizzle),
+                              &(client.client_con_list[x].con),
+                              host, port, user, password, db,
+                              client.mysql_protocol ? DRIZZLE_CON_MYSQL : 0);
+    if (con == NULL)
+      CLIENT_ERROR("drizzle_con_add_tcp", 0, &client);
+    drizzle_con_set_context(&(client.client_con_list[x].con),
+                            &(client.client_con_list[x]));
+
+    if (client_process(&client, &(client.client_con_list[x])) == 1)
+      wait_for_connections++;
+  }
+
+  /* If in non-blocking mode, continue to process connections as they become
+     ready. Loop exits when all connections have completed. */
+  while (wait_for_connections != 0)
+  {
+    ret= drizzle_con_wait(&(client.drizzle));
+    if (ret != DRIZZLE_RETURN_OK)
+      CLIENT_ERROR("drizzle_con_wait", ret, &client);
+
+    while ((con= drizzle_con_ready(&(client.drizzle))) != NULL)
+    {
+      client_con= (client_con_st *)drizzle_con_context(con);
+
+      if (client_process(&client, client_con) == 0)
+        wait_for_connections--;
+    }
+  }
+
+  for (x= 0; x < client.client_con_count; x++)
+    drizzle_con_free(&(client.client_con_list[x].con));
+
+  drizzle_free(&(client.drizzle));
+
+  if (client.client_con_list != NULL)
+    free(client.client_con_list);
+
+  return 0;
+}
+
+char client_process(client_st *client, client_con_st *client_con)
+{
+  drizzle_return_t ret;
+  drizzle_column_st *column;
+  size_t offset= 0;
+  size_t length;
+  size_t total;
+  drizzle_field_t field;
+  drizzle_row_t row;
+  size_t *field_sizes;
+  uint16_t x;
+
+  switch (client_con->state)
+  {
+  case CLIENT_QUERY:
+    if (client->query == NULL)
+      break;
+
+    /* This may fail if some allocation fails, but it will set ret. */
+    (void)drizzle_query(&(client_con->con), &(client_con->result),
+                        client->query, client->query_len, &ret);
+    if (ret == DRIZZLE_RETURN_IO_WAIT)
+      return 1;
+    else if (ret != DRIZZLE_RETURN_OK)
+      CLIENT_ERROR("drizzle_query", ret, client);
+
+    result_info(&(client_con->result));
+
+    if (drizzle_result_column_count(&(client_con->result)) == 0)
+      break;
+
+    client_con->state= CLIENT_FIELDS;
+
+  case CLIENT_FIELDS:
+    if (client->level == BUFFER_ALL)
+    {
+      ret= drizzle_result_buffer(&(client_con->result));
+      if (ret == DRIZZLE_RETURN_IO_WAIT)
+        return 1;
+      else if (ret != DRIZZLE_RETURN_OK)
+        CLIENT_ERROR("drizzle_result_buffer", ret, client);
+
+      while ((column= drizzle_column_next(&(client_con->result))) != NULL)
+        column_info(column);
+    }
+    else
+    {
+      while (1)
+      {
+        column= drizzle_column_read(&(client_con->result),
+                                    &(client_con->column), &ret);
+        if (ret == DRIZZLE_RETURN_IO_WAIT)
+          return 1;
+        else if (ret != DRIZZLE_RETURN_OK)
+          CLIENT_ERROR("drizzle_column_read", ret, client);
+
+        if (column == NULL)
+          break;
+
+        column_info(column);
+        drizzle_column_free(column);
+      }
+    }
+
+    client_con->state= CLIENT_ROWS;
+
+  case CLIENT_ROWS:
+    if (client->level == BUFFER_ALL)
+    {
+      /* Everything has been buffered, just loop through and print. */
+      while ((row= drizzle_row_next(&(client_con->result))) != NULL)
+      {
+        field_sizes= drizzle_row_field_sizes(&(client_con->result));
+
+        printf("Row: %" PRId64 "\n",
+               drizzle_row_current(&(client_con->result)));
+
+        for (x= 0; x < drizzle_result_column_count(&(client_con->result)); x++)
+        {
+          if (row[x] == NULL)
+            printf("     (NULL)\n");
+          else
+          {
+            printf("     (%zd) %.*s\n", field_sizes[x], (int32_t)field_sizes[x],
+                   row[x]);
+          }
+        }
+
+        printf("\n");
+      }
+
+      drizzle_result_free(&(client_con->result));
+      break;
+    }
+
+    while (1)
+    {
+      if (client->level == BUFFER_NONE || client->level == BUFFER_FIELD)
+      {
+        /* Still need to read a row at a time, and then each field. */
+        if (client_con->row == 0)
+        {
+          client_con->row= drizzle_row_read(&(client_con->result), &ret);
+          if (ret == DRIZZLE_RETURN_IO_WAIT)
+          {
+            client_con->row= 0;
+            return 1;
+          }
+          else if (ret != DRIZZLE_RETURN_OK)
+            CLIENT_ERROR("drizzle_row", ret, client);
+
+          if (client_con->row == 0)
+          {
+            drizzle_result_free(&(client_con->result));
+            break;
+          }
+
+          printf("Row: %" PRId64 "\n", client_con->row);
+        }
+
+        while (1)
+        {
+          if (client->level == BUFFER_FIELD)
+          {
+            /* Since an entire field is buffered, we don't need to worry about
+               partial reads. */
+            field= drizzle_field_buffer(&(client_con->result), &total, &ret);
+            length= total;
+          }
+          else
+          {
+            field= drizzle_field_read(&(client_con->result), &offset, &length,
+                                      &total, &ret);
+          }
+
+          if (ret == DRIZZLE_RETURN_IO_WAIT)
+            return 1;
+          else if (ret == DRIZZLE_RETURN_ROW_END)
+            break;
+          else if (ret != DRIZZLE_RETURN_OK)
+            CLIENT_ERROR("drizzle_field_read", ret, client);
+
+          if (field == NULL)
+            printf("     (NULL)");
+          else if (offset > 0)
+            printf("%.*s", (int32_t)length, field);
+          else
+            printf("     (%zd) %.*s", total, (int32_t)length, field);
+
+          if (offset + length == total)
+            printf("\n");
+
+          /* If we buffered the entire field, be sure to free it. */
+          if (client->level == BUFFER_FIELD)
+            drizzle_field_free(field);
+        }
+
+        client_con->row= 0;
+        printf("\n");
+      }
+      else if (client->level == BUFFER_ROW)
+      {
+        /* The entire row will be buffered here, so no need to worry about
+           partial reads. */
+        row = drizzle_row_buffer(&(client_con->result), &ret);
+        if (ret == DRIZZLE_RETURN_IO_WAIT)
+          return 1;
+        else if (ret != DRIZZLE_RETURN_OK)
+          CLIENT_ERROR("drizzle_row", ret, client);
+
+        /* This marks the end of rows. */
+        if (row == NULL)
+          break;
+
+        field_sizes= drizzle_row_field_sizes(&(client_con->result));
+
+        printf("Row: %" PRId64 "\n",
+               drizzle_row_current(&(client_con->result)));
+
+        for (x= 0; x < drizzle_result_column_count(&(client_con->result)); x++)
+        {
+          if (row[x] == NULL)
+            printf("     (NULL)\n");
+          else
+          {
+            printf("     (%zd) %.*s\n", field_sizes[x], (int32_t)field_sizes[x],
+                   row[x]);
+          }
+        }
+
+        drizzle_row_free(&(client_con->result), row);
+        printf("\n");
+      }
+    }
+
+    drizzle_result_free(&(client_con->result));
+    break;
+
+  default:
+    /* This should be impossible. */
+    return 1;
+  }
+
+  return 0;
+}
+
+void con_info(drizzle_con_st *con)
+{
+  printf("Connected: protocol_version=%u\n"
+         "                    version=%s\n"
+         "                  thread_id=%u\n"
+         "               capabilities=%u\n"
+         "                   language=%u\n"
+         "                     status=%u\n\n",
+         drizzle_con_protocol_version(con), drizzle_con_server_version(con),
+         drizzle_con_thread_id(con), drizzle_con_capabilities(con),
+         drizzle_con_charset(con), drizzle_con_status(con));
+}
+
+void result_info(drizzle_result_st *result)
+{
+  printf("Result:     row_count=%" PRId64 "\n"
+         "            insert_id=%" PRId64 "\n"
+         "        warning_count=%u\n"
+         "         column_count=%u\n"
+         "        affected_rows=%" PRId64 "\n\n",
+         drizzle_result_row_count(result),
+         drizzle_result_insert_id(result),
+         drizzle_result_warning_count(result),
+         drizzle_result_column_count(result),
+         drizzle_result_affected_rows(result));
+}
+
+void column_info(drizzle_column_st *column)
+{
+  printf("Field:   catalog=%s\n"
+         "              db=%s\n"
+         "           table=%s\n"
+         "       org_table=%s\n"
+         "            name=%s\n"
+         "        org_name=%s\n"
+         "         charset=%u\n"
+         "            size=%u\n"
+         "        max_size=%zu\n"
+         "            type=%u\n"
+         "           flags=%u\n\n",
+         drizzle_column_catalog(column), drizzle_column_db(column),
+         drizzle_column_table(column), drizzle_column_orig_table(column),
+         drizzle_column_name(column), drizzle_column_orig_name(column),
+         drizzle_column_charset(column), drizzle_column_size(column),
+         drizzle_column_max_size(column), drizzle_column_type(column),
+         drizzle_column_flags(column));
+}

=== added file 'examples/pipe_query.c'
--- examples/pipe_query.c	1970-01-01 00:00:00 +0000
+++ examples/pipe_query.c	2010-08-12 20:27:32 +0000
@@ -0,0 +1,173 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+#include "config.h"
+
+#include <errno.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <unistd.h>
+
+#include <libdrizzle/drizzle_client.h>
+
+#define BUFFER_CHUNK 8192
+
+int main(int argc, char *argv[])
+{
+  int c;
+  char *host= NULL;
+  in_port_t port= 0;
+  char *user= NULL;
+  char *password= NULL;
+  char *buffer= NULL;
+  size_t buffer_size= 0;
+  size_t buffer_total= 0;
+  ssize_t read_size= 0;
+  drizzle_st drizzle;
+  drizzle_con_st con;
+  drizzle_result_st result;
+  drizzle_return_t ret;
+  drizzle_field_t field;
+  size_t offset;
+  size_t size;
+  size_t total;
+
+  /* The docs say this might fail, so check for errors. */
+  if (drizzle_create(&drizzle) == NULL)
+  {
+    printf("drizzle_create:failed\n");
+    exit(1);
+  }
+
+  if (drizzle_con_create(&drizzle, &con) == NULL)
+  {
+    printf("drizzle_con_create:%s\n", drizzle_error(&drizzle));
+    exit(1);
+  }
+
+  while ((c = getopt(argc, argv, "d:h:Hmp:P:u:")) != -1)
+  {
+    switch(c)
+    {
+    case 'd':
+      drizzle_con_set_db(&con, optarg);
+      break;
+
+    case 'h':
+      host= optarg;
+      break;
+
+    case 'm':
+      drizzle_con_add_options(&con, DRIZZLE_CON_MYSQL);
+      break;
+
+    case 'p':
+      password= optarg;
+      break;
+
+    case 'P':
+      port= (in_port_t)atoi(optarg);
+      break;
+
+    case 'u':
+      user= optarg;
+      break;
+
+    case 'H':
+    default:
+      printf("\nUsage: %s [options] [query]\n", argv[0]);
+      printf("\t-d <db>       - Use <db> for the connection\n");
+      printf("\t-h <host>     - Connect to <host>\n");
+      printf("\t-H            - Print this help menu\n");
+      printf("\t-m            - Use MySQL protocol\n");
+      printf("\t-p <password> - Use <password> for authentication\n");
+      printf("\t-P <port>     - Connect to <port>\n");
+      printf("\t-u <user>     - Use <user> for authentication\n");
+      exit(0);
+    }
+  }
+
+  drizzle_con_set_tcp(&con, host, port);
+  drizzle_con_set_auth(&con, user, password);
+
+  do
+  {
+    if (read_size == -1)
+    {
+      printf("read:%d\n", errno);
+      return 1;
+    }
+
+    buffer_size+= (size_t)read_size;
+
+    buffer= realloc(buffer, buffer_size + BUFFER_CHUNK);
+    if (buffer == NULL)
+    {
+      printf("realloc:%d\n", errno);
+      return 1;
+    }
+
+    buffer_total= buffer_size + BUFFER_CHUNK;
+  } while ((read_size= read(0, buffer + buffer_size, BUFFER_CHUNK)) != 0);
+
+  (void)drizzle_query(&con, &result, buffer, buffer_size, &ret);
+  if (ret != DRIZZLE_RETURN_OK)
+  {
+    printf("drizzle_query:%s\n", drizzle_error(&drizzle));
+    return 1;
+  }
+
+  free(buffer);
+
+  ret= drizzle_column_skip(&result);
+  if (ret != DRIZZLE_RETURN_OK)
+  {
+    printf("drizzle_column_skip:%s\n", drizzle_error(&drizzle));
+    return 1;
+  }
+
+  while (drizzle_row_read(&result, &ret) != 0 && ret == DRIZZLE_RETURN_OK)
+  {
+    while (1)
+    {
+      field= drizzle_field_read(&result, &offset, &size, &total, &ret);
+      if (ret == DRIZZLE_RETURN_ROW_END)
+        break;
+      else if (ret != DRIZZLE_RETURN_OK)
+      {
+        printf("drizzle_field_read:%s\n", drizzle_error(&drizzle));
+        return 1;
+      }
+
+      if (field == NULL)
+        printf("NULL");
+      else
+        printf("%.*s", (int)size, field);
+
+      if (offset + size == total)
+        printf("\t");
+    }
+
+    printf("\n");
+  }
+
+  if (ret != DRIZZLE_RETURN_OK)
+  {
+    printf("drizzle_row_read:%s\n", drizzle_error(&drizzle));
+    return 1;
+  }
+
+  drizzle_result_free(&result);
+  drizzle_con_free(&con);
+  drizzle_free(&drizzle);
+
+  return 0;
+}

=== added file 'examples/proxy.c'
--- examples/proxy.c	1970-01-01 00:00:00 +0000
+++ examples/proxy.c	2010-08-12 20:27:32 +0000
@@ -0,0 +1,359 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+#include <errno.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <unistd.h>
+
+#include <libdrizzle/drizzle_client.h>
+#include <libdrizzle/drizzle_server.h>
+
+#define DRIZZLE_RETURN_CHECK(__ret, __function, __drizzle) \
+{ \
+  if ((__ret) != DRIZZLE_RETURN_OK) \
+    DRIZZLE_RETURN_ERROR(__function, __drizzle) \
+}
+
+#define DRIZZLE_RETURN_ERROR(__function, __drizzle) \
+{ \
+  printf(__function ":%s\n", drizzle_error(__drizzle)); \
+  return; \
+}
+
+static void proxy(drizzle_st *drizzle, drizzle_con_st *server,
+                  drizzle_con_st *client, drizzle_result_st *server_result,
+                  drizzle_result_st *client_result, drizzle_column_st *column);
+
+int main(int argc, char *argv[])
+{
+  int c;
+  uint32_t count= 0;
+  const char *server_host= NULL;
+  const char *client_host= NULL;
+  bool server_mysql= false;
+  bool client_mysql= false;
+  in_port_t server_port= 0;
+  in_port_t client_port= 0;
+  drizzle_verbose_t verbose= DRIZZLE_VERBOSE_NEVER;
+  drizzle_return_t ret;
+  drizzle_st drizzle;
+  drizzle_con_st con_listen;
+  drizzle_con_st server;
+  drizzle_con_st client;
+  drizzle_result_st server_result;
+  drizzle_result_st client_result;
+  drizzle_column_st column;
+
+  while((c = getopt(argc, argv, "c:h:H:mMp:P:v")) != -1)
+  {
+    switch(c)
+    {
+    case 'c':
+      count= (uint32_t)atoi(optarg);
+      break;
+
+    case 'h':
+      server_host= optarg;
+      break;
+
+    case 'H':
+      client_host= optarg;
+      break;
+
+    case 'm':
+      server_mysql= true;
+      break;
+
+    case 'M':
+      client_mysql= true;
+      break;
+
+    case 'p':
+      server_port= (in_port_t)atoi(optarg);
+      break;
+
+    case 'P':
+      client_port= (in_port_t)atoi(optarg);
+      break;
+
+    case 'v':
+      verbose++;
+      break;
+
+    default:
+      printf("\nusage: %s [-c <count>] [-h <host>] [-H <host>] [-m] [-M] "
+             "[-p <port>] [-p <port>] [-v]\n", argv[0]);
+      printf("\t-c <count> - Number of connections to accept before exiting\n");
+      printf("\t-h <host>  - Host to listen on\n");
+      printf("\t-H <host>  - Host to connect to\n");
+      printf("\t-m         - Use MySQL protocol for incoming connections\n");
+      printf("\t-M         - Use MySQL protocol for outgoing connectionsn\n");
+      printf("\t-p <port>  - Port to listen on\n");
+      printf("\t-P <port>  - Port to connect to\n");
+      printf("\t-v         - Increase verbosity level\n");
+      return 1;
+    }
+  }
+
+  if (drizzle_create(&drizzle) == NULL)
+  {
+    printf("drizzle_create:NULL\n");
+    return 1;
+  }
+
+  drizzle_add_options(&drizzle, DRIZZLE_FREE_OBJECTS);
+  drizzle_set_verbose(&drizzle, verbose);
+
+  if (drizzle_con_create(&drizzle, &con_listen) == NULL)
+  {
+    printf("drizzle_con_create:NULL\n");
+    return 1;
+  }
+
+  drizzle_con_add_options(&con_listen, DRIZZLE_CON_LISTEN);
+  drizzle_con_set_tcp(&con_listen, server_host, server_port);
+
+  if (server_mysql)
+    drizzle_con_add_options(&con_listen, DRIZZLE_CON_MYSQL);
+
+  if (drizzle_con_listen(&con_listen) != DRIZZLE_RETURN_OK)
+  {
+    printf("drizzle_con_listen:%s\n", drizzle_error(&drizzle));
+    return 1;
+  }
+
+  while (1)
+  {
+    (void)drizzle_con_accept(&drizzle, &server, &ret);
+    if (ret != DRIZZLE_RETURN_OK)
+    {
+      printf("drizzle_con_accept:%s\n", drizzle_error(&drizzle));
+      return 1;
+    }
+
+    if (drizzle_con_create(&drizzle, &client) == NULL)
+    {
+      printf("drizzle_con_create:NULL\n");
+      return 1;
+    }
+
+    drizzle_con_add_options(&client,
+                            DRIZZLE_CON_RAW_PACKET | DRIZZLE_CON_RAW_SCRAMBLE);
+    if (client_mysql)
+      drizzle_con_add_options(&client, DRIZZLE_CON_MYSQL);
+    drizzle_con_set_tcp(&client, client_host, client_port);
+
+    ret= drizzle_con_connect(&client);
+    if (ret != DRIZZLE_RETURN_OK)
+    {
+      printf("drizzle_con_connect:%s\n", drizzle_error(&drizzle));
+      return 1;
+    }
+
+    proxy(&drizzle, &server, &client, &server_result, &client_result, &column);
+
+    drizzle_con_free(&client);
+    drizzle_con_free(&server);
+
+    if (count > 0)
+    {
+      count--;
+
+      if (count == 0)
+        break;
+    }
+  }
+
+  drizzle_con_free(&con_listen);
+  drizzle_free(&drizzle);
+
+  return 0;
+}
+
+static void proxy(drizzle_st *drizzle, drizzle_con_st *server,
+                  drizzle_con_st *client, drizzle_result_st *server_result,
+                  drizzle_result_st *client_result, drizzle_column_st *column)
+{
+  drizzle_return_t ret;
+  drizzle_command_t command;
+  const uint8_t *data;
+  size_t offset;
+  size_t size;
+  size_t total;
+  uint64_t row;
+  bool row_break;
+  drizzle_field_t field;
+
+  /* Handshake packets. */
+  ret= drizzle_handshake_server_read(client);
+  DRIZZLE_RETURN_CHECK(ret, "drizzle_handshake_server_read", drizzle)
+
+  drizzle_con_copy_handshake(server, client);
+
+  ret= drizzle_handshake_server_write(server);
+  DRIZZLE_RETURN_CHECK(ret, "drizzle_handshake_server_write", drizzle)
+
+  ret= drizzle_handshake_client_read(server);
+  DRIZZLE_RETURN_CHECK(ret, "drizzle_handshake_client_read", drizzle)
+
+  drizzle_con_copy_handshake(client, server);
+
+  ret= drizzle_handshake_client_write(client);
+  DRIZZLE_RETURN_CHECK(ret, "drizzle_handshake_client_write", drizzle)
+
+  (void)drizzle_result_read(client, client_result, &ret);
+  DRIZZLE_RETURN_CHECK(ret, "drizzle_result_read", drizzle)
+
+  drizzle_con_set_status(server, drizzle_con_status(client));
+
+  if (drizzle_result_clone(server, server_result, client_result) == NULL)
+    DRIZZLE_RETURN_ERROR("drizzle_result_clone", drizzle)
+
+  ret= drizzle_result_write(server, server_result, true);
+  DRIZZLE_RETURN_CHECK(ret, "drizzle_result_write", drizzle)
+
+  if (drizzle_result_error_code(client_result) != 0 ||
+      drizzle_result_eof(client_result))
+  {
+    /* There was a handshake or authentication error. */
+    return;
+  }
+
+  drizzle_con_add_options(client, DRIZZLE_CON_READY);
+
+  /* Command loop. */
+  while (1)
+  {
+    drizzle_result_free(server_result);
+    drizzle_result_free(client_result);
+
+    while (1)
+    {
+      data= drizzle_con_command_read(server, &command, &offset, &size, &total,
+                                     &ret);
+      if (ret == DRIZZLE_RETURN_LOST_CONNECTION)
+        return;
+
+      DRIZZLE_RETURN_CHECK(ret, "drizzle_con_command_read", drizzle)
+
+      (void)drizzle_con_command_write(client, NULL, command, data, size, total,
+                                      &ret);
+      DRIZZLE_RETURN_CHECK(ret, "drizzle_con_command_write", drizzle)
+
+      if ((offset + size) == total)
+        break;
+    }
+
+    if (command == DRIZZLE_COMMAND_QUIT)
+      return;
+    else if (command == DRIZZLE_COMMAND_FIELD_LIST)
+    {
+      if (drizzle_result_create(client, client_result) == NULL)
+        DRIZZLE_RETURN_ERROR("drizzle_result_create", drizzle)
+
+      if (drizzle_result_create(server, server_result) == NULL)
+        DRIZZLE_RETURN_ERROR("drizzle_result_create", drizzle)
+    }
+    else
+    {
+      (void)drizzle_result_read(client, client_result, &ret);
+      DRIZZLE_RETURN_CHECK(ret, "drizzle_result_read", drizzle)
+
+      drizzle_con_set_status(server, drizzle_con_status(client));
+      if (drizzle_result_clone(server, server_result, client_result) == NULL)
+        DRIZZLE_RETURN_ERROR("drizzle_result_clone", drizzle)
+
+      if (drizzle_result_column_count(client_result) == 0)
+      {
+        /* Simple result with no column, row, or field data. */
+        ret= drizzle_result_write(server, server_result, true);
+        DRIZZLE_RETURN_CHECK(ret, "drizzle_result_write", drizzle)
+
+        continue;
+      }
+
+      ret= drizzle_result_write(server, server_result, false);
+      DRIZZLE_RETURN_CHECK(ret, "drizzle_result_write", drizzle)
+    }
+
+    /* Columns. */
+    while (1)
+    {
+      if (drizzle_column_read(client_result, column, &ret) == NULL)
+      {
+        DRIZZLE_RETURN_CHECK(ret, "drizzle_column_read", drizzle)
+        break;
+      }
+
+      DRIZZLE_RETURN_CHECK(ret, "drizzle_column_read", drizzle)
+
+      ret= drizzle_column_write(server_result, column);
+      DRIZZLE_RETURN_CHECK(ret, "drizzle_column_write", drizzle)
+
+      drizzle_column_free(column);
+    }
+
+    drizzle_con_set_status(server, drizzle_con_status(client));
+    drizzle_result_set_eof(server_result, true);
+
+    if (command == DRIZZLE_COMMAND_FIELD_LIST)
+    {
+      ret= drizzle_result_write(server, server_result, true);
+      DRIZZLE_RETURN_CHECK(ret, "drizzle_result_write", drizzle)
+      continue;
+    }
+    else
+    {
+      ret= drizzle_result_write(server, server_result, false);
+      DRIZZLE_RETURN_CHECK(ret, "drizzle_result_write", drizzle)
+    }
+
+    /* Rows. */
+    while (1)
+    {
+      row= drizzle_row_read(client_result, &ret);
+      DRIZZLE_RETURN_CHECK(ret, "drizzle_row_read", drizzle)
+
+      if (row == 0)
+        break;
+
+      drizzle_result_set_row_size(server_result,
+                                  drizzle_result_row_size(client_result));
+
+      ret= drizzle_row_write(server_result);
+      DRIZZLE_RETURN_CHECK(ret, "drizzle_row_write", drizzle)
+
+      /* Fields. */
+      row_break= false;
+      while (row_break == false)
+      {
+        field= drizzle_field_read(client_result, &offset, &size, &total, &ret);
+        if (ret == DRIZZLE_RETURN_ROW_END)
+          break;
+        else if (ret == DRIZZLE_RETURN_ROW_BREAK)
+        {
+          if (size == 0)
+            break;
+
+          row_break= true;
+        }
+        else
+          DRIZZLE_RETURN_CHECK(ret, "drizzle_field_read", drizzle)
+
+        ret= drizzle_field_write(server_result, field, size, total);
+        DRIZZLE_RETURN_CHECK(ret, "drizzle_field_write", drizzle)
+      }
+    }
+
+    ret= drizzle_result_write(server, server_result, true);
+    DRIZZLE_RETURN_CHECK(ret, "drizzle_result_write", drizzle)
+  }
+}

=== added file 'examples/server.c'
--- examples/server.c	1970-01-01 00:00:00 +0000
+++ examples/server.c	2010-08-12 20:27:32 +0000
@@ -0,0 +1,274 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+#include <errno.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <unistd.h>
+
+#include <libdrizzle/drizzle_server.h>
+
+#define DRIZZLE_FIELD_MAX 32
+#define DRIZZLE_RESULT_ROWS 20
+
+#define DRIZZLE_RETURN_CHECK(__ret, __function, __drizzle) \
+{ \
+  if ((__ret) != DRIZZLE_RETURN_OK) \
+    DRIZZLE_RETURN_ERROR(__function, __drizzle) \
+}
+
+#define DRIZZLE_RETURN_ERROR(__function, __drizzle) \
+{ \
+  printf(__function ":%s\n", drizzle_error(__drizzle)); \
+  return; \
+}
+
+static void server(drizzle_st *drizzle, drizzle_con_st *con,
+                   drizzle_result_st *result, drizzle_column_st *column);
+
+int main(int argc, char *argv[])
+{
+  int c;
+  uint32_t count= 0;
+  const char *host= NULL;
+  bool mysql= false;
+  in_port_t port= 0;
+  drizzle_verbose_t verbose= DRIZZLE_VERBOSE_NEVER;
+  drizzle_return_t ret;
+  drizzle_st drizzle;
+  drizzle_con_st con_listen;
+  drizzle_con_st con;
+  drizzle_result_st result;
+  drizzle_column_st column;
+
+  while((c = getopt(argc, argv, "c:h:mp:v")) != -1)
+  {
+    switch(c)
+    {
+    case 'c':
+      count= (uint32_t)atoi(optarg);
+      break;
+
+    case 'h':
+      host= optarg;
+      break;
+
+    case 'm':
+      mysql= true;
+      break;
+
+    case 'p':
+      port= (in_port_t)atoi(optarg);
+      break;
+
+    case 'v':
+      verbose++;
+      break;
+
+    default:
+      printf("\nusage: %s [-c <count>] [-h <host>] [-m] [-p <port>] [-v]\n",
+             argv[0]);
+      printf("\t-c <count> - Number of connections to accept before exiting\n");
+      printf("\t-h <host>  - Host to listen on\n");
+      printf("\t-m         - Use the MySQL protocol\n");
+      printf("\t-p <port>  - Port to listen on\n");
+      printf("\t-v         - Increase verbosity level\n");
+      return 1;
+    }
+  }
+
+  if (drizzle_create(&drizzle) == NULL)
+  {
+    printf("drizzle_create:NULL\n");
+    return 1;
+  }
+
+  drizzle_add_options(&drizzle, DRIZZLE_FREE_OBJECTS);
+  drizzle_set_verbose(&drizzle, verbose);
+
+  if (drizzle_con_create(&drizzle, &con_listen) == NULL)
+  {
+    printf("drizzle_con_create:NULL\n");
+    return 1;
+  }
+
+  drizzle_con_add_options(&con_listen, DRIZZLE_CON_LISTEN);
+  drizzle_con_set_tcp(&con_listen, host, port);
+
+  if (mysql)
+    drizzle_con_add_options(&con_listen, DRIZZLE_CON_MYSQL);
+
+  if (drizzle_con_listen(&con_listen) != DRIZZLE_RETURN_OK)
+  {
+    printf("drizzle_con_listen:%s\n", drizzle_error(&drizzle));
+    return 1;
+  }
+
+  while (1)
+  {
+    (void)drizzle_con_accept(&drizzle, &con, &ret);
+    if (ret != DRIZZLE_RETURN_OK)
+    {
+      printf("drizzle_con_accept:%s\n", drizzle_error(&drizzle));
+      return 1;
+    }
+
+    server(&drizzle, &con, &result, &column);
+
+    drizzle_con_free(&con);
+
+    if (count > 0)
+    {
+      count--;
+
+      if (count == 0)
+        break;
+    }
+  }
+
+  drizzle_con_free(&con_listen);
+  drizzle_free(&drizzle);
+
+  return 0;
+}
+
+static void server(drizzle_st *drizzle, drizzle_con_st *con,
+                   drizzle_result_st *result, drizzle_column_st *column)
+{
+  drizzle_return_t ret;
+  drizzle_command_t command;
+  uint8_t *data= NULL;
+  size_t total;
+  char *field[2];
+  char field1[DRIZZLE_FIELD_MAX];
+  char field2[DRIZZLE_FIELD_MAX];
+  size_t size[2];
+  uint64_t x;
+
+  field[0]= field1;
+  field[1]= field2;
+
+  /* Handshake packets. */
+  drizzle_con_set_protocol_version(con, 10);
+  drizzle_con_set_server_version(con, "libdrizzle example 1.2.3");
+  drizzle_con_set_thread_id(con, 1);
+  drizzle_con_set_scramble(con, (const uint8_t *)"ABCDEFGHIJKLMNOPQRST");
+  drizzle_con_set_capabilities(con, DRIZZLE_CAPABILITIES_NONE);
+  drizzle_con_set_charset(con, 8);
+  drizzle_con_set_status(con, DRIZZLE_CON_STATUS_NONE);
+  drizzle_con_set_max_packet_size(con, DRIZZLE_MAX_PACKET_SIZE);
+
+  ret= drizzle_handshake_server_write(con);
+  DRIZZLE_RETURN_CHECK(ret, "drizzle_handshake_server_write", drizzle)
+
+  ret= drizzle_handshake_client_read(con);
+  DRIZZLE_RETURN_CHECK(ret, "drizzle_handshake_client_read", drizzle)
+
+  if (drizzle_result_create(con, result) == NULL)
+    DRIZZLE_RETURN_ERROR("drizzle_result_create", drizzle)
+
+  ret= drizzle_result_write(con, result, true);
+  DRIZZLE_RETURN_CHECK(ret, "drizzle_result_write", drizzle)
+
+  /* Command loop. */
+  while (1)
+  {
+    drizzle_result_free(result);
+    if (data != NULL)
+      free(data);
+
+    data= drizzle_con_command_buffer(con, &command, &total, &ret);
+    if (ret == DRIZZLE_RETURN_LOST_CONNECTION ||
+        (ret == DRIZZLE_RETURN_OK && command == DRIZZLE_COMMAND_QUIT))
+    {
+      if (data != NULL)
+        free(data);
+      return;
+    }
+    DRIZZLE_RETURN_CHECK(ret, "drizzle_con_command_buffer", drizzle)
+
+    if (drizzle_result_create(con, result) == NULL)
+      DRIZZLE_RETURN_ERROR("drizzle_result_create", drizzle)
+
+    if (command != DRIZZLE_COMMAND_QUERY)
+    {
+      ret= drizzle_result_write(con, result, true);
+      DRIZZLE_RETURN_CHECK(ret, "drizzle_result_write", drizzle)
+      continue;
+    }
+
+    drizzle_result_set_column_count(result, 2);
+
+    ret= drizzle_result_write(con, result, false);
+    DRIZZLE_RETURN_CHECK(ret, "drizzle_result_write", drizzle)
+
+    /* Columns. */
+    if (drizzle_column_create(result, column) == NULL)
+      DRIZZLE_RETURN_ERROR("drizzle_column_create", drizzle)
+
+    drizzle_column_set_catalog(column, "default");
+    drizzle_column_set_db(column, "drizzle_test_db");
+    drizzle_column_set_table(column, "drizzle_test_table");
+    drizzle_column_set_orig_table(column, "drizzle_test_table");
+    drizzle_column_set_name(column, "test_column_1");
+    drizzle_column_set_orig_name(column, "test_column_1");
+    drizzle_column_set_charset(column, 8);
+    drizzle_column_set_size(column, DRIZZLE_FIELD_MAX);
+    drizzle_column_set_type(column, DRIZZLE_COLUMN_TYPE_VARCHAR);
+
+    ret= drizzle_column_write(result, column);
+    DRIZZLE_RETURN_CHECK(ret, "drizzle_column_write", drizzle)
+
+    drizzle_column_set_name(column, "test_column_2");
+    drizzle_column_set_orig_name(column, "test_column_2");
+
+    ret= drizzle_column_write(result, column);
+    DRIZZLE_RETURN_CHECK(ret, "drizzle_column_write", drizzle)
+
+    drizzle_column_free(column);
+
+    drizzle_result_set_eof(result, true);
+
+    ret= drizzle_result_write(con, result, false);
+    DRIZZLE_RETURN_CHECK(ret, "drizzle_result_write", drizzle)
+
+    /* Rows. */
+    for (x= 0; x < DRIZZLE_RESULT_ROWS; x++)
+    {
+      size[0]= (size_t)snprintf(field[0], DRIZZLE_FIELD_MAX,
+                                "field %" PRIu64 "-1", x);
+      if (size[0] >= DRIZZLE_FIELD_MAX)
+        size[0]= DRIZZLE_FIELD_MAX - 1;
+
+      size[1]= (size_t)snprintf(field[1], DRIZZLE_FIELD_MAX,
+                                "field %" PRIu64 "-2", x);
+      if (size[1] >= DRIZZLE_FIELD_MAX)
+        size[1]= DRIZZLE_FIELD_MAX - 1;
+
+      /* This is needed for MySQL and old Drizzle protocol. */
+      drizzle_result_calc_row_size(result, (drizzle_field_t *)field, size);
+
+      ret= drizzle_row_write(result);
+      DRIZZLE_RETURN_CHECK(ret, "drizzle_row_write", drizzle)
+
+      /* Fields. */
+      ret= drizzle_field_write(result, (drizzle_field_t)field[0], size[0],
+                               size[0]);
+      DRIZZLE_RETURN_CHECK(ret, "drizzle_field_write", drizzle)
+
+      ret= drizzle_field_write(result, (drizzle_field_t)field[1], size[1],
+                               size[1]);
+      DRIZZLE_RETURN_CHECK(ret, "drizzle_field_write", drizzle)
+    }
+
+    ret= drizzle_result_write(con, result, true);
+    DRIZZLE_RETURN_CHECK(ret, "drizzle_result_write", drizzle)
+  }
+}

=== added file 'examples/simple.c'
--- examples/simple.c	1970-01-01 00:00:00 +0000
+++ examples/simple.c	2010-08-12 20:27:32 +0000
@@ -0,0 +1,121 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <unistd.h>
+
+#include <libdrizzle/drizzle_client.h>
+
+int main(int argc, char *argv[])
+{
+  int c;
+  const char *db= "INFORMATION_SCHEMA";
+  const char *host= NULL;
+  bool mysql= false;
+  in_port_t port= 0;
+  const char *query= "SELECT TABLE_SCHEMA,TABLE_NAME FROM TABLES";
+  drizzle_verbose_t verbose= DRIZZLE_VERBOSE_NEVER;
+  drizzle_st drizzle;
+  drizzle_con_st con;
+  drizzle_result_st result;
+  drizzle_return_t ret;
+  int x;
+  char **row;
+
+  while ((c = getopt(argc, argv, "d:h:mp:q:v")) != -1)
+  {
+    switch(c)
+    {
+    case 'd':
+      db= optarg;
+      break;
+
+    case 'h':
+      host= optarg;
+      break;
+
+    case 'm':
+      mysql= true;
+      break;
+
+    case 'p':
+      port= (in_port_t)atoi(optarg);
+      break;
+
+    case 'q':
+      query= optarg;
+      break;
+
+    case 'v':
+      verbose++;
+      break;
+
+    default:
+      printf("usage: %s [-d <db>] [-h <host>] [-m] [-p <port>] [-q <query>] "
+             "[-v]\n", argv[0]);
+      printf("\t-d <db>    - Database to use for query\n");
+      printf("\t-h <host>  - Host to listen on\n");
+      printf("\t-m         - Use the MySQL protocol\n");
+      printf("\t-p <port>  - Port to listen on\n");
+      printf("\t-q <query> - Query to run\n");
+      printf("\t-v         - Increase verbosity level\n");
+      return 1;
+    }
+  }
+
+  if (drizzle_create(&drizzle) == NULL)
+  {
+    printf("drizzle_create:NULL\n");
+    return 1;
+  }
+
+  drizzle_set_verbose(&drizzle, verbose);
+
+  if (drizzle_con_create(&drizzle, &con) == NULL)
+  {
+    printf("drizzle_con_create:NULL\n");
+    return 1;
+  }
+
+  if (mysql)
+    drizzle_con_add_options(&con, DRIZZLE_CON_MYSQL);
+
+  drizzle_con_set_tcp(&con, host, port);
+  drizzle_con_set_db(&con, db);
+
+  (void)drizzle_query_str(&con, &result, query, &ret);
+  if (ret != DRIZZLE_RETURN_OK)
+  {
+    printf("drizzle_query:%s\n", drizzle_con_error(&con));
+    return 1;
+  }
+
+  ret= drizzle_result_buffer(&result);
+  if (ret != DRIZZLE_RETURN_OK)
+  {
+    printf("drizzle_result_buffer:%s\n", drizzle_con_error(&con));
+    return 1;
+  }
+
+  while ((row= (char **)drizzle_row_next(&result)) != NULL)
+  {
+    for (x= 0; x < drizzle_result_column_count(&result); x++)
+      printf("%s%s", x == 0 ? "" : ":", row[x] == NULL ? "NULL" : row[x]);
+    printf("\n");
+  }
+
+  drizzle_result_free(&result);
+  drizzle_con_free(&con);
+  drizzle_free(&drizzle);
+
+  return 0;
+}

=== added file 'examples/simple_multi.c'
--- examples/simple_multi.c	1970-01-01 00:00:00 +0000
+++ examples/simple_multi.c	2010-08-12 20:27:32 +0000
@@ -0,0 +1,96 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+#include <stdio.h>
+#include <string.h>
+
+#include <libdrizzle/drizzle_client.h>
+
+#define SIMPLE_MULTI_COUNT 10
+
+int main(int argc, char *argv[])
+{
+  const char *query= "SELECT table_schema,table_name FROM tables";
+  drizzle_st drizzle;
+  drizzle_con_st con[SIMPLE_MULTI_COUNT];
+  drizzle_result_st result[SIMPLE_MULTI_COUNT];
+  drizzle_query_st ql[SIMPLE_MULTI_COUNT];
+  drizzle_return_t ret;
+  drizzle_row_t row;
+  int x;
+
+  if (drizzle_create(&drizzle) == NULL)
+  {
+    printf("drizzle_create:NULL\n");
+    return 1;
+  }
+
+  /* Create SIMPLE_MULTI_COUNT connections and initialize query list. */
+  for (x= 0; x < SIMPLE_MULTI_COUNT; x++)
+  {
+    if (x == 0)
+    {
+      if (drizzle_con_create(&drizzle, &(con[0])) == NULL)
+      {
+        printf("drizzle_con_create:%s\n", drizzle_error(&drizzle));
+        return 1;
+      }
+
+      if (argc == 2 && !strcmp(argv[1], "-m"))
+        drizzle_con_add_options(&(con[0]), DRIZZLE_CON_MYSQL);
+      else if (argc != 1)
+      {
+        printf("usage: %s [-m]\n", argv[0]);
+        return 1;
+      }
+
+      drizzle_con_set_db(&(con[0]), "information_schema");
+    }
+    else
+    {
+      if (drizzle_con_clone(&drizzle, &(con[x]), &(con[0])) == NULL)
+      {
+        printf("drizzle_con_clone:%s\n", drizzle_error(&drizzle));
+        return 1;
+      }
+    }
+
+    if (drizzle_query_add(&drizzle, &(ql[x]), &(con[x]), &(result[x]), query,
+                          strlen(query), 0, NULL) == NULL)
+    {
+      printf("drizzle_query_add:%s\n", drizzle_error(&drizzle));
+      return 1;
+    }
+  }
+
+  ret= drizzle_query_run_all(&drizzle);
+  if (ret != DRIZZLE_RETURN_OK)
+  {
+    printf("drizzle_query_run_all:%s\n", drizzle_error(&drizzle));
+    return 1;
+  }
+
+  for (x= 0; x < SIMPLE_MULTI_COUNT; x++)
+  {
+    if (drizzle_result_error_code(&(result[x])) != 0)
+    {
+      printf("%d:%s\n", drizzle_result_error_code(&(result[x])),
+             drizzle_result_error(&(result[x])));
+      continue;
+    }
+
+    while ((row= drizzle_row_next(&(result[x]))) != NULL)
+      printf("%d %s:%s\n", x, row[0], row[1]);
+  }
+
+  drizzle_free(&drizzle);
+
+  return 0;
+}

=== added file 'examples/sqlite_server.c'
--- examples/sqlite_server.c	1970-01-01 00:00:00 +0000
+++ examples/sqlite_server.c	2010-08-12 20:27:32 +0000
@@ -0,0 +1,445 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+#include <errno.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <strings.h>
+#include <unistd.h>
+
+#include <libdrizzle/drizzle_server.h>
+#include <sqlite3.h>
+
+#define SQLITE_SERVER_VERSION "SQLite Server using libdrizzle 0.1"
+
+#define DRIZZLE_RETURN_CHECK(__ret, __function, __drizzle) \
+{ \
+  if ((__ret) != DRIZZLE_RETURN_OK) \
+    DRIZZLE_RETURN_ERROR(__function, __drizzle) \
+}
+
+#define DRIZZLE_RETURN_ERROR(__function, __drizzle) \
+{ \
+  printf(__function ":%s\n", drizzle_error(__drizzle)); \
+  return; \
+}
+
+#define DRIZZLE_RETURN_CHECK_VAL(__ret, __function, __drizzle) \
+{ \
+  if ((__ret) != DRIZZLE_RETURN_OK) \
+  { \
+    printf(__function ":%s\n", drizzle_error(__drizzle)); \
+    return ret; \
+  } \
+}
+
+typedef struct
+{
+  drizzle_st drizzle;
+  drizzle_con_st con;
+  drizzle_result_st result;
+  drizzle_column_st column;
+  sqlite3* db;
+  bool send_columns;
+  drizzle_verbose_t verbose;
+  uint64_t rows;
+} sqlite_server;
+
+static void server_run(sqlite_server *server);
+static int row_cb(void *data, int field_count, char **fields, char **columns);
+static drizzle_return_t send_version(sqlite_server *server);
+static void usage(char *name);
+
+int main(int argc, char *argv[])
+{
+  int c;
+  uint32_t count= 0;
+  const char *host= NULL;
+  bool mysql= false;
+  in_port_t port= 0;
+  drizzle_return_t ret;
+  sqlite_server server;
+  drizzle_con_st con_listen;
+
+  server.db= NULL;
+  server.verbose= DRIZZLE_VERBOSE_NEVER;
+
+  while((c = getopt(argc, argv, "c:h:mp:v")) != -1)
+  {
+    switch(c)
+    {
+    case 'c':
+      count= (uint32_t)atoi(optarg);
+      break;
+
+    case 'h':
+      host= optarg;
+      break;
+
+    case 'm':
+      mysql= true;
+      break;
+
+    case 'p':
+      port= (in_port_t)atoi(optarg);
+      break;
+
+    case 'v':
+      server.verbose++;
+      break;
+
+    default:
+      usage(argv[0]);
+      return 1;
+    }
+  }
+
+  if (argc != (optind + 1))
+  {
+    usage(argv[0]);
+    return 1;
+  }
+
+  sqlite3_open(argv[optind], &(server.db));
+  if (server.db == NULL)
+  {
+    printf("sqlite3_open: could not open sqlite3 db\n");
+    return 1;
+  }
+
+  if (drizzle_create(&server.drizzle) == NULL)
+  {
+    printf("drizzle_create:NULL\n");
+    return 1;
+  }
+
+  drizzle_add_options(&server.drizzle, DRIZZLE_FREE_OBJECTS);
+  drizzle_set_verbose(&server.drizzle, server.verbose);
+
+  if (drizzle_con_create(&server.drizzle, &con_listen) == NULL)
+  {
+    printf("drizzle_con_create:NULL\n");
+    return 1;
+  }
+
+  drizzle_con_add_options(&con_listen, DRIZZLE_CON_LISTEN);
+  drizzle_con_set_tcp(&con_listen, host, port);
+
+  if (mysql)
+    drizzle_con_add_options(&con_listen, DRIZZLE_CON_MYSQL);
+
+  if (drizzle_con_listen(&con_listen) != DRIZZLE_RETURN_OK)
+  {
+    printf("drizzle_con_listen:%s\n", drizzle_error(&server.drizzle));
+    return 1;
+  }
+
+  while (1)
+  {
+    (void)drizzle_con_accept(&server.drizzle, &server.con, &ret);
+    if (ret != DRIZZLE_RETURN_OK)
+    {
+      printf("drizzle_con_accept:%s\n", drizzle_error(&server.drizzle));
+      return 1;
+    }
+
+    server_run(&server);
+
+    drizzle_con_free(&server.con);
+
+    if (count > 0)
+    {
+      count--;
+
+      if (count == 0)
+        break;
+    }
+  }
+
+  drizzle_con_free(&con_listen);
+  drizzle_free(&server.drizzle);
+  sqlite3_close(server.db);
+
+  return 0;
+}
+
+static void server_run(sqlite_server *server)
+{
+  drizzle_return_t ret;
+  drizzle_command_t command;
+  uint8_t *data= NULL;
+  size_t total;
+  int sqlite_ret;
+  char *sqlite_err;
+
+  /* Handshake packets. */
+  drizzle_con_set_protocol_version(&(server->con), 10);
+  drizzle_con_set_server_version(&(server->con), "libdrizzle+SQLite");
+  drizzle_con_set_thread_id(&(server->con), 1);
+  drizzle_con_set_scramble(&(server->con),
+                           (const uint8_t *)"ABCDEFGHIJKLMNOPQRST");
+  drizzle_con_set_capabilities(&(server->con), DRIZZLE_CAPABILITIES_NONE);
+  drizzle_con_set_charset(&(server->con), 8);
+  drizzle_con_set_status(&(server->con), DRIZZLE_CON_STATUS_NONE);
+  drizzle_con_set_max_packet_size(&(server->con), DRIZZLE_MAX_PACKET_SIZE);
+
+  ret= drizzle_handshake_server_write(&(server->con));
+  DRIZZLE_RETURN_CHECK(ret, "drizzle_handshake_server_write",
+                       &(server->drizzle))
+
+  ret= drizzle_handshake_client_read(&(server->con));
+  DRIZZLE_RETURN_CHECK(ret, "drizzle_handshake_client_read", &(server->drizzle))
+
+  if (drizzle_result_create(&(server->con), &(server->result)) == NULL)
+    DRIZZLE_RETURN_ERROR("drizzle_result_create", &(server->drizzle))
+
+  ret= drizzle_result_write(&(server->con), &(server->result), true);
+  DRIZZLE_RETURN_CHECK(ret, "drizzle_result_write", &(server->drizzle))
+
+  /* Command loop. */
+  while (1)
+  {
+    drizzle_result_free(&(server->result));
+    if (data != NULL)
+      free(data);
+
+    data= drizzle_con_command_buffer(&(server->con), &command, &total, &ret);
+    if (ret == DRIZZLE_RETURN_LOST_CONNECTION ||
+        (ret == DRIZZLE_RETURN_OK && command == DRIZZLE_COMMAND_QUIT))
+    {
+      if (data != NULL)
+        free(data);
+      return;
+    }
+    DRIZZLE_RETURN_CHECK(ret, "drizzle_con_command_buffer", &(server->drizzle))
+
+    if (server->verbose >= DRIZZLE_VERBOSE_INFO)
+    {
+      printf("Command=%u Data=%s\n", command,
+             data == NULL ? "NULL" : (char *)data);
+    }
+
+    if (drizzle_result_create(&(server->con), &(server->result)) == NULL)
+      DRIZZLE_RETURN_ERROR("drizzle_result_create", &(server->drizzle))
+
+    if (command != DRIZZLE_COMMAND_QUERY ||
+        !strcasecmp((char *)data, "SHOW DATABASES"))
+    {
+      ret= drizzle_result_write(&(server->con), &(server->result), true);
+      DRIZZLE_RETURN_CHECK(ret, "drizzle_result_write", &(server->drizzle))
+
+      if (command == DRIZZLE_COMMAND_FIELD_LIST)
+      {
+        drizzle_result_set_eof(&(server->result), true);
+        ret= drizzle_result_write(&(server->con), &(server->result), true);
+        DRIZZLE_RETURN_CHECK(ret, "drizzle_result_write", &(server->drizzle))
+      }
+
+      continue;
+    }
+
+    if (strstr((char *)data, "@@version") != NULL)
+    {
+      ret= send_version(server);
+      if (ret != DRIZZLE_RETURN_OK)
+        return;
+
+      continue;
+    }
+
+    server->send_columns= true;
+    server->rows= 0;
+
+    if (!strcasecmp((char *)data, "SHOW TABLES"))
+    {
+      sqlite_ret= sqlite3_exec(server->db,
+                            "SELECT name FROM sqlite_master WHERE type='table'",
+                               row_cb, server, &sqlite_err);
+    }
+    else
+    {
+      sqlite_ret= sqlite3_exec(server->db, (char *)data, row_cb, server,
+                               &sqlite_err);
+    }
+
+    if (sqlite_ret != SQLITE_OK)
+    {
+      if (sqlite_err == NULL)
+        printf("sqlite3_exec failed\n");
+      else
+      {
+        drizzle_result_set_error_code(&(server->result), (uint16_t)sqlite_ret);
+        drizzle_result_set_error(&(server->result), sqlite_err);
+        ret= drizzle_result_write(&(server->con), &(server->result), true);
+        DRIZZLE_RETURN_CHECK(ret, "drizzle_result_write", &(server->drizzle))
+
+        printf("sqlite3_exec:%s\n", sqlite_err);
+        sqlite3_free(sqlite_err);
+      }
+
+      return;
+    }
+
+    if (server->rows == 0)
+    {
+      drizzle_result_set_column_count(&(server->result), 0);
+      ret= drizzle_result_write(&(server->con), &(server->result), true);
+      DRIZZLE_RETURN_CHECK(ret, "drizzle_result_write", &(server->drizzle))
+    }
+    else
+    {
+      drizzle_result_set_eof(&(server->result), true);
+      ret= drizzle_result_write(&(server->con), &(server->result), true);
+      DRIZZLE_RETURN_CHECK(ret, "drizzle_result_write", &(server->drizzle))
+    }
+  }
+}
+
+static int row_cb(void *data, int field_count, char **fields, char **columns)
+{
+  sqlite_server *server= (sqlite_server *)data;
+  drizzle_return_t ret;
+  int x;
+  size_t sizes[8192];
+
+  if (server->send_columns == true)
+  {
+    server->send_columns= false;
+    drizzle_result_set_column_count(&(server->result), (uint16_t)field_count);
+
+    ret= drizzle_result_write(&(server->con), &(server->result), false);
+    DRIZZLE_RETURN_CHECK_VAL(ret, "drizzle_result_write", &(server->drizzle))
+
+    if (drizzle_column_create(&(server->result), &(server->column)) == NULL)
+    {
+      DRIZZLE_RETURN_CHECK_VAL(DRIZZLE_RETURN_MEMORY, "drizzle_column_create",
+                               &(server->drizzle))
+    }
+
+    drizzle_column_set_catalog(&(server->column), "sqlite");
+    drizzle_column_set_db(&(server->column), "sqlite_db");
+    drizzle_column_set_table(&(server->column), "sqlite_table");
+    drizzle_column_set_orig_table(&(server->column), "sqlite_table");
+    drizzle_column_set_charset(&(server->column), 8);
+    drizzle_column_set_type(&(server->column), DRIZZLE_COLUMN_TYPE_VARCHAR);
+
+    for (x= 0; x < field_count; x++)
+    {
+      drizzle_column_set_size(&(server->column),
+                              fields[x] == NULL ?
+                              0 : (uint32_t)strlen(fields[x]));
+      drizzle_column_set_name(&(server->column), columns[x]);
+      drizzle_column_set_orig_name(&(server->column), columns[x]);
+
+      ret= drizzle_column_write(&(server->result), &(server->column));
+      DRIZZLE_RETURN_CHECK_VAL(ret, "drizzle_column_write", &(server->drizzle))
+    }
+
+    drizzle_column_free(&(server->column));
+
+    drizzle_result_set_eof(&(server->result), true);
+
+    ret= drizzle_result_write(&(server->con), &(server->result), false);
+    DRIZZLE_RETURN_CHECK_VAL(ret, "drizzle_result_write", &(server->drizzle))
+  }
+
+  for (x= 0; x < field_count; x++)
+  {
+    if (fields[x] == NULL)
+      sizes[x]= 0;
+    else
+      sizes[x]= strlen(fields[x]);
+  }
+
+  /* This is needed for MySQL and old Drizzle protocol. */
+  drizzle_result_calc_row_size(&(server->result), (drizzle_field_t *)fields,
+                               sizes);
+
+  ret= drizzle_row_write(&(server->result));
+  DRIZZLE_RETURN_CHECK_VAL(ret, "drizzle_row_write", &(server->drizzle))
+
+  for (x= 0; x < field_count; x++)
+  {
+    ret= drizzle_field_write(&(server->result), (drizzle_field_t)fields[x],
+                             sizes[x], sizes[x]);
+    DRIZZLE_RETURN_CHECK_VAL(ret, "drizzle_field_write", &(server->drizzle))
+  }
+
+  server->rows++;
+
+  return 0;
+}
+
+static drizzle_return_t send_version(sqlite_server *server)
+{
+  drizzle_return_t ret;
+  drizzle_field_t fields[1];
+  size_t sizes[1];
+
+  fields[0]= (drizzle_field_t)SQLITE_SERVER_VERSION;
+  sizes[0]= strlen(SQLITE_SERVER_VERSION);
+
+  drizzle_result_set_column_count(&(server->result), 1);
+
+  ret= drizzle_result_write(&(server->con), &(server->result), false);
+  DRIZZLE_RETURN_CHECK_VAL(ret, "drizzle_result_write", &(server->drizzle))
+
+  if (drizzle_column_create(&(server->result), &(server->column)) == NULL)
+  {
+    DRIZZLE_RETURN_CHECK_VAL(DRIZZLE_RETURN_MEMORY, "drizzle_column_create",
+                             &(server->drizzle))
+  }
+
+  drizzle_column_set_catalog(&(server->column), "sqlite");
+  drizzle_column_set_db(&(server->column), "sqlite_db");
+  drizzle_column_set_table(&(server->column), "sqlite_table");
+  drizzle_column_set_orig_table(&(server->column), "sqlite_table");
+  drizzle_column_set_charset(&(server->column), 8);
+  drizzle_column_set_type(&(server->column), DRIZZLE_COLUMN_TYPE_VARCHAR);
+  drizzle_column_set_size(&(server->column), (uint32_t)sizes[0]);
+  drizzle_column_set_name(&(server->column), "version");
+  drizzle_column_set_orig_name(&(server->column), "version");
+
+  ret= drizzle_column_write(&(server->result), &(server->column));
+  DRIZZLE_RETURN_CHECK_VAL(ret, "drizzle_column_write", &(server->drizzle))
+
+  drizzle_column_free(&(server->column));
+
+  drizzle_result_set_eof(&(server->result), true);
+
+  ret= drizzle_result_write(&(server->con), &(server->result), false);
+  DRIZZLE_RETURN_CHECK_VAL(ret, "drizzle_result_write", &(server->drizzle))
+
+  /* This is needed for MySQL and old Drizzle protocol. */
+  drizzle_result_calc_row_size(&(server->result), fields, sizes);
+
+  ret= drizzle_row_write(&(server->result));
+  DRIZZLE_RETURN_CHECK_VAL(ret, "drizzle_row_write", &(server->drizzle))
+
+  ret= drizzle_field_write(&(server->result), fields[0], sizes[0], sizes[0]);
+  DRIZZLE_RETURN_CHECK_VAL(ret, "drizzle_field_write", &(server->drizzle))
+
+  ret= drizzle_result_write(&(server->con), &(server->result), true);
+  DRIZZLE_RETURN_CHECK_VAL(ret, "drizzle_result_write", &(server->drizzle))
+
+  return DRIZZLE_RETURN_OK;
+}
+
+static void usage(char *name)
+{
+  printf("\nusage: %s [-c <count>] [-h <host>] [-m] [-p <port>] [-v] "
+         "<sqlite3 db file>\n", name);
+  printf("\t-c <count> - Number of connections to accept before exiting\n");
+  printf("\t-h <host>  - Host to listen on\n");
+  printf("\t-m         - Use the MySQL protocol\n");
+  printf("\t-p <port>  - Port to listen on\n");
+  printf("\t-v         - Increase verbosity level\n");
+}

=== modified file 'extra/include.am'
--- extra/include.am	2010-02-16 07:05:28 +0000
+++ extra/include.am	2010-08-12 20:27:32 +0000
@@ -5,3 +5,5 @@
 
 extra_my_print_defaults_SOURCES= extra/my_print_defaults.cc
 extra_my_print_defaults_LDADD= $(CLIENT_LDADD)
+
+extra_mysql_password_hash_LDADD= libdrizzle/libdrizzle.la

=== added file 'extra/mysql_password_hash.c'
--- extra/mysql_password_hash.c	1970-01-01 00:00:00 +0000
+++ extra/mysql_password_hash.c	2010-08-12 20:27:32 +0000
@@ -0,0 +1,35 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+#include "config.h"
+
+#include <stdio.h>
+#include <string.h>
+
+#include <libdrizzle/drizzle_client.h>
+
+#define BUFFER_CHUNK 8192
+
+int main(int argc, char *argv[])
+{
+  char hashed_password[DRIZZLE_MYSQL_PASSWORD_HASH];
+
+  if (argc != 2)
+  {
+    printf("Usage: %s <password to hash>\n", argv[0]);
+    return 1;
+  }
+
+  drizzle_mysql_password_hash(hashed_password, argv[1], strlen(argv[1]));
+
+  printf("%s\n", hashed_password);
+
+  return 0;
+}

=== added directory 'libdrizzle'
=== added file 'libdrizzle/column.c'
--- libdrizzle/column.c	1970-01-01 00:00:00 +0000
+++ libdrizzle/column.c	2010-08-12 20:27:32 +0000
@@ -0,0 +1,894 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Column definitions
+ */
+
+#include "common.h"
+
+/*
+ * Private variables.
+ */
+
+static drizzle_column_type_t _column_type_drizzle_map_to[]=
+{
+ DRIZZLE_COLUMN_TYPE_TINY,
+ DRIZZLE_COLUMN_TYPE_LONG,
+ DRIZZLE_COLUMN_TYPE_DOUBLE,
+ DRIZZLE_COLUMN_TYPE_NULL,
+ DRIZZLE_COLUMN_TYPE_TIMESTAMP,
+ DRIZZLE_COLUMN_TYPE_LONGLONG,
+ DRIZZLE_COLUMN_TYPE_DATETIME,
+ DRIZZLE_COLUMN_TYPE_NEWDATE,
+ DRIZZLE_COLUMN_TYPE_VARCHAR,
+ DRIZZLE_COLUMN_TYPE_NEWDECIMAL,
+ DRIZZLE_COLUMN_TYPE_ENUM,
+ DRIZZLE_COLUMN_TYPE_BLOB
+};
+
+static drizzle_column_type_drizzle_t _column_type_drizzle_map_from[]=
+{
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX, /* 0 */
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_TINY,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_LONG,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_DOUBLE,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_NULL,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_TIMESTAMP,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_LONGLONG,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX, /* 10 */
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_DATETIME,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_DATE,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_VARCHAR,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX, /* 20 */
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX, /* 30 */
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX, /* 40 */
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX, /* 50 */
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX, /* 60 */
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX, /* 70 */
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX, /* 80 */
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX, /* 90 */
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX, /* 100 */
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX, /* 110 */
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX, /* 120 */
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX, /* 130 */
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX, /* 140 */
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX, /* 150 */
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX, /* 160 */
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX, /* 170 */
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX, /* 180 */
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX, /* 190 */
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX, /* 200 */
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX, /* 210 */
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX, /* 220 */
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX, /* 230 */
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX, /* 240 */
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_NEWDECIMAL,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_ENUM,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX, /* 250 */
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_BLOB,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
+ DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX
+};
+
+/*
+ * Common definitions
+ */
+
+drizzle_column_st *drizzle_column_create(drizzle_result_st *result,
+                                         drizzle_column_st *column)
+{
+  if (column == NULL)
+  {
+    column= malloc(sizeof(drizzle_column_st));
+    if (column == NULL)
+    {
+      drizzle_set_error(result->con->drizzle, "drizzle_column_create",
+                        "malloc");
+      return NULL;
+    }
+
+    memset(column, 0, sizeof(drizzle_column_st));
+    column->options|= DRIZZLE_COLUMN_ALLOCATED;
+  }
+  else
+    memset(column, 0, sizeof(drizzle_column_st));
+
+  column->result= result;
+
+  if (result->column_list)
+    result->column_list->prev= column;
+  column->next= result->column_list;
+  result->column_list= column;
+
+  return column;
+}
+
+void drizzle_column_free(drizzle_column_st *column)
+{
+  if (column->result->column_list == column)
+    column->result->column_list= column->next;
+  if (column->prev)
+    column->prev->next= column->next;
+  if (column->next)
+    column->next->prev= column->prev;
+
+  if (column->options & DRIZZLE_COLUMN_ALLOCATED)
+    free(column);
+}
+
+drizzle_result_st *drizzle_column_drizzle_result(drizzle_column_st *column)
+{
+  return column->result;
+}
+
+const char *drizzle_column_catalog(drizzle_column_st *column)
+{
+  return column->catalog;
+}
+
+const char *drizzle_column_db(drizzle_column_st *column)
+{
+  return column->db;
+}
+
+const char *drizzle_column_table(drizzle_column_st *column)
+{
+  return column->table;
+}
+
+const char *drizzle_column_orig_table(drizzle_column_st *column)
+{
+  return column->orig_table;
+}
+
+const char *drizzle_column_name(drizzle_column_st *column)
+{
+  return column->name;
+}
+
+const char *drizzle_column_orig_name(drizzle_column_st *column)
+{
+  return column->orig_name;
+}
+
+drizzle_charset_t drizzle_column_charset(drizzle_column_st *column)
+{
+  return column->charset;
+}
+
+uint32_t drizzle_column_size(drizzle_column_st *column)
+{
+  return column->size;
+}
+
+size_t drizzle_column_max_size(drizzle_column_st *column)
+{
+  return column->max_size;
+}
+
+void drizzle_column_set_max_size(drizzle_column_st *column, size_t size)
+{
+  column->max_size= size;
+}
+
+drizzle_column_type_t drizzle_column_type(drizzle_column_st *column)
+{
+  return column->type;
+}
+
+drizzle_column_type_drizzle_t
+drizzle_column_type_drizzle(drizzle_column_st *column)
+{
+  return _column_type_drizzle_map_from[column->type];
+}
+
+drizzle_column_flags_t drizzle_column_flags(drizzle_column_st *column)
+{
+  return column->flags;
+}
+
+uint8_t drizzle_column_decimals(drizzle_column_st *column)
+{
+  return column->decimals;
+}
+
+const uint8_t *drizzle_column_default_value(drizzle_column_st *column,
+                                            size_t *size)
+{
+  *size= column->default_value_size;
+  return column->default_value;
+}
+
+/*
+ * Client definitions
+ */
+
+drizzle_return_t drizzle_column_skip(drizzle_result_st *result)
+{
+  if (drizzle_state_none(result->con))
+  {
+    result->options|= DRIZZLE_RESULT_SKIP_COLUMN;
+
+    drizzle_state_push(result->con, drizzle_state_column_read);
+    drizzle_state_push(result->con, drizzle_state_packet_read);
+  }
+
+  return drizzle_state_loop(result->con);
+}
+
+drizzle_column_st *drizzle_column_read(drizzle_result_st *result,
+                                       drizzle_column_st *column,
+                                       drizzle_return_t *ret_ptr)
+{
+  if (drizzle_state_none(result->con))
+  {
+    result->column= column;
+
+    drizzle_state_push(result->con, drizzle_state_column_read);
+    drizzle_state_push(result->con, drizzle_state_packet_read);
+  }
+
+  *ret_ptr= drizzle_state_loop(result->con);
+  return result->column;
+}
+
+drizzle_return_t drizzle_column_buffer(drizzle_result_st *result)
+{
+  drizzle_return_t ret;
+
+  if (result->column_buffer == NULL)
+  {
+    if (result->column_count == 0)
+    {
+      result->options|= DRIZZLE_RESULT_BUFFER_COLUMN;
+      return DRIZZLE_RETURN_OK;
+    }
+
+    result->column_buffer= malloc(sizeof(drizzle_column_st) *
+                                  result->column_count);
+    if (result->column_buffer == NULL)
+    {
+      drizzle_set_error(result->con->drizzle, "drizzle_column_buffer",
+                        "malloc");
+      return DRIZZLE_RETURN_MEMORY;
+    }
+  }
+
+  /* No while body, just keep calling to buffer columns. */
+  while (drizzle_column_read(result,
+                             &(result->column_buffer[result->column_current]),
+                             &ret) != NULL && ret == DRIZZLE_RETURN_OK);
+  if (ret == DRIZZLE_RETURN_OK)
+  {
+    result->column_current= 0;
+    result->options|= DRIZZLE_RESULT_BUFFER_COLUMN;
+  }
+
+  return ret;
+}
+
+drizzle_column_st *drizzle_column_next(drizzle_result_st *result)
+{
+  if (result->column_current == result->column_count)
+    return NULL;
+
+  result->column_current++;
+  return &(result->column_buffer[result->column_current - 1]);
+}
+
+drizzle_column_st *drizzle_column_prev(drizzle_result_st *result)
+{
+  if (result->column_current == 0)
+    return NULL;
+
+  result->column_current--;
+  return &(result->column_buffer[result->column_current]);
+}
+
+void drizzle_column_seek(drizzle_result_st *result, uint16_t column)
+{
+  if (column <= result->column_count)
+    result->column_current= column;
+}
+
+drizzle_column_st *drizzle_column_index(drizzle_result_st *result,
+                                        uint16_t column)
+{
+  if (column >= result->column_count)
+    return NULL;
+
+  return &(result->column_buffer[column]);
+}
+
+uint16_t drizzle_column_current(drizzle_result_st *result)
+{
+  return result->column_current;
+}
+
+/*
+ * Server definitions
+ */
+
+drizzle_return_t drizzle_column_write(drizzle_result_st *result,
+                                      drizzle_column_st *column)
+{
+  if (drizzle_state_none(result->con))
+  {
+    result->column= column;
+
+    drizzle_state_push(result->con, drizzle_state_column_write);
+  }
+
+  return drizzle_state_loop(result->con);
+}
+
+void drizzle_column_set_catalog(drizzle_column_st *column, const char *catalog)
+{
+  if (catalog == NULL)
+    column->catalog[0]= 0;
+  else
+  {
+    strncpy(column->catalog, catalog, DRIZZLE_MAX_CATALOG_SIZE);
+    column->catalog[DRIZZLE_MAX_CATALOG_SIZE - 1]= 0;
+  }
+}
+
+void drizzle_column_set_db(drizzle_column_st *column, const char *db)
+{
+  if (db == NULL)
+    column->db[0]= 0;
+  else
+  {
+    strncpy(column->db, db, DRIZZLE_MAX_DB_SIZE);
+    column->db[DRIZZLE_MAX_DB_SIZE - 1]= 0;
+  }
+}
+
+void drizzle_column_set_table(drizzle_column_st *column, const char *table)
+{
+  if (table == NULL)
+    column->table[0]= 0;
+  else
+  {
+    strncpy(column->table, table, DRIZZLE_MAX_TABLE_SIZE);
+    column->table[DRIZZLE_MAX_TABLE_SIZE - 1]= 0;
+  }
+}
+
+void drizzle_column_set_orig_table(drizzle_column_st *column,
+                                   const char *orig_table)
+{
+  if (orig_table == NULL)
+    column->orig_table[0]= 0;
+  else
+  {
+    strncpy(column->orig_table, orig_table, DRIZZLE_MAX_TABLE_SIZE);
+    column->orig_table[DRIZZLE_MAX_TABLE_SIZE - 1]= 0;
+  }
+}
+
+void drizzle_column_set_name(drizzle_column_st *column, const char *name)
+{
+  if (name == NULL)
+    column->name[0]= 0;
+  else
+  {
+    strncpy(column->name, name, DRIZZLE_MAX_COLUMN_NAME_SIZE);
+    column->name[DRIZZLE_MAX_COLUMN_NAME_SIZE - 1]= 0;
+  }
+}
+
+void drizzle_column_set_orig_name(drizzle_column_st *column,
+                                  const char *orig_name)
+{
+  if (orig_name == NULL)
+    column->orig_name[0]= 0;
+  else
+  {
+    strncpy(column->orig_name, orig_name, DRIZZLE_MAX_COLUMN_NAME_SIZE);
+    column->orig_name[DRIZZLE_MAX_COLUMN_NAME_SIZE - 1]= 0;
+  }
+}
+
+void drizzle_column_set_charset(drizzle_column_st *column,
+                                drizzle_charset_t charset)
+{
+  column->charset= charset;
+}
+
+void drizzle_column_set_size(drizzle_column_st *column, uint32_t size)
+{
+  column->size= size;
+}
+
+void drizzle_column_set_type(drizzle_column_st *column,
+                             drizzle_column_type_t type)
+{
+  column->type= type;
+}
+
+void drizzle_column_set_flags(drizzle_column_st *column,
+                              drizzle_column_flags_t flags)
+{
+  column->flags= flags;
+}
+
+void drizzle_column_set_decimals(drizzle_column_st *column, uint8_t decimals)
+{
+  column->decimals= decimals;
+}
+
+void drizzle_column_set_default_value(drizzle_column_st *column,
+                                      const uint8_t *default_value,
+                                      size_t size)
+{
+  if (default_value == NULL)
+    column->default_value[0]= 0;
+  else
+  {
+    if (size < DRIZZLE_MAX_DEFAULT_VALUE_SIZE)
+    {
+      memcpy(column->default_value, default_value, size);
+      column->default_value[size]= 0;
+      column->default_value_size= size;
+    }
+    else
+    {
+      memcpy(column->default_value, default_value,
+             DRIZZLE_MAX_DEFAULT_VALUE_SIZE - 1);
+      column->default_value[DRIZZLE_MAX_DEFAULT_VALUE_SIZE - 1]= 0;
+      column->default_value_size= DRIZZLE_MAX_DEFAULT_VALUE_SIZE;
+    }
+  }
+}
+
+/*
+ * Internal state functions.
+ */
+
+drizzle_return_t drizzle_state_column_read(drizzle_con_st *con)
+{
+  drizzle_column_st *column;
+  drizzle_column_type_drizzle_t drizzle_type;
+
+  drizzle_log_debug(con->drizzle, "drizzle_state_column_read");
+
+  /* Assume the entire column packet will fit in the buffer. */
+  if (con->buffer_size < con->packet_size)
+  {
+    drizzle_state_push(con, drizzle_state_read);
+    return DRIZZLE_RETURN_OK;
+  }
+
+  if (con->packet_size == 5 && con->buffer_ptr[0] == 254)
+  {
+    /* EOF packet marking end of columns. */
+    con->result->column= NULL;
+    con->result->warning_count= drizzle_get_byte2(con->buffer_ptr + 1);
+    con->status= drizzle_get_byte2(con->buffer_ptr + 3);
+    con->buffer_ptr+= 5;
+    con->buffer_size-= 5;
+
+    drizzle_state_pop(con);
+  }
+  else if (con->result->options & DRIZZLE_RESULT_SKIP_COLUMN)
+  {
+    con->buffer_ptr+= con->packet_size;
+    con->buffer_size-= con->packet_size;
+    con->packet_size= 0;
+
+    drizzle_state_push(con, drizzle_state_packet_read);
+  }
+  else
+  {
+    column= drizzle_column_create(con->result, con->result->column);
+    if (column == NULL)
+      return DRIZZLE_RETURN_MEMORY;
+
+    con->result->column= column;
+
+    /* These functions can only fail if they need to read data, but we know we
+       buffered the entire packet, so ignore returns. */
+    (void)drizzle_unpack_string(con, column->catalog, DRIZZLE_MAX_CATALOG_SIZE);
+    (void)drizzle_unpack_string(con, column->db, DRIZZLE_MAX_DB_SIZE);
+    (void)drizzle_unpack_string(con, column->table, DRIZZLE_MAX_TABLE_SIZE);
+    (void)drizzle_unpack_string(con, column->orig_table,
+                                DRIZZLE_MAX_TABLE_SIZE);
+    (void)drizzle_unpack_string(con, column->name,
+                                DRIZZLE_MAX_COLUMN_NAME_SIZE);
+    (void)drizzle_unpack_string(con, column->orig_name,
+                                DRIZZLE_MAX_COLUMN_NAME_SIZE);
+
+    /* Skip one filler byte. */
+    column->charset= (drizzle_charset_t)drizzle_get_byte2(con->buffer_ptr + 1);
+    column->size= drizzle_get_byte4(con->buffer_ptr + 3);
+
+    if (con->options & DRIZZLE_CON_MYSQL)
+      column->type= con->buffer_ptr[7];
+    else
+    {
+      drizzle_type= con->buffer_ptr[7];
+      if (drizzle_type >= DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX)
+        drizzle_type= DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX;
+      column->type= _column_type_drizzle_map_to[drizzle_type];
+    }
+
+    column->flags= drizzle_get_byte2(con->buffer_ptr + 8);
+    if (column->type <= DRIZZLE_COLUMN_TYPE_INT24 &&
+        column->type != DRIZZLE_COLUMN_TYPE_TIMESTAMP)
+    {
+      column->flags|= DRIZZLE_COLUMN_FLAGS_NUM;
+    }
+
+    column->decimals= con->buffer_ptr[10];
+    /* Skip two reserved bytes. */
+
+    con->buffer_ptr+= 13;
+    con->buffer_size-= 13;
+    con->packet_size-= 13;
+
+    if (con->packet_size > 0)
+    {
+      drizzle_column_set_default_value(column, con->buffer_ptr,
+                                       con->packet_size);
+
+      con->buffer_ptr+= con->packet_size;
+      con->buffer_size-= con->packet_size;
+    }
+    else
+      column->default_value[0]= 0;
+
+    con->result->column_current++;
+
+    drizzle_state_pop(con);
+  }
+
+  return DRIZZLE_RETURN_OK;
+}
+
+drizzle_return_t drizzle_state_column_write(drizzle_con_st *con)
+{
+  uint8_t *start= con->buffer_ptr + con->buffer_size;
+  uint8_t *ptr;
+  drizzle_column_st *column= con->result->column;
+
+  drizzle_log_debug(con->drizzle, "drizzle_state_column_write");
+
+  /* Calculate max packet size. */
+  con->packet_size= 9 + strlen(column->catalog)
+                  + 9 + strlen(column->db)
+                  + 9 + strlen(column->table)
+                  + 9 + strlen(column->orig_table)
+                  + 9 + strlen(column->name)
+                  + 9 + strlen(column->orig_name)
+                  + 1   /* Unused */
+                  + 2   /* Charset */
+                  + 4   /* Size */
+                  + 1   /* Type */
+                  + 2   /* Flags */
+                  + 1   /* Decimals */
+                  + 2   /* Unused */
+                  + column->default_value_size;
+
+  /* Assume the entire column packet will fit in the buffer. */
+  if ((con->packet_size + 4) > DRIZZLE_MAX_BUFFER_SIZE)
+  {
+    drizzle_set_error(con->drizzle, "drizzle_state_column_write",
+                      "buffer too small:%zu", con->packet_size + 4);
+    return DRIZZLE_RETURN_INTERNAL_ERROR;
+  }
+
+  /* Flush buffer if there is not enough room. */
+  if (((size_t)DRIZZLE_MAX_BUFFER_SIZE - (size_t)(start - con->buffer)) <
+      con->packet_size)
+  {
+    drizzle_state_push(con, drizzle_state_write);
+    return DRIZZLE_RETURN_OK;
+  }
+
+  /* Store packet size at the end since it may change. */
+  ptr= start;
+  ptr[3]= con->packet_number;
+  con->packet_number++;
+  ptr+= 4;
+
+  ptr= drizzle_pack_string(column->catalog, ptr);
+  ptr= drizzle_pack_string(column->db, ptr);
+  ptr= drizzle_pack_string(column->table, ptr);
+  ptr= drizzle_pack_string(column->orig_table, ptr);
+  ptr= drizzle_pack_string(column->name, ptr);
+  ptr= drizzle_pack_string(column->orig_name, ptr);
+
+  /* This unused byte is set to 12 for some reason. */
+  ptr[0]= 12;
+  ptr++;
+
+  drizzle_set_byte2(ptr, column->charset);
+  ptr+= 2;
+
+  drizzle_set_byte4(ptr, column->size);
+  ptr+= 4;
+
+  if (con->options & DRIZZLE_CON_MYSQL)
+    ptr[0]= column->type;
+  else
+    ptr[0]= _column_type_drizzle_map_from[column->type];
+  ptr++;
+
+  drizzle_set_byte2(ptr, column->flags);
+  ptr+= 2;
+
+  ptr[0]= column->decimals;
+  ptr++;
+
+  memset(ptr, 0, 2);
+  ptr+= 2;
+
+  if (column->default_value_size > 0)
+  {
+    memcpy(ptr, column->default_value, column->default_value_size);
+    ptr+= column->default_value_size;
+  }
+
+  con->packet_size= ((size_t)(ptr - start) - 4);
+  con->buffer_size+= (4 + con->packet_size);
+
+  /* Store packet size now. */
+  drizzle_set_byte3(start, con->packet_size);
+
+  con->result->column_current++;
+
+  drizzle_state_pop(con);
+
+  return DRIZZLE_RETURN_OK;
+}

=== added file 'libdrizzle/column.h'
--- libdrizzle/column.h	1970-01-01 00:00:00 +0000
+++ libdrizzle/column.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,151 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Column Declarations
+ */
+
+#ifndef __DRIZZLE_COLUMN_H
+#define __DRIZZLE_COLUMN_H
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/**
+ * @addtogroup drizzle_column Column Declarations
+ * @ingroup drizzle_client_interface
+ * @ingroup drizzle_server_interface
+ *
+ * These functions are used to get detailed column information. This information
+ * is usually sent as the first part of a result set. There are multiple ways
+ * for column information to be buffered depending on the functions being used.
+ * @{
+ */
+
+/**
+ * Initialize a column structure.
+ */
+DRIZZLE_API
+drizzle_column_st *drizzle_column_create(drizzle_result_st *result,
+                                         drizzle_column_st *column);
+
+/**
+ * Free a column structure.
+ */
+DRIZZLE_API
+void drizzle_column_free(drizzle_column_st *column);
+
+/**
+ * Get the drizzle_result_st struct that the column belongs to.
+ */
+DRIZZLE_API
+drizzle_result_st *drizzle_column_drizzle_result(drizzle_column_st *column);
+
+/**
+ * Get catalog name for a column.
+ */
+DRIZZLE_API
+const char *drizzle_column_catalog(drizzle_column_st *column);
+
+/**
+ * Get database name for a column.
+ */
+DRIZZLE_API
+const char *drizzle_column_db(drizzle_column_st *column);
+
+/**
+ * Get table name for a column.
+ */
+DRIZZLE_API
+const char *drizzle_column_table(drizzle_column_st *column);
+
+/**
+ * Get original table name for a column.
+ */
+DRIZZLE_API
+const char *drizzle_column_orig_table(drizzle_column_st *column);
+
+/**
+ * Get column name for a column.
+ */
+DRIZZLE_API
+const char *drizzle_column_name(drizzle_column_st *column);
+
+/**
+ * Get original column name for a column.
+ */
+DRIZZLE_API
+const char *drizzle_column_orig_name(drizzle_column_st *column);
+
+/**
+ * Get charset for a column.
+ */
+DRIZZLE_API
+drizzle_charset_t drizzle_column_charset(drizzle_column_st *column);
+
+/**
+ * Get size of a column.
+ */
+DRIZZLE_API
+uint32_t drizzle_column_size(drizzle_column_st *column);
+
+/**
+ * Get max size of a column.
+ */
+DRIZZLE_API
+size_t drizzle_column_max_size(drizzle_column_st *column);
+
+/**
+ * Set max size of a column.
+ */
+DRIZZLE_API
+void drizzle_column_set_max_size(drizzle_column_st *column, size_t size);
+
+/**
+ * Get the type of a column.
+ */
+DRIZZLE_API
+drizzle_column_type_t drizzle_column_type(drizzle_column_st *column);
+
+/**
+ * Get the Drizzle type of a column.
+ */
+DRIZZLE_API
+drizzle_column_type_drizzle_t
+drizzle_column_type_drizzle(drizzle_column_st *column);
+
+/**
+ * Get flags for a column.
+ */
+DRIZZLE_API
+drizzle_column_flags_t drizzle_column_flags(drizzle_column_st *column);
+
+/**
+ * Get the number of decimals for numeric columns.
+ */
+DRIZZLE_API
+uint8_t drizzle_column_decimals(drizzle_column_st *column);
+
+/**
+ * Get default value for a column.
+ */
+DRIZZLE_API
+const uint8_t *drizzle_column_default_value(drizzle_column_st *column,
+                                            size_t *size);
+
+/** @} */
+
+#ifdef __cplusplus
+}
+#endif
+
+#endif /* __DRIZZLE_COLUMN_H */

=== added file 'libdrizzle/column_client.h'
--- libdrizzle/column_client.h	1970-01-01 00:00:00 +0000
+++ libdrizzle/column_client.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,90 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Column Declarations for Clients
+ */
+
+#ifndef __DRIZZLE_COLUMN_CLIENT_H
+#define __DRIZZLE_COLUMN_CLIENT_H
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/**
+ * @addtogroup drizzle_column_client Column Declarations for Clients
+ * @ingroup drizzle_client_interface
+ *
+ * These functions are used to get detailed column information. This information
+ * is usually sent as the first part of a result set. There are both buffered
+ * and unbuffered functions provided.
+ * @{
+ */
+
+/**
+ * Skip all columns in result.
+ */
+DRIZZLE_API
+drizzle_return_t drizzle_column_skip(drizzle_result_st *result);
+
+/**
+ * Read column information.
+ */
+DRIZZLE_API
+drizzle_column_st *drizzle_column_read(drizzle_result_st *result,
+                                       drizzle_column_st *column,
+                                       drizzle_return_t *ret_ptr);
+
+/**
+ * Buffer all columns in result structure.
+ */
+DRIZZLE_API
+drizzle_return_t drizzle_column_buffer(drizzle_result_st *result);
+
+/**
+ * Get next buffered column from a result structure.
+ */
+DRIZZLE_API
+drizzle_column_st *drizzle_column_next(drizzle_result_st *result);
+
+/**
+ * Get previous buffered column from a result structure.
+ */
+DRIZZLE_API
+drizzle_column_st *drizzle_column_prev(drizzle_result_st *result);
+
+/**
+ * Seek to the given buffered column in a result structure.
+ */
+DRIZZLE_API
+void drizzle_column_seek(drizzle_result_st *result, uint16_t column);
+
+/**
+ * Get the given buffered column from a result structure.
+ */
+DRIZZLE_API
+drizzle_column_st *drizzle_column_index(drizzle_result_st *result,
+                                        uint16_t column);
+
+/**
+ * Get current column number in a buffered or unbuffered result.
+ */
+DRIZZLE_API
+uint16_t drizzle_column_current(drizzle_result_st *result);
+
+/** @} */
+
+#ifdef __cplusplus
+}
+#endif
+
+#endif /* __DRIZZLE_COLUMN_CLIENT_H */

=== added file 'libdrizzle/column_server.h'
--- libdrizzle/column_server.h	1970-01-01 00:00:00 +0000
+++ libdrizzle/column_server.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,123 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Column Declarations for Servers
+ */
+
+#ifndef __DRIZZLE_COLUMN_SERVER_H
+#define __DRIZZLE_COLUMN_SERVER_H
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/**
+ * @addtogroup drizzle_column_server Column Declarations for Servers
+ * @ingroup drizzle_server_interface
+ *
+ * These functions allow you to send column information over a connection.
+ * @{
+ */
+
+/**
+ * Write column information.
+ */
+DRIZZLE_API
+drizzle_return_t drizzle_column_write(drizzle_result_st *result,
+                                      drizzle_column_st *column);
+
+/**
+ * Set catalog name for a column.
+ */
+DRIZZLE_API
+void drizzle_column_set_catalog(drizzle_column_st *column, const char *catalog);
+ 
+/**
+ * Set database name for a column.
+ */
+DRIZZLE_API
+void drizzle_column_set_db(drizzle_column_st *column, const char *db);
+ 
+/**
+ * Set table name for a column.
+ */
+DRIZZLE_API
+void drizzle_column_set_table(drizzle_column_st *column, const char *table);
+
+/**
+ * Set original table name for a column.
+ */
+DRIZZLE_API
+void drizzle_column_set_orig_table(drizzle_column_st *column,
+                                   const char *orig_table);
+
+/**
+ * Set column name for a column.
+ */
+DRIZZLE_API
+void drizzle_column_set_name(drizzle_column_st *column, const char *name);
+
+/**
+ * Set original column name for a column.
+ */
+DRIZZLE_API
+void drizzle_column_set_orig_name(drizzle_column_st *column,
+                                  const char *orig_name);
+
+/**
+ * Set charset for a column.
+ */
+DRIZZLE_API
+void drizzle_column_set_charset(drizzle_column_st *column,
+                                drizzle_charset_t charset);
+
+/**
+ * Set size of a column.
+ */
+DRIZZLE_API
+void drizzle_column_set_size(drizzle_column_st *column, uint32_t size);
+
+/**
+ * Set the type of a column.
+ */
+DRIZZLE_API
+void drizzle_column_set_type(drizzle_column_st *column,
+                             drizzle_column_type_t type);
+
+/**
+ * Set flags for a column.
+ */
+DRIZZLE_API
+void drizzle_column_set_flags(drizzle_column_st *column,
+                              drizzle_column_flags_t flags);
+
+/**
+ * Set the number of decimals for numeric columns.
+ */
+DRIZZLE_API
+void drizzle_column_set_decimals(drizzle_column_st *column, uint8_t decimals);
+
+/**
+ * Set default value for a column.
+ */
+DRIZZLE_API
+void drizzle_column_set_default_value(drizzle_column_st *column,
+                                      const uint8_t *default_value,
+                                      size_t size);
+
+/** @} */
+
+#ifdef __cplusplus
+}
+#endif
+
+#endif /* __DRIZZLE_COLUMN_SERVER_H */

=== added file 'libdrizzle/command.c'
--- libdrizzle/command.c	1970-01-01 00:00:00 +0000
+++ libdrizzle/command.c	2010-08-12 20:27:32 +0000
@@ -0,0 +1,219 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Command definitions
+ */
+
+#include "common.h"
+
+/*
+ * Private variables.
+ */
+
+static drizzle_command_drizzle_t _command_drizzle_map[]=
+{
+ DRIZZLE_COMMAND_DRIZZLE_END,
+ DRIZZLE_COMMAND_DRIZZLE_QUIT,
+ DRIZZLE_COMMAND_DRIZZLE_INIT_DB,
+ DRIZZLE_COMMAND_DRIZZLE_QUERY,
+ DRIZZLE_COMMAND_DRIZZLE_END,
+ DRIZZLE_COMMAND_DRIZZLE_END,
+ DRIZZLE_COMMAND_DRIZZLE_END,
+ DRIZZLE_COMMAND_DRIZZLE_END,
+ DRIZZLE_COMMAND_DRIZZLE_SHUTDOWN,
+ DRIZZLE_COMMAND_DRIZZLE_END,
+ DRIZZLE_COMMAND_DRIZZLE_END,
+ DRIZZLE_COMMAND_DRIZZLE_END,
+ DRIZZLE_COMMAND_DRIZZLE_END,
+ DRIZZLE_COMMAND_DRIZZLE_END,
+ DRIZZLE_COMMAND_DRIZZLE_PING,
+ DRIZZLE_COMMAND_DRIZZLE_END,
+ DRIZZLE_COMMAND_DRIZZLE_END,
+ DRIZZLE_COMMAND_DRIZZLE_END,
+ DRIZZLE_COMMAND_DRIZZLE_END,
+ DRIZZLE_COMMAND_DRIZZLE_END,
+ DRIZZLE_COMMAND_DRIZZLE_END,
+ DRIZZLE_COMMAND_DRIZZLE_END,
+ DRIZZLE_COMMAND_DRIZZLE_END,
+ DRIZZLE_COMMAND_DRIZZLE_END,
+ DRIZZLE_COMMAND_DRIZZLE_END,
+ DRIZZLE_COMMAND_DRIZZLE_END,
+ DRIZZLE_COMMAND_DRIZZLE_END,
+ DRIZZLE_COMMAND_DRIZZLE_END,
+ DRIZZLE_COMMAND_DRIZZLE_END,
+ DRIZZLE_COMMAND_DRIZZLE_END,
+ DRIZZLE_COMMAND_DRIZZLE_END
+};
+
+/*
+ * State Definitions
+ */
+
+drizzle_return_t drizzle_state_command_read(drizzle_con_st *con)
+{
+  drizzle_log_debug(con->drizzle, "drizzle_state_command_read");
+
+  if (con->buffer_size == 0)
+  {
+    drizzle_state_push(con, drizzle_state_read);
+    return DRIZZLE_RETURN_OK;
+  }
+
+  if (con->command_total == 0)
+  {
+    con->command= (drizzle_command_t)(con->buffer_ptr[0]);
+    con->buffer_ptr++;
+    con->buffer_size--;
+
+    con->command_total= (con->packet_size - 1);
+  }
+
+  if (con->buffer_size < (con->command_total - con->command_offset))
+  {
+    con->command_size= con->buffer_size;
+    con->command_offset+= con->command_size;
+  }
+  else
+  {
+    con->command_size= (con->command_total - con->command_offset);
+    con->command_offset= con->command_total;
+  }
+
+  con->command_data= con->buffer_ptr;
+  con->buffer_ptr+= con->command_size;
+  con->buffer_size-= con->command_size;
+
+  if (con->command_offset == con->command_total)
+    drizzle_state_pop(con);
+  else
+    return DRIZZLE_RETURN_PAUSE;
+
+  return DRIZZLE_RETURN_OK;
+}
+
+drizzle_return_t drizzle_state_command_write(drizzle_con_st *con)
+{
+  uint8_t *start;
+  uint8_t *ptr;
+  size_t free_size;
+  drizzle_return_t ret;
+
+  drizzle_log_debug(con->drizzle, "drizzle_state_command_write");
+
+  if (con->command_data == NULL && con->command_total != 0 &&
+      con->command != DRIZZLE_COMMAND_CHANGE_USER)
+  {
+    return DRIZZLE_RETURN_PAUSE;
+  }
+
+  if (con->buffer_size == 0)
+  {
+    con->buffer_ptr= con->buffer;
+    start= con->buffer;
+  }
+  else
+    start= con->buffer_ptr + con->buffer_size;
+
+  if (con->command_offset == 0)
+  {
+    /* Make sure we can fit the largest non-streaming packet, currently a
+       DRIZZLE_COMMAND_CHANGE_USER command. */
+
+    con->packet_size= 1  /* Command */
+                    + strlen(con->user) + 1
+                    + 1  /* Scramble size */
+                    + DRIZZLE_MAX_SCRAMBLE_SIZE
+                    + strlen(con->db) + 1;
+
+    /* Flush buffer if there is not enough room. */
+    free_size= (size_t)DRIZZLE_MAX_BUFFER_SIZE - (size_t)(start - con->buffer);
+    if (free_size < con->packet_size)
+    {
+      drizzle_state_push(con, drizzle_state_write);
+      return DRIZZLE_RETURN_OK;
+    }
+
+    /* Store packet size at the end since it may change. */
+    con->packet_number= 1;
+    ptr= start;
+    ptr[3]= 0;
+    if (con->options & DRIZZLE_CON_MYSQL)
+      ptr[4]= (uint8_t)(con->command);
+    else
+      ptr[4]= (uint8_t)(_command_drizzle_map[con->command]);
+    ptr+= 5;
+
+    if (con->command == DRIZZLE_COMMAND_CHANGE_USER)
+    {
+      ptr= drizzle_pack_auth(con, ptr, &ret);
+      if (ret != DRIZZLE_RETURN_OK)
+        return ret;
+
+      con->buffer_size+= (4 + con->packet_size);
+    }
+    else if (con->command_total == 0)
+    {
+      con->packet_size= 1;
+      con->buffer_size+= 5;
+    }
+    else
+    {
+      con->packet_size= 1 + con->command_total;
+      free_size-= 5;
+
+      /* Copy as much of the data in as we can into the write buffer. */
+      if (con->command_size <= free_size)
+      {
+        memcpy(ptr, con->command_data, con->command_size);
+        con->command_offset= con->command_size;
+        con->command_data= NULL;
+        con->buffer_size+= 5 + con->command_size;
+      }
+      else
+      {
+        memcpy(ptr, con->command_data, free_size);
+        con->command_offset= free_size;
+        con->command_data+= free_size;
+        con->command_size-= free_size;
+        con->buffer_size+= 5 + free_size;
+      }
+    }
+
+    /* Store packet size now. */
+    drizzle_set_byte3(start, con->packet_size);
+  }
+  else
+  {
+    /* Write directly from the caller buffer for the rest. */
+    con->buffer_ptr= con->command_data;
+    con->buffer_size= con->command_size;
+    con->command_offset+= con->command_size;
+    con->command_data= NULL;
+  }
+
+  if (con->command_offset == con->command_total)
+  {
+    drizzle_state_pop(con);
+
+    if (!(con->options & (DRIZZLE_CON_RAW_PACKET |
+                          DRIZZLE_CON_NO_RESULT_READ)) &&
+        con->command != DRIZZLE_COMMAND_FIELD_LIST)
+    {
+      drizzle_state_push(con, drizzle_state_result_read);
+      drizzle_state_push(con, drizzle_state_packet_read);
+    }
+  }
+
+  drizzle_state_push(con, drizzle_state_write);
+
+  return DRIZZLE_RETURN_OK;
+}

=== added file 'libdrizzle/command_client.h'
--- libdrizzle/command_client.h	1970-01-01 00:00:00 +0000
+++ libdrizzle/command_client.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,38 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Command Declarations for Clients
+ */
+
+#ifndef __DRIZZLE_COMMAND_CLIENT_H
+#define __DRIZZLE_COMMAND_CLIENT_H
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/**
+ * @addtogroup drizzle_command_client Command Declarations for Clients
+ * @ingroup drizzle_client_interface
+ *
+ * These functions are used to issue commands on a connection. Normal SQL
+ * queries are issued using the drizzle_query* functions defined in query.h.
+ * @{
+ */
+
+/** @} */
+
+#ifdef __cplusplus
+}
+#endif
+
+#endif /* __DRIZZLE_COMMAND_CLIENT_H */

=== added file 'libdrizzle/command_server.h'
--- libdrizzle/command_server.h	1970-01-01 00:00:00 +0000
+++ libdrizzle/command_server.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,37 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Command Declarations for Servers
+ */
+
+#ifndef __DRIZZLE_COMMAND_SERVER_H
+#define __DRIZZLE_COMMAND_SERVER_H
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/**
+ * @addtogroup drizzle_command_server Command Declarations for Servers
+ * @ingroup drizzle_server_interface
+ *
+ * These functions allow you to read comands on a connection.
+ * @{
+ */
+
+/** @} */
+
+#ifdef __cplusplus
+}
+#endif
+
+#endif /* __DRIZZLE_COMMAND_SERVER_H */

=== added file 'libdrizzle/common.h'
--- libdrizzle/common.h	1970-01-01 00:00:00 +0000
+++ libdrizzle/common.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,43 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief System Include Files
+ */
+
+#ifndef __DRIZZLE_COMMON_H
+#define __DRIZZLE_COMMON_H
+
+#include "config.h"
+
+#define HAVE_VISIBILITY 1
+
+#include "drizzle_client.h"
+#include "drizzle_server.h"
+
+#include <assert.h>
+#include <errno.h>
+#include <fcntl.h>
+#include <netinet/tcp.h>
+#include <stdarg.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/uio.h>
+#include <unistd.h>
+
+#include "drizzle_local.h"
+#include "conn_local.h"
+#include "pack.h"
+#include "state.h"
+#include "sha1.h"
+
+#endif /* __DRIZZLE_COMMON_H */

=== added file 'libdrizzle/conn.c'
--- libdrizzle/conn.c	1970-01-01 00:00:00 +0000
+++ libdrizzle/conn.c	2010-08-12 20:27:32 +0000
@@ -0,0 +1,1300 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Connection Definitions
+ */
+
+#include "common.h"
+
+/**
+ * @addtogroup drizzle_con_static Static Connection Declarations
+ * @ingroup drizzle_con
+ * @{
+ */
+
+/**
+ * Set socket options for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @return Standard drizzle return value.
+ */
+static drizzle_return_t _con_setsockopt(drizzle_con_st *con);
+
+/** @} */
+
+/*
+ * Common Definitions
+ */
+
+int drizzle_con_fd(const drizzle_con_st *con)
+{
+  return con->fd;
+}
+
+drizzle_return_t drizzle_con_set_fd(drizzle_con_st *con, int fd)
+{
+  drizzle_return_t ret;
+
+  con->fd= fd;
+
+  ret= _con_setsockopt(con);
+  if (ret != DRIZZLE_RETURN_OK)
+    con->drizzle->last_errno= errno;
+
+  return ret;
+}
+
+void drizzle_con_close(drizzle_con_st *con)
+{
+  if (con->fd == -1)
+    return;
+
+  (void)close(con->fd);
+  con->fd= -1;
+
+  con->options&= (drizzle_con_options_t)~DRIZZLE_CON_READY;
+  con->packet_number= 0;
+  con->buffer_ptr= con->buffer;
+  con->buffer_size= 0;
+  con->events= 0;
+  con->revents= 0;
+
+  drizzle_state_reset(con);
+}
+
+drizzle_return_t drizzle_con_set_events(drizzle_con_st *con, short events)
+{
+  drizzle_return_t ret;
+
+  if ((con->events | events) == con->events)
+    return DRIZZLE_RETURN_OK;
+
+  con->events|= events;
+
+  if (con->drizzle->event_watch_fn != NULL)
+  {
+    ret= con->drizzle->event_watch_fn(con, con->events,
+                                      con->drizzle->event_watch_context);
+    if (ret != DRIZZLE_RETURN_OK)
+    {
+      drizzle_con_close(con);
+      return ret;
+    }
+  }
+
+  return DRIZZLE_RETURN_OK;
+}
+
+drizzle_return_t drizzle_con_set_revents(drizzle_con_st *con, short revents)
+{
+  drizzle_return_t ret;
+
+  if (revents != 0)
+    con->options|= DRIZZLE_CON_IO_READY;
+
+  con->revents= revents;
+
+  /* Remove external POLLOUT watch if we didn't ask for it. Otherwise we spin
+     forever until another POLLIN state change. This is much more efficient
+     than removing POLLOUT on every state change since some external polling
+     mechanisms need to use a system call to change flags (like Linux epoll). */
+  if (revents & POLLOUT && !(con->events & POLLOUT) &&
+      con->drizzle->event_watch_fn != NULL)
+  {
+    ret= con->drizzle->event_watch_fn(con, con->events,
+                                      con->drizzle->event_watch_context);
+    if (ret != DRIZZLE_RETURN_OK)
+    {
+      drizzle_con_close(con);
+      return ret;
+    }
+  }
+
+  con->events&= (short)~revents;
+
+  return DRIZZLE_RETURN_OK;
+}
+
+drizzle_st *drizzle_con_drizzle(const drizzle_con_st *con)
+{
+  return con->drizzle;
+}
+
+const char *drizzle_con_error(const drizzle_con_st *con)
+{
+  return drizzle_error(con->drizzle);
+}
+
+int drizzle_con_errno(const drizzle_con_st *con)
+{
+  return drizzle_errno(con->drizzle);
+}
+
+uint16_t drizzle_con_error_code(const drizzle_con_st *con)
+{
+  return drizzle_error_code(con->drizzle);
+}
+
+const char *drizzle_con_sqlstate(const drizzle_con_st *con)
+{
+  return drizzle_sqlstate(con->drizzle);
+}
+
+drizzle_con_options_t drizzle_con_options(const drizzle_con_st *con)
+{
+  return con->options;
+}
+
+void drizzle_con_set_options(drizzle_con_st *con,
+                             drizzle_con_options_t options)
+{
+  con->options= options;
+}
+
+void drizzle_con_add_options(drizzle_con_st *con,
+                             drizzle_con_options_t options)
+{
+  con->options|= options;
+
+  /* If asking for the experimental Drizzle protocol, clean the MySQL flag. */
+  if (con->options & DRIZZLE_CON_EXPERIMENTAL)
+    con->options&= (drizzle_con_options_t)~DRIZZLE_CON_MYSQL;
+}
+
+void drizzle_con_remove_options(drizzle_con_st *con,
+                                drizzle_con_options_t options)
+{
+  con->options&= ~options;
+}
+
+const char *drizzle_con_host(const drizzle_con_st *con)
+{
+  if (con->socket_type == DRIZZLE_CON_SOCKET_TCP)
+  {
+    if (con->socket.tcp.host == NULL && !(con->options & DRIZZLE_CON_LISTEN))
+      return DRIZZLE_DEFAULT_TCP_HOST;
+
+    return con->socket.tcp.host;
+  }
+
+  return NULL;
+}
+
+in_port_t drizzle_con_port(const drizzle_con_st *con)
+{
+  if (con->socket_type == DRIZZLE_CON_SOCKET_TCP)
+  {
+    if (con->socket.tcp.port != 0)
+      return con->socket.tcp.port;
+
+    if (con->options & DRIZZLE_CON_MYSQL)
+      return DRIZZLE_DEFAULT_TCP_PORT_MYSQL;
+
+    return DRIZZLE_DEFAULT_TCP_PORT;
+  }
+
+  return 0;
+}
+
+void drizzle_con_set_tcp(drizzle_con_st *con, const char *host, in_port_t port)
+{
+  drizzle_con_reset_addrinfo(con);
+
+  con->socket_type= DRIZZLE_CON_SOCKET_TCP;
+
+  if (host == NULL)
+    con->socket.tcp.host= NULL;
+  else
+  {
+    con->socket.tcp.host= con->socket.tcp.host_buffer;
+    strncpy(con->socket.tcp.host, host, NI_MAXHOST);
+    con->socket.tcp.host[NI_MAXHOST - 1]= 0;
+  }
+
+  con->socket.tcp.port= port;
+}
+
+const char *drizzle_con_user(const drizzle_con_st *con)
+{
+  return con->user;
+}
+
+const char *drizzle_con_password(const drizzle_con_st *con)
+{
+  return con->password;
+}
+
+void drizzle_con_set_auth(drizzle_con_st *con, const char *user,
+                          const char *password)
+{
+  if (user == NULL)
+    con->user[0]= 0;
+  else
+  {
+    strncpy(con->user, user, DRIZZLE_MAX_USER_SIZE);
+    con->user[DRIZZLE_MAX_USER_SIZE - 1]= 0;
+  }
+
+  if (password == NULL)
+    con->password[0]= 0;
+  else
+  {
+    strncpy(con->password, password, DRIZZLE_MAX_PASSWORD_SIZE);
+    con->password[DRIZZLE_MAX_PASSWORD_SIZE - 1]= 0;
+  }
+}
+
+const char *drizzle_con_db(const drizzle_con_st *con)
+{
+  return con->db;
+}
+
+void drizzle_con_set_db(drizzle_con_st *con, const char *db)
+{
+  if (db == NULL)
+    con->db[0]= 0;
+  else
+  {
+    strncpy(con->db, db, DRIZZLE_MAX_DB_SIZE);
+    con->db[DRIZZLE_MAX_DB_SIZE - 1]= 0;
+  }
+}
+
+void *drizzle_con_context(const drizzle_con_st *con)
+{
+  return con->context;
+}
+
+void drizzle_con_set_context(drizzle_con_st *con, void *context)
+{
+  con->context= context;
+}
+
+void drizzle_con_set_context_free_fn(drizzle_con_st *con,
+                                     drizzle_con_context_free_fn *function)
+{
+  con->context_free_fn= function;
+}
+
+uint8_t drizzle_con_protocol_version(const drizzle_con_st *con)
+{
+  return con->protocol_version;
+}
+
+const char *drizzle_con_server_version(const drizzle_con_st *con)
+{
+  return con->server_version;
+}
+
+uint32_t drizzle_con_server_version_number(const drizzle_con_st *con)
+{
+  uint32_t major;
+  uint32_t minor;
+  uint32_t version;
+  const char *current;
+  char *end;
+
+  current= con->server_version;
+
+  major= (uint32_t)strtoul(current, &end, 10);
+  current= end + 1;
+  minor= (uint32_t)strtoul(current, &end, 10);
+  current= end + 1;
+  version= (uint32_t)strtoul(current, &end, 10);
+
+  return (major * 10000) + (minor * 100) + version;
+}
+
+uint32_t drizzle_con_thread_id(const drizzle_con_st *con)
+{
+  return con->thread_id;
+}
+
+const uint8_t *drizzle_con_scramble(const drizzle_con_st *con)
+{
+  return con->scramble;
+}
+
+drizzle_capabilities_t drizzle_con_capabilities(const drizzle_con_st *con)
+{
+  return con->capabilities;
+}
+
+drizzle_charset_t drizzle_con_charset(const drizzle_con_st *con)
+{
+  return con->charset;
+}
+
+drizzle_con_status_t drizzle_con_status(const drizzle_con_st *con)
+{
+  return con->status;
+}
+
+uint32_t drizzle_con_max_packet_size(const drizzle_con_st *con)
+{
+  return con->max_packet_size;
+}
+
+/*
+ * Client Definitions
+ */
+
+drizzle_return_t drizzle_con_connect(drizzle_con_st *con)
+{
+  if (con->options & DRIZZLE_CON_READY)
+    return DRIZZLE_RETURN_OK;
+
+  if (drizzle_state_none(con))
+  {
+    if (!(con->options & DRIZZLE_CON_RAW_PACKET))
+    {
+      drizzle_state_push(con, drizzle_state_handshake_server_read);
+      drizzle_state_push(con, drizzle_state_packet_read);
+    }
+
+    drizzle_state_push(con, drizzle_state_connect);
+    drizzle_state_push(con, drizzle_state_addrinfo);
+  }
+
+  return drizzle_state_loop(con);
+}
+
+drizzle_result_st *drizzle_con_quit(drizzle_con_st *con,
+                                    drizzle_result_st *result,
+                                    drizzle_return_t *ret_ptr)
+{
+  return drizzle_con_command_write(con, result, DRIZZLE_COMMAND_QUIT, NULL, 0,
+                                   0, ret_ptr);
+}
+
+drizzle_result_st *drizzle_quit(drizzle_con_st *con,
+                                drizzle_result_st *result,
+                                drizzle_return_t *ret_ptr)
+{
+  return drizzle_con_quit(con, result, ret_ptr);
+}
+
+drizzle_result_st *drizzle_con_select_db(drizzle_con_st *con,
+                                         drizzle_result_st *result,
+                                         const char *db,
+                                         drizzle_return_t *ret_ptr)
+{
+  drizzle_con_set_db(con, db);
+  return drizzle_con_command_write(con, result, DRIZZLE_COMMAND_INIT_DB,
+                                   db, strlen(db), strlen(db), ret_ptr);
+}
+
+drizzle_result_st *drizzle_select_db(drizzle_con_st *con,
+                                     drizzle_result_st *result,
+                                     const char *db,
+                                     drizzle_return_t *ret_ptr)
+{
+  return drizzle_con_select_db(con, result, db, ret_ptr);
+}
+
+drizzle_result_st *drizzle_con_shutdown(drizzle_con_st *con,
+                                        drizzle_result_st *result,
+                                        drizzle_return_t *ret_ptr)
+{
+  if (con->options & DRIZZLE_CON_MYSQL)
+  {
+    return drizzle_con_command_write(con, result, DRIZZLE_COMMAND_SHUTDOWN,
+                                     "0", 1, 1, ret_ptr);
+  }
+
+  return drizzle_con_command_write(con, result, DRIZZLE_COMMAND_SHUTDOWN, NULL,
+                                   0, 0, ret_ptr);
+}
+
+drizzle_result_st *drizzle_shutdown(drizzle_con_st *con,
+                                    drizzle_result_st *result, uint32_t level,
+                                    drizzle_return_t *ret_ptr)
+{
+  (void) level;
+  return drizzle_con_shutdown(con, result, ret_ptr);
+}
+
+drizzle_result_st *drizzle_con_ping(drizzle_con_st *con,
+                                    drizzle_result_st *result,
+                                    drizzle_return_t *ret_ptr)
+{
+  return drizzle_con_command_write(con, result, DRIZZLE_COMMAND_PING, NULL, 0,
+                                   0, ret_ptr);
+}
+
+drizzle_result_st *drizzle_ping(drizzle_con_st *con,
+                                drizzle_result_st *result,
+                                drizzle_return_t *ret_ptr)
+{
+  return drizzle_con_ping(con, result, ret_ptr);
+}
+
+drizzle_result_st *drizzle_con_command_write(drizzle_con_st *con,
+                                             drizzle_result_st *result,
+                                             drizzle_command_t command,
+                                             const void *data, size_t size,
+                                             size_t total,
+                                             drizzle_return_t *ret_ptr)
+{
+  if (!(con->options & DRIZZLE_CON_READY))
+  {
+    if (con->options & DRIZZLE_CON_RAW_PACKET)
+    {
+      drizzle_set_error(con->drizzle, "drizzle_command_write",
+                        "connection not ready");
+      *ret_ptr= DRIZZLE_RETURN_NOT_READY;
+      return result;
+    }
+
+    *ret_ptr= drizzle_con_connect(con);
+    if (*ret_ptr != DRIZZLE_RETURN_OK)
+      return result;
+  }
+
+  if (drizzle_state_none(con))
+  {
+    if (con->options & (DRIZZLE_CON_RAW_PACKET | DRIZZLE_CON_NO_RESULT_READ))
+      con->result= NULL;
+    else
+    {
+      con->result= drizzle_result_create(con, result);
+      if (con->result == NULL)
+      {
+        *ret_ptr= DRIZZLE_RETURN_MEMORY;
+        return NULL;
+      }
+    }
+
+    con->command= command;
+    con->command_data= (uint8_t *)data;
+    con->command_size= size;
+    con->command_offset= 0;
+    con->command_total= total;
+
+    drizzle_state_push(con, drizzle_state_command_write);
+  }
+  else if (con->command_data == NULL)
+  {
+    con->command_data= (uint8_t *)data;
+    con->command_size= size;
+  }
+
+  *ret_ptr= drizzle_state_loop(con);
+  if (*ret_ptr == DRIZZLE_RETURN_PAUSE)
+    *ret_ptr= DRIZZLE_RETURN_OK;
+  else if (*ret_ptr != DRIZZLE_RETURN_OK &&
+           *ret_ptr != DRIZZLE_RETURN_IO_WAIT &&
+           *ret_ptr != DRIZZLE_RETURN_ERROR_CODE)
+  {
+    drizzle_result_free(con->result);
+    con->result= result;
+  }
+
+  return con->result;
+}
+
+/*
+ * Server Definitions
+ */
+
+drizzle_return_t drizzle_con_listen(drizzle_con_st *con)
+{
+  if (con->options & DRIZZLE_CON_READY)
+    return DRIZZLE_RETURN_OK;
+
+  if (drizzle_state_none(con))
+  {
+    drizzle_state_push(con, drizzle_state_listen);
+    drizzle_state_push(con, drizzle_state_addrinfo);
+  }
+
+  return drizzle_state_loop(con);
+}
+
+int drizzle_con_backlog(const drizzle_con_st *con)
+{
+  return con->backlog;
+}
+
+void drizzle_con_set_backlog(drizzle_con_st *con, int backlog)
+{
+  con->backlog= backlog;
+}
+
+void drizzle_con_set_protocol_version(drizzle_con_st *con,
+                                      uint8_t protocol_version)
+{
+  con->protocol_version= protocol_version;
+}
+
+void drizzle_con_set_server_version(drizzle_con_st *con,
+                                    const char *server_version)
+{
+  if (server_version == NULL)
+    con->server_version[0]= 0;
+  else
+  {
+    strncpy(con->server_version, server_version,
+            DRIZZLE_MAX_SERVER_VERSION_SIZE);
+    con->server_version[DRIZZLE_MAX_SERVER_VERSION_SIZE - 1]= 0;
+  }
+}
+
+void drizzle_con_set_thread_id(drizzle_con_st *con, uint32_t thread_id)
+{
+  con->thread_id= thread_id;
+}
+
+void drizzle_con_set_scramble(drizzle_con_st *con, const uint8_t *scramble)
+{
+  if (scramble == NULL)
+    con->scramble= NULL;
+  else
+  {
+    con->scramble= con->scramble_buffer;
+    memcpy(con->scramble, scramble, DRIZZLE_MAX_SCRAMBLE_SIZE);
+  }
+}
+
+void drizzle_con_set_capabilities(drizzle_con_st *con,
+                                  drizzle_capabilities_t capabilities)
+{
+  con->capabilities= capabilities;
+}
+
+void drizzle_con_set_charset(drizzle_con_st *con, drizzle_charset_t charset)
+{
+  con->charset= charset;
+}
+
+void drizzle_con_set_status(drizzle_con_st *con, drizzle_con_status_t status)
+{
+  con->status= status;
+}
+
+void drizzle_con_set_max_packet_size(drizzle_con_st *con,
+                                     uint32_t max_packet_size)
+{
+  con->max_packet_size= max_packet_size;
+}
+
+void drizzle_con_copy_handshake(drizzle_con_st *con, drizzle_con_st *from)
+{
+  drizzle_con_set_auth(con, from->user, NULL);
+  drizzle_con_set_scramble(con, from->scramble);
+  drizzle_con_set_db(con, from->db);
+  drizzle_con_set_protocol_version(con, from->protocol_version);
+  drizzle_con_set_server_version(con, from->server_version);
+  drizzle_con_set_thread_id(con, from->thread_id);
+  drizzle_con_set_scramble(con, from->scramble);
+  drizzle_con_set_capabilities(con, from->capabilities);
+  drizzle_con_set_charset(con, from->charset);
+  drizzle_con_set_status(con, from->status);
+  drizzle_con_set_max_packet_size(con, from->max_packet_size);
+}
+
+void *drizzle_con_command_read(drizzle_con_st *con,
+                               drizzle_command_t *command, size_t *offset,
+                               size_t *size, size_t *total,
+                               drizzle_return_t *ret_ptr)
+{
+  if (drizzle_state_none(con))
+  {
+    con->packet_number= 0;
+    con->command_offset= 0;
+    con->command_total= 0;
+
+    drizzle_state_push(con, drizzle_state_command_read);
+    drizzle_state_push(con, drizzle_state_packet_read);
+  }
+
+  *offset= con->command_offset;
+
+  *ret_ptr= drizzle_state_loop(con);
+  if (*ret_ptr == DRIZZLE_RETURN_PAUSE)
+    *ret_ptr= DRIZZLE_RETURN_OK;
+
+  *command= con->command;
+  *size= con->command_size;
+  *total= con->command_total;
+
+  return con->command_data;
+}
+
+void *drizzle_con_command_buffer(drizzle_con_st *con,
+                                 drizzle_command_t *command, size_t *total,
+                                 drizzle_return_t *ret_ptr)
+{
+  uint8_t *command_data;
+  size_t offset= 0;
+  size_t size= 0;
+
+  command_data= drizzle_con_command_read(con, command, &offset, &size, total,
+                                         ret_ptr);
+  if (*ret_ptr != DRIZZLE_RETURN_OK)
+    return NULL;
+
+  if (command_data == NULL)
+  {
+    *total= 0;
+    return NULL;
+  }
+
+  if (con->command_buffer == NULL)
+  {
+    con->command_buffer= malloc((*total) + 1);
+    if (con->command_buffer == NULL)
+    {
+      drizzle_set_error(con->drizzle, "drizzle_command_buffer", "malloc");
+      *ret_ptr= DRIZZLE_RETURN_MEMORY;
+      return NULL;
+    }
+  }
+
+  memcpy(con->command_buffer + offset, command_data, size);
+
+  while ((offset + size) != (*total))
+  {
+    command_data= drizzle_con_command_read(con, command, &offset, &size, total,
+                                           ret_ptr);
+    if (*ret_ptr != DRIZZLE_RETURN_OK)
+      return NULL;
+
+    memcpy(con->command_buffer + offset, command_data, size);
+  }
+
+  command_data= con->command_buffer;
+  con->command_buffer= NULL;
+  command_data[*total]= 0;
+
+  return command_data;
+}
+
+/*
+ * Local Definitions
+ */
+
+void drizzle_con_reset_addrinfo(drizzle_con_st *con)
+{
+  switch (con->socket_type)
+  {
+  case DRIZZLE_CON_SOCKET_TCP:
+    if (con->socket.tcp.addrinfo != NULL)
+    {
+      freeaddrinfo(con->socket.tcp.addrinfo);
+      con->socket.tcp.addrinfo= NULL;
+    }
+    break;
+
+  case DRIZZLE_CON_SOCKET_UDS:
+    con->socket.uds.addrinfo.ai_addr= NULL;
+    break;
+
+  default:
+    break;
+  }
+
+  con->addrinfo_next= NULL;
+}
+
+/*
+ * State Definitions
+ */
+
+drizzle_return_t drizzle_state_addrinfo(drizzle_con_st *con)
+{
+  drizzle_con_tcp_st *tcp;
+  const char *host;
+  char port[NI_MAXSERV];
+  struct addrinfo ai;
+  int ret;
+
+  drizzle_log_debug(con->drizzle, "drizzle_state_addrinfo");
+
+  switch (con->socket_type)
+  {
+  case DRIZZLE_CON_SOCKET_TCP:
+    tcp= &(con->socket.tcp);
+
+    if (tcp->addrinfo != NULL)
+    {
+      freeaddrinfo(tcp->addrinfo);
+      tcp->addrinfo= NULL;
+    }
+
+    if (tcp->port != 0)
+      snprintf(port, NI_MAXSERV, "%u", tcp->port);
+    else if (con->options & DRIZZLE_CON_MYSQL)
+      snprintf(port, NI_MAXSERV, "%u", DRIZZLE_DEFAULT_TCP_PORT_MYSQL);
+    else
+      snprintf(port, NI_MAXSERV, "%u", DRIZZLE_DEFAULT_TCP_PORT);
+
+    memset(&ai, 0, sizeof(struct addrinfo));
+    ai.ai_socktype= SOCK_STREAM;
+    ai.ai_protocol= IPPROTO_TCP;
+
+    if (con->options & DRIZZLE_CON_LISTEN)
+    {
+      ai.ai_flags = AI_PASSIVE;
+      ai.ai_family = AF_UNSPEC;
+      host= tcp->host;
+    }
+    else
+    {
+      if (tcp->host == NULL)
+        host= DRIZZLE_DEFAULT_TCP_HOST;
+      else
+        host= tcp->host;
+    }
+
+    ret= getaddrinfo(host, port, &ai, &(tcp->addrinfo));
+    if (ret != 0)
+    {
+      drizzle_set_error(con->drizzle, "drizzle_state_addrinfo",
+                        "getaddrinfo:%s", gai_strerror(ret));
+      return DRIZZLE_RETURN_GETADDRINFO;
+    }
+
+    con->addrinfo_next= tcp->addrinfo;
+
+    break;
+
+  case DRIZZLE_CON_SOCKET_UDS:
+    con->addrinfo_next= &(con->socket.uds.addrinfo);
+    break;
+
+  default:
+    break;
+  }
+
+  drizzle_state_pop(con);
+  return DRIZZLE_RETURN_OK;
+}
+
+drizzle_return_t drizzle_state_connect(drizzle_con_st *con)
+{
+  int ret;
+  drizzle_return_t dret;
+
+  drizzle_log_debug(con->drizzle, "drizzle_state_connect");
+
+  if (con->fd != -1)
+  {
+    (void)close(con->fd);
+    con->fd= -1;
+  }
+
+  if (con->addrinfo_next == NULL)
+  {
+    drizzle_set_error(con->drizzle, "drizzle_state_connect",
+                      "could not connect");
+    drizzle_state_reset(con);
+    return DRIZZLE_RETURN_COULD_NOT_CONNECT;
+  }
+
+  con->fd= socket(con->addrinfo_next->ai_family,
+                  con->addrinfo_next->ai_socktype,
+                  con->addrinfo_next->ai_protocol);
+  if (con->fd == -1)
+  {
+    drizzle_set_error(con->drizzle, "drizzle_state_connect", "socket:%d",
+                      errno);
+    con->drizzle->last_errno= errno;
+    return DRIZZLE_RETURN_ERRNO;
+  }
+
+  dret= _con_setsockopt(con);
+  if (dret != DRIZZLE_RETURN_OK)
+  {
+    con->drizzle->last_errno= errno;
+    return dret;
+  }
+
+  while (1)
+  {
+    ret= connect(con->fd, con->addrinfo_next->ai_addr,
+                 con->addrinfo_next->ai_addrlen);
+
+#ifdef _WIN32
+    /*Mapping windows specific error codes to Posix*/
+    errno = WSAGetLastError();
+    switch(errno) {
+    case WSAEINVAL:
+    case WSAEALREADY:
+    case WSAEWOULDBLOCK:
+      errno = EINPROGRESS;
+      break;
+    default:
+      break;
+    }
+#endif /* _WIN32 */
+	
+    drizzle_log_crazy(con->drizzle, "connect return=%d errno=%d", ret, errno);
+
+    if (ret == 0)
+    {
+      con->addrinfo_next= NULL;
+      break;
+    }
+
+    if (errno == EAGAIN || errno == EINTR)
+      continue;
+
+    if (errno == EINPROGRESS)
+    {
+      drizzle_state_pop(con);
+      drizzle_state_push(con, drizzle_state_connecting);
+      return DRIZZLE_RETURN_OK;
+    }
+
+    if (errno == ECONNREFUSED || errno == ENETUNREACH || errno == ETIMEDOUT)
+    {
+      con->addrinfo_next= con->addrinfo_next->ai_next;
+      return DRIZZLE_RETURN_OK;
+    }
+
+    drizzle_set_error(con->drizzle, "drizzle_state_connect", "connect:%d",
+                      errno);
+    con->drizzle->last_errno= errno;
+    return DRIZZLE_RETURN_ERRNO;
+  }
+
+  drizzle_state_pop(con);
+  return DRIZZLE_RETURN_OK;
+}
+
+drizzle_return_t drizzle_state_connecting(drizzle_con_st *con)
+{
+  drizzle_return_t ret;
+
+  drizzle_log_debug(con->drizzle, "drizzle_state_connecting");
+
+  while (1)
+  {
+    if (con->revents & POLLOUT)
+    {
+      drizzle_state_pop(con);
+      return DRIZZLE_RETURN_OK;
+    }
+    else if (con->revents & (POLLERR | POLLHUP | POLLNVAL))
+    {
+      con->revents= 0;
+      drizzle_state_pop(con);
+      drizzle_state_push(con, drizzle_state_connect);
+      con->addrinfo_next= con->addrinfo_next->ai_next;
+      return DRIZZLE_RETURN_OK;
+    }
+
+    ret= drizzle_con_set_events(con, POLLOUT);
+    if (ret != DRIZZLE_RETURN_OK)
+      return ret;
+
+    if (con->drizzle->options & DRIZZLE_NON_BLOCKING)
+      return DRIZZLE_RETURN_IO_WAIT;
+
+    ret= drizzle_con_wait(con->drizzle);
+    if (ret != DRIZZLE_RETURN_OK)
+      return ret;
+  }
+}
+
+drizzle_return_t drizzle_state_read(drizzle_con_st *con)
+{
+  drizzle_return_t ret;
+  ssize_t read_size;
+
+  drizzle_log_debug(con->drizzle, "drizzle_state_read");
+
+  if (con->buffer_size == 0)
+    con->buffer_ptr= con->buffer;
+  else if ((con->buffer_ptr - con->buffer) > (DRIZZLE_MAX_BUFFER_SIZE / 2))
+  {
+    memmove(con->buffer, con->buffer_ptr, con->buffer_size);
+    con->buffer_ptr= con->buffer;
+  }
+
+  while (1)
+  {
+    read_size = recv(con->fd, (char *)con->buffer_ptr + con->buffer_size,
+                     (size_t)DRIZZLE_MAX_BUFFER_SIZE -
+                     ((size_t)(con->buffer_ptr - con->buffer) +
+                      con->buffer_size),0); 
+#ifdef _WIN32
+    /*Get windows error codes and map it to Posix*/
+    errno = WSAGetLastError();
+    switch(errno) {
+    case WSAEWOULDBLOCK:
+    case 10057:
+      errno = EAGAIN;
+      break;
+    default:
+      break;
+    }
+#endif /* _WIN32 */	
+    drizzle_log_crazy(con->drizzle, "read fd=%d return=%zd errno=%d", con->fd,
+                      read_size, errno);
+
+    if (read_size == 0)
+    {
+      drizzle_set_error(con->drizzle, "drizzle_state_read",
+                        "lost connection to server (EOF)");
+      return DRIZZLE_RETURN_LOST_CONNECTION;
+    }
+    else if (read_size == -1)
+    {
+      if (errno == EAGAIN)
+      {
+        ret= drizzle_con_set_events(con, POLLIN);
+        if (ret != DRIZZLE_RETURN_OK)
+          return 0;
+
+        if (con->drizzle->options & DRIZZLE_NON_BLOCKING)
+          return DRIZZLE_RETURN_IO_WAIT;
+
+        ret= drizzle_con_wait(con->drizzle);
+        if (ret != DRIZZLE_RETURN_OK)
+          return ret;
+
+        continue;
+      }
+      else if (errno == ECONNREFUSED)
+      {
+        con->revents= 0;
+        drizzle_state_pop(con);
+        drizzle_state_push(con, drizzle_state_connect);
+        con->addrinfo_next= con->addrinfo_next->ai_next;
+        return DRIZZLE_RETURN_OK;
+      }
+      else if (errno == EINTR)
+        continue;
+      else if (errno == EPIPE || errno == ECONNRESET)
+      {
+        drizzle_set_error(con->drizzle, "drizzle_state_read",
+                          "lost connection to server (%d)", errno);
+        return DRIZZLE_RETURN_LOST_CONNECTION;
+      }
+
+      drizzle_set_error(con->drizzle, "drizzle_state_read", "read:%d", errno);
+      con->drizzle->last_errno= errno;
+      return DRIZZLE_RETURN_ERRNO;
+    }
+
+    con->buffer_size+= (size_t)read_size;
+    break;
+  }
+
+  drizzle_state_pop(con);;
+  return DRIZZLE_RETURN_OK;
+}
+
+drizzle_return_t drizzle_state_write(drizzle_con_st *con)
+{
+  drizzle_return_t ret;
+  ssize_t write_size;
+
+  drizzle_log_debug(con->drizzle, "drizzle_state_write");
+
+  while (con->buffer_size != 0)
+  {
+  
+    write_size = send(con->fd,(char *) con->buffer_ptr, con->buffer_size,0);
+
+    drizzle_log_crazy(con->drizzle, "write fd=%d return=%zd errno=%d", con->fd,
+                      write_size, errno);
+
+    if (write_size == 0)
+    {
+      drizzle_set_error(con->drizzle, "drizzle_state_write",
+                        "lost connection to server (EOF)");
+      return DRIZZLE_RETURN_LOST_CONNECTION;
+    }
+    else if (write_size == -1)
+    {
+      if (errno == EAGAIN)
+      {
+        ret= drizzle_con_set_events(con, POLLOUT);
+        if (ret != DRIZZLE_RETURN_OK)
+          return ret;
+
+        if (con->drizzle->options & DRIZZLE_NON_BLOCKING)
+          return DRIZZLE_RETURN_IO_WAIT;
+
+        ret= drizzle_con_wait(con->drizzle);
+        if (ret != DRIZZLE_RETURN_OK)
+          return ret;
+
+        continue;
+      }
+      else if (errno == EINTR)
+        continue;
+      else if (errno == EPIPE || errno == ECONNRESET)
+      {
+        drizzle_set_error(con->drizzle, "drizzle_state_write",
+                          "lost connection to server (%d)", errno);
+        return DRIZZLE_RETURN_LOST_CONNECTION;
+      }
+
+      drizzle_set_error(con->drizzle, "drizzle_state_write", "write:%d", errno);
+      con->drizzle->last_errno= errno;
+      return DRIZZLE_RETURN_ERRNO;
+    }
+
+    con->buffer_ptr+= write_size;
+    con->buffer_size-= (size_t)write_size;
+    if (con->buffer_size == 0)
+      break;
+  }
+
+  con->buffer_ptr= con->buffer;
+
+  drizzle_state_pop(con);
+  return DRIZZLE_RETURN_OK;
+}
+
+drizzle_return_t drizzle_state_listen(drizzle_con_st *con)
+{
+  char host[NI_MAXHOST];
+  char port[NI_MAXSERV];
+  int ret;
+  int fd;
+  int opt;
+  drizzle_con_st *new_con;
+
+  for (; con->addrinfo_next != NULL;
+       con->addrinfo_next= con->addrinfo_next->ai_next)
+  {
+    ret= getnameinfo(con->addrinfo_next->ai_addr,
+                     con->addrinfo_next->ai_addrlen, host, NI_MAXHOST, port,
+                     NI_MAXSERV, NI_NUMERICHOST | NI_NUMERICSERV);
+    if (ret != 0)
+    {
+      drizzle_set_error(con->drizzle, "drizzle_state_listen", "getnameinfo:%s",
+                        gai_strerror(ret));
+      return DRIZZLE_RETURN_GETADDRINFO;
+    }
+
+    /* Call to socket() can fail for some getaddrinfo results, try another. */
+    fd= socket(con->addrinfo_next->ai_family, con->addrinfo_next->ai_socktype,
+               con->addrinfo_next->ai_protocol);
+    if (fd == -1)
+    {
+      drizzle_log_info(con->drizzle, "could not listen on %s:%s", host, port);
+      drizzle_set_error(con->drizzle, "drizzle_state_listen", "socket:%d",
+                        errno);
+      continue;
+    }
+	
+	opt= 1;
+#ifdef _WIN32
+        ret= setsockopt(fd, SOL_SOCKET, SO_REUSEADDR,(const char*) &opt, sizeof(opt));
+#else
+        ret= setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof(opt));
+#endif /* _WIN32 */
+    if (ret == -1)
+    {
+      close(fd);
+      drizzle_set_error(con->drizzle, "drizzle_state_listen", "setsockopt:%d",
+                        errno);
+      return DRIZZLE_RETURN_ERRNO;
+    }
+
+    ret= bind(fd, con->addrinfo_next->ai_addr, con->addrinfo_next->ai_addrlen);
+    if (ret == -1)
+    {
+      close(fd);
+      drizzle_set_error(con->drizzle, "drizzle_state_listen", "bind:%d", errno);
+      if (errno == EADDRINUSE)
+      {
+        if (con->fd == -1)
+        {
+          drizzle_log_info(con->drizzle, "could not listen on %s:%s", host,
+                           port);
+        }
+
+        continue;
+      }
+
+      return DRIZZLE_RETURN_ERRNO;
+    }
+
+    if (listen(fd, con->backlog) == -1)
+    {
+      close(fd);
+      drizzle_set_error(con->drizzle, "drizzle_state_listen", "listen:%d",
+                        errno);
+      return DRIZZLE_RETURN_ERRNO;
+    }
+
+    if (con->fd == -1)
+    {
+      con->fd= fd;
+      new_con= con;
+    }
+    else
+    {
+      new_con= drizzle_con_clone(con->drizzle, NULL, con);
+      if (new_con == NULL)
+      {
+        close(fd);
+        return DRIZZLE_RETURN_MEMORY;
+      }
+
+      new_con->fd= fd;
+    }
+
+    /* Wait for read events on the listening socket. */
+    ret= drizzle_con_set_events(new_con, POLLIN);
+    if (ret != DRIZZLE_RETURN_OK)
+    {
+      drizzle_con_free(new_con);
+      return ret;
+    }
+
+    drizzle_log_info(con->drizzle, "listening on %s:%s", host, port);
+  }
+
+  /* Report last socket() error if we couldn't find an address to bind. */
+  if (con->fd == -1)
+    return DRIZZLE_RETURN_ERRNO;
+
+  drizzle_state_pop(con);
+  return DRIZZLE_RETURN_OK;
+}
+
+/*
+ * Static Definitions
+ */
+
+static drizzle_return_t _con_setsockopt(drizzle_con_st *con)
+{
+  int ret;
+  struct linger linger;
+  struct timeval waittime;
+
+  ret= 1;
+
+#ifdef _WIN32
+  ret= setsockopt(con->fd, IPPROTO_TCP, TCP_NODELAY, (const char*)&ret,
+                  (socklen_t)sizeof(int));
+#else
+  ret= setsockopt(con->fd, IPPROTO_TCP, TCP_NODELAY, &ret,
+                  (socklen_t)sizeof(int));
+#endif /* _WIN32 */
+
+  if (ret == -1 && errno != EOPNOTSUPP)
+  {
+    drizzle_set_error(con->drizzle, "_con_setsockopt",
+                      "setsockopt:TCP_NODELAY:%d", errno);
+    return DRIZZLE_RETURN_ERRNO;
+  }
+
+  linger.l_onoff= 1;
+  linger.l_linger= DRIZZLE_DEFAULT_SOCKET_TIMEOUT;
+
+#ifdef _WIN32
+  ret= setsockopt(con->fd, SOL_SOCKET, SO_LINGER, (const char*)&linger,
+                  (socklen_t)sizeof(struct linger));
+#else
+  ret= setsockopt(con->fd, SOL_SOCKET, SO_LINGER, &linger,
+                  (socklen_t)sizeof(struct linger));
+#endif /* _WIN32 */
+
+  if (ret == -1)
+  {
+    drizzle_set_error(con->drizzle, "_con_setsockopt",
+                      "setsockopt:SO_LINGER:%d", errno);
+    return DRIZZLE_RETURN_ERRNO;
+  }
+
+  waittime.tv_sec= DRIZZLE_DEFAULT_SOCKET_TIMEOUT;
+  waittime.tv_usec= 0;
+
+#ifdef _WIN32
+  ret= setsockopt(con->fd, SOL_SOCKET, SO_SNDTIMEO, (const char*)&waittime,
+                  (socklen_t)sizeof(struct timeval));
+#else
+  ret= setsockopt(con->fd, SOL_SOCKET, SO_SNDTIMEO, &waittime,
+                  (socklen_t)sizeof(struct timeval));
+#endif /* _WIN32 */
+
+  if (ret == -1 && errno != ENOPROTOOPT)
+  {
+    drizzle_set_error(con->drizzle, "_con_setsockopt",
+                      "setsockopt:SO_SNDTIMEO:%d", errno);
+    return DRIZZLE_RETURN_ERRNO;
+  }
+
+#ifdef _WIN32
+  ret= setsockopt(con->fd, SOL_SOCKET, SO_RCVTIMEO, (const char*)&waittime,
+                  (socklen_t)sizeof(struct timeval));
+#else
+  ret= setsockopt(con->fd, SOL_SOCKET, SO_RCVTIMEO, &waittime,
+                  (socklen_t)sizeof(struct timeval));
+#endif /* _WIN32 */
+
+  if (ret == -1 && errno != ENOPROTOOPT)
+  {
+    drizzle_set_error(con->drizzle, "_con_setsockopt",
+                      "setsockopt:SO_RCVTIMEO:%d", errno);
+    return DRIZZLE_RETURN_ERRNO;
+  }
+
+  ret= DRIZZLE_DEFAULT_SOCKET_SEND_SIZE;
+#ifdef _WIN32
+  ret= setsockopt(con->fd, SOL_SOCKET, SO_SNDBUF, (const char*)&ret, (socklen_t)sizeof(int));
+#else
+  ret= setsockopt(con->fd, SOL_SOCKET, SO_SNDBUF, &ret, (socklen_t)sizeof(int));
+#endif /* _WIN32 */
+  if (ret == -1)
+  {
+    drizzle_set_error(con->drizzle, "_con_setsockopt",
+                      "setsockopt:SO_SNDBUF:%d", errno);
+    return DRIZZLE_RETURN_ERRNO;
+  }
+
+  ret= DRIZZLE_DEFAULT_SOCKET_RECV_SIZE;
+#ifdef _WIN32
+  ret= setsockopt(con->fd, SOL_SOCKET, SO_RCVBUF, (const char*)&ret, (socklen_t)sizeof(int));
+#else
+  ret= setsockopt(con->fd, SOL_SOCKET, SO_RCVBUF, &ret, (socklen_t)sizeof(int));
+#endif /* _WIN32 */
+  if (ret == -1)
+  {
+    drizzle_set_error(con->drizzle, "_con_setsockopt",
+                      "setsockopt:SO_RCVBUF:%d", errno);
+    return DRIZZLE_RETURN_ERRNO;
+  }
+
+#if defined (_WIN32)
+  unsigned long asyncmode = 1;
+  ioctlsocket(con->fd, FIONBIO, &asyncmode);
+#else
+  ret= fcntl(con->fd, F_GETFL, 0);
+  if (ret == -1)
+  {
+    drizzle_set_error(con->drizzle, "_con_setsockopt", "fcntl:F_GETFL:%d",
+                      errno);
+    return DRIZZLE_RETURN_ERRNO;
+  }
+
+  ret= fcntl(con->fd, F_SETFL, ret | O_NONBLOCK);
+  if (ret == -1)
+  {
+    drizzle_set_error(con->drizzle, "_con_setsockopt", "fcntl:F_SETFL:%d",
+                      errno);
+    return DRIZZLE_RETURN_ERRNO;
+  }
+#endif
+
+  return DRIZZLE_RETURN_OK;
+}

=== added file 'libdrizzle/conn.h'
--- libdrizzle/conn.h	1970-01-01 00:00:00 +0000
+++ libdrizzle/conn.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,407 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Connection Declarations
+ */
+
+#ifndef __DRIZZLE_CONN_H
+#define __DRIZZLE_CONN_H
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/**
+ * @addtogroup drizzle_con Connection Declarations
+ * @ingroup drizzle_client_interface
+ * @ingroup drizzle_server_interface
+ * @{
+ */
+
+/**
+ * Get file descriptor for connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @return File descriptor of connection, or -1 if not active.
+ */
+DRIZZLE_API
+int drizzle_con_fd(const drizzle_con_st *con);
+
+/**
+ * Use given file descriptor for connction.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @param[in] fd File descriptor for connection.
+ * @return Standard drizzle return value.
+ */
+DRIZZLE_API
+drizzle_return_t drizzle_con_set_fd(drizzle_con_st *con, int fd);
+
+/**
+ * Close a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ */
+DRIZZLE_API
+void drizzle_con_close(drizzle_con_st *con);
+
+/**
+ * Set events to be watched for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @param[in] events Bitfield of poll() events to watch.
+ * @return Standard drizzle return value.
+ */
+DRIZZLE_API
+drizzle_return_t drizzle_con_set_events(drizzle_con_st *con, short events);
+
+/**
+ * Set events that are ready for a connection. This is used with the external
+ * event callbacks. See drizzle_set_event_watch_fn().
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @param[in] revents Bitfield of poll() events that were detected.
+ * @return Standard drizzle return value.
+ */
+DRIZZLE_API
+drizzle_return_t drizzle_con_set_revents(drizzle_con_st *con, short revents);
+
+/**
+ * Get the drizzle_st struct that the connection belongs to.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @return Drizzle object that this connection is part of.
+ */
+DRIZZLE_API
+drizzle_st *drizzle_con_drizzle(const drizzle_con_st *con);
+
+/**
+ * Return an error string for last error encountered.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @return Pointer to static buffer in library that holds an error string.
+ */
+DRIZZLE_API
+const char *drizzle_con_error(const drizzle_con_st *con);
+
+/**
+ * Value of errno in the case of a DRIZZLE_RETURN_ERRNO return value.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @return An errno value as defined in your system errno.h file.
+ */
+DRIZZLE_API
+int drizzle_con_errno(const drizzle_con_st *con);
+
+/**
+ * Get server defined error code for the last result read.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @return An error code given back in the server response.
+ */
+DRIZZLE_API
+uint16_t drizzle_con_error_code(const drizzle_con_st *con);
+
+/**
+ * Get SQL state code for the last result read.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @return A SQLSTATE code given back in the server response.
+ */
+DRIZZLE_API
+const char *drizzle_con_sqlstate(const drizzle_con_st *con);
+
+/**
+ * Get options for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @return Options set for the connection structure.
+ */
+DRIZZLE_API
+drizzle_con_options_t drizzle_con_options(const drizzle_con_st *con);
+
+/**
+ * Set options for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @param[in] options Available options for connection structure to set.
+ */
+DRIZZLE_API
+void drizzle_con_set_options(drizzle_con_st *con,
+                             drizzle_con_options_t options);
+
+/**
+ * Add options for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @param[in] options Available options for connection structure to set.
+ */
+DRIZZLE_API
+void drizzle_con_add_options(drizzle_con_st *con,
+                             drizzle_con_options_t options);
+
+/**
+ * Remove options for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @param[in] options Available options for connection structure to remove.
+ */
+DRIZZLE_API
+void drizzle_con_remove_options(drizzle_con_st *con,
+                                drizzle_con_options_t options);
+
+/**
+ * Get TCP host for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @return Host this connection is configured for, or NULL if not set.
+ */
+DRIZZLE_API
+const char *drizzle_con_host(const drizzle_con_st *con);
+
+/**
+ * Get TCP port for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @return Port this connection is configured for, 0 if not set.
+ */
+DRIZZLE_API
+in_port_t drizzle_con_port(const drizzle_con_st *con);
+
+/**
+ * Set TCP host and port for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @param[in] host Host to use for this connection, NULL for default value.
+ * @param[in] port Port to use for this connection, 0 for default value.
+ */
+DRIZZLE_API
+void drizzle_con_set_tcp(drizzle_con_st *con, const char *host, in_port_t port);
+
+/**
+ * Get unix domain socket for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @return Unix domain socket set for this connection, NULL if not set.
+ */
+DRIZZLE_API
+const char *drizzle_con_uds(const drizzle_con_st *con);
+
+/**
+ * Set unix domain socket for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @param[in] uds Unix domain socket to use for this connection, NULL for
+ *  defailt value.
+ */
+DRIZZLE_API
+void drizzle_con_set_uds(drizzle_con_st *con, const char *uds);
+
+/**
+ * Get username for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @return User associated with this connection.
+ */
+DRIZZLE_API
+const char *drizzle_con_user(const drizzle_con_st *con);
+
+/**
+ * Get password for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @return Password associated with this connection.
+ */
+DRIZZLE_API
+const char *drizzle_con_password(const drizzle_con_st *con);
+
+/**
+ * Set username and password for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @param[in] user Username to use for this connection.
+ * @param[in] password Password to use for this connection.
+ */
+DRIZZLE_API
+void drizzle_con_set_auth(drizzle_con_st *con, const char *user,
+                          const char *password);
+
+/**
+ * Get database for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @return Database associated with this connection.
+ */
+DRIZZLE_API
+const char *drizzle_con_db(const drizzle_con_st *con);
+
+/**
+ * Set database for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @param[in] db Database to use with this connection.
+ */
+DRIZZLE_API
+void drizzle_con_set_db(drizzle_con_st *con, const char *db);
+
+/**
+ * Get application context pointer for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @return Application context with this connection.
+ */
+DRIZZLE_API
+void *drizzle_con_context(const drizzle_con_st *con);
+
+/**
+ * Set application context pointer for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @param[in] context Application context to use with this connection.
+ */
+DRIZZLE_API
+void drizzle_con_set_context(drizzle_con_st *con, void *context);
+
+/**
+ * Set callback function when the context pointer should be freed.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @param[in] function Function to call to clean up connection context.
+ */
+DRIZZLE_API
+void drizzle_con_set_context_free_fn(drizzle_con_st *con,
+                                     drizzle_con_context_free_fn *function);
+
+/**
+ * Get protocol version for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @return Protocol version for connection.
+ */
+DRIZZLE_API
+uint8_t drizzle_con_protocol_version(const drizzle_con_st *con);
+
+/**
+ * Get server version string for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @return Server version string for connection.
+ */
+DRIZZLE_API
+const char *drizzle_con_server_version(const drizzle_con_st *con);
+
+/**
+ * Get server version number for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @return Server version number for connection.
+ */
+DRIZZLE_API
+uint32_t drizzle_con_server_version_number(const drizzle_con_st *con);
+
+/**
+ * Get thread ID for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @return Thread ID for connection.
+ */
+DRIZZLE_API
+uint32_t drizzle_con_thread_id(const drizzle_con_st *con);
+
+/**
+ * Get scramble buffer for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @return Scramble buffer for connection.
+ */
+DRIZZLE_API
+const uint8_t *drizzle_con_scramble(const drizzle_con_st *con);
+
+/**
+ * Get capabilities for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @return Capabilities for connection.
+ */
+DRIZZLE_API
+drizzle_capabilities_t drizzle_con_capabilities(const drizzle_con_st *con);
+
+/**
+ * Get character set for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @return Character set for connection.
+ */
+DRIZZLE_API
+drizzle_charset_t drizzle_con_charset(const drizzle_con_st *con);
+
+/**
+ * Get status for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @return Status for connection.
+ */
+DRIZZLE_API
+drizzle_con_status_t drizzle_con_status(const drizzle_con_st *con);
+
+/**
+ * Get max packet size for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @return Max packet size for connection.
+ */
+DRIZZLE_API
+uint32_t drizzle_con_max_packet_size(const drizzle_con_st *con);
+
+/** @} */
+
+#ifdef __cplusplus
+}
+#endif
+
+#endif /* __DRIZZLE_CONN_H */

=== added file 'libdrizzle/conn_client.h'
--- libdrizzle/conn_client.h	1970-01-01 00:00:00 +0000
+++ libdrizzle/conn_client.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,163 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Connection Declarations for Clients
+ */
+
+#ifndef __DRIZZLE_CON_CLIENT_H
+#define __DRIZZLE_CON_CLIENT_H
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/**
+ * @addtogroup drizzle_con_client Connection Declarations for Clients
+ * @ingroup drizzle_client_interface
+ * @{
+ */
+
+/**
+ * Connect to server.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @return Standard drizzle return value.
+ */
+DRIZZLE_API
+drizzle_return_t drizzle_con_connect(drizzle_con_st *con);
+
+/**
+ * Send quit command to server for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @param[in] result Caller allocated structure, or NULL to allocate one.
+ * @param[out] ret_ptr Standard drizzle return value.
+ * @return On success, a pointer to the (possibly allocated) structure. On
+ *  failure this will be NULL.
+ */
+DRIZZLE_API
+drizzle_result_st *drizzle_con_quit(drizzle_con_st *con,
+                                    drizzle_result_st *result,
+                                    drizzle_return_t *ret_ptr);
+
+/**
+ * @todo Remove this with next major API change.
+ */
+DRIZZLE_API
+drizzle_result_st *drizzle_quit(drizzle_con_st *con,
+                                drizzle_result_st *result,
+                                drizzle_return_t *ret_ptr);
+
+/**
+ * Select a new default database for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @param[in] result Caller allocated structure, or NULL to allocate one.
+ * @param[in] db Default database to select.
+ * @param[out] ret_ptr Standard drizzle return value.
+ * @return On success, a pointer to the (possibly allocated) structure. On
+ *  failure this will be NULL.
+ */
+DRIZZLE_API
+drizzle_result_st *drizzle_con_select_db(drizzle_con_st *con,
+                                         drizzle_result_st *result,
+                                         const char *db,
+                                         drizzle_return_t *ret_ptr);
+
+/**
+ * @todo Remove this with next major API change.
+ */
+DRIZZLE_API
+drizzle_result_st *drizzle_select_db(drizzle_con_st *con,
+                                     drizzle_result_st *result,
+                                     const char *db,
+                                     drizzle_return_t *ret_ptr);
+
+/**
+ * Send a shutdown message to the server.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @param[in] result Caller allocated structure, or NULL to allocate one.
+ * @param[out] ret_ptr Standard drizzle return value.
+ * @return On success, a pointer to the (possibly allocated) structure. On
+ *  failure this will be NULL.
+ */
+DRIZZLE_API
+drizzle_result_st *drizzle_con_shutdown(drizzle_con_st *con,
+                                        drizzle_result_st *result,
+                                        drizzle_return_t *ret_ptr);
+
+/**
+ * @todo Remove this with next major API change.
+ */
+#define DRIZZLE_SHUTDOWN_DEFAULT 0
+DRIZZLE_API
+drizzle_result_st *drizzle_shutdown(drizzle_con_st *con,
+                                    drizzle_result_st *result, uint32_t level,
+                                    drizzle_return_t *ret_ptr);
+
+/**
+ * Send a ping request to the server.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @param[in] result Caller allocated structure, or NULL to allocate one.
+ * @param[out] ret_ptr Standard drizzle return value.
+ * @return On success, a pointer to the (possibly allocated) structure. On
+ *  failure this will be NULL.
+ */
+DRIZZLE_API
+drizzle_result_st *drizzle_con_ping(drizzle_con_st *con,
+                                    drizzle_result_st *result,
+                                    drizzle_return_t *ret_ptr);
+
+/**
+ * @todo Remove this with next major API change.
+ */
+DRIZZLE_API
+drizzle_result_st *drizzle_ping(drizzle_con_st *con,
+                                drizzle_result_st *result,
+                                drizzle_return_t *ret_ptr);
+
+/**
+ * Send raw command to server, possibly in parts.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @param[in] result Caller allocated structure, or NULL to allocate one.
+ * @param[in] command Command to run on server.
+ * @param[in] data Data to send along with the command.
+ * @param[in] size Size of the current chunk of data being sent.
+ * @param[in] total Total size of all data being sent for command.
+ * @param[out] ret_ptr Standard drizzle return value.
+ * @return On success, a pointer to the (possibly allocated) structure. On
+ *  failure this will be NULL.
+ */
+DRIZZLE_API
+drizzle_result_st *drizzle_con_command_write(drizzle_con_st *con,
+                                             drizzle_result_st *result,
+                                             drizzle_command_t command,
+                                             const void *data, size_t size,
+                                             size_t total,
+                                             drizzle_return_t *ret_ptr);
+
+/** @} */
+
+#ifdef __cplusplus
+}
+#endif
+
+#endif /* __DRIZZLE_CON_CLIENT_H */

=== added file 'libdrizzle/conn_local.h'
--- libdrizzle/conn_local.h	1970-01-01 00:00:00 +0000
+++ libdrizzle/conn_local.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,95 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Local Connection Declarations
+ */
+
+#ifndef __DRIZZLE_CONN_LOCAL_H
+#define __DRIZZLE_CONN_LOCAL_H
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/**
+ * @addtogroup drizzle_con_local Local Connection Declarations
+ * @ingroup drizzle_con
+ * @{
+ */
+
+/**
+ * Clear address info, freeing structs if needed.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ */
+DRIZZLE_LOCAL
+void drizzle_con_reset_addrinfo(drizzle_con_st *con);
+ 
+/**
+ * Check if state stack is empty.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @return True if empty, false if something is on the stack.
+ */
+static inline bool drizzle_state_none(drizzle_con_st *con)
+{
+  return con->state_current == 0;
+}
+
+/**
+ * Push a function onto the stack.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @param[in] function Function to push.
+ */
+static inline void drizzle_state_push(drizzle_con_st *con,
+                                      drizzle_state_fn *function)
+{
+  /* The maximum stack depth can be determined at compile time, so bump this
+     constant if needed to avoid the dynamic memory management. */
+  assert(con->state_current < DRIZZLE_STATE_STACK_SIZE);
+  con->state_stack[con->state_current]= function;
+  con->state_current++;
+}
+
+/**
+ * Pop a function off of the stack.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ */
+static inline void drizzle_state_pop(drizzle_con_st *con)
+{
+  con->state_current--;
+}
+
+/**
+ * Reset the stack so it is empty.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ */
+static inline void drizzle_state_reset(drizzle_con_st *con)
+{
+  con->state_current= 0;
+}
+
+/** @} */
+
+#ifdef __cplusplus
+}
+#endif
+
+#endif /* __DRIZZLE_CONN_LOCAL_H */

=== added file 'libdrizzle/conn_server.h'
--- libdrizzle/conn_server.h	1970-01-01 00:00:00 +0000
+++ libdrizzle/conn_server.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,198 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Connection Declarations for Servers
+ */
+
+#ifndef __DRIZZLE_CON_SERVER_H
+#define __DRIZZLE_CON_SERVER_H
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/**
+ * @addtogroup drizzle_con_server Connection Declarations for Servers
+ * @ingroup drizzle_server_interface
+ *
+ * These functions extend the core connection functions with a set of functions
+ * for server application use. These functions allow you to set raw handshake
+ * information for use with the handshake write functions.
+ * @{
+ */
+
+/**
+ * Put a connection into listening mode.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @return Standard drizzle return value.
+ */
+DRIZZLE_API
+drizzle_return_t drizzle_con_listen(drizzle_con_st *con);
+
+/**
+ * Get connection backlog queue length.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @return Backlog for connection
+ */
+DRIZZLE_API
+int drizzle_con_backlog(const drizzle_con_st *con);
+
+/**
+ * Set connection backlog queue length.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @param[in] backlog Backlog to use for connection
+ */
+DRIZZLE_API
+void drizzle_con_set_backlog(drizzle_con_st *con, int backlog);
+
+/**
+ * Set protocol version for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @param[in] protocol_version Protocol version to use for connection
+ */
+DRIZZLE_API
+void drizzle_con_set_protocol_version(drizzle_con_st *con,
+                                      uint8_t protocol_version);
+
+/**
+ * Set server version string for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @param[in] server_version Server version to use for connection
+ */
+DRIZZLE_API
+void drizzle_con_set_server_version(drizzle_con_st *con,
+                                    const char *server_version);
+
+/**
+ * Set thread ID for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @param[in] thread_id Thread ID to use for connection
+ */
+DRIZZLE_API
+void drizzle_con_set_thread_id(drizzle_con_st *con, uint32_t thread_id);
+
+/**
+ * Set scramble buffer for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @param[in] scramble Scramble to use for connection
+ */
+DRIZZLE_API
+void drizzle_con_set_scramble(drizzle_con_st *con, const uint8_t *scramble);
+
+/**
+ * Set capabilities for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @param[in] capabilities Capabilities to use for connection
+ */
+DRIZZLE_API
+void drizzle_con_set_capabilities(drizzle_con_st *con,
+                                  drizzle_capabilities_t capabilities);
+
+/**
+ * Set charset for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @param[in] charset Character set to use for connection
+ */
+DRIZZLE_API
+void drizzle_con_set_charset(drizzle_con_st *con, drizzle_charset_t charset);
+
+/**
+ * Set status for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @param[in] status Status to use for connection
+ */
+DRIZZLE_API
+void drizzle_con_set_status(drizzle_con_st *con, drizzle_con_status_t status);
+
+/**
+ * Set max packet size for a connection.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @param[in] max_packet_size Max packet size to use for connection
+ */
+DRIZZLE_API
+void drizzle_con_set_max_packet_size(drizzle_con_st *con,
+                                     uint32_t max_packet_size);
+
+/**
+ * Copy all handshake information from one connection into another.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @param[in] from Connection structure to copy from.
+ */
+DRIZZLE_API
+void drizzle_con_copy_handshake(drizzle_con_st *con, drizzle_con_st *from);
+
+/**
+ * Read command without buffering.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @param[out] command Command that was read.
+ * @param[out] offset Where the data being returned begins in the command data.
+ * @param[out] size The size of the data chunk being returned.
+ * @param[out] total The total size of all command data being read.
+ * @param[out] ret_ptr Standard drizzle return value.
+ * @return On success, a pointer to an internal buffer with the command data.
+ *  It will be *size bytes in length.
+ */
+DRIZZLE_API
+void *drizzle_con_command_read(drizzle_con_st *con,
+                               drizzle_command_t *command, size_t *offset,
+                               size_t *size, size_t *total,
+                               drizzle_return_t *ret_ptr);
+
+/**
+ * Read command and buffer it.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @param[out] command Command that was read.
+ * @param[out] total The total size of all command data being read.
+ * @param[out] ret_ptr Standard drizzle return value.
+ * @return On success, allocated buffer that holds the command data of length
+ *  *total.
+ */
+DRIZZLE_API
+void *drizzle_con_command_buffer(drizzle_con_st *con,
+                                 drizzle_command_t *command, size_t *total,
+                                 drizzle_return_t *ret_ptr);
+
+/** @} */
+
+#ifdef __cplusplus
+}
+#endif
+
+#endif /* __DRIZZLE_CON_SERVER_H */

=== added file 'libdrizzle/conn_uds.c'
--- libdrizzle/conn_uds.c	1970-01-01 00:00:00 +0000
+++ libdrizzle/conn_uds.c	2010-08-12 20:27:32 +0000
@@ -0,0 +1,53 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Connection Definitions for Unix Domain Sockets
+ */
+
+#include "common.h"
+
+const char *drizzle_con_uds(const drizzle_con_st *con)
+{
+  if (con->socket_type == DRIZZLE_CON_SOCKET_UDS)
+  {
+    if (con->socket.uds.sockaddr.sun_path[0] != 0)
+      return con->socket.uds.sockaddr.sun_path;
+
+    if (con->options & DRIZZLE_CON_MYSQL)
+      return DRIZZLE_DEFAULT_UDS_MYSQL;
+
+    return DRIZZLE_DEFAULT_UDS;
+  }
+
+  return NULL;
+}
+
+void drizzle_con_set_uds(drizzle_con_st *con, const char *uds)
+{
+  drizzle_con_reset_addrinfo(con);
+
+  con->socket_type= DRIZZLE_CON_SOCKET_UDS;
+
+  if (uds == NULL)
+    uds= "";
+
+  con->socket.uds.sockaddr.sun_family= AF_UNIX;
+  strncpy(con->socket.uds.sockaddr.sun_path, uds,
+          sizeof(con->socket.uds.sockaddr.sun_path));
+  con->socket.uds.sockaddr.sun_path[sizeof(con->socket.uds.sockaddr.sun_path) - 1]= 0;
+
+  con->socket.uds.addrinfo.ai_family= AF_UNIX;
+  con->socket.uds.addrinfo.ai_socktype= SOCK_STREAM;
+  con->socket.uds.addrinfo.ai_protocol= 0;
+  con->socket.uds.addrinfo.ai_addrlen= sizeof(struct sockaddr_un);
+  con->socket.uds.addrinfo.ai_addr= (struct sockaddr *)&(con->socket.uds.sockaddr);
+}

=== added file 'libdrizzle/constants.h'
--- libdrizzle/constants.h	1970-01-01 00:00:00 +0000
+++ libdrizzle/constants.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,515 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Defines, typedefs, enums, and macros
+ */
+
+#ifndef __DRIZZLE_CONSTANTS_H
+#define __DRIZZLE_CONSTANTS_H
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/**
+ * @addtogroup drizzle_constants Constants
+ * @ingroup drizzle_client_interface
+ * @ingroup drizzle_server_interface
+ * @{
+ */
+
+/**
+ * @todo Remove these with next major API change.
+ */
+#define DRIZZLE_RETURN_SERVER_GONE DRIZZLE_RETURN_LOST_CONNECTION
+#define DRIZZLE_RETURN_EOF DRIZZLE_RETURN_LOST_CONNECTION
+#define DRIZZLE_COLUMN_TYPE_VIRTUAL 17
+
+/* Defines. */
+#define DRIZZLE_DEFAULT_TCP_HOST         "127.0.0.1"
+#define DRIZZLE_DEFAULT_TCP_PORT         4427
+#define DRIZZLE_DEFAULT_TCP_PORT_MYSQL   3306
+#define DRIZZLE_DEFAULT_UDS              "/tmp/drizzle.sock"
+#define DRIZZLE_DEFAULT_UDS_MYSQL        "/tmp/mysql.sock"
+#define DRIZZLE_DEFAULT_BACKLOG          64
+#define DRIZZLE_MAX_ERROR_SIZE           2048
+#define DRIZZLE_MAX_USER_SIZE            64
+#define DRIZZLE_MAX_PASSWORD_SIZE        32
+#define DRIZZLE_MAX_DB_SIZE              64
+#define DRIZZLE_MAX_INFO_SIZE            2048
+#define DRIZZLE_MAX_SQLSTATE_SIZE        5
+#define DRIZZLE_MAX_CATALOG_SIZE         128
+#define DRIZZLE_MAX_TABLE_SIZE           128
+#define DRIZZLE_MAX_COLUMN_NAME_SIZE     2048
+#define DRIZZLE_MAX_DEFAULT_VALUE_SIZE   2048
+#define DRIZZLE_MAX_PACKET_SIZE          UINT32_MAX
+#define DRIZZLE_MAX_BUFFER_SIZE          32768
+#define DRIZZLE_BUFFER_COPY_THRESHOLD    8192
+#define DRIZZLE_MAX_SERVER_VERSION_SIZE  32
+#define DRIZZLE_MAX_SCRAMBLE_SIZE        20
+#define DRIZZLE_STATE_STACK_SIZE         8
+#define DRIZZLE_ROW_GROW_SIZE            8192
+#define DRIZZLE_DEFAULT_SOCKET_TIMEOUT   10
+#define DRIZZLE_DEFAULT_SOCKET_SEND_SIZE 32768
+#define DRIZZLE_DEFAULT_SOCKET_RECV_SIZE 32768
+#define DRIZZLE_MYSQL_PASSWORD_HASH      41
+
+/**
+ * Return codes.
+ */
+typedef enum
+{
+  DRIZZLE_RETURN_OK,
+  DRIZZLE_RETURN_IO_WAIT,
+  DRIZZLE_RETURN_PAUSE,
+  DRIZZLE_RETURN_ROW_BREAK,
+  DRIZZLE_RETURN_MEMORY,
+  DRIZZLE_RETURN_ERRNO,
+  DRIZZLE_RETURN_INTERNAL_ERROR,
+  DRIZZLE_RETURN_GETADDRINFO,
+  DRIZZLE_RETURN_NOT_READY,
+  DRIZZLE_RETURN_BAD_PACKET_NUMBER,
+  DRIZZLE_RETURN_BAD_HANDSHAKE_PACKET,
+  DRIZZLE_RETURN_BAD_PACKET,
+  DRIZZLE_RETURN_PROTOCOL_NOT_SUPPORTED,
+  DRIZZLE_RETURN_UNEXPECTED_DATA,
+  DRIZZLE_RETURN_NO_SCRAMBLE,
+  DRIZZLE_RETURN_AUTH_FAILED,
+  DRIZZLE_RETURN_NULL_SIZE,
+  DRIZZLE_RETURN_ERROR_CODE,
+  DRIZZLE_RETURN_TOO_MANY_COLUMNS,
+  DRIZZLE_RETURN_ROW_END,
+  DRIZZLE_RETURN_LOST_CONNECTION,
+  DRIZZLE_RETURN_COULD_NOT_CONNECT,
+  DRIZZLE_RETURN_NO_ACTIVE_CONNECTIONS,
+  DRIZZLE_RETURN_HANDSHAKE_FAILED,
+  DRIZZLE_RETURN_TIMEOUT,
+  DRIZZLE_RETURN_MAX /* Always add new codes to the end before this one. */
+} drizzle_return_t;
+
+/**
+ * Verbosity levels.
+ */
+typedef enum
+{
+  DRIZZLE_VERBOSE_NEVER,
+  DRIZZLE_VERBOSE_FATAL,
+  DRIZZLE_VERBOSE_ERROR,
+  DRIZZLE_VERBOSE_INFO,
+  DRIZZLE_VERBOSE_DEBUG,
+  DRIZZLE_VERBOSE_CRAZY,
+  DRIZZLE_VERBOSE_MAX
+} drizzle_verbose_t;
+
+/** @} */
+
+/**
+ * @ingroup drizzle
+ * Options for drizzle_st.
+ */
+typedef enum
+{
+  DRIZZLE_NONE=            0,
+  DRIZZLE_ALLOCATED=       (1 << 0),
+  DRIZZLE_NON_BLOCKING=    (1 << 1),
+  DRIZZLE_FREE_OBJECTS=    (1 << 2),
+  DRIZZLE_ASSERT_DANGLING= (1 << 3)
+} drizzle_options_t;
+
+/**
+ * @ingroup drizzle_con
+ * Options for drizzle_con_st.
+ */
+typedef enum
+{
+  DRIZZLE_CON_NONE=           0,
+  DRIZZLE_CON_ALLOCATED=      (1 << 0),
+  DRIZZLE_CON_MYSQL=          (1 << 1),
+  DRIZZLE_CON_RAW_PACKET=     (1 << 2),
+  DRIZZLE_CON_RAW_SCRAMBLE=   (1 << 3),
+  DRIZZLE_CON_READY=          (1 << 4),
+  DRIZZLE_CON_NO_RESULT_READ= (1 << 5),
+  DRIZZLE_CON_IO_READY=       (1 << 6),
+  DRIZZLE_CON_LISTEN=         (1 << 7),
+  DRIZZLE_CON_EXPERIMENTAL=   (1 << 8),
+  DRIZZLE_CON_FOUND_ROWS=     (1 << 9)
+} drizzle_con_options_t;
+
+/**
+ * @ingroup drizzle_con
+ * Socket types for drizzle_con_st.
+ */
+typedef enum
+{
+  DRIZZLE_CON_SOCKET_TCP= 0,
+  DRIZZLE_CON_SOCKET_UDS= (1 << 0)
+} drizzle_con_socket_t;
+
+/**
+ * @ingroup drizzle_con
+ * Status flags for drizle_con_st.
+ */
+typedef enum
+{
+  DRIZZLE_CON_STATUS_NONE=                     0,
+  DRIZZLE_CON_STATUS_IN_TRANS=                 (1 << 0),
+  DRIZZLE_CON_STATUS_AUTOCOMMIT=               (1 << 1),
+  DRIZZLE_CON_STATUS_MORE_RESULTS_EXISTS=      (1 << 3),
+  DRIZZLE_CON_STATUS_QUERY_NO_GOOD_INDEX_USED= (1 << 4),
+  DRIZZLE_CON_STATUS_QUERY_NO_INDEX_USED=      (1 << 5),
+  DRIZZLE_CON_STATUS_CURSOR_EXISTS=            (1 << 6),
+  DRIZZLE_CON_STATUS_LAST_ROW_SENT=            (1 << 7),
+  DRIZZLE_CON_STATUS_DB_DROPPED=               (1 << 8),
+  DRIZZLE_CON_STATUS_NO_BACKSLASH_ESCAPES=     (1 << 9),
+  DRIZZLE_CON_STATUS_QUERY_WAS_SLOW=           (1 << 10)
+} drizzle_con_status_t;
+
+/**
+ * @ingroup drizzle_con
+ * Capabilities for drizzle_con_st.
+ */
+typedef enum
+{
+  DRIZZLE_CAPABILITIES_NONE=                   0,
+  DRIZZLE_CAPABILITIES_LONG_PASSWORD=          (1 << 0),
+  DRIZZLE_CAPABILITIES_FOUND_ROWS=             (1 << 1),
+  DRIZZLE_CAPABILITIES_LONG_FLAG=              (1 << 2),
+  DRIZZLE_CAPABILITIES_CONNECT_WITH_DB=        (1 << 3),
+  DRIZZLE_CAPABILITIES_NO_SCHEMA=              (1 << 4),
+  DRIZZLE_CAPABILITIES_COMPRESS=               (1 << 5),
+  DRIZZLE_CAPABILITIES_ODBC=                   (1 << 6),
+  DRIZZLE_CAPABILITIES_LOCAL_FILES=            (1 << 7),
+  DRIZZLE_CAPABILITIES_IGNORE_SPACE=           (1 << 8),
+  DRIZZLE_CAPABILITIES_PROTOCOL_41=            (1 << 9),
+  DRIZZLE_CAPABILITIES_INTERACTIVE=            (1 << 10),
+  DRIZZLE_CAPABILITIES_SSL=                    (1 << 11),
+  DRIZZLE_CAPABILITIES_IGNORE_SIGPIPE=         (1 << 12),
+  DRIZZLE_CAPABILITIES_TRANSACTIONS=           (1 << 13),
+  DRIZZLE_CAPABILITIES_RESERVED=               (1 << 14),
+  DRIZZLE_CAPABILITIES_SECURE_CONNECTION=      (1 << 15),
+  DRIZZLE_CAPABILITIES_MULTI_STATEMENTS=       (1 << 16),
+  DRIZZLE_CAPABILITIES_MULTI_RESULTS=          (1 << 17),
+  DRIZZLE_CAPABILITIES_SSL_VERIFY_SERVER_CERT= (1 << 30),
+  DRIZZLE_CAPABILITIES_REMEMBER_OPTIONS=       (1 << 31),
+  DRIZZLE_CAPABILITIES_CLIENT= (DRIZZLE_CAPABILITIES_LONG_PASSWORD |
+                                DRIZZLE_CAPABILITIES_FOUND_ROWS |
+                                DRIZZLE_CAPABILITIES_LONG_FLAG |
+                                DRIZZLE_CAPABILITIES_CONNECT_WITH_DB |
+                                DRIZZLE_CAPABILITIES_TRANSACTIONS |
+                                DRIZZLE_CAPABILITIES_PROTOCOL_41 |
+                                DRIZZLE_CAPABILITIES_SECURE_CONNECTION)
+} drizzle_capabilities_t;
+
+/**
+ * @ingroup drizzle_command 
+ * Commands for drizzle_command functions.
+ */
+typedef enum
+{
+  DRIZZLE_COMMAND_SLEEP,               /* Not used currently. */
+  DRIZZLE_COMMAND_QUIT,
+  DRIZZLE_COMMAND_INIT_DB,
+  DRIZZLE_COMMAND_QUERY,
+  DRIZZLE_COMMAND_FIELD_LIST,          /* Deprecated. */
+  DRIZZLE_COMMAND_CREATE_DB,           /* Deprecated. */
+  DRIZZLE_COMMAND_DROP_DB,             /* Deprecated. */
+  DRIZZLE_COMMAND_REFRESH,
+  DRIZZLE_COMMAND_SHUTDOWN,
+  DRIZZLE_COMMAND_STATISTICS,
+  DRIZZLE_COMMAND_PROCESS_INFO,        /* Deprecated. */
+  DRIZZLE_COMMAND_CONNECT,             /* Not used currently. */
+  DRIZZLE_COMMAND_PROCESS_KILL,        /* Deprecated. */
+  DRIZZLE_COMMAND_DEBUG,
+  DRIZZLE_COMMAND_PING,
+  DRIZZLE_COMMAND_TIME,                /* Not used currently. */
+  DRIZZLE_COMMAND_DELAYED_INSERT,      /* Not used currently. */
+  DRIZZLE_COMMAND_CHANGE_USER,
+  DRIZZLE_COMMAND_BINLOG_DUMP,         /* Not used currently. */
+  DRIZZLE_COMMAND_TABLE_DUMP,          /* Not used currently. */
+  DRIZZLE_COMMAND_CONNECT_OUT,         /* Not used currently. */
+  DRIZZLE_COMMAND_REGISTER_SLAVE,      /* Not used currently. */
+  DRIZZLE_COMMAND_STMT_PREPARE,        /* Not used currently. */
+  DRIZZLE_COMMAND_STMT_EXECUTE,        /* Not used currently. */
+  DRIZZLE_COMMAND_STMT_SEND_LONG_DATA, /* Not used currently. */
+  DRIZZLE_COMMAND_STMT_CLOSE,          /* Not used currently. */
+  DRIZZLE_COMMAND_STMT_RESET,          /* Not used currently. */
+  DRIZZLE_COMMAND_SET_OPTION,          /* Not used currently. */
+  DRIZZLE_COMMAND_STMT_FETCH,          /* Not used currently. */
+  DRIZZLE_COMMAND_DAEMON,              /* Not used currently. */
+  DRIZZLE_COMMAND_END                  /* Not used currently. */
+} drizzle_command_t;
+
+/**
+ * @ingroup drizzle_command 
+ * Commands for the Drizzle protocol functions.
+ */
+typedef enum
+{
+  DRIZZLE_COMMAND_DRIZZLE_SLEEP,
+  DRIZZLE_COMMAND_DRIZZLE_QUIT,
+  DRIZZLE_COMMAND_DRIZZLE_INIT_DB,
+  DRIZZLE_COMMAND_DRIZZLE_QUERY,
+  DRIZZLE_COMMAND_DRIZZLE_SHUTDOWN,
+  DRIZZLE_COMMAND_DRIZZLE_CONNECT,
+  DRIZZLE_COMMAND_DRIZZLE_PING,
+  DRIZZLE_COMMAND_DRIZZLE_END
+} drizzle_command_drizzle_t;
+
+/**
+ * @ingroup drizzle_query
+ * Options for drizzle_query_st.
+ */
+typedef enum
+{
+  DRIZZLE_QUERY_ALLOCATED= (1 << 0)
+} drizzle_query_options_t;
+
+/**
+ * @ingroup drizzle_query
+ * States for drizle_query_st.
+ */
+typedef enum
+{
+  DRIZZLE_QUERY_STATE_INIT,
+  DRIZZLE_QUERY_STATE_QUERY,
+  DRIZZLE_QUERY_STATE_RESULT,
+  DRIZZLE_QUERY_STATE_DONE
+} drizzle_query_state_t;
+
+/**
+ * @ingroup drizzle_result
+ * Options for drizzle_result_st.
+ */
+typedef enum
+{
+  DRIZZLE_RESULT_NONE=          0,
+  DRIZZLE_RESULT_ALLOCATED=     (1 << 0),
+  DRIZZLE_RESULT_SKIP_COLUMN=   (1 << 1),
+  DRIZZLE_RESULT_BUFFER_COLUMN= (1 << 2),
+  DRIZZLE_RESULT_BUFFER_ROW=    (1 << 3),
+  DRIZZLE_RESULT_EOF_PACKET=    (1 << 4),
+  DRIZZLE_RESULT_ROW_BREAK=     (1 << 5)
+} drizzle_result_options_t;
+
+/**
+ * @ingroup drizzle_column
+ * Options for drizzle_column_st.
+ */
+typedef enum
+{
+  DRIZZLE_COLUMN_ALLOCATED= (1 << 0)
+} drizzle_column_options_t;
+
+/**
+ * @ingroup drizzle_column
+ * Types for drizzle_column_st.
+ */
+typedef enum
+{
+  DRIZZLE_COLUMN_TYPE_DECIMAL,
+  DRIZZLE_COLUMN_TYPE_TINY,
+  DRIZZLE_COLUMN_TYPE_SHORT,
+  DRIZZLE_COLUMN_TYPE_LONG,
+  DRIZZLE_COLUMN_TYPE_FLOAT,
+  DRIZZLE_COLUMN_TYPE_DOUBLE,
+  DRIZZLE_COLUMN_TYPE_NULL,
+  DRIZZLE_COLUMN_TYPE_TIMESTAMP,
+  DRIZZLE_COLUMN_TYPE_LONGLONG,
+  DRIZZLE_COLUMN_TYPE_INT24,
+  DRIZZLE_COLUMN_TYPE_DATE,
+  DRIZZLE_COLUMN_TYPE_TIME,
+  DRIZZLE_COLUMN_TYPE_DATETIME,
+  DRIZZLE_COLUMN_TYPE_YEAR,
+  DRIZZLE_COLUMN_TYPE_NEWDATE,
+  DRIZZLE_COLUMN_TYPE_VARCHAR,
+  DRIZZLE_COLUMN_TYPE_BIT,
+  DRIZZLE_COLUMN_TYPE_NEWDECIMAL=  246,
+  DRIZZLE_COLUMN_TYPE_ENUM=        247,
+  DRIZZLE_COLUMN_TYPE_SET=         248,
+  DRIZZLE_COLUMN_TYPE_TINY_BLOB=   249,
+  DRIZZLE_COLUMN_TYPE_MEDIUM_BLOB= 250,
+  DRIZZLE_COLUMN_TYPE_LONG_BLOB=   251,
+  DRIZZLE_COLUMN_TYPE_BLOB=        252,
+  DRIZZLE_COLUMN_TYPE_VAR_STRING=  253,
+  DRIZZLE_COLUMN_TYPE_STRING=      254,
+  DRIZZLE_COLUMN_TYPE_GEOMETRY=    255
+} drizzle_column_type_t;
+
+/**
+ * @ingroup drizzle_column
+ * Types for drizzle_column_st for Drizzle.
+ */
+typedef enum
+{
+  DRIZZLE_COLUMN_TYPE_DRIZZLE_TINY,
+  DRIZZLE_COLUMN_TYPE_DRIZZLE_LONG,
+  DRIZZLE_COLUMN_TYPE_DRIZZLE_DOUBLE,
+  DRIZZLE_COLUMN_TYPE_DRIZZLE_NULL,
+  DRIZZLE_COLUMN_TYPE_DRIZZLE_TIMESTAMP,
+  DRIZZLE_COLUMN_TYPE_DRIZZLE_LONGLONG,
+  DRIZZLE_COLUMN_TYPE_DRIZZLE_DATETIME,
+  DRIZZLE_COLUMN_TYPE_DRIZZLE_DATE,
+  DRIZZLE_COLUMN_TYPE_DRIZZLE_VARCHAR,
+  DRIZZLE_COLUMN_TYPE_DRIZZLE_NEWDECIMAL,
+  DRIZZLE_COLUMN_TYPE_DRIZZLE_ENUM,
+  DRIZZLE_COLUMN_TYPE_DRIZZLE_BLOB,
+  DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX=DRIZZLE_COLUMN_TYPE_DRIZZLE_BLOB
+} drizzle_column_type_drizzle_t;
+
+/**
+ * @ingroup drizzle_column
+ * Flags for drizzle_column_st.
+ */
+typedef enum
+{
+  DRIZZLE_COLUMN_FLAGS_NONE=             0,
+  DRIZZLE_COLUMN_FLAGS_NOT_NULL=         (1 << 0),
+  DRIZZLE_COLUMN_FLAGS_PRI_KEY=          (1 << 1),
+  DRIZZLE_COLUMN_FLAGS_UNIQUE_KEY=       (1 << 2),
+  DRIZZLE_COLUMN_FLAGS_MULTIPLE_KEY=     (1 << 3),
+  DRIZZLE_COLUMN_FLAGS_BLOB=             (1 << 4),
+  DRIZZLE_COLUMN_FLAGS_UNSIGNED=         (1 << 5),
+  DRIZZLE_COLUMN_FLAGS_ZEROFILL=         (1 << 6),
+  DRIZZLE_COLUMN_FLAGS_BINARY=           (1 << 7),
+  DRIZZLE_COLUMN_FLAGS_ENUM=             (1 << 8),
+  DRIZZLE_COLUMN_FLAGS_AUTO_INCREMENT=   (1 << 9),
+  DRIZZLE_COLUMN_FLAGS_TIMESTAMP=        (1 << 10),
+  DRIZZLE_COLUMN_FLAGS_SET=              (1 << 11),
+  DRIZZLE_COLUMN_FLAGS_NO_DEFAULT_VALUE= (1 << 12),
+  DRIZZLE_COLUMN_FLAGS_ON_UPDATE_NOW=    (1 << 13),
+  DRIZZLE_COLUMN_FLAGS_PART_KEY=         (1 << 14),
+  DRIZZLE_COLUMN_FLAGS_NUM=              (1 << 15),
+  DRIZZLE_COLUMN_FLAGS_GROUP=            (1 << 15), /* NUM & GROUP the same. */
+  DRIZZLE_COLUMN_FLAGS_UNIQUE=           (1 << 16),
+  DRIZZLE_COLUMN_FLAGS_BINCMP=           (1 << 17),
+  DRIZZLE_COLUMN_FLAGS_GET_FIXED_FIELDS= (1 << 18),
+  DRIZZLE_COLUMN_FLAGS_IN_PART_FUNC=     (1 << 19),
+  DRIZZLE_COLUMN_FLAGS_IN_ADD_INDEX=     (1 << 20),
+  DRIZZLE_COLUMN_FLAGS_RENAMED=          (1 << 21)
+} drizzle_column_flags_t;
+
+/**
+ * @addtogroup drizzle_types Types
+ * @ingroup drizzle_client_interface
+ * @ingroup drizzle_server_interface
+ * @{
+ */
+
+/* Types. */
+typedef struct drizzle_st drizzle_st;
+typedef struct drizzle_con_tcp_st drizzle_con_tcp_st;
+typedef struct drizzle_con_uds_st drizzle_con_uds_st;
+typedef struct drizzle_con_st drizzle_con_st;
+typedef struct drizzle_query_st drizzle_query_st;
+typedef struct drizzle_result_st drizzle_result_st;
+typedef struct drizzle_column_st drizzle_column_st;
+typedef char *drizzle_field_t;
+typedef drizzle_field_t *drizzle_row_t;
+typedef uint8_t drizzle_charset_t;
+
+/* Function types. */
+typedef void (drizzle_context_free_fn)(drizzle_st *drizzle,
+                                       void *context);
+typedef void (drizzle_log_fn)(const char *line, drizzle_verbose_t verbose,
+                              void *context);
+typedef drizzle_return_t (drizzle_state_fn)(drizzle_con_st *con);
+typedef void (drizzle_con_context_free_fn)(drizzle_con_st *con,
+                                           void *context);
+typedef void (drizzle_query_context_free_fn)(drizzle_query_st *query,
+                                             void *context);
+/**
+ * Custom function to register or deregister interest in file descriptor
+ * events. See drizzle_set_event_watch_fn().
+ *
+ * @param[in] con Connection that has changed the events it is interested in.
+ *  Use drizzle_con_fd() to get the file descriptor.
+ * @param[in] events A bit mask of POLLIN | POLLOUT, specifying if the
+ *  connection is waiting for read or write events.
+ * @param[in] context Application context pointer registered with
+ *  drizzle_set_event_watch_fn().
+ * @return DRIZZLE_RETURN_OK if successful.
+ */
+typedef drizzle_return_t (drizzle_event_watch_fn)(drizzle_con_st *con,
+                                                  short events,
+                                                  void *context);
+
+/** @} */
+
+/**
+ * @addtogroup drizzle_macros Macros
+ * @ingroup drizzle_client_interface
+ * @ingroup drizzle_server_interface
+ * @{
+ */
+
+/* Protocol unpacking macros. */
+#define drizzle_get_byte2(__buffer) \
+  (uint16_t)((__buffer)[0] | \
+            ((__buffer)[1] << 8))
+#define drizzle_get_byte3(__buffer) \
+  (uint32_t)((__buffer)[0] | \
+            ((__buffer)[1] << 8) | \
+            ((__buffer)[2] << 16))
+#define drizzle_get_byte4(__buffer) \
+  (uint32_t)((__buffer)[0] | \
+            ((__buffer)[1] << 8) | \
+            ((__buffer)[2] << 16) | \
+            ((__buffer)[3] << 24))
+#define drizzle_get_byte8(__buffer) \
+  ((uint64_t)(__buffer)[0] | \
+  ((uint64_t)(__buffer)[1] << 8) | \
+  ((uint64_t)(__buffer)[2] << 16) | \
+  ((uint64_t)(__buffer)[3] << 24) | \
+  ((uint64_t)(__buffer)[4] << 32) | \
+  ((uint64_t)(__buffer)[5] << 40) | \
+  ((uint64_t)(__buffer)[6] << 48) | \
+  ((uint64_t)(__buffer)[7] << 56))
+
+/* Protocol packing macros. */
+#define drizzle_set_byte2(__buffer, __int) do { \
+  (__buffer)[0]= (uint8_t)((__int) & 0xFF); \
+  (__buffer)[1]= (uint8_t)(((__int) >> 8) & 0xFF); } while (0)
+#define drizzle_set_byte3(__buffer, __int) do { \
+  (__buffer)[0]= (uint8_t)((__int) & 0xFF); \
+  (__buffer)[1]= (uint8_t)(((__int) >> 8) & 0xFF); \
+  (__buffer)[2]= (uint8_t)(((__int) >> 16) & 0xFF); } while (0)
+#define drizzle_set_byte4(__buffer, __int) do { \
+  (__buffer)[0]= (uint8_t)((__int) & 0xFF); \
+  (__buffer)[1]= (uint8_t)(((__int) >> 8) & 0xFF); \
+  (__buffer)[2]= (uint8_t)(((__int) >> 16) & 0xFF); \
+  (__buffer)[3]= (uint8_t)(((__int) >> 24) & 0xFF); } while (0)
+#define drizzle_set_byte8(__buffer, __int) do { \
+  (__buffer)[0]= (uint8_t)((__int) & 0xFF); \
+  (__buffer)[1]= (uint8_t)(((__int) >> 8) & 0xFF); \
+  (__buffer)[2]= (uint8_t)(((__int) >> 16) & 0xFF); \
+  (__buffer)[3]= (uint8_t)(((__int) >> 24) & 0xFF); \
+  (__buffer)[4]= (uint8_t)(((__int) >> 32) & 0xFF); \
+  (__buffer)[5]= (uint8_t)(((__int) >> 40) & 0xFF); \
+  (__buffer)[6]= (uint8_t)(((__int) >> 48) & 0xFF); \
+  (__buffer)[7]= (uint8_t)(((__int) >> 56) & 0xFF); } while (0)
+
+/* Multi-byte character macros. */
+#define drizzle_mb_char(__c) (((__c) & 0x80) != 0)
+#define drizzle_mb_length(__c) \
+  ((uint32_t)(__c) <= 0x7f ? 1 : \
+  ((uint32_t)(__c) <= 0x7ff ? 2 : \
+  ((uint32_t)(__c) <= 0xd7ff ? 3 : \
+  ((uint32_t)(__c) <= 0xdfff || (uint32_t)(__c) > 0x10ffff ? 0 : \
+  ((uint32_t)(__c) <= 0xffff ? 3 : 4)))))
+
+/** @} */
+
+#ifdef __cplusplus
+}
+#endif
+
+#endif /* __DRIZZLE_CONSTANTS_H */

=== added file 'libdrizzle/drizzle.c'
--- libdrizzle/drizzle.c	1970-01-01 00:00:00 +0000
+++ libdrizzle/drizzle.c	2010-08-12 20:27:32 +0000
@@ -0,0 +1,690 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Drizzle Definitions
+ */
+
+#include "libdrizzle/common.h"
+
+
+/**
+ * @addtogroup drizzle_static Static Drizzle Declarations
+ * @ingroup drizzle
+ * @{
+ */
+
+/**
+ * Names of the verbose levels provided.
+ */
+static const char *_verbose_name[DRIZZLE_VERBOSE_MAX]=
+{
+  "NEVER",
+  "FATAL",
+  "ERROR",
+  "INFO",
+  "DEBUG",
+  "CRAZY"
+};
+
+/** @} */
+
+/*
+ * Common Definitions
+ */
+
+const char *drizzle_version(void)
+{
+  return PACKAGE_VERSION;
+}
+
+const char *drizzle_bugreport(void)
+{
+  return PACKAGE_BUGREPORT;
+}
+
+const char *drizzle_verbose_name(drizzle_verbose_t verbose)
+{
+  if (verbose >= DRIZZLE_VERBOSE_MAX)
+    return "UNKNOWN";
+
+  return _verbose_name[verbose];
+}
+
+drizzle_st *drizzle_create(drizzle_st *drizzle)
+{
+#if defined(_WIN32)
+  /* if it is MS windows, invoke WSAStartup */
+  WSADATA wsaData;
+  if ( WSAStartup( MAKEWORD(2,2), &wsaData ) != 0 )
+    printf("Error at WSAStartup()\n");
+#endif
+  if (drizzle == NULL)
+  {
+    drizzle= malloc(sizeof(drizzle_st));
+    if (drizzle == NULL)
+      return NULL;
+
+    drizzle->options= DRIZZLE_ALLOCATED;
+  }
+  else
+    drizzle->options= DRIZZLE_NONE;
+
+  /* @todo remove this default free flag with new API. */
+  drizzle->options|= DRIZZLE_FREE_OBJECTS;
+  drizzle->error_code= 0;
+  /* drizzle->options set above */
+  drizzle->verbose= DRIZZLE_VERBOSE_NEVER;
+  drizzle->con_count= 0;
+  drizzle->pfds_size= 0;
+  drizzle->query_count= 0;
+  drizzle->query_new= 0;
+  drizzle->query_running= 0;
+  drizzle->last_errno= 0;
+  drizzle->timeout= -1;
+  drizzle->con_list= NULL;
+  drizzle->context= NULL;
+  drizzle->context_free_fn= NULL;
+  drizzle->event_watch_fn= NULL;
+  drizzle->event_watch_context= NULL;
+  drizzle->log_fn= NULL;
+  drizzle->log_context= NULL;
+  drizzle->pfds= NULL;
+  drizzle->query_list= NULL;
+  drizzle->sqlstate[0]= 0;
+  drizzle->last_error[0]= 0;
+
+  return drizzle;
+}
+
+drizzle_st *drizzle_clone(drizzle_st *drizzle, const drizzle_st *from)
+{
+  drizzle_con_st *con;
+
+  drizzle= drizzle_create(drizzle);
+  if (drizzle == NULL)
+    return NULL;
+
+  drizzle->options|= (from->options & (drizzle_options_t)~DRIZZLE_ALLOCATED);
+
+  for (con= from->con_list; con != NULL; con= con->next)
+  {
+    if (drizzle_con_clone(drizzle, NULL, con) == NULL)
+    {
+      drizzle_free(drizzle);
+      return NULL;
+    }
+  }
+
+  return drizzle;
+}
+
+void drizzle_free(drizzle_st *drizzle)
+{
+  if (drizzle->context != NULL && drizzle->context_free_fn != NULL)
+    drizzle->context_free_fn(drizzle, drizzle->context);
+
+  if (drizzle->options & DRIZZLE_FREE_OBJECTS)
+  {
+    drizzle_con_free_all(drizzle);
+    drizzle_query_free_all(drizzle);
+  }
+  else if (drizzle->options & DRIZZLE_ASSERT_DANGLING)
+  {
+    assert(drizzle->con_list == NULL);
+    assert(drizzle->con_list == NULL);
+  }
+
+  if (drizzle->pfds != NULL)
+    free(drizzle->pfds);
+
+  if (drizzle->options & DRIZZLE_ALLOCATED)
+    free(drizzle);
+#if defined(_WIN32)
+  /* if it is MS windows, invoke WSACleanup() at the end*/
+  WSACleanup();
+#endif
+}
+
+const char *drizzle_error(const drizzle_st *drizzle)
+{
+  return (const char *)drizzle->last_error;
+}
+
+int drizzle_errno(const drizzle_st *drizzle)
+{
+  return drizzle->last_errno;
+}
+
+uint16_t drizzle_error_code(const drizzle_st *drizzle)
+{
+  return drizzle->error_code;
+}
+
+const char *drizzle_sqlstate(const drizzle_st *drizzle)
+{
+  return drizzle->sqlstate;
+}
+
+drizzle_options_t drizzle_options(const drizzle_st *drizzle)
+{
+  return drizzle->options;
+}
+
+void drizzle_set_options(drizzle_st *drizzle, drizzle_options_t options)
+{
+  drizzle->options= options;
+}
+
+void drizzle_add_options(drizzle_st *drizzle, drizzle_options_t options)
+{
+  drizzle->options|= options;
+}
+
+void drizzle_remove_options(drizzle_st *drizzle, drizzle_options_t options)
+{
+  drizzle->options&= ~options;
+}
+
+void *drizzle_context(const drizzle_st *drizzle)
+{
+  return drizzle->context;
+}
+
+void drizzle_set_context(drizzle_st *drizzle, void *context)
+{
+  drizzle->context= context;
+}
+
+void drizzle_set_context_free_fn(drizzle_st *drizzle,
+                                 drizzle_context_free_fn *function)
+{
+  drizzle->context_free_fn= function;
+}
+
+int drizzle_timeout(const drizzle_st *drizzle)
+{
+  return drizzle->timeout;
+}
+
+void drizzle_set_timeout(drizzle_st *drizzle, int timeout)
+{
+  drizzle->timeout= timeout;
+}
+
+drizzle_verbose_t drizzle_verbose(const drizzle_st *drizzle)
+{
+  return drizzle->verbose;
+}
+
+void drizzle_set_verbose(drizzle_st *drizzle, drizzle_verbose_t verbose)
+{
+  drizzle->verbose= verbose;
+}
+
+void drizzle_set_log_fn(drizzle_st *drizzle, drizzle_log_fn *function,
+                        void *context)
+{
+  drizzle->log_fn= function;
+  drizzle->log_context= context;
+}
+
+void drizzle_set_event_watch_fn(drizzle_st *drizzle,
+                                drizzle_event_watch_fn *function,
+                                void *context)
+{
+  drizzle->event_watch_fn= function;
+  drizzle->event_watch_context= context;
+}
+
+drizzle_con_st *drizzle_con_create(drizzle_st *drizzle, drizzle_con_st *con)
+{
+  if (con == NULL)
+  {
+    con= malloc(sizeof(drizzle_con_st));
+    if (con == NULL)
+    {
+      if (drizzle != NULL)
+        drizzle_set_error(drizzle, "drizzle_con_create", "malloc");
+      return NULL;
+    }
+
+    con->options= DRIZZLE_CON_ALLOCATED;
+  }
+  else
+    con->options= 0;
+
+  if (drizzle->con_list != NULL)
+    drizzle->con_list->prev= con;
+  con->next= drizzle->con_list;
+  con->prev= NULL;
+  drizzle->con_list= con;
+  drizzle->con_count++;
+
+  con->packet_number= 0;
+  con->protocol_version= 0;
+  con->state_current= 0;
+  con->events= 0;
+  con->revents= 0;
+  con->capabilities= DRIZZLE_CAPABILITIES_NONE;
+  con->charset= 0;
+  con->command= 0;
+  con->options|= DRIZZLE_CON_MYSQL;
+  con->socket_type= DRIZZLE_CON_SOCKET_TCP;
+  con->status= DRIZZLE_CON_STATUS_NONE;
+  con->max_packet_size= DRIZZLE_MAX_PACKET_SIZE;
+  con->result_count= 0;
+  con->thread_id= 0;
+  con->backlog= DRIZZLE_DEFAULT_BACKLOG;
+  con->fd= -1;
+  con->buffer_size= 0;
+  con->command_offset= 0;
+  con->command_size= 0;
+  con->command_total= 0;
+  con->packet_size= 0;
+  con->addrinfo_next= NULL;
+  con->buffer_ptr= con->buffer;
+  con->command_buffer= NULL;
+  con->command_data= NULL;
+  con->context= NULL;
+  con->context_free_fn= NULL;
+  con->drizzle= drizzle;
+  /* con->next set above */
+  /* con->prev set above */
+  con->query= NULL;
+  /* con->result doesn't need to be set */
+  con->result_list= NULL;
+  con->scramble= NULL;
+  con->socket.tcp.addrinfo= NULL;
+  con->socket.tcp.host= NULL;
+  con->socket.tcp.port= 0;
+  /* con->buffer doesn't need to be set */
+  con->db[0]= 0;
+  con->password[0]= 0;
+  /* con->scramble_buffer doesn't need to be set */
+  con->server_version[0]= 0;
+  /* con->state_stack doesn't need to be set */
+  con->user[0]= 0;
+
+  return con;
+}
+
+drizzle_con_st *drizzle_con_clone(drizzle_st *drizzle, drizzle_con_st *con,
+                                  const drizzle_con_st *from)
+{
+  con= drizzle_con_create(drizzle, con);
+  if (con == NULL)
+    return NULL;
+
+  /* Clear "operational" options such as IO status. */
+  con->options|= (from->options & (drizzle_con_options_t)~(
+                  DRIZZLE_CON_ALLOCATED|DRIZZLE_CON_READY|
+                  DRIZZLE_CON_NO_RESULT_READ|DRIZZLE_CON_IO_READY|
+                  DRIZZLE_CON_LISTEN));
+  con->backlog= from->backlog;
+  strcpy(con->db, from->db);
+  strcpy(con->password, from->password);
+  strcpy(con->user, from->user);
+
+  switch (from->socket_type)
+  {
+  case DRIZZLE_CON_SOCKET_TCP:
+    drizzle_con_set_tcp(con, from->socket.tcp.host, from->socket.tcp.port);
+    break;
+
+  case DRIZZLE_CON_SOCKET_UDS:
+    drizzle_con_set_uds(con, from->socket.uds.sockaddr.sun_path);
+    break;
+
+  default:
+    break;
+  }
+
+  return con;
+}
+
+void drizzle_con_free(drizzle_con_st *con)
+{
+  if (con->context != NULL && con->context_free_fn != NULL)
+    con->context_free_fn(con, con->context);
+
+  if (con->drizzle->options & DRIZZLE_FREE_OBJECTS)
+    drizzle_result_free_all(con);
+  else if (con->drizzle->options & DRIZZLE_ASSERT_DANGLING)
+    assert(con->result_list == NULL);
+
+  if (con->fd != -1)
+    drizzle_con_close(con);
+
+  drizzle_con_reset_addrinfo(con);
+
+  if (con->drizzle->con_list == con)
+    con->drizzle->con_list= con->next;
+  if (con->prev != NULL)
+    con->prev->next= con->next;
+  if (con->next != NULL)
+    con->next->prev= con->prev;
+  con->drizzle->con_count--;
+
+  if (con->options & DRIZZLE_CON_ALLOCATED)
+    free(con);
+}
+
+void drizzle_con_free_all(drizzle_st *drizzle)
+{
+  while (drizzle->con_list != NULL)
+    drizzle_con_free(drizzle->con_list);
+}
+
+drizzle_return_t drizzle_con_wait(drizzle_st *drizzle)
+{
+  drizzle_con_st *con;
+  struct pollfd *pfds;
+  uint32_t x;
+  int ret;
+  drizzle_return_t dret;
+
+  if (drizzle->pfds_size < drizzle->con_count)
+  {
+    pfds= realloc(drizzle->pfds, drizzle->con_count * sizeof(struct pollfd));
+    if (pfds == NULL)
+    {
+      drizzle_set_error(drizzle, "drizzle_con_wait", "realloc");
+      return DRIZZLE_RETURN_MEMORY;
+    }
+
+    drizzle->pfds= pfds;
+    drizzle->pfds_size= drizzle->con_count;
+  }
+  else
+    pfds= drizzle->pfds;
+
+  x= 0;
+  for (con= drizzle->con_list; con != NULL; con= con->next)
+  {
+    if (con->events == 0)
+      continue;
+
+    pfds[x].fd= con->fd;
+    pfds[x].events= con->events;
+    pfds[x].revents= 0;
+    x++;
+  }
+
+  if (x == 0)
+  {
+    drizzle_set_error(drizzle, "drizzle_con_wait",
+                      "no active file descriptors");
+    return DRIZZLE_RETURN_NO_ACTIVE_CONNECTIONS;
+  }
+
+  while (1)
+  {
+    drizzle_log_crazy(drizzle, "poll count=%d timeout=%d", x,
+                      drizzle->timeout);
+
+    ret= poll(pfds, x, drizzle->timeout);
+
+    drizzle_log_crazy(drizzle, "poll return=%d errno=%d", ret, errno);
+
+    if (ret == -1)
+    {
+      if (errno == EINTR)
+        continue;
+
+      drizzle_set_error(drizzle, "drizzle_con_wait", "poll:%d", errno);
+      drizzle->last_errno= errno;
+      return DRIZZLE_RETURN_ERRNO;
+    }
+
+    break;
+  }
+
+  if (ret == 0)
+  {
+    drizzle_set_error(drizzle, "drizzle_con_wait", "timeout reached");
+    return DRIZZLE_RETURN_TIMEOUT;
+  }
+
+  x= 0;
+  for (con= drizzle->con_list; con != NULL; con= con->next)
+  {
+    if (con->events == 0)
+      continue;
+
+    dret= drizzle_con_set_revents(con, pfds[x].revents);
+    if (dret != DRIZZLE_RETURN_OK)
+      return dret;
+
+    x++;
+  }
+
+  return DRIZZLE_RETURN_OK;
+}
+
+drizzle_con_st *drizzle_con_ready(drizzle_st *drizzle)
+{
+  drizzle_con_st *con;
+
+  /* We can't keep state between calls since connections may be removed during
+     processing. If this list ever gets big, we may want something faster. */
+
+  for (con= drizzle->con_list; con != NULL; con= con->next)
+  {
+    if (con->options & DRIZZLE_CON_IO_READY)
+    {
+      con->options&= (drizzle_con_options_t)~DRIZZLE_CON_IO_READY;
+      return con;
+    }
+  }
+
+  return NULL;
+}
+
+drizzle_con_st *drizzle_con_ready_listen(drizzle_st *drizzle)
+{
+  drizzle_con_st *con;
+
+  /* We can't keep state between calls since connections may be removed during
+     processing. If this list ever gets big, we may want something faster. */
+
+  for (con= drizzle->con_list; con != NULL; con= con->next)
+  {
+    if ((con->options & (DRIZZLE_CON_IO_READY | DRIZZLE_CON_LISTEN)) ==
+        (DRIZZLE_CON_IO_READY | DRIZZLE_CON_LISTEN))
+    {
+      con->options&= (drizzle_con_options_t)~DRIZZLE_CON_IO_READY;
+      return con;
+    }
+  }
+
+  return NULL;
+}
+
+/*
+ * Client Definitions
+ */
+
+drizzle_con_st *drizzle_con_add_tcp(drizzle_st *drizzle, drizzle_con_st *con,
+                                    const char *host, in_port_t port,
+                                    const char *user, const char *password,
+                                    const char *db,
+                                    drizzle_con_options_t options)
+{
+  con= drizzle_con_create(drizzle, con);
+  if (con == NULL)
+    return NULL;
+
+  drizzle_con_set_tcp(con, host, port);
+  drizzle_con_set_auth(con, user, password);
+  drizzle_con_set_db(con, db);
+  drizzle_con_add_options(con, options);
+
+  return con;
+}
+
+drizzle_con_st *drizzle_con_add_uds(drizzle_st *drizzle, drizzle_con_st *con,
+                                    const char *uds, const char *user,
+                                    const char *password, const char *db,
+                                    drizzle_con_options_t options)
+{
+  con= drizzle_con_create(drizzle, con);
+  if (con == NULL)
+    return NULL;
+
+  drizzle_con_set_uds(con, uds);
+  drizzle_con_set_auth(con, user, password);
+  drizzle_con_set_db(con, db);
+  drizzle_con_add_options(con, options);
+
+  return con;
+}
+
+/*
+ * Server Definitions
+ */
+
+drizzle_con_st *drizzle_con_add_tcp_listen(drizzle_st *drizzle,
+                                           drizzle_con_st *con,
+                                           const char *host, in_port_t port,
+                                           int backlog,
+                                           drizzle_con_options_t options)
+{
+  con= drizzle_con_create(drizzle, con);
+  if (con == NULL)
+    return NULL;
+
+  drizzle_con_set_tcp(con, host, port);
+  drizzle_con_set_backlog(con, backlog);
+  drizzle_con_add_options(con, DRIZZLE_CON_LISTEN | options);
+
+  return con;
+}
+
+drizzle_con_st *drizzle_con_add_uds_listen(drizzle_st *drizzle,
+                                           drizzle_con_st *con,
+                                           const char *uds, int backlog,
+                                           drizzle_con_options_t options)
+{
+  con= drizzle_con_create(drizzle, con);
+  if (con == NULL)
+    return NULL;
+
+  drizzle_con_set_uds(con, uds);
+  drizzle_con_set_backlog(con, backlog);
+  drizzle_con_add_options(con, DRIZZLE_CON_LISTEN | options);
+
+  return con;
+}
+
+drizzle_con_st *drizzle_con_accept(drizzle_st *drizzle, drizzle_con_st *con,
+                                   drizzle_return_t *ret_ptr)
+{
+  drizzle_con_st *ready;
+  int fd;
+
+  while (1)
+  {
+    if ((ready= drizzle_con_ready_listen(drizzle)) != NULL)
+    {
+      fd= accept(ready->fd, NULL, NULL);
+
+      con= drizzle_con_create(drizzle, con);
+      if (con == NULL)
+      {
+        (void)close(fd);
+        *ret_ptr= DRIZZLE_RETURN_MEMORY;
+        return NULL;
+      }
+
+      *ret_ptr= drizzle_con_set_fd(con, fd);
+      if (*ret_ptr != DRIZZLE_RETURN_OK)
+      {
+        (void)close(fd);
+        return NULL;
+      }
+
+      if (ready->options & DRIZZLE_CON_MYSQL)
+        drizzle_con_add_options(con, DRIZZLE_CON_MYSQL);
+
+      *ret_ptr= DRIZZLE_RETURN_OK;
+      return con;
+    }
+
+    if (drizzle->options & DRIZZLE_NON_BLOCKING)
+    {
+      *ret_ptr= DRIZZLE_RETURN_IO_WAIT;
+      return NULL;
+    }
+
+    for (ready= drizzle->con_list; ready != NULL; ready= ready->next)
+    {
+      if (ready->options & DRIZZLE_CON_LISTEN)
+        drizzle_con_set_events(ready, POLLIN);
+    }
+
+    *ret_ptr= drizzle_con_wait(drizzle);
+    if (*ret_ptr != DRIZZLE_RETURN_OK)
+      return NULL;
+  }
+}
+
+/*
+ * Local Definitions
+ */
+
+void drizzle_set_error(drizzle_st *drizzle, const char *function,
+                       const char *format, ...)
+{
+  size_t size;
+  char *ptr;
+  char log_buffer[DRIZZLE_MAX_ERROR_SIZE];
+  va_list args;
+
+  size= strlen(function);
+  ptr= memcpy(log_buffer, function, size);
+  ptr+= size;
+  ptr[0]= ':';
+  size++;
+  ptr++;
+
+  va_start(args, format);
+  size+= (size_t)vsnprintf(ptr, DRIZZLE_MAX_ERROR_SIZE - size, format, args);
+  va_end(args);
+
+  if (drizzle->log_fn == NULL)
+  {
+    if (size >= DRIZZLE_MAX_ERROR_SIZE)
+      size= DRIZZLE_MAX_ERROR_SIZE - 1;
+
+    memcpy(drizzle->last_error, log_buffer, size + 1);
+  }
+  else
+    drizzle->log_fn(log_buffer, DRIZZLE_VERBOSE_ERROR, drizzle->log_context);
+}
+
+void drizzle_log(drizzle_st *drizzle, drizzle_verbose_t verbose,
+                 const char *format, va_list args)
+{
+  char log_buffer[DRIZZLE_MAX_ERROR_SIZE];
+
+  if (drizzle->log_fn == NULL)
+  {
+    printf("%5s: ", drizzle_verbose_name(verbose));
+    vprintf(format, args);
+    printf("\n");
+  }
+  else
+  {
+    vsnprintf(log_buffer, DRIZZLE_MAX_ERROR_SIZE, format, args);
+    drizzle->log_fn(log_buffer, verbose, drizzle->log_context);
+  }
+}

=== added file 'libdrizzle/drizzle.h'
--- libdrizzle/drizzle.h	1970-01-01 00:00:00 +0000
+++ libdrizzle/drizzle.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,403 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Drizzle Declarations
+ */
+
+#ifndef __DRIZZLE_H
+#define __DRIZZLE_H
+
+#if !defined(__cplusplus)
+# include <stdbool.h>
+#endif
+
+#include <inttypes.h>
+#include <sys/types.h>
+
+#ifdef _WIN32
+# include <winsock2.h>
+# include <ws2tcpip.h>
+
+# include <errno.h>
+# define EINPROGRESS WSAEINPROGRESS
+# define EALREADY WSAEALREADY
+# define EISCONN WSAEISCONN
+# define ENOBUFS WSAENOBUFS
+# define ECONNREFUSED WSAECONNREFUSED
+# define ENETUNREACH WSAENETUNREACH
+# define ETIMEDOUT WSAETIMEDOUT
+# define ECONNRESET WSAECONNRESET 
+# define EADDRINUSE WSAEADDRINUSE
+# define EOPNOTSUPP WSAEOPNOTSUPP
+# define ENOPROTOOPT WSAENOPROTOOPT
+
+typedef unsigned int in_port_t;
+
+struct sockaddr_un
+{
+  short int sun_family;
+  char sun_path[108];
+};
+
+
+#else
+# include <sys/socket.h>
+# include <netinet/in.h>
+# include <arpa/inet.h>
+# include <sys/un.h>
+# include <netdb.h>
+#endif
+
+#include <poll.h>
+
+#include <libdrizzle/visibility.h>
+#include <libdrizzle/constants.h>
+#include <libdrizzle/structs.h>
+#include <libdrizzle/conn.h>
+#include <libdrizzle/result.h>
+#include <libdrizzle/column.h>
+
+#ifdef  __cplusplus
+extern "C" {
+#endif
+
+/**
+ * @addtogroup drizzle Drizzle Declarations
+ * @ingroup drizzle_client_interface
+ * @ingroup drizzle_server_interface
+ *
+ * This is the core library structure that other structures (such as
+ * connections) are created from.
+ *
+ * There is no locking within a single drizzle_st structure, so for threaded
+ * applications you must either ensure isolation in the application or use
+ * multiple drizzle_st structures (for example, one for each thread).
+ * @{
+ */
+
+/**
+ * Get library version string.
+ *
+ * @return Pointer to static buffer in library that holds the version string.
+ */
+DRIZZLE_API
+const char *drizzle_version(void);
+
+/**
+ * Get bug report URL.
+ *
+ * @return Bug report URL string.
+ */
+DRIZZLE_API
+const char *drizzle_bugreport(void);
+
+/**
+ * Get string with the name of the given verbose level.
+ *
+ * @param[in] verbose Verbose logging level.
+ * @return String form of verbose level.
+ */
+DRIZZLE_API
+const char *drizzle_verbose_name(drizzle_verbose_t verbose);
+
+/**
+ * Initialize a drizzle structure. Always check the return value even if passing
+ * in a pre-allocated structure. Some other initialization may have failed.
+ *
+ * @param[in] drizzle Caller allocated structure, or NULL to allocate one.
+ * @return On success, a pointer to the (possibly allocated) structure. On
+ * failure this will be NULL.
+ */
+DRIZZLE_API
+drizzle_st *drizzle_create(drizzle_st *drizzle);
+
+/**
+ * Clone a drizzle structure.
+ *
+ * @param[in] drizzle Caller allocated structure, or NULL to allocate one.
+ * @param[in] from Drizzle structure to use as a source to clone from.
+ * @return Same return as drizzle_create().
+ */
+DRIZZLE_API
+drizzle_st *drizzle_clone(drizzle_st *drizzle, const drizzle_st *from);
+
+/**
+ * Free a drizzle structure.
+ *
+ * @param[in] drizzle Drizzle structure previously initialized with
+ *  drizzle_create() or drizzle_clone().
+ */
+DRIZZLE_API
+void drizzle_free(drizzle_st *drizzle);
+
+/**
+ * Return an error string for last error encountered.
+ *
+ * @param[in] drizzle Drizzle structure previously initialized with
+ *  drizzle_create() or drizzle_clone().
+ * @return Pointer to static buffer in library that holds an error string.
+ */
+DRIZZLE_API
+const char *drizzle_error(const drizzle_st *drizzle);
+
+/**
+ * Value of errno in the case of a DRIZZLE_RETURN_ERRNO return value.
+ *
+ * @param[in] drizzle Drizzle structure previously initialized with
+ *  drizzle_create() or drizzle_clone().
+ * @return An errno value as defined in your system errno.h file.
+ */
+DRIZZLE_API
+int drizzle_errno(const drizzle_st *drizzle);
+
+/**
+ * Get server defined error code for the last result read.
+ *
+ * @param[in] drizzle Drizzle structure previously initialized with
+ *  drizzle_create() or drizzle_clone().
+ * @return An error code given back in the server response.
+ */
+DRIZZLE_API
+uint16_t drizzle_error_code(const drizzle_st *drizzle);
+
+/**
+ * Get SQL state code for the last result read.
+ *
+ * @param[in] drizzle Drizzle structure previously initialized with
+ *  drizzle_create() or drizzle_clone().
+ * @return A SQLSTATE code given back in the server response.
+ */
+DRIZZLE_API
+const char *drizzle_sqlstate(const drizzle_st *drizzle);
+
+/**
+ * Get options for a drizzle structure.
+ *
+ * @param[in] drizzle Drizzle structure previously initialized with
+ *  drizzle_create() or drizzle_clone().
+ * @return Options set for the drizzle structure.
+ */
+DRIZZLE_API
+drizzle_options_t drizzle_options(const drizzle_st *drizzle);
+
+/**
+ * Set options for a drizzle structure.
+ *
+ * @param[in] drizzle Drizzle structure previously initialized with
+ *  drizzle_create() or drizzle_clone().
+ * @param[in] options Available options for drizzle structure to set.
+ */
+DRIZZLE_API
+void drizzle_set_options(drizzle_st *drizzle, drizzle_options_t options);
+
+/**
+ * Add options for a drizzle structure.
+ *
+ * @param[in] drizzle Drizzle structure previously initialized with
+ *  drizzle_create() or drizzle_clone().
+ * @param[in] options Available options for drizzle structure to add.
+ */
+DRIZZLE_API
+void drizzle_add_options(drizzle_st *drizzle, drizzle_options_t options);
+
+/**
+ * Remove options for a drizzle structure.
+ *
+ * @param[in] drizzle Drizzle structure previously initialized with
+ *  drizzle_create() or drizzle_clone().
+ * @param[in] options Available options for drizzle structure to remove.
+ */
+DRIZZLE_API
+void drizzle_remove_options(drizzle_st *drizzle, drizzle_options_t options);
+
+/**
+ * Get application context pointer.
+ *
+ * @param[in] drizzle Drizzle structure previously initialized with
+ *  drizzle_create() or drizzle_clone().
+ * @return Application context that was previously set, or NULL.
+ */
+DRIZZLE_API
+void *drizzle_context(const drizzle_st *drizzle);
+
+/**
+ * Set application context pointer.
+ *
+ * @param[in] drizzle Drizzle structure previously initialized with
+ *  drizzle_create() or drizzle_clone().
+ * @param[in] context Application context to set.
+ */
+DRIZZLE_API
+void drizzle_set_context(drizzle_st *drizzle, void *context);
+
+/**
+ * Set function to call when the drizzle structure is being cleaned up so
+ * the application can clean up the context pointer.
+ *
+ * @param[in] drizzle Drizzle structure previously initialized with
+ *  drizzle_create() or drizzle_clone().
+ * @param[in] function Function to call to clean up drizzle context.
+ */
+DRIZZLE_API
+void drizzle_set_context_free_fn(drizzle_st *drizzle,
+                                 drizzle_context_free_fn *function);
+
+/**
+ * Get current socket I/O activity timeout value.
+ *
+ * @param[in] drizzle Drizzle structure previously initialized with
+ *  drizzle_create() or drizzle_clone().
+ * @return Timeout in milliseconds to wait for I/O activity. A negative value
+ *  means an infinite timeout.
+ */
+DRIZZLE_API
+int drizzle_timeout(const drizzle_st *drizzle);
+
+/**
+ * Set socket I/O activity timeout for connections in a Drizzle structure.
+ *
+ * @param[in] drizzle Drizzle structure previously initialized with
+ *  drizzle_create() or drizzle_clone().
+ * @param[in] timeout Milliseconds to wait for I/O activity. A negative value
+ *  means an infinite timeout.
+ */
+DRIZZLE_API
+void drizzle_set_timeout(drizzle_st *drizzle, int timeout);
+
+/**
+ * Get current verbosity threshold for logging messages.
+ *
+ * @param[in] drizzle Drizzle structure previously initialized with
+ *  drizzle_create() or drizzle_clone().
+ * @return Current verbosity threshold.
+ */
+DRIZZLE_API
+drizzle_verbose_t drizzle_verbose(const drizzle_st *drizzle);
+
+/**
+ * Set verbosity threshold for logging messages. If this is set above
+ * DRIZZLE_VERBOSE_NEVER and the drizzle_set_log_fn() callback is set to NULL,
+ * messages are printed to STDOUT.
+ *
+ * @param[in] drizzle Drizzle structure previously initialized with
+ *  drizzle_create() or drizzle_clone().
+ * @param[in] verbose Verbosity threshold of what to log.
+ */
+DRIZZLE_API
+void drizzle_set_verbose(drizzle_st *drizzle, drizzle_verbose_t verbose);
+
+/**
+ * Set logging function for a drizzle structure. This function is only called
+ * for log messages that are above the verbosity threshold set with
+ * drizzle_set_verbose().
+ *
+ * @param[in] drizzle Drizzle structure previously initialized with
+ *  drizzle_create() or drizzle_clone().
+ * @param[in] function Function to call when there is a logging message.
+ * @param[in] context Argument to pass into the callback function.
+ */
+DRIZZLE_API
+void drizzle_set_log_fn(drizzle_st *drizzle, drizzle_log_fn *function,
+                        void *context);
+
+/**
+ * Set a custom I/O event watcher function for a drizzle structure. Used to
+ * integrate libdrizzle with a custom event loop. The callback will be invoked
+ * to register or deregister interest in events for a connection. When the
+ * events are triggered, drizzle_con_set_revents() should be called to
+ * indicate which events are ready. The event loop should stop waiting for
+ * these events, as libdrizzle will call the callback again if it is still
+ * interested. To resume processing, the libdrizzle function that returned
+ * DRIZZLE_RETURN_IO_WAIT should be called again. See drizzle_event_watch_fn().
+ *
+ * @param[in] drizzle Drizzle structure previously initialized with
+ *  drizzle_create() or drizzle_clone().
+ * @param[in] function Function to call when there is an I/O event.
+ * @param[in] context Argument to pass into the callback function.
+ */
+DRIZZLE_API
+void drizzle_set_event_watch_fn(drizzle_st *drizzle,
+                                drizzle_event_watch_fn *function,
+                                void *context);
+
+/**
+ * Initialize a connection structure. Always check the return value even if
+ * passing in a pre-allocated structure. Some other initialization may have
+ * failed.
+ *
+ * @param[in] drizzle Drizzle structure previously initialized with
+ *  drizzle_create() or drizzle_clone().
+ * @param[in] con Caller allocated structure, or NULL to allocate one.
+ * @return On success, a pointer to the (possibly allocated) structure. On
+ *  failure this will be NULL.
+ */
+DRIZZLE_API
+drizzle_con_st *drizzle_con_create(drizzle_st *drizzle, drizzle_con_st *con);
+
+/**
+ * Clone a connection structure.
+ *
+ * @param[in] drizzle Drizzle structure previously initialized with
+ *  drizzle_create() or drizzle_clone().
+ * @param[in] con Caller allocated structure, or NULL to allocate one.
+ * @param[in] from Connection structure to use as a source to clone from.
+ * @return Same return as drizzle_con_create().
+ */
+DRIZZLE_API
+drizzle_con_st *drizzle_con_clone(drizzle_st *drizzle, drizzle_con_st *con,
+                                  const drizzle_con_st *from);
+
+/**
+ * Free a connection structure.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ */
+DRIZZLE_API
+void drizzle_con_free(drizzle_con_st *con);
+
+/**
+ * Free all connections in a drizzle structure.
+ *
+ * @param[in] drizzle Drizzle structure previously initialized with
+ *  drizzle_create() or drizzle_clone().
+ */
+DRIZZLE_API
+void drizzle_con_free_all(drizzle_st *drizzle);
+
+/**
+ * Wait for I/O on connections.
+ *
+ * @param[in] drizzle Drizzle structure previously initialized with
+ *  drizzle_create() or drizzle_clone().
+ * @return Standard drizzle return value.
+ */
+DRIZZLE_API
+drizzle_return_t drizzle_con_wait(drizzle_st *drizzle);
+
+/**
+ * Get next connection that is ready for I/O.
+ *
+ * @param[in] drizzle Drizzle structure previously initialized with
+ *  drizzle_create() or drizzle_clone().
+ * @return Connection that is ready for I/O, or NULL if there are none.
+ */
+DRIZZLE_API
+drizzle_con_st *drizzle_con_ready(drizzle_st *drizzle);
+
+/** @} */
+
+#ifdef  __cplusplus
+}
+#endif
+
+#endif /* __DRIZZLE_H */

=== added file 'libdrizzle/drizzle_client.h'
--- libdrizzle/drizzle_client.h	1970-01-01 00:00:00 +0000
+++ libdrizzle/drizzle_client.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,90 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Drizzle Declarations for Clients
+ */
+
+#ifndef __DRIZZLE_CLIENT_H
+#define __DRIZZLE_CLIENT_H
+
+#include <libdrizzle/drizzle.h>
+#include <libdrizzle/conn_client.h>
+#include <libdrizzle/handshake_client.h>
+#include <libdrizzle/command_client.h>
+#include <libdrizzle/query.h>
+#include <libdrizzle/result_client.h>
+#include <libdrizzle/column_client.h>
+#include <libdrizzle/row_client.h>
+#include <libdrizzle/field_client.h>
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/**
+ * @defgroup drizzle_client_interface Drizzle Client Interface
+ */
+
+/**
+ * @addtogroup drizzle_client Drizzle Declarations for Clients
+ * @ingroup drizzle_client_interface
+ * @{
+ */
+
+/**
+ * Add TCP (IPv4 or IPv6) connection with common arguments.
+ *
+ * @param[in] drizzle Drizzle structure previously initialized with
+ *  drizzle_create() or drizzle_clone().
+ * @param[in] con Caller allocated structure, or NULL to allocate one.
+ * @param[in] host Host to connect to. This may be a hostname to resolve, an
+ *  IPv4 address, or an IPv6 address. This is passed directly to getaddrinfo().
+ * @param[in] port Remote port to connect to.
+ * @param[in] user User to use while establishing the connection.
+ * @param[in] password Password to use while establishing the connection.
+ * @param[in] db Initial database to connect to.
+ * @param[in] options Drizzle connection options to add.
+ * @return Same return as drizzle_con_create().
+ */
+DRIZZLE_API
+drizzle_con_st *drizzle_con_add_tcp(drizzle_st *drizzle, drizzle_con_st *con,
+                                    const char *host, in_port_t port,
+                                    const char *user, const char *password,
+                                    const char *db,
+                                    drizzle_con_options_t options);
+
+/**
+ * Add unix domain socket connection with common arguments.
+ *
+ * @param[in] drizzle Drizzle structure previously initialized with
+ *  drizzle_create() or drizzle_clone().
+ * @param[in] con Caller allocated structure, or NULL to allocate one.
+ * @param[in] uds Path to unix domain socket to use for connection.
+ * @param[in] user User to use while establishing the connection.
+ * @param[in] password Password to use while establishing the connection.
+ * @param[in] db Initial database to connect to.
+ * @param[in] options Drizzle connection options to add.
+ * @return Same return as drizzle_con_create().
+ */
+DRIZZLE_API
+drizzle_con_st *drizzle_con_add_uds(drizzle_st *drizzle, drizzle_con_st *con,
+                                    const char *uds, const char *user,
+                                    const char *password, const char *db,
+                                    drizzle_con_options_t options);
+
+/** @} */
+
+#ifdef  __cplusplus
+}
+#endif
+
+#endif /* __DRIZZLE_CLIENT_H */

=== added file 'libdrizzle/drizzle_local.h'
--- libdrizzle/drizzle_local.h	1970-01-01 00:00:00 +0000
+++ libdrizzle/drizzle_local.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,140 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Local Drizzle Declarations
+ */
+
+#ifndef __DRIZZLE_LOCAL_H
+#define __DRIZZLE_LOCAL_H
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/**
+ * @addtogroup drizzle_local Local Drizzle Declarations
+ * @ingroup drizzle
+ * @{
+ */
+
+/**
+ * Set the error string.
+ *
+ * @param[in] drizzle Drizzle structure previously initialized with
+ *  drizzle_create() or drizzle_clone().
+ * @param[in] function Name of function the error happened in. 
+ * @param[in] format Format and variable argument list of message.
+ */
+DRIZZLE_LOCAL
+void drizzle_set_error(drizzle_st *drizzle, const char *function,
+                       const char *format, ...);
+
+/**
+ * Log a message.
+ *
+ * @param[in] drizzle Drizzle structure previously initialized with
+ *  drizzle_create() or drizzle_clone().
+ * @param[in] verbose Logging level of the message.
+ * @param[in] format Format and variable argument list of message.
+ * @param[in] args Variable argument list that has been initialized.
+ */
+DRIZZLE_LOCAL
+void drizzle_log(drizzle_st *drizzle, drizzle_verbose_t verbose,
+                 const char *format, va_list args);
+
+/**
+ * Log a fatal message, see drizzle_log() for argument details.
+ */
+static inline void drizzle_log_fatal(drizzle_st *drizzle, const char *format,
+                                     ...)
+{
+  va_list args;
+
+  if (drizzle->verbose >= DRIZZLE_VERBOSE_FATAL)
+  {
+    va_start(args, format);
+    drizzle_log(drizzle, DRIZZLE_VERBOSE_FATAL, format, args);
+    va_end(args);
+  }
+}
+
+/**
+ * Log an error message, see drizzle_log() for argument details.
+ */
+static inline void drizzle_log_error(drizzle_st *drizzle, const char *format,
+                                     ...)
+{
+  va_list args;
+
+  if (drizzle->verbose >= DRIZZLE_VERBOSE_ERROR)
+  {
+    va_start(args, format);
+    drizzle_log(drizzle, DRIZZLE_VERBOSE_ERROR, format, args);
+    va_end(args);
+  }
+}
+
+/**
+ * Log an info message, see drizzle_log() for argument details.
+ */
+static inline void drizzle_log_info(drizzle_st *drizzle, const char *format,
+                                    ...)
+{
+  va_list args;
+
+  if (drizzle->verbose >= DRIZZLE_VERBOSE_INFO)
+  {
+    va_start(args, format);
+    drizzle_log(drizzle, DRIZZLE_VERBOSE_INFO, format, args);
+    va_end(args);
+  }
+}
+
+/**
+ * Log a debug message, see drizzle_log() for argument details.
+ */
+static inline void drizzle_log_debug(drizzle_st *drizzle, const char *format,
+                                     ...)
+{
+  va_list args;
+
+  if (drizzle->verbose >= DRIZZLE_VERBOSE_DEBUG)
+  {
+    va_start(args, format);
+    drizzle_log(drizzle, DRIZZLE_VERBOSE_DEBUG, format, args);
+    va_end(args);
+  }
+}
+
+/**
+ * Log a crazy message, see drizzle_log() for argument details.
+ */
+static inline void drizzle_log_crazy(drizzle_st *drizzle, const char *format,
+                                     ...)
+{
+  va_list args;
+
+  if (drizzle->verbose >= DRIZZLE_VERBOSE_CRAZY)
+  {
+    va_start(args, format);
+    drizzle_log(drizzle, DRIZZLE_VERBOSE_CRAZY, format, args);
+    va_end(args);
+  }
+}
+
+/** @} */
+
+#ifdef __cplusplus
+}
+#endif
+
+#endif /* __DRIZZLE_LOCAL_H */

=== added file 'libdrizzle/drizzle_server.h'
--- libdrizzle/drizzle_server.h	1970-01-01 00:00:00 +0000
+++ libdrizzle/drizzle_server.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,108 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Drizzle Declarations for Servers
+ */
+
+#ifndef __DRIZZLE_SERVER_H
+#define __DRIZZLE_SERVER_H
+
+#include <libdrizzle/drizzle.h>
+#include <libdrizzle/conn_server.h>
+#include <libdrizzle/handshake_server.h>
+#include <libdrizzle/command_server.h>
+#include <libdrizzle/result_server.h>
+#include <libdrizzle/column_server.h>
+#include <libdrizzle/row_server.h>
+#include <libdrizzle/field_server.h>
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/**
+ * @defgroup drizzle_server_interface Drizzle Server Interface
+ */
+
+/**
+ * @addtogroup drizzle_server Drizzle Declarations for Servers
+ * @ingroup drizzle_server_interface
+ * @{
+ */
+
+/**
+ * Add TCP (IPv4 or IPv6) connection for listening with common arguments.
+ *
+ * @param[in] drizzle Drizzle structure previously initialized with
+ *  drizzle_create() or drizzle_clone().
+ * @param[in] con Caller allocated structure, or NULL to allocate one.
+ * @param[in] host Host to listen on. This may be a hostname to resolve, an
+ *  IPv4 address, or an IPv6 address. This is passed directly to getaddrinfo().
+ * @param[in] port Port to connect to.
+ * @param[in] backlog Number of backlog connections passed to listen().
+ * @param[in] options Drizzle connection options to add.
+ * @return Same return as drizzle_con_create().
+ */
+DRIZZLE_API
+drizzle_con_st *drizzle_con_add_tcp_listen(drizzle_st *drizzle,
+                                           drizzle_con_st *con,
+                                           const char *host, in_port_t port,
+                                           int backlog,
+                                           drizzle_con_options_t options);
+
+/**
+ * Add unix domain socket connection for listening with common arguments.
+ *
+ * @param[in] drizzle Drizzle structure previously initialized with
+ *  drizzle_create() or drizzle_clone().
+ * @param[in] con Caller allocated structure, or NULL to allocate one.
+ * @param[in] uds Path to unix domain socket to use for listening.
+ * @param[in] backlog Number of backlog connections passed to listen().
+ * @param[in] options Drizzle connection options to add.
+ * @return Same return as drizzle_con_create().
+ */
+DRIZZLE_API
+drizzle_con_st *drizzle_con_add_uds_listen(drizzle_st *drizzle,
+                                           drizzle_con_st *con,
+                                           const char *uds, int backlog,
+                                           drizzle_con_options_t options);
+
+/**
+ * Get next connection marked for listening that is ready for I/O.
+ *
+ * @param[in] drizzle Drizzle structure previously initialized with
+ *  drizzle_create() or drizzle_clone().
+ * @return Connection that is ready to accept, or NULL if there are none.
+ */
+DRIZZLE_API
+drizzle_con_st *drizzle_con_ready_listen(drizzle_st *drizzle);
+
+/**
+ * Accept a new connection and initialize the connection structure for it.
+ *
+ * @param[in] drizzle Drizzle structure previously initialized with
+ *  drizzle_create() or drizzle_clone().
+ * @param[in] con Caller allocated structure, or NULL to allocate one.
+ * @param[out] ret_ptr Standard drizzle return value.
+ * @return Same return as drizzle_con_create().
+ */
+DRIZZLE_API
+drizzle_con_st *drizzle_con_accept(drizzle_st *drizzle, drizzle_con_st *con,
+                                   drizzle_return_t *ret_ptr);
+
+/** @} */
+
+#ifdef  __cplusplus
+}
+#endif
+
+#endif /* __DRIZZLE_SERVER_H */

=== added file 'libdrizzle/field.c'
--- libdrizzle/field.c	1970-01-01 00:00:00 +0000
+++ libdrizzle/field.c	2010-08-12 20:27:32 +0000
@@ -0,0 +1,361 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Field definitions
+ */
+
+#include "common.h"
+
+/*
+ * Client definitions
+ */
+
+drizzle_field_t drizzle_field_read(drizzle_result_st *result, size_t *offset,
+                                   size_t *size, size_t *total,
+                                   drizzle_return_t *ret_ptr)
+{
+  if (drizzle_state_none(result->con))
+  {
+    if (result->field_current == result->column_count)
+    {
+      *ret_ptr= DRIZZLE_RETURN_ROW_END;
+      return NULL;
+    }
+
+    drizzle_state_push(result->con, drizzle_state_field_read);
+  }
+
+  *ret_ptr= drizzle_state_loop(result->con);
+  if (*ret_ptr == DRIZZLE_RETURN_OK &&
+      result->options & DRIZZLE_RESULT_ROW_BREAK)
+  {
+    *ret_ptr= DRIZZLE_RETURN_ROW_BREAK;
+  }
+
+  *offset= result->field_offset;
+  *size= result->field_size;
+  *total= result->field_total;
+
+  return result->field;
+}
+
+drizzle_field_t drizzle_field_buffer(drizzle_result_st *result, size_t *total,
+                                     drizzle_return_t *ret_ptr)
+{
+  drizzle_field_t field;
+  size_t offset= 0;
+  size_t size= 0;
+
+  field= drizzle_field_read(result, &offset, &size, total, ret_ptr);
+  if (*ret_ptr != DRIZZLE_RETURN_OK)
+    return NULL;
+
+  if (field == NULL)
+  {
+    *total= 0;
+    return NULL;
+  }
+
+  if (result->field_buffer == NULL)
+  {
+    result->field_buffer= malloc((*total) + 1);
+    if (result->field_buffer == NULL)
+    {
+      drizzle_set_error(result->con->drizzle, "drizzle_field_buffer", "malloc");
+      *ret_ptr= DRIZZLE_RETURN_MEMORY;
+      return NULL;
+    }
+  }
+
+  memcpy(result->field_buffer + offset, field, size);
+
+  while ((offset + size) != (*total))
+  {
+    field= drizzle_field_read(result, &offset, &size, total, ret_ptr);
+    if (*ret_ptr != DRIZZLE_RETURN_OK)
+      return NULL;
+
+    memcpy(result->field_buffer + offset, field, size);
+  }
+
+  field= result->field_buffer;
+  result->field_buffer= NULL;
+  field[*total]= 0;
+
+  return field;
+}
+
+void drizzle_field_free(drizzle_field_t field)
+{
+  if (field != NULL)
+    free(field);
+}
+
+/*
+ * Server definitions
+ */
+
+drizzle_return_t drizzle_field_write(drizzle_result_st *result,
+                                     const drizzle_field_t field, size_t size,
+                                     size_t total)
+{
+  drizzle_return_t ret;
+
+  if (drizzle_state_none(result->con))
+  {
+    if (result->options & DRIZZLE_RESULT_ROW_BREAK)
+    {
+      result->options&= (drizzle_result_options_t)~DRIZZLE_RESULT_ROW_BREAK;
+      result->field= field;
+      result->field_size= size;
+    }
+    else
+    {
+      result->field= field;
+      result->field_size= size;
+      result->field_offset= 0;
+      result->field_total= total;
+    }
+
+    drizzle_state_push(result->con, drizzle_state_field_write);
+  }
+  else if (result->field == NULL)
+  {
+    result->field= field;
+    result->field_size= size;
+  }
+
+  ret= drizzle_state_loop(result->con);
+  if (ret == DRIZZLE_RETURN_PAUSE)
+    ret= DRIZZLE_RETURN_OK;
+
+  return ret;
+}
+
+/*
+ * Internal state functions.
+ */
+
+drizzle_return_t drizzle_state_field_read(drizzle_con_st *con)
+{
+  drizzle_return_t ret;
+
+  drizzle_log_debug(con->drizzle, "drizzle_state_field_read");
+
+  if (con->buffer_size == 0)
+  {
+    drizzle_state_push(con, drizzle_state_read);
+    return DRIZZLE_RETURN_OK;
+  }
+
+  con->result->field_offset+= con->result->field_size;
+  if (con->result->field_offset == con->result->field_total)
+  {
+    con->result->field_offset= 0;
+    con->result->field_size= 0;
+
+    con->result->field_total= (size_t)drizzle_unpack_length(con, &ret);
+    if (ret == DRIZZLE_RETURN_NULL_SIZE)
+    {
+      con->result->field= NULL;
+      con->result->field_current++;
+      drizzle_state_pop(con);
+      return DRIZZLE_RETURN_OK;
+    }
+    else if (ret != DRIZZLE_RETURN_OK)
+    {
+      if (ret == DRIZZLE_RETURN_IO_WAIT)
+      {
+        drizzle_state_push(con, drizzle_state_read);
+        return DRIZZLE_RETURN_OK;
+      }
+
+      return ret;
+    }
+
+    drizzle_log_debug(con->drizzle,
+                      "field_offset= %zu, field_size= %zu, field_total= %zu",
+                      con->result->field_offset, con->result->field_size,
+                      con->result->field_total);
+
+    if ((size_t)(con->buffer_size) >= con->result->field_total)
+      con->result->field_size= con->result->field_total;
+    else
+      con->result->field_size= con->buffer_size;
+  }
+  else
+  {
+    if ((con->result->field_offset + con->buffer_size) >=
+        con->result->field_total)
+    {
+      con->result->field_size= (con->result->field_total -
+                                con->result->field_offset);
+    }
+    else
+      con->result->field_size= con->buffer_size;
+  }
+
+  /* This is a special case when a row is larger than the packet size. */
+  if (con->result->field_size > (size_t)con->packet_size)
+  {
+    con->result->field_size= con->packet_size;
+
+    if (con->options & DRIZZLE_CON_RAW_PACKET)
+      con->result->options|= DRIZZLE_RESULT_ROW_BREAK;
+    else
+    {
+      drizzle_state_pop(con);
+      drizzle_state_push(con, drizzle_state_packet_read);
+      drizzle_state_push(con, drizzle_state_field_read);
+    }
+  }
+
+  con->result->field= (char *)con->buffer_ptr;
+  con->buffer_ptr+= con->result->field_size;
+  con->buffer_size-= con->result->field_size;
+  con->packet_size-= con->result->field_size;
+
+  drizzle_log_debug(con->drizzle,
+                    "field_offset= %zu, field_size= %zu, field_total= %zu",
+                    con->result->field_offset, con->result->field_size,
+                    con->result->field_total);
+
+  if ((con->result->field_offset + con->result->field_size) ==
+      con->result->field_total)
+  {
+    if (con->result->column_buffer != NULL &&
+        con->result->column_buffer[con->result->field_current].max_size <
+        con->result->field_total)
+    {
+      con->result->column_buffer[con->result->field_current].max_size=
+                                                       con->result->field_total;
+    }
+
+    con->result->field_current++;
+  }
+
+  if (con->result->field_total == 0 || con->result->field_size > 0 ||
+      con->packet_size == 0)
+  {
+    drizzle_state_pop(con);
+  }
+
+  return DRIZZLE_RETURN_OK;
+}
+
+drizzle_return_t drizzle_state_field_write(drizzle_con_st *con)
+{
+  uint8_t *start= con->buffer_ptr + con->buffer_size;
+  uint8_t *ptr;
+  size_t free_size;
+  drizzle_result_st *result= con->result;
+
+  drizzle_log_debug(con->drizzle, "drizzle_state_field_write");
+
+  if (result->field == NULL && result->field_total != 0)
+    return DRIZZLE_RETURN_PAUSE;
+
+  free_size= (size_t)DRIZZLE_MAX_BUFFER_SIZE - (size_t)(start - con->buffer);
+  ptr= start;
+
+  if (result->field_offset == 0)
+  {
+    /* Make sure we can fit the max length and 1 byte of data in (9 + 1). */
+    if (free_size < 10)
+    {
+      drizzle_state_push(con, drizzle_state_write);
+      return DRIZZLE_RETURN_OK;
+    }
+
+    if (result->field == NULL)
+    {
+      ptr[0]= 251;
+      ptr++;
+    }
+    else if (result->field_total == 0)
+    {
+      ptr[0]= 0;
+      ptr++;
+    }
+    else
+      ptr= drizzle_pack_length(result->field_total, ptr);
+
+    free_size-= (size_t)(ptr - start);
+    con->buffer_size+= (size_t)(ptr - start);
+    con->packet_size-= (size_t)(ptr - start);
+  }
+  else if (result->field_size > DRIZZLE_BUFFER_COPY_THRESHOLD)
+  {
+    /* Flush the internal buffer first. */
+    if (con->buffer_size != 0)
+    {
+      drizzle_state_push(con, drizzle_state_write);
+      return DRIZZLE_RETURN_OK;
+    }
+
+    /* We do this to write directly from the field buffer to avoid memcpy(). */
+    con->buffer_ptr= (uint8_t *)result->field;
+    con->buffer_size= result->field_size;
+    con->packet_size-= result->field_size;
+    result->field_offset+= result->field_size;
+    result->field= NULL;
+
+    if (result->field_offset == result->field_total)
+      drizzle_state_pop(con);
+    else if (con->packet_size == 0)
+    {
+      con->result->options|= DRIZZLE_RESULT_ROW_BREAK;
+      drizzle_state_pop(con);
+    }
+
+    drizzle_state_push(con, drizzle_state_write);
+    return DRIZZLE_RETURN_OK;
+  }
+
+  if (result->field_size == 0)
+    drizzle_state_pop(con);
+  else
+  {
+    if (result->field_size < free_size)
+      free_size= result->field_size;
+
+    memcpy(ptr, result->field, free_size);
+    result->field_offset+= free_size;
+    con->buffer_size+= free_size;
+    con->packet_size-= free_size;
+
+    if (result->field_offset == result->field_total)
+    {
+      result->field= NULL;
+      drizzle_state_pop(con);
+    }
+    else
+    {
+      if (con->packet_size == 0)
+      {
+        con->result->options|= DRIZZLE_RESULT_ROW_BREAK;
+        drizzle_state_pop(con);
+      }
+
+      if (result->field_size == free_size)
+        result->field= NULL;
+      else
+      {
+        result->field+= free_size;
+        result->field_size-= free_size;
+        drizzle_state_push(con, drizzle_state_write);
+      }
+    }
+  }
+
+  return DRIZZLE_RETURN_OK;
+}

=== added file 'libdrizzle/field_client.h'
--- libdrizzle/field_client.h	1970-01-01 00:00:00 +0000
+++ libdrizzle/field_client.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,62 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Field Declarations for Clients
+ */
+
+#ifndef __DRIZZLE_FIELD_CLIENT_H
+#define __DRIZZLE_FIELD_CLIENT_H
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/**
+ * @addtogroup drizzle_field_client Field Declarations for Clients
+ * @ingroup drizzle_client_interface
+ *
+ * These functions allow you to access fields in a result set if the result is
+ * unbuffered. If the result is buffered, you can access the fields through the
+ * row functions.
+ * @{
+ */
+
+/**
+ * Read field for unbuffered result, possibly in parts. This is especially
+ * useful for blob streaming, since the client does not need to buffer the
+ * entire blob.
+ */
+DRIZZLE_API
+drizzle_field_t drizzle_field_read(drizzle_result_st *result, size_t *offset,
+                                   size_t *size, size_t *total,
+                                   drizzle_return_t *ret_ptr);
+
+/**
+ * Buffer one field.
+ */
+DRIZZLE_API
+drizzle_field_t drizzle_field_buffer(drizzle_result_st *result, size_t *total,
+                                     drizzle_return_t *ret_ptr);
+
+/**
+ * Free a buffered field.
+ */
+DRIZZLE_API
+void drizzle_field_free(drizzle_field_t field);
+
+/** @} */
+
+#ifdef __cplusplus
+}
+#endif
+
+#endif /* __DRIZZLE_FIELD_CLIENT_H */

=== added file 'libdrizzle/field_server.h'
--- libdrizzle/field_server.h	1970-01-01 00:00:00 +0000
+++ libdrizzle/field_server.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,42 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Field Declarations for Servers
+ */
+
+#ifndef __DRIZZLE_FIELD_SERVER_H
+#define __DRIZZLE_FIELD_SERVER_H
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/**
+ * @addtogroup drizzle_field_server Field Declarations for Servers
+ * @ingroup drizzle_server_interface
+ *
+ * These functions allow you to send a field over a connection.
+ * @{
+ */
+
+DRIZZLE_API
+drizzle_return_t drizzle_field_write(drizzle_result_st *result,
+                                     const drizzle_field_t field, size_t size,
+                                     size_t total);
+
+/** @} */
+
+#ifdef __cplusplus
+}
+#endif
+
+#endif /* __DRIZZLE_FIELD_SERVER_H */

=== added file 'libdrizzle/handshake.c'
--- libdrizzle/handshake.c	1970-01-01 00:00:00 +0000
+++ libdrizzle/handshake.c	2010-08-12 20:27:32 +0000
@@ -0,0 +1,545 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Handshake Definitions
+ */
+
+#include "common.h"
+
+/*
+ * Client Definitions
+ */
+
+drizzle_return_t drizzle_handshake_server_read(drizzle_con_st *con)
+{
+  if (drizzle_state_none(con))
+  {
+    drizzle_state_push(con, drizzle_state_handshake_server_read);
+    drizzle_state_push(con, drizzle_state_packet_read);
+  }
+
+  return drizzle_state_loop(con);
+}
+
+drizzle_return_t drizzle_handshake_client_write(drizzle_con_st *con)
+{
+  if (drizzle_state_none(con))
+  {
+    drizzle_state_push(con, drizzle_state_write);
+    drizzle_state_push(con, drizzle_state_handshake_client_write);
+  }
+
+  return drizzle_state_loop(con);
+}
+
+/*
+ * Server Definitions
+ */
+
+drizzle_return_t drizzle_handshake_server_write(drizzle_con_st *con)
+{
+  if (drizzle_state_none(con))
+  {
+    drizzle_state_push(con, drizzle_state_write);
+    drizzle_state_push(con, drizzle_state_handshake_server_write);
+  }
+
+  return drizzle_state_loop(con);
+}
+
+drizzle_return_t drizzle_handshake_client_read(drizzle_con_st *con)
+{
+  if (drizzle_state_none(con))
+  {
+    drizzle_state_push(con, drizzle_state_handshake_client_read);
+    drizzle_state_push(con, drizzle_state_packet_read);
+  }
+
+  return drizzle_state_loop(con);
+}
+
+/*
+ * State Definitions
+ */
+
+drizzle_return_t drizzle_state_handshake_server_read(drizzle_con_st *con)
+{
+  uint8_t *ptr;
+
+  drizzle_log_debug(con->drizzle, "drizzle_state_handshake_server_read");
+
+  /* Assume the entire handshake packet will fit in the buffer. */
+  if (con->buffer_size < con->packet_size)
+  {
+    drizzle_state_push(con, drizzle_state_read);
+    return DRIZZLE_RETURN_OK;
+  }
+
+  if (con->packet_size < 46)
+  {
+    drizzle_set_error(con->drizzle, "drizzle_state_handshake_server_read",
+                      "bad packet size:>=46:%zu", con->packet_size);
+    return DRIZZLE_RETURN_BAD_HANDSHAKE_PACKET;
+  }
+
+  con->protocol_version= con->buffer_ptr[0];
+  con->buffer_ptr++;
+
+  if (con->protocol_version != 10)
+  {
+    /* This is a special case where the server determines that authentication
+       will be impossible and denies any attempt right away. */
+    if (con->protocol_version == 255)
+    {
+      drizzle_set_error(con->drizzle, "drizzle_state_handshake_server_read",
+                        "%.*s", (int32_t)con->packet_size - 3,
+                        con->buffer_ptr + 2);
+      return DRIZZLE_RETURN_AUTH_FAILED;
+    }
+
+    drizzle_set_error(con->drizzle, "drizzle_state_handshake_server_read",
+                      "protocol version not supported:%d",
+                      con->protocol_version);
+    return DRIZZLE_RETURN_PROTOCOL_NOT_SUPPORTED;
+  }
+
+  /* Look for null-terminated server version string. */
+  ptr= memchr(con->buffer_ptr, 0, con->buffer_size - 1);
+  if (ptr == NULL)
+  {
+    drizzle_set_error(con->drizzle, "drizzle_state_handshake_server_read",
+                      "server version string not found");
+    return DRIZZLE_RETURN_BAD_HANDSHAKE_PACKET;
+  }
+
+  if (con->packet_size != (46 + (size_t)(ptr - con->buffer_ptr)))
+  {
+    drizzle_set_error(con->drizzle, "drizzle_state_handshake_server_read",
+                      "bad packet size:%zu:%zu",
+                      (46 + (size_t)(ptr - con->buffer_ptr)), con->packet_size);
+    return DRIZZLE_RETURN_BAD_HANDSHAKE_PACKET;
+  }
+
+  strncpy(con->server_version, (char *)con->buffer_ptr,
+          DRIZZLE_MAX_SERVER_VERSION_SIZE);
+  con->server_version[DRIZZLE_MAX_SERVER_VERSION_SIZE - 1]= 0;
+  con->buffer_ptr+= ((ptr - con->buffer_ptr) + 1);
+
+  con->thread_id= (uint32_t)drizzle_get_byte4(con->buffer_ptr);
+  con->buffer_ptr+= 4;
+
+  con->scramble= con->scramble_buffer;
+  memcpy(con->scramble, con->buffer_ptr, 8);
+  /* Skip scramble and filler. */
+  con->buffer_ptr+= 9;
+
+  /* Even though drizzle_capabilities is more than 2 bytes, the protocol only
+     allows for 2. This means some capabilities are not possible during this
+     handshake step. The options beyond 2 bytes are for client response only. */
+  con->capabilities= (drizzle_capabilities_t)drizzle_get_byte2(con->buffer_ptr);
+  con->buffer_ptr+= 2;
+
+  if (con->options & DRIZZLE_CON_MYSQL &&
+      !(con->capabilities & DRIZZLE_CAPABILITIES_PROTOCOL_41))
+  {
+    drizzle_set_error(con->drizzle, "drizzle_state_handshake_server_read",
+                      "protocol version not supported, must be MySQL 4.1+");
+    return DRIZZLE_RETURN_PROTOCOL_NOT_SUPPORTED;
+  }
+
+  con->charset= con->buffer_ptr[0];
+  con->buffer_ptr+= 1;
+
+  con->status= drizzle_get_byte2(con->buffer_ptr);
+  /* Skip status and filler. */
+  con->buffer_ptr+= 15;
+
+  memcpy(con->scramble + 8, con->buffer_ptr, 12);
+  con->buffer_ptr+= 13;
+
+  con->buffer_size-= con->packet_size;
+  if (con->buffer_size != 0)
+  {
+    drizzle_set_error(con->drizzle, "drizzle_state_handshake_server_read",
+                      "unexpected data after packet:%zu", con->buffer_size);
+    return DRIZZLE_RETURN_UNEXPECTED_DATA;
+  }
+
+  con->buffer_ptr= con->buffer;
+
+  drizzle_state_pop(con);
+
+  if (!(con->options & DRIZZLE_CON_RAW_PACKET))
+  {
+    drizzle_state_push(con, drizzle_state_handshake_result_read);
+    drizzle_state_push(con, drizzle_state_packet_read);
+    drizzle_state_push(con, drizzle_state_write);
+    drizzle_state_push(con, drizzle_state_handshake_client_write);
+  }
+
+  return DRIZZLE_RETURN_OK;
+}
+
+drizzle_return_t drizzle_state_handshake_server_write(drizzle_con_st *con)
+{
+  uint8_t *ptr;
+
+  drizzle_log_debug(con->drizzle, "drizzle_state_handshake_server_write");
+
+  /* Calculate max packet size. */
+  con->packet_size= 1   /* Protocol version */
+                  + strlen(con->server_version) + 1
+                  + 4   /* Thread ID */
+                  + 8   /* Scramble */
+                  + 1   /* NULL */
+                  + 2   /* Capabilities */
+                  + 1   /* Language */
+                  + 2   /* Status */
+                  + 13  /* Unused */
+                  + 12  /* Scramble */
+                  + 1;  /* NULL */
+
+  /* Assume the entire handshake packet will fit in the buffer. */
+  if ((con->packet_size + 4) > DRIZZLE_MAX_BUFFER_SIZE)
+  {
+    drizzle_set_error(con->drizzle, "drizzle_state_handshake_server_write",
+                      "buffer too small:%zu", con->packet_size + 4);
+    return DRIZZLE_RETURN_INTERNAL_ERROR;
+  }
+
+  ptr= con->buffer_ptr;
+
+  /* Store packet size and packet number first. */
+  drizzle_set_byte3(ptr, con->packet_size);
+  ptr[3]= 0;
+  con->packet_number= 1;
+  ptr+= 4;
+
+  ptr[0]= con->protocol_version;
+  ptr++;
+
+  memcpy(ptr, con->server_version, strlen(con->server_version));
+  ptr+= strlen(con->server_version);
+
+  ptr[0]= 0;
+  ptr++;
+
+  drizzle_set_byte4(ptr, con->thread_id);
+  ptr+= 4;
+
+  if (con->scramble == NULL)
+    memset(ptr, 0, 8);
+  else
+    memcpy(ptr, con->scramble, 8);
+  ptr+= 8;
+
+  ptr[0]= 0;
+  ptr++;
+
+  if (con->options & DRIZZLE_CON_MYSQL)
+    con->capabilities|= DRIZZLE_CAPABILITIES_PROTOCOL_41;
+
+  /* We can only send two bytes worth, this is a protocol limitation. */
+  drizzle_set_byte2(ptr, con->capabilities);
+  ptr+= 2;
+
+  ptr[0]= con->charset;
+  ptr++;
+
+  drizzle_set_byte2(ptr, con->status);
+  ptr+= 2;
+
+  memset(ptr, 0, 13);
+  ptr+= 13;
+
+  if (con->scramble == NULL)
+    memset(ptr, 0, 12);
+  else
+    memcpy(ptr, con->scramble + 8, 12);
+  ptr+= 12;
+
+  ptr[0]= 0;
+  ptr++;
+
+  con->buffer_size+= (4 + con->packet_size);
+
+  /* Make sure we packed it correctly. */
+  if ((size_t)(ptr - con->buffer_ptr) != (4 + con->packet_size))
+  {
+    drizzle_set_error(con->drizzle, "drizzle_state_handshake_server_write",
+                      "error packing server handshake:%zu:%zu",
+                      (size_t)(ptr - con->buffer_ptr), 4 + con->packet_size);
+    return DRIZZLE_RETURN_INTERNAL_ERROR;
+  }
+
+  drizzle_state_pop(con);
+  return DRIZZLE_RETURN_OK;
+}
+
+drizzle_return_t drizzle_state_handshake_client_read(drizzle_con_st *con)
+{
+  size_t real_size;
+  uint8_t *ptr;
+  uint8_t scramble_size;
+
+  drizzle_log_debug(con->drizzle, "drizzle_state_handshake_client_read");
+
+  /* Assume the entire handshake packet will fit in the buffer. */
+  if (con->buffer_size < con->packet_size)
+  {
+    drizzle_state_push(con, drizzle_state_read);
+    return DRIZZLE_RETURN_OK;
+  }
+
+  /* This is the minimum packet size. */
+  if (con->packet_size < 34)
+  {
+    drizzle_set_error(con->drizzle, "drizzle_state_handshake_client_read",
+                      "bad packet size:>=34:%zu", con->packet_size);
+    return DRIZZLE_RETURN_BAD_HANDSHAKE_PACKET;
+  }
+
+  real_size= 34;
+
+  con->capabilities= drizzle_get_byte4(con->buffer_ptr);
+  con->buffer_ptr+= 4;
+
+  if (con->options & DRIZZLE_CON_MYSQL &&
+      !(con->capabilities & DRIZZLE_CAPABILITIES_PROTOCOL_41))
+  {
+    drizzle_set_error(con->drizzle, "drizzle_state_handshake_client_read",
+                      "protocol version not supported, must be MySQL 4.1+");
+    return DRIZZLE_RETURN_PROTOCOL_NOT_SUPPORTED;
+  }
+
+  con->max_packet_size= (uint32_t)drizzle_get_byte4(con->buffer_ptr);
+  con->buffer_ptr+= 4;
+
+  con->charset= con->buffer_ptr[0];
+  con->buffer_ptr+= 1;
+
+  /* Skip unused. */
+  con->buffer_ptr+= 23;
+
+  /* Look for null-terminated user string. */
+  ptr= memchr(con->buffer_ptr, 0, con->buffer_size - 32);
+  if (ptr == NULL)
+  {
+    drizzle_set_error(con->drizzle, "drizzle_state_handshake_client_read",
+                      "user string not found");
+    return DRIZZLE_RETURN_BAD_HANDSHAKE_PACKET;
+  }
+
+  if (con->buffer_ptr == ptr)
+  {
+    con->user[0]= 0;
+    con->buffer_ptr++;
+  }
+  else
+  {
+    real_size+= (size_t)(ptr - con->buffer_ptr);
+    if (con->packet_size < real_size)
+    {
+      drizzle_set_error(con->drizzle, "drizzle_state_handshake_client_read",
+                        "bad packet size:>=%zu:%zu", real_size,
+                        con->packet_size);
+      return DRIZZLE_RETURN_BAD_HANDSHAKE_PACKET;
+    }
+
+    strncpy(con->user, (char *)con->buffer_ptr, DRIZZLE_MAX_USER_SIZE);
+    con->user[DRIZZLE_MAX_USER_SIZE - 1]= 0;
+    con->buffer_ptr+= ((ptr - con->buffer_ptr) + 1);
+  }
+
+  scramble_size= con->buffer_ptr[0];
+  con->buffer_ptr+= 1;
+
+  if (scramble_size == 0)
+    con->scramble= NULL;
+  else
+  {
+    if (scramble_size != DRIZZLE_MAX_SCRAMBLE_SIZE)
+    {
+      drizzle_set_error(con->drizzle, "drizzle_state_handshake_client_read",
+                        "wrong scramble size");
+      return DRIZZLE_RETURN_BAD_HANDSHAKE_PACKET;
+    }
+
+    real_size+= scramble_size;
+    con->scramble= con->scramble_buffer;
+    memcpy(con->scramble, con->buffer_ptr, DRIZZLE_MAX_SCRAMBLE_SIZE);
+
+    con->buffer_ptr+= DRIZZLE_MAX_SCRAMBLE_SIZE;
+  }
+
+  /* Look for null-terminated db string. */
+  if ((34 + strlen(con->user) + scramble_size) == con->packet_size)
+    con->db[0]= 0;
+  else
+  {
+    ptr= memchr(con->buffer_ptr, 0, con->buffer_size -
+                                    (34 + strlen(con->user) + scramble_size));
+    if (ptr == NULL)
+    {
+      drizzle_set_error(con->drizzle, "drizzle_state_handshake_client_read",
+                        "db string not found");
+      return DRIZZLE_RETURN_BAD_HANDSHAKE_PACKET;
+    }
+
+    real_size+= ((size_t)(ptr - con->buffer_ptr) + 1);
+    if (con->packet_size != real_size)
+    {
+      drizzle_set_error(con->drizzle, "drizzle_state_handshake_client_read",
+                        "bad packet size:%zu:%zu", real_size, con->packet_size);
+      return DRIZZLE_RETURN_BAD_HANDSHAKE_PACKET;
+    }
+
+    if (con->buffer_ptr == ptr)
+    {
+      con->db[0]= 0;
+      con->buffer_ptr++;
+    }
+    else
+    {
+      strncpy(con->db, (char *)con->buffer_ptr, DRIZZLE_MAX_DB_SIZE);
+      con->db[DRIZZLE_MAX_DB_SIZE - 1]= 0;
+      con->buffer_ptr+= ((ptr - con->buffer_ptr) + 1);
+    }
+  }
+
+  con->buffer_size-= con->packet_size;
+  if (con->buffer_size != 0)
+  {
+    drizzle_set_error(con->drizzle, "drizzle_state_handshake_client_read",
+                      "unexpected data after packet:%zu", con->buffer_size);
+    return DRIZZLE_RETURN_UNEXPECTED_DATA;
+  }
+
+  con->buffer_ptr= con->buffer;
+
+  drizzle_state_pop(con);
+  return DRIZZLE_RETURN_OK;
+}
+
+drizzle_return_t drizzle_state_handshake_client_write(drizzle_con_st *con)
+{
+  uint8_t *ptr;
+  drizzle_capabilities_t capabilities;
+  drizzle_return_t ret;
+
+  drizzle_log_debug(con->drizzle, "drizzle_state_handshake_client_write");
+
+  /* Calculate max packet size. */
+  con->packet_size= 4   /* Capabilities */
+                  + 4   /* Max packet size */
+                  + 1   /* Charset */
+                  + 23  /* Unused */
+                  + strlen(con->user) + 1
+                  + 1   /* Scramble size */
+                  + DRIZZLE_MAX_SCRAMBLE_SIZE
+                  + strlen(con->db) + 1;
+
+  /* Assume the entire handshake packet will fit in the buffer. */
+  if ((con->packet_size + 4) > DRIZZLE_MAX_BUFFER_SIZE)
+  {
+    drizzle_set_error(con->drizzle, "drizzle_state_handshake_client_write",
+                      "buffer too small:%zu", con->packet_size + 4);
+    return DRIZZLE_RETURN_INTERNAL_ERROR;
+  }
+
+  ptr= con->buffer_ptr;
+
+  /* Store packet size at the end since it may change. */
+  ptr[3]= con->packet_number;
+  con->packet_number++;
+  ptr+= 4;
+
+  if (con->options & DRIZZLE_CON_MYSQL)
+    con->capabilities|= DRIZZLE_CAPABILITIES_PROTOCOL_41;
+
+  capabilities= con->capabilities & DRIZZLE_CAPABILITIES_CLIENT;
+  if (!(con->options & DRIZZLE_CON_FOUND_ROWS))
+    capabilities&= ~DRIZZLE_CAPABILITIES_FOUND_ROWS;
+
+  capabilities&= ~(DRIZZLE_CAPABILITIES_COMPRESS | DRIZZLE_CAPABILITIES_SSL);
+  if (con->db[0] == 0)
+    capabilities&= ~DRIZZLE_CAPABILITIES_CONNECT_WITH_DB;
+
+  drizzle_set_byte4(ptr, capabilities);
+  ptr+= 4;
+
+  drizzle_set_byte4(ptr, con->max_packet_size);
+  ptr+= 4;
+
+  ptr[0]= con->charset;
+  ptr++;
+
+  memset(ptr, 0, 23);
+  ptr+= 23;
+
+  ptr= drizzle_pack_auth(con, ptr, &ret);
+  if (ret != DRIZZLE_RETURN_OK)
+    return ret;
+
+  con->buffer_size+= (4 + con->packet_size);
+
+  /* Make sure we packed it correctly. */
+  if ((size_t)(ptr - con->buffer_ptr) != (4 + con->packet_size))
+  {
+    drizzle_set_error(con->drizzle, "drizzle_state_handshake_client_write",
+                      "error packing client handshake:%zu:%zu",
+                      (size_t)(ptr - con->buffer_ptr), 4 + con->packet_size);
+    return DRIZZLE_RETURN_INTERNAL_ERROR;
+  }
+
+  /* Store packet size now. */
+  drizzle_set_byte3(con->buffer_ptr, con->packet_size);
+
+  drizzle_state_pop(con);
+  return DRIZZLE_RETURN_OK;
+}
+
+drizzle_return_t drizzle_state_handshake_result_read(drizzle_con_st *con)
+{
+  drizzle_return_t ret;
+  drizzle_result_st result;
+
+  drizzle_log_debug(con->drizzle, "drizzle_state_handshake_result_read");
+
+  if (drizzle_result_create(con, &result) == NULL)
+    return DRIZZLE_RETURN_MEMORY;
+
+  con->result= &result;
+
+  ret= drizzle_state_result_read(con);
+  if (drizzle_state_none(con))
+  {
+    if (ret == DRIZZLE_RETURN_OK)
+    {
+      if (drizzle_result_eof(&result))
+      {
+        drizzle_set_error(con->drizzle, "drizzle_state_handshake_result_read",
+                         "old insecure authentication mechanism not supported");
+        ret= DRIZZLE_RETURN_AUTH_FAILED;
+      }
+      else
+        con->options|= DRIZZLE_CON_READY;
+    }
+  }
+
+  drizzle_result_free(&result);
+
+  if (ret == DRIZZLE_RETURN_ERROR_CODE)
+    return DRIZZLE_RETURN_HANDSHAKE_FAILED;
+
+  return ret;
+}

=== added file 'libdrizzle/handshake_client.h'
--- libdrizzle/handshake_client.h	1970-01-01 00:00:00 +0000
+++ libdrizzle/handshake_client.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,59 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Handshake Declarations for Clients
+ */
+
+#ifndef __DRIZZLE_HANDSHAKE_CLIENT_H
+#define __DRIZZLE_HANDSHAKE_CLIENT_H
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/**
+ * @addtogroup drizzle_handshake_client Handshake Declarations for Clients
+ * @ingroup drizzle_client_interface
+ *
+ * These functions are used to send and receive handshake packets for a client.
+ * These are only used by low-level clients when the DRIZZLE_CON_RAW_PACKET
+ * option is set, so most applications will never need to use these.
+ * @{
+ */
+
+/**
+ * Read handshake packet from the server in a client.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @return Standard drizzle return value.
+ */
+DRIZZLE_API
+drizzle_return_t drizzle_handshake_server_read(drizzle_con_st *con);
+
+/**
+ * Write client handshake packet to a server.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @return Standard drizzle return value.
+ */
+DRIZZLE_API
+drizzle_return_t drizzle_handshake_client_write(drizzle_con_st *con);
+
+/** @} */
+
+#ifdef __cplusplus
+}
+#endif
+
+#endif /* __DRIZZLE_HANDSHAKE_CLIENT_H */

=== added file 'libdrizzle/handshake_server.h'
--- libdrizzle/handshake_server.h	1970-01-01 00:00:00 +0000
+++ libdrizzle/handshake_server.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,57 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Handshake Declarations for Servers
+ */
+
+#ifndef __DRIZZLE_HANDSHAKE_SERVER_H
+#define __DRIZZLE_HANDSHAKE_SERVER_H
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/**
+ * @addtogroup drizzle_handshake_server Handshake Declarations for Servers
+ * @ingroup drizzle_server_interface
+ *
+ * These functions are used to send and receive handshake packets in a server.
+ * @{
+ */
+
+/**
+ * Write server handshake packet to a client.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @return Standard drizzle return value.
+ */
+DRIZZLE_API
+drizzle_return_t drizzle_handshake_server_write(drizzle_con_st *con);
+
+/**
+ * Read handshake packet from the client in a server.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @return Standard drizzle return value.
+ */
+DRIZZLE_API
+drizzle_return_t drizzle_handshake_client_read(drizzle_con_st *con);
+
+/** @} */
+
+#ifdef __cplusplus
+}
+#endif
+
+#endif /* __DRIZZLE_HANDSHAKE_SERVER_H */

=== added file 'libdrizzle/include.am'
--- libdrizzle/include.am	1970-01-01 00:00:00 +0000
+++ libdrizzle/include.am	2010-08-12 20:27:32 +0000
@@ -0,0 +1,126 @@
+#
+# Drizzle Client Library
+#
+# Copyright (C) 2008 Eric Day (eday@oddments.org)
+# All rights reserved.
+#
+# Use and distribution licensed under the BSD license.  See
+# the COPYING file in this directory for full text.
+#
+#
+
+lib_LTLIBRARIES+= libdrizzle/libdrizzle.la
+
+if HAVE_LIBSQLITE3
+SQLITE_PROGS= examples/sqlite_server
+endif
+
+noinst_PROGRAMS+= \
+	extra/mysql_password_hash \
+	examples/client \
+	examples/simple \
+	examples/simple_multi \
+	examples/pipe_query \
+	examples/server \
+	examples/proxy \
+	$(SQLITE_PROGS)
+
+libdrizzle_libdrizzle_la_CFLAGS= \
+	${AM_CFLAGS} \
+	-DBUILDING_LIBDRIZZLE
+
+libdrizzle_libdrizzle_la_LDFLAGS= \
+	$(AM_LDFLAGS) \
+	-version-info \
+	$(LIBDRIZZLE_LIBRARY_VERSION)
+
+libdrizzle_libdrizzle_la_SOURCES= \
+	libdrizzle/drizzle.c \
+	libdrizzle/conn.c \
+	libdrizzle/conn_uds.c \
+	libdrizzle/handshake.c \
+	libdrizzle/command.c \
+	libdrizzle/query.c \
+	libdrizzle/result.c \
+	libdrizzle/column.c \
+	libdrizzle/row.c \
+	libdrizzle/field.c \
+	libdrizzle/pack.c \
+	libdrizzle/state.c \
+	libdrizzle/sha1.c
+
+if BUILD_WIN32
+libdrizzle_libdrizzle_la_SOURCES+= win32/poll.c
+endif
+
+examples_server_CFLAGS= $(AM_CFLAGS)
+examples_server_SOURCES= examples/server.c
+examples_server_LDADD= libdrizzle/libdrizzle.la
+
+examples_proxy_CFLAGS= $(AM_CFLAGS)
+examples_proxy_SOURCES= examples/proxy.c
+examples_proxy_LDADD= libdrizzle/libdrizzle.la
+
+examples_client_LDADD= libdrizzle/libdrizzle.la
+examples_simple_LDADD= libdrizzle/libdrizzle.la
+examples_simple_multi_LDADD= libdrizzle/libdrizzle.la
+examples_pipe_query_LDADD= libdrizzle/libdrizzle.la
+
+if HAVE_LIBSQLITE3
+examples_sqlite_server_CFLAGS= $(AM_CFLAGS)
+examples_sqlite_server_LDFLAGS= libdrizzle/libdrizzle.la $(LTLIBSQLITE3)
+examples_sqlite_server_DEPENDENCIES= libdrizzle/libdrizzle.la
+examples_sqlite_server_SOURCES= examples/sqlite_server.c
+endif
+
+nobase_dist_include_HEADERS+= \
+	libdrizzle/drizzle.h \
+	libdrizzle/drizzle_client.h \
+	libdrizzle/drizzle_server.h \
+	libdrizzle/conn.h \
+	libdrizzle/conn_client.h \
+	libdrizzle/conn_server.h \
+	libdrizzle/handshake_client.h \
+	libdrizzle/handshake_server.h \
+	libdrizzle/command_client.h \
+	libdrizzle/command_server.h \
+	libdrizzle/query.h \
+	libdrizzle/result.h \
+	libdrizzle/result_client.h \
+	libdrizzle/result_server.h \
+	libdrizzle/column.h \
+	libdrizzle/column_client.h \
+	libdrizzle/column_server.h \
+	libdrizzle/row_client.h \
+	libdrizzle/row_server.h \
+	libdrizzle/field_client.h \
+	libdrizzle/field_server.h \
+	libdrizzle/constants.h \
+	libdrizzle/structs.h \
+	libdrizzle/visibility.h
+
+noinst_HEADERS+= \
+	libdrizzle/common.h \
+	libdrizzle/drizzle_local.h \
+	libdrizzle/conn_local.h \
+	libdrizzle/pack.h \
+	libdrizzle/state.h \
+	libdrizzle/sha1.h \
+	unittests/common.h
+
+CLEANFILES+= \
+	doxerr.log \
+	libdrizzle/drizzle_client.h.gch \
+	libdrizzle/drizzle_server.h.gch \
+	libdrizzle/common.h.gch
+
+EXTRA_DIST+= \
+	PROTOCOL \
+	config/config.rpath \
+	support/libdrizzle.spec \
+	docs/Doxyfile.api \
+	docs/Doxyfile.dev \
+	docs/dev_header.html \
+	docs/api_header.html \
+	docs/doxygen.h
+

=== added file 'libdrizzle/pack.c'
--- libdrizzle/pack.c	1970-01-01 00:00:00 +0000
+++ libdrizzle/pack.c	2010-08-12 20:27:32 +0000
@@ -0,0 +1,276 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Packing definitions
+ */
+
+#include "common.h"
+
+/*
+ * Private declarations
+ */
+
+/**
+ * @addtogroup drizzle_pack_private Private Packing Functions
+ * @ingroup drizzle_pack
+ * @{
+ */
+
+/**
+ * Compute hash from password and scramble.
+ */
+static drizzle_return_t _pack_scramble_hash(drizzle_con_st *con,
+                                            uint8_t *buffer);
+
+/** @} */
+
+/*
+ * Public definitions
+ */
+
+uint8_t *drizzle_pack_length(uint64_t number, uint8_t *ptr)
+{
+  if (number < 251)
+  {
+    ptr[0]= (uint8_t)number;
+    ptr++;
+  }
+  else if (number < 65536)
+  {
+    ptr[0]= 252;
+    ptr++;
+    drizzle_set_byte2(ptr, number);
+    ptr+= 2;
+  }
+  else if (number < 16777216)
+  {
+    ptr[0]= 253;
+    ptr++;
+    drizzle_set_byte3(ptr, number);
+    ptr+= 3;
+  }
+  else
+  {
+    ptr[0]= 254;
+    ptr++;
+    drizzle_set_byte8(ptr, number);
+    ptr+= 8;
+  }
+
+  return ptr;
+}
+
+uint64_t drizzle_unpack_length(drizzle_con_st *con, drizzle_return_t *ret_ptr)
+{
+  uint64_t length;
+  uint8_t bytes;
+
+  if (con->buffer_ptr[0] < 251)
+  {
+    length= (uint64_t)(con->buffer_ptr[0]);
+    bytes= 1;
+  }
+  else if (con->buffer_ptr[0] == 251)
+  {
+    con->buffer_ptr++;
+    con->buffer_size--;
+    con->packet_size--;
+
+    *ret_ptr= DRIZZLE_RETURN_NULL_SIZE;
+    return 0;
+  }
+  else if (con->buffer_ptr[0] == 252 && con->buffer_size > 2)
+  {
+    length= drizzle_get_byte2(con->buffer_ptr + 1);
+    bytes= 3;
+  }
+  else if (con->buffer_ptr[0] == 253 && con->buffer_size > 3)
+  {
+    length= drizzle_get_byte3(con->buffer_ptr + 1);
+    bytes= 4;
+  }
+  else if (con->buffer_size > 8)
+  {
+    length= drizzle_get_byte8(con->buffer_ptr + 1);
+    bytes= 9;
+  }
+  else
+  {
+    *ret_ptr= DRIZZLE_RETURN_IO_WAIT;
+    return 0;
+  }
+
+  con->buffer_ptr+= bytes;
+  con->buffer_size-= bytes;
+  con->packet_size-= bytes;
+
+  *ret_ptr= DRIZZLE_RETURN_OK;
+  return length;
+}
+
+uint8_t *drizzle_pack_string(char *string, uint8_t *ptr)
+{
+  uint64_t size= strlen(string);
+
+  ptr= drizzle_pack_length(size, ptr);
+  if (size > 0)
+  {
+    memcpy(ptr, string, (size_t)size);
+    ptr+= size;
+  }
+
+  return ptr;
+}
+
+drizzle_return_t drizzle_unpack_string(drizzle_con_st *con, char *buffer,
+                                       uint64_t max_length)
+{
+  drizzle_return_t ret= DRIZZLE_RETURN_OK;
+  uint64_t length;
+
+  length= drizzle_unpack_length(con, &ret);
+  if (ret != DRIZZLE_RETURN_OK)
+  {
+    if (ret == DRIZZLE_RETURN_NULL_SIZE)
+    {
+      drizzle_set_error(con->drizzle, "drizzle_unpack_string",
+                        "unexpected NULL length");
+    }
+
+    return ret;
+  }
+
+  if (length < max_length)
+  {
+    if (length > 0)
+      memcpy(buffer, con->buffer_ptr, (size_t)length);
+
+    buffer[length]= 0;
+  }
+  else
+  {
+    memcpy(buffer, con->buffer_ptr, (size_t)(max_length - 1));
+    buffer[max_length - 1]= 0;
+  }
+  
+  con->buffer_ptr+= length;
+  con->buffer_size-= (size_t)length;
+  con->packet_size-= (size_t)length;
+
+  return DRIZZLE_RETURN_OK;
+}
+
+uint8_t *drizzle_pack_auth(drizzle_con_st *con, uint8_t *ptr,
+                           drizzle_return_t *ret_ptr)
+{
+  if (con->user[0] != 0)
+  {
+    memcpy(ptr, con->user, strlen(con->user));
+    ptr+= strlen(con->user);
+  }
+
+  ptr[0]= 0;
+  ptr++;
+
+  if (con->options & DRIZZLE_CON_RAW_SCRAMBLE && con->scramble != NULL)
+  {
+    ptr[0]= DRIZZLE_MAX_SCRAMBLE_SIZE;
+    ptr++;
+
+    memcpy(ptr, con->scramble, DRIZZLE_MAX_SCRAMBLE_SIZE);
+    ptr+= DRIZZLE_MAX_SCRAMBLE_SIZE;
+  }
+  else if (con->password[0] == 0)
+  {
+    ptr[0]= 0;
+    ptr++;
+    con->packet_size-= DRIZZLE_MAX_SCRAMBLE_SIZE;
+  }
+  else
+  {
+    ptr[0]= DRIZZLE_MAX_SCRAMBLE_SIZE;
+    ptr++;
+
+    if (con->options & DRIZZLE_CON_MYSQL)
+    {
+      *ret_ptr= _pack_scramble_hash(con, ptr);
+      if (*ret_ptr != DRIZZLE_RETURN_OK)
+        return ptr;
+    }
+    else
+      snprintf((char *)ptr, DRIZZLE_MAX_SCRAMBLE_SIZE, "%s", con->password);
+
+    ptr+= DRIZZLE_MAX_SCRAMBLE_SIZE;
+  }
+
+  if (con->db[0] != 0)
+  {
+    memcpy(ptr, con->db, strlen(con->db));
+    ptr+= strlen(con->db);
+  }
+
+  ptr[0]= 0;
+  ptr++;
+
+  *ret_ptr= DRIZZLE_RETURN_OK;
+  return ptr;
+}
+
+/*
+ * Private definitions
+ */
+
+static drizzle_return_t _pack_scramble_hash(drizzle_con_st *con,
+                                            uint8_t *buffer)
+{
+  SHA1_CTX ctx;
+  uint8_t hash_tmp1[SHA1_DIGEST_LENGTH];
+  uint8_t hash_tmp2[SHA1_DIGEST_LENGTH];
+  uint32_t x;
+
+  if (SHA1_DIGEST_LENGTH != DRIZZLE_MAX_SCRAMBLE_SIZE)
+  {
+    drizzle_set_error(con->drizzle, "_pack_scramble_hash",
+                      "SHA1 hash size mismatch:%u:%u", SHA1_DIGEST_LENGTH,
+                      DRIZZLE_MAX_SCRAMBLE_SIZE);
+    return DRIZZLE_RETURN_INTERNAL_ERROR;
+  }
+
+  if (con->scramble == NULL)
+  {
+    drizzle_set_error(con->drizzle, "_pack_scramble_hash",
+                      "no scramble buffer");
+    return DRIZZLE_RETURN_NO_SCRAMBLE;
+  }
+
+  /* First hash the password. */
+  SHA1Init(&ctx);
+  SHA1Update(&ctx, (uint8_t *)(con->password), strlen(con->password));
+  SHA1Final(hash_tmp1, &ctx);
+
+  /* Second, hash the password hash. */
+  SHA1Init(&ctx);
+  SHA1Update(&ctx, hash_tmp1, SHA1_DIGEST_LENGTH);
+  SHA1Final(hash_tmp2, &ctx);
+
+  /* Third, hash the scramble and the double password hash. */
+  SHA1Init(&ctx);
+  SHA1Update(&ctx, con->scramble, SHA1_DIGEST_LENGTH);
+  SHA1Update(&ctx, hash_tmp2, SHA1_DIGEST_LENGTH);
+  SHA1Final(buffer, &ctx);
+
+  /* Fourth, xor the last hash against the first password hash. */
+  for (x= 0; x < SHA1_DIGEST_LENGTH; x++)
+    buffer[x]= buffer[x] ^ hash_tmp1[x];
+
+  return DRIZZLE_RETURN_OK;
+}

=== added file 'libdrizzle/pack.h'
--- libdrizzle/pack.h	1970-01-01 00:00:00 +0000
+++ libdrizzle/pack.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,70 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Packing Declarations
+ */
+
+#ifndef __DRIZZLE_PACK_H
+#define __DRIZZLE_PACK_H
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/**
+ * @addtogroup drizzle_pack Packing Declarations
+ *
+ * These functions are used internally to pack various parts of the protocol.
+ * Not all functions are defined in pack.c, they are in the most appropriate
+ * source file (for example, handshake.c for drizzle_pack_client_handshake).
+ * @{
+ */
+
+/**
+ * Pack length-encoded number.
+ */
+DRIZZLE_API
+uint8_t *drizzle_pack_length(uint64_t number, uint8_t *ptr);
+
+/**
+ * Unpack length-encoded number.
+ */
+DRIZZLE_API
+uint64_t drizzle_unpack_length(drizzle_con_st *con, drizzle_return_t *ret_ptr);
+
+/**
+ * Pack length-encoded string.
+ */
+DRIZZLE_API
+uint8_t *drizzle_pack_string(char *string, uint8_t *ptr);
+
+/**
+ * Unpack length-encoded string.
+ */
+DRIZZLE_API
+drizzle_return_t drizzle_unpack_string(drizzle_con_st *con, char *buffer,
+                                       uint64_t max_size);
+
+/**
+ * Pack user, scramble, and db.
+ */
+DRIZZLE_API
+uint8_t *drizzle_pack_auth(drizzle_con_st *con, uint8_t *ptr,
+                           drizzle_return_t *ret_ptr);
+
+/** @} */
+
+#ifdef __cplusplus
+}
+#endif
+
+#endif /* __DRIZZLE_PACK_H */

=== added file 'libdrizzle/query.c'
--- libdrizzle/query.c	1970-01-01 00:00:00 +0000
+++ libdrizzle/query.c	2010-08-12 20:27:32 +0000
@@ -0,0 +1,391 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Query definitions
+ */
+
+#include "common.h"
+
+drizzle_result_st *drizzle_query(drizzle_con_st *con, drizzle_result_st *result,
+                                 const char *query, size_t size,
+                                 drizzle_return_t *ret_ptr)
+{
+  return drizzle_con_command_write(con, result, DRIZZLE_COMMAND_QUERY,
+                                   (uint8_t *)query, size, size, ret_ptr);
+}
+
+drizzle_result_st *drizzle_query_str(drizzle_con_st *con,
+                                     drizzle_result_st *result,
+                                     const char *query, 
+                                     drizzle_return_t *ret_ptr)
+{
+  size_t size;
+
+  size= strlen(query);
+
+  return drizzle_con_command_write(con, result, DRIZZLE_COMMAND_QUERY,
+                                   (uint8_t *)query, size, size, ret_ptr);
+}
+
+drizzle_result_st *drizzle_query_inc(drizzle_con_st *con,
+                                     drizzle_result_st *result,
+                                     const char *query, size_t size,
+                                     size_t total, drizzle_return_t *ret_ptr)
+{
+  return drizzle_con_command_write(con, result, DRIZZLE_COMMAND_QUERY,
+                                   (uint8_t *)query, size, total, ret_ptr);
+}
+
+drizzle_query_st *drizzle_query_add(drizzle_st *drizzle,
+                                    drizzle_query_st *query,
+                                    drizzle_con_st *con,
+                                    drizzle_result_st *result,
+                                    const char *query_string, size_t size,
+                                    drizzle_query_options_t options,
+                                    void *context)
+{
+  query= drizzle_query_create(drizzle, query);
+  if (query == NULL)
+    return NULL;
+
+  drizzle_query_set_con(query, con);
+  drizzle_query_set_result(query, result);
+  drizzle_query_set_string(query, query_string, size);
+  drizzle_query_add_options(query, options);
+  drizzle_query_set_context(query, context);
+
+  return query;
+}
+
+drizzle_query_st *drizzle_query_create(drizzle_st *drizzle,
+                                       drizzle_query_st *query)
+{
+  if (query == NULL)
+  {
+    query= malloc(sizeof(drizzle_query_st));
+    if (query == NULL)
+    {
+      drizzle_set_error(drizzle, "drizzle_query_create", "malloc");
+      return NULL;
+    }
+
+    memset(query, 0, sizeof(drizzle_query_st));
+    query->options|= DRIZZLE_CON_ALLOCATED;
+  }
+  else
+    memset(query, 0, sizeof(drizzle_query_st));
+
+  query->drizzle= drizzle;
+
+  if (drizzle->query_list)
+    drizzle->query_list->prev= query;
+  query->next= drizzle->query_list;
+  drizzle->query_list= query;
+  drizzle->query_count++;
+  drizzle->query_new++;
+
+  return query;
+}
+
+void drizzle_query_free(drizzle_query_st *query)
+{
+  if (query->context != NULL && query->context_free_fn != NULL)
+    query->context_free_fn(query, query->context);
+
+  if (query->drizzle->query_list == query)
+    query->drizzle->query_list= query->next;
+  if (query->prev)
+    query->prev->next= query->next;
+  if (query->next)
+    query->next->prev= query->prev;
+  query->drizzle->query_count--;
+
+  if (query->options & DRIZZLE_QUERY_ALLOCATED)
+    free(query);
+}
+
+void drizzle_query_free_all(drizzle_st *drizzle)
+{
+  while (drizzle->query_list != NULL)
+    drizzle_query_free(drizzle->query_list);
+}
+
+drizzle_con_st *drizzle_query_con(drizzle_query_st *query)
+{
+  return query->con;
+}
+
+void drizzle_query_set_con(drizzle_query_st *query, drizzle_con_st *con)
+{
+  query->con= con;
+}
+
+drizzle_result_st *drizzle_query_result(drizzle_query_st *query)
+{
+  return query->result;
+}
+
+void drizzle_query_set_result(drizzle_query_st *query,
+                              drizzle_result_st *result)
+{
+  query->result= result;
+}
+
+char *drizzle_query_string(drizzle_query_st *query, size_t *size)
+{
+  *size= query->size;
+  return (char *)(query->string);
+}
+
+void drizzle_query_set_string(drizzle_query_st *query, const char *string,
+                              size_t size)
+{
+  query->string= string;
+  query->size= size;
+}
+
+drizzle_query_options_t drizzle_query_options(drizzle_query_st *query)
+{
+  return query->options;
+}
+
+void drizzle_query_set_options(drizzle_query_st *query,
+                               drizzle_query_options_t options)
+{
+  query->options= options;
+}
+
+void drizzle_query_add_options(drizzle_query_st *query,
+                               drizzle_query_options_t options)
+{
+  query->options|= options;
+}
+
+void drizzle_query_remove_options(drizzle_query_st *query,
+                                  drizzle_query_options_t options)
+{
+  query->options&= ~options;
+}
+
+void *drizzle_query_context(drizzle_query_st *query)
+{
+  return query->context;
+}
+
+void drizzle_query_set_context(drizzle_query_st *query, void *context)
+{
+  query->context= context;
+}
+
+void drizzle_query_set_context_free_fn(drizzle_query_st *query,
+                                       drizzle_query_context_free_fn *function)
+{
+  query->context_free_fn= function;
+}
+
+static void drizzle_query_run_state(drizzle_query_st* query,
+                                    drizzle_return_t* ret_ptr)
+{
+  switch (query->state)
+  {
+  case DRIZZLE_QUERY_STATE_INIT:
+    query->state= DRIZZLE_QUERY_STATE_QUERY;
+  case DRIZZLE_QUERY_STATE_QUERY:
+    query->result= drizzle_query(query->con, query->result, query->string,
+                                 query->size, ret_ptr);
+    if (*ret_ptr == DRIZZLE_RETURN_IO_WAIT)
+    {
+      return;
+    }
+    else if (*ret_ptr != DRIZZLE_RETURN_OK)
+    {
+      query->state= DRIZZLE_QUERY_STATE_DONE;
+      return;
+    }
+
+    query->state= DRIZZLE_QUERY_STATE_RESULT;
+
+  case DRIZZLE_QUERY_STATE_RESULT:
+    *ret_ptr= drizzle_result_buffer(query->result);
+    if (*ret_ptr == DRIZZLE_RETURN_IO_WAIT)
+    {
+      return;
+    }
+
+    query->state= DRIZZLE_QUERY_STATE_DONE;
+    return;
+
+  default:
+  case DRIZZLE_QUERY_STATE_DONE:
+    return;
+  }
+}
+
+drizzle_query_st *drizzle_query_run(drizzle_st *drizzle,
+                                    drizzle_return_t *ret_ptr)
+{
+  drizzle_options_t options;
+  drizzle_query_st *query;
+  drizzle_con_st *con;
+
+  if (drizzle->query_new == 0 && drizzle->query_running == 0)
+  {
+    *ret_ptr= DRIZZLE_RETURN_OK;
+    return NULL;
+  }
+
+  options= drizzle->options;
+  drizzle->options|= DRIZZLE_NON_BLOCKING;
+
+  /* Check to see if any queries need to be started. */
+  if (drizzle->query_new > 0)
+  {
+    for (query= drizzle->query_list; query != NULL; query= query->next)
+    {
+      if (query->state != DRIZZLE_QUERY_STATE_INIT)
+        continue;
+
+      drizzle->query_new--;
+      drizzle->query_running++;
+      assert(query->con->query == NULL);
+      query->con->query= query;
+
+      drizzle_query_run_state(query, ret_ptr);
+      if (*ret_ptr != DRIZZLE_RETURN_IO_WAIT)
+      {
+        assert(query->state == DRIZZLE_QUERY_STATE_DONE);
+        drizzle->query_running--;
+        drizzle->options= options;
+        query->con->query= NULL;
+        if (*ret_ptr == DRIZZLE_RETURN_ERROR_CODE || *ret_ptr == DRIZZLE_RETURN_OK)
+        {
+          return query;
+        }
+        return NULL;
+      }
+    }
+    assert(drizzle->query_new == 0);
+  }
+
+  while (1)
+  {
+    /* Loop through each active connection. */
+    while ((con= drizzle_con_ready(drizzle)) != NULL)
+    {
+      query= con->query;
+      drizzle_query_run_state(query, ret_ptr);
+      if (query->state == DRIZZLE_QUERY_STATE_DONE)
+      {
+        drizzle->query_running--;
+        drizzle->options= options;
+        con->query= NULL;
+        return query;
+      }
+      assert(*ret_ptr == DRIZZLE_RETURN_IO_WAIT);
+    }
+
+    if (options & DRIZZLE_NON_BLOCKING)
+    {
+      *ret_ptr= DRIZZLE_RETURN_IO_WAIT;
+      return NULL;
+    }
+
+    *ret_ptr= drizzle_con_wait(drizzle);
+    if (*ret_ptr != DRIZZLE_RETURN_OK)
+    {
+      drizzle->options= options;
+      return NULL;
+    }
+  }
+}
+
+drizzle_return_t drizzle_query_run_all(drizzle_st *drizzle)
+{
+  drizzle_return_t ret;
+
+  while (drizzle->query_new > 0 || drizzle->query_running > 0)
+  {
+    (void)drizzle_query_run(drizzle, &ret);
+    if (ret != DRIZZLE_RETURN_OK && ret != DRIZZLE_RETURN_ERROR_CODE)
+      return ret;
+  }
+
+  return DRIZZLE_RETURN_OK;
+}
+
+size_t drizzle_escape_string(char *to, const char *from, size_t from_size)
+{
+  size_t to_size= 0;
+
+  while (from_size > 0)
+  {
+    /* All multi-byte UTF8 characters have the high bit set for all bytes. */
+    if (!(*from & 0x80))
+    {
+      switch (*from)
+      {
+      case 0:
+      case '\n':
+      case '\r':
+      case '\\':
+      case '\'':
+      case '"':
+      case '\032':
+        *to++= '\\';
+        to_size++;
+      default:
+        break;
+      }
+    }
+
+    *to++= *from++;
+    from_size--;
+    to_size++;
+  }
+
+  *to= 0;
+
+  return to_size;
+}
+
+size_t drizzle_hex_string(char *to, const char *from, size_t from_size)
+{
+  static const char hex_map[]= "0123456789ABCDEF";
+  const char *from_end;
+
+  for (from_end= from + from_size; from != from_end; from++)
+  {
+    *to++= hex_map[((unsigned char) *from) >> 4];
+    *to++= hex_map[((unsigned char) *from) & 0xF];
+  }
+
+  *to= 0;
+
+  return from_size * 2;
+}
+
+void drizzle_mysql_password_hash(char *to, const char *from, size_t from_size)
+{
+  SHA1_CTX ctx;
+  uint8_t hash_tmp1[SHA1_DIGEST_LENGTH];
+  uint8_t hash_tmp2[SHA1_DIGEST_LENGTH];
+
+  SHA1Init(&ctx);
+  SHA1Update(&ctx, (const uint8_t*)from, from_size);
+  SHA1Final(hash_tmp1, &ctx);
+
+  SHA1Init(&ctx);
+  SHA1Update(&ctx, hash_tmp1, SHA1_DIGEST_LENGTH);
+  SHA1Final(hash_tmp2, &ctx);
+
+  (void)drizzle_hex_string(to, (char*)hash_tmp2, SHA1_DIGEST_LENGTH);
+}

=== added file 'libdrizzle/query.h'
--- libdrizzle/query.h	1970-01-01 00:00:00 +0000
+++ libdrizzle/query.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,217 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Query Declarations
+ */
+
+#ifndef __DRIZZLE_QUERY_H
+#define __DRIZZLE_QUERY_H
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/**
+ * @addtogroup drizzle_query Query Declarations
+ *
+ * @ingroup drizzle_client_interface
+ * These functions are used to issue queries on a connection. Single queries are
+ * made using the drizzle_query function, or you can queue multiple queries and
+ * run them concurrently using the other query functions.
+ * @{
+ */
+
+/**
+ * Send query to server. A \ref drizzle_result_st will be created for the
+ * results.
+ *
+ * @param[in] con connection to use to send the query.
+ * @param[in,out] result pointer to an unused structure that will be used for
+ *                       the results, or NULL to allocate a new structure.
+ * @param[in] query query string to send.
+ * @param[in] size length of the query string in bytes.
+ * @param[out] ret_ptr pointer to the result code.
+ * @return result, a pointer to the newly allocated result structure, or NULL
+ *         if the allocation failed.
+ */
+DRIZZLE_API
+drizzle_result_st *drizzle_query(drizzle_con_st *con, drizzle_result_st *result,
+                                 const char *query, size_t size,
+                                 drizzle_return_t *ret_ptr);
+
+/**
+ * Send query to server, using strlen to get the size of query buffer..
+ */
+DRIZZLE_API
+drizzle_result_st *drizzle_query_str(drizzle_con_st *con,
+                                     drizzle_result_st *result,
+                                     const char *query,
+                                     drizzle_return_t *ret_ptr);
+
+/**
+ * Send query incrementally.
+ */
+DRIZZLE_API
+drizzle_result_st *drizzle_query_inc(drizzle_con_st *con,
+                                     drizzle_result_st *result,
+                                     const char *query, size_t size,
+                                     size_t total, drizzle_return_t *ret_ptr);
+
+/**
+ * Add a query to be run concurrently.
+ */
+DRIZZLE_API
+drizzle_query_st *drizzle_query_add(drizzle_st *drizzle,
+                                    drizzle_query_st *query,
+                                    drizzle_con_st *con,
+                                    drizzle_result_st *result,
+                                    const char *query_string, size_t size,
+                                    drizzle_query_options_t options,
+                                    void *context);
+
+/**
+ * Initialize a query structure.
+ */
+DRIZZLE_API
+drizzle_query_st *drizzle_query_create(drizzle_st *drizzle,
+                                       drizzle_query_st *query);
+
+/**
+ * Free a query structure.
+ */
+DRIZZLE_API
+void drizzle_query_free(drizzle_query_st *query);
+
+/**
+ * Free a query structure.
+ */
+DRIZZLE_API
+void drizzle_query_free_all(drizzle_st *drizzle);
+
+/**
+ * Get connection struct for a query.
+ */
+DRIZZLE_API
+drizzle_con_st *drizzle_query_con(drizzle_query_st *query);
+
+/**
+ * Set connection struct for a query.
+ */
+DRIZZLE_API
+void drizzle_query_set_con(drizzle_query_st *query, drizzle_con_st *con);
+
+/**
+ * Get result struct for a query.
+ */
+DRIZZLE_API
+drizzle_result_st *drizzle_query_result(drizzle_query_st *query);
+
+/**
+ * Set result struct for a query.
+ */
+DRIZZLE_API
+void drizzle_query_set_result(drizzle_query_st *query,
+                              drizzle_result_st *result);
+
+/**
+ * Get query string for a query.
+ */
+DRIZZLE_API
+char *drizzle_query_string(drizzle_query_st *query, size_t *size);
+
+/**
+ * Set query string for a query.
+ */
+DRIZZLE_API
+void drizzle_query_set_string(drizzle_query_st *query, const char *string,
+                              size_t size);
+
+/**
+ * Get options for a query. 
+ */
+DRIZZLE_API
+drizzle_query_options_t drizzle_query_options(drizzle_query_st *query);
+
+/**
+ * Set options for a query.
+ */
+DRIZZLE_API
+void drizzle_query_set_options(drizzle_query_st *query,
+                               drizzle_query_options_t options);
+
+/**
+ * Add options for a query.
+ */
+DRIZZLE_API
+void drizzle_query_add_options(drizzle_query_st *query,
+                               drizzle_query_options_t options);
+
+/**
+ * Remove options for a query.
+ */
+DRIZZLE_API
+void drizzle_query_remove_options(drizzle_query_st *query,
+                                  drizzle_query_options_t options);
+
+/**
+ * Get application context for a query.
+ */
+DRIZZLE_API
+void *drizzle_query_context(drizzle_query_st *query);
+
+/**
+ * Set application context for a query.
+ */
+DRIZZLE_API
+void drizzle_query_set_context(drizzle_query_st *query, void *context);
+
+/**
+ * Set callback function when the context pointer should be freed.
+ */
+DRIZZLE_API
+void drizzle_query_set_context_free_fn(drizzle_query_st *query,
+                                       drizzle_query_context_free_fn *function);
+
+/**
+ * Run queries concurrently, returning when one is complete.
+ */
+DRIZZLE_API
+drizzle_query_st *drizzle_query_run(drizzle_st *drizzle,
+                                    drizzle_return_t *ret_ptr);
+
+/**
+ * Run queries until they are all complete. Returns \ref DRIZZLE_RETURN_OK if
+ * all queries complete, even if some return errors. This returns immediately
+ * if some other error occurs, leaving some queries unprocessed. You must call
+ * drizzle_result_error_code() to check if each query succeeded.
+ */
+DRIZZLE_API
+drizzle_return_t drizzle_query_run_all(drizzle_st *drizzle);
+
+/*
+ * Escape a string or encode a string in hexadecimal. The return value is the
+ * size of the output string in to.
+ */
+DRIZZLE_API
+size_t drizzle_escape_string(char *to, const char *from, size_t from_size);
+DRIZZLE_API
+size_t drizzle_hex_string(char *to, const char *from, size_t from_size);
+DRIZZLE_API
+void drizzle_mysql_password_hash(char *to, const char *from, size_t from_size);
+
+/** @} */
+
+#ifdef __cplusplus
+}
+#endif
+
+#endif /* __DRIZZLE_QUERY_H */

=== added file 'libdrizzle/result.c'
--- libdrizzle/result.c	1970-01-01 00:00:00 +0000
+++ libdrizzle/result.c	2010-08-12 20:27:32 +0000
@@ -0,0 +1,558 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Result definitions
+ */
+
+#include "common.h"
+
+/*
+ * Common definitions
+ */
+
+drizzle_result_st *drizzle_result_create(drizzle_con_st *con,
+                                         drizzle_result_st *result)
+{
+  if (result == NULL)
+  {
+    result= malloc(sizeof(drizzle_result_st));
+    if (result == NULL)
+    {
+      drizzle_set_error(con->drizzle, "drizzle_result_create", "malloc");
+      return NULL;
+    }
+
+    memset(result, 0, sizeof(drizzle_result_st));
+    result->options|= DRIZZLE_RESULT_ALLOCATED;
+  }
+  else
+    memset(result, 0, sizeof(drizzle_result_st));
+
+  result->con= con;
+  con->result= result;
+
+  if (con->result_list)
+    con->result_list->prev= result;
+  result->next= con->result_list;
+  con->result_list= result;
+  con->result_count++;
+
+  return result;
+}
+
+drizzle_result_st *drizzle_result_clone(drizzle_con_st *con,
+                                        drizzle_result_st *result,
+                                        drizzle_result_st *from)
+{
+  result= drizzle_result_create(con, result);
+  if (result == NULL)
+    return NULL;
+
+  result->options|= (from->options &
+                     (drizzle_result_options_t)~DRIZZLE_RESULT_ALLOCATED);
+
+  drizzle_result_set_info(result, from->info);
+  result->error_code= from->error_code;
+  drizzle_result_set_sqlstate(result, from->sqlstate);
+  result->warning_count= from->warning_count;
+  result->insert_id= from->insert_id;
+  result->affected_rows= from->affected_rows;
+  result->column_count= from->column_count;
+  result->row_count= from->row_count;
+
+  return result;
+}
+
+void drizzle_result_free(drizzle_result_st *result)
+{
+  drizzle_column_st *column;
+  uint64_t x;
+
+  for (column= result->column_list; column != NULL; column= result->column_list)
+    drizzle_column_free(column);
+
+  if (result->column_buffer != NULL)
+    free(result->column_buffer);
+
+  if (result->options & DRIZZLE_RESULT_BUFFER_ROW)
+  {
+    for (x= 0; x < result->row_count; x++)
+      drizzle_row_free(result, result->row_list[x]);
+
+    free(result->row_list);
+    free(result->field_sizes_list);
+  }
+
+  if (result->con->result_list == result)
+    result->con->result_list= result->next;
+  if (result->prev)
+    result->prev->next= result->next;
+  if (result->next)
+    result->next->prev= result->prev;
+  result->con->result_count--;
+
+  if (result->options & DRIZZLE_RESULT_ALLOCATED)
+    free(result);
+}
+
+void drizzle_result_free_all(drizzle_con_st *con)
+{
+  while (con->result_list != NULL)
+    drizzle_result_free(con->result_list);
+}
+
+drizzle_con_st *drizzle_result_drizzle_con(drizzle_result_st *result)
+{
+  return result->con;
+}
+
+bool drizzle_result_eof(drizzle_result_st *result)
+{
+  return result->options & DRIZZLE_RESULT_EOF_PACKET;
+}
+
+const char *drizzle_result_info(drizzle_result_st *result)
+{
+  return result->info;
+}
+
+const char *drizzle_result_error(drizzle_result_st *result)
+{
+  return result->info;
+}
+
+uint16_t drizzle_result_error_code(drizzle_result_st *result)
+{
+  return result->error_code;
+}
+
+const char *drizzle_result_sqlstate(drizzle_result_st *result)
+{
+  return result->sqlstate;
+}
+
+uint16_t drizzle_result_warning_count(drizzle_result_st *result)
+{
+  return result->warning_count;
+}
+
+uint64_t drizzle_result_insert_id(drizzle_result_st *result)
+{
+  return result->insert_id;
+}
+
+uint64_t drizzle_result_affected_rows(drizzle_result_st *result)
+{
+  return result->affected_rows;
+}
+
+uint16_t drizzle_result_column_count(drizzle_result_st *result)
+{
+  return result->column_count;
+}
+
+uint64_t drizzle_result_row_count(drizzle_result_st *result)
+{
+  return result->row_count;
+}
+
+/*
+ * Client definitions
+ */
+
+drizzle_result_st *drizzle_result_read(drizzle_con_st *con,
+                                       drizzle_result_st *result,
+                                       drizzle_return_t *ret_ptr)
+{
+  if (drizzle_state_none(con))
+  {
+    con->result= drizzle_result_create(con, result);
+    if (con->result == NULL)
+    {
+      *ret_ptr= DRIZZLE_RETURN_MEMORY;
+      return NULL;
+    }
+
+    drizzle_state_push(con, drizzle_state_result_read);
+    drizzle_state_push(con, drizzle_state_packet_read);
+  }
+
+  *ret_ptr= drizzle_state_loop(con);
+  return con->result;
+}
+
+drizzle_return_t drizzle_result_buffer(drizzle_result_st *result)
+{
+  drizzle_return_t ret;
+  drizzle_row_t row;
+  drizzle_row_t *row_list;
+  size_t **field_sizes_list;
+
+  if (!(result->options & DRIZZLE_RESULT_BUFFER_COLUMN))
+  {
+    ret= drizzle_column_buffer(result);
+    if (ret != DRIZZLE_RETURN_OK)
+      return ret;
+  }
+
+  if (result->column_count == 0)
+  {
+    result->options|= DRIZZLE_RESULT_BUFFER_ROW;
+    return DRIZZLE_RETURN_OK;
+  }
+
+  while (1)
+  {
+    row= drizzle_row_buffer(result, &ret);
+    if (ret != DRIZZLE_RETURN_OK)
+      return ret;
+
+    if (row == NULL)
+      break;
+
+    if (result->row_list_size < result->row_count)
+    {
+      row_list= realloc(result->row_list, sizeof(drizzle_row_t) *
+                        ((size_t)(result->row_list_size) +
+                         DRIZZLE_ROW_GROW_SIZE));
+      if (row_list == NULL)
+      {
+        drizzle_row_free(result, row);
+        drizzle_set_error(result->con->drizzle, "drizzle_result_buffer",
+                          "realloc");
+        return DRIZZLE_RETURN_MEMORY;
+      }
+
+      result->row_list= row_list;
+
+      field_sizes_list= realloc(result->field_sizes_list, sizeof(size_t *) *
+                                ((size_t)(result->row_list_size) +
+                                 DRIZZLE_ROW_GROW_SIZE));
+      if (field_sizes_list == NULL)
+      {
+        drizzle_row_free(result, row);
+        drizzle_set_error(result->con->drizzle, "drizzle_result_buffer",
+                          "realloc");
+        return DRIZZLE_RETURN_MEMORY;
+      }
+
+      result->field_sizes_list= field_sizes_list;
+
+      result->row_list_size+= DRIZZLE_ROW_GROW_SIZE;
+    }
+
+    result->row_list[result->row_current - 1]= row;
+    result->field_sizes_list[result->row_current - 1]= result->field_sizes;
+  }
+
+  result->options|= DRIZZLE_RESULT_BUFFER_ROW;
+  return DRIZZLE_RETURN_OK;
+}
+
+size_t drizzle_result_row_size(drizzle_result_st *result)
+{
+  return result->con->packet_size;
+}
+
+/*
+ * Server definitions
+ */
+
+drizzle_return_t drizzle_result_write(drizzle_con_st *con,
+                                      drizzle_result_st *result, bool flush)
+{
+  if (drizzle_state_none(con))
+  {
+    con->result= result;
+
+    if (flush)
+      drizzle_state_push(con, drizzle_state_write);
+
+    drizzle_state_push(con, drizzle_state_result_write);
+  }
+
+  return drizzle_state_loop(con);
+}
+
+void drizzle_result_set_row_size(drizzle_result_st *result, size_t size)
+{
+  result->con->packet_size= size;
+}
+
+void drizzle_result_calc_row_size(drizzle_result_st *result,
+                                  const drizzle_field_t *field,
+                                  const size_t *size)
+{
+  uint16_t x;
+
+  result->con->packet_size= 0;
+
+  for (x= 0; x < result->column_count; x++)
+  {
+    if (field[x] == NULL)
+      result->con->packet_size++;
+    else if (size[x] < 251)
+      result->con->packet_size+= (1 + size[x]);
+    else if (size[x] < 65536)
+      result->con->packet_size+= (3 + size[x]);
+    else if (size[x] < 16777216)
+      result->con->packet_size+= (4 + size[x]);
+    else
+      result->con->packet_size+= (9 + size[x]);
+  }
+}
+
+void drizzle_result_set_eof(drizzle_result_st *result, bool is_eof)
+{
+  if (is_eof)
+    result->options|= DRIZZLE_RESULT_EOF_PACKET;
+  else
+    result->options&= (drizzle_result_options_t)~DRIZZLE_RESULT_EOF_PACKET;
+}
+
+void drizzle_result_set_info(drizzle_result_st *result, const char *info)
+{
+  if (info == NULL)
+    result->info[0]= 0;
+  else
+  {
+    strncpy(result->info, info, DRIZZLE_MAX_INFO_SIZE);
+    result->info[DRIZZLE_MAX_INFO_SIZE - 1]= 0;
+  }
+}
+
+void drizzle_result_set_error(drizzle_result_st *result, const char *error)
+{
+  drizzle_result_set_info(result, error);
+}
+
+void drizzle_result_set_error_code(drizzle_result_st *result,
+                                   uint16_t error_code)
+{
+  result->error_code= error_code;
+}
+
+void drizzle_result_set_sqlstate(drizzle_result_st *result,
+                                 const char *sqlstate)
+{
+  if (sqlstate == NULL)
+    result->sqlstate[0]= 0;
+  else
+  {
+    strncpy(result->sqlstate, sqlstate, DRIZZLE_MAX_SQLSTATE_SIZE + 1);
+    result->sqlstate[DRIZZLE_MAX_SQLSTATE_SIZE]= 0;
+  }
+}
+
+void drizzle_result_set_warning_count(drizzle_result_st *result,
+                                      uint16_t warning_count)
+{
+  result->warning_count= warning_count;
+}
+
+void drizzle_result_set_insert_id(drizzle_result_st *result,
+                                  uint64_t insert_id)
+{
+  result->insert_id= insert_id;
+}
+
+void drizzle_result_set_affected_rows(drizzle_result_st *result,
+                                      uint64_t affected_rows)
+{
+  result->affected_rows= affected_rows;
+}
+
+void drizzle_result_set_column_count(drizzle_result_st *result,
+                                     uint16_t column_count)
+{
+  result->column_count= column_count;
+}
+
+/*
+ * Internal state functions.
+ */
+
+drizzle_return_t drizzle_state_result_read(drizzle_con_st *con)
+{
+  drizzle_return_t ret;
+
+  drizzle_log_debug(con->drizzle, "drizzle_state_result_read");
+
+  /* Assume the entire result packet will fit in the buffer. */
+  if (con->buffer_size < con->packet_size)
+  {
+    drizzle_state_push(con, drizzle_state_read);
+    return DRIZZLE_RETURN_OK;
+  }
+
+  if (con->buffer_ptr[0] == 0)
+  {
+    con->buffer_ptr++;
+    /* We can ignore the returns since we've buffered the entire packet. */
+    con->result->affected_rows= drizzle_unpack_length(con, &ret);
+    con->result->insert_id= drizzle_unpack_length(con, &ret);
+    con->status= drizzle_get_byte2(con->buffer_ptr);
+    con->result->warning_count= drizzle_get_byte2(con->buffer_ptr + 2);
+    con->buffer_ptr+= 4;
+    con->buffer_size-= 5;
+    con->packet_size-= 5;
+    if (con->packet_size > 0)
+    {
+      /* Skip one byte for message size. */
+      con->buffer_ptr+= 1;
+      con->buffer_size-= 1;
+      con->packet_size-= 1;
+    }
+    ret= DRIZZLE_RETURN_OK;
+  }
+  else if (con->buffer_ptr[0] == 254)
+  {
+    con->result->options= DRIZZLE_RESULT_EOF_PACKET;
+    con->result->warning_count= drizzle_get_byte2(con->buffer_ptr + 1);
+    con->status= drizzle_get_byte2(con->buffer_ptr + 3);
+    con->buffer_ptr+= 5;
+    con->buffer_size-= 5;
+    con->packet_size-= 5;
+    ret= DRIZZLE_RETURN_OK;
+  }
+  else if (con->buffer_ptr[0] == 255)
+  {
+    con->result->error_code= drizzle_get_byte2(con->buffer_ptr + 1);
+    con->drizzle->error_code= con->result->error_code;
+    /* Byte 3 is always a '#' character, skip it. */
+    memcpy(con->result->sqlstate, con->buffer_ptr + 4,
+           DRIZZLE_MAX_SQLSTATE_SIZE);
+    con->result->sqlstate[DRIZZLE_MAX_SQLSTATE_SIZE]= 0;
+    memcpy(con->drizzle->sqlstate, con->result->sqlstate,
+           DRIZZLE_MAX_SQLSTATE_SIZE + 1);
+    con->buffer_ptr+= 9;
+    con->buffer_size-= 9;
+    con->packet_size-= 9;
+    ret= DRIZZLE_RETURN_ERROR_CODE;
+  }
+  else
+  {
+    /* We can ignore the return since we've buffered the entire packet. */
+    con->result->column_count= (uint16_t)drizzle_unpack_length(con, &ret);
+    ret= DRIZZLE_RETURN_OK;
+  }
+
+  if (con->packet_size > 0)
+  {
+    snprintf(con->drizzle->last_error, DRIZZLE_MAX_ERROR_SIZE, "%.*s",
+             (int32_t)con->packet_size, con->buffer_ptr);
+    snprintf(con->result->info, DRIZZLE_MAX_INFO_SIZE, "%.*s",
+             (int32_t)con->packet_size, con->buffer_ptr);
+    con->buffer_ptr+= con->packet_size;
+    con->buffer_size-= con->packet_size;
+    con->packet_size= 0;
+  }
+
+  drizzle_state_pop(con);
+  return ret;
+}
+
+drizzle_return_t drizzle_state_result_write(drizzle_con_st *con)
+{
+  uint8_t *start= con->buffer_ptr + con->buffer_size;
+  uint8_t *ptr;
+  drizzle_result_st *result= con->result;
+
+  drizzle_log_debug(con->drizzle, "drizzle_state_result_write");
+
+  /* Calculate max packet size. */
+  con->packet_size= 1 /* OK/Field Count/EOF/Error */
+                  + 9 /* Affected rows */
+                  + 9 /* Insert ID */
+                  + 2 /* Status */
+                  + 2 /* Warning count */
+                  + strlen(result->info); /* Info/error message */
+
+  /* Assume the entire result packet will fit in the buffer. */
+  if ((con->packet_size + 4) > DRIZZLE_MAX_BUFFER_SIZE)
+  {
+    drizzle_set_error(con->drizzle, "drizzle_state_result_write",
+                      "buffer too small:%zu", con->packet_size + 4);
+    return DRIZZLE_RETURN_INTERNAL_ERROR;
+  }
+
+  /* Flush buffer if there is not enough room. */
+  if (((size_t)DRIZZLE_MAX_BUFFER_SIZE - (size_t)(start - con->buffer)) <
+      con->packet_size)
+  {
+    drizzle_state_push(con, drizzle_state_write);
+    return DRIZZLE_RETURN_OK;
+  }
+
+  /* Store packet size at the end since it may change. */
+  ptr= start;
+  ptr[3]= con->packet_number;
+  con->packet_number++;
+  ptr+= 4;
+
+  if (result->options & DRIZZLE_RESULT_EOF_PACKET)
+  {
+    ptr[0]= 254;
+    ptr++;
+
+    drizzle_set_byte2(ptr, result->warning_count);
+    ptr+= 2;
+
+    drizzle_set_byte2(ptr, con->status);
+    ptr+= 2;
+  }
+  else if (result->error_code != 0)
+  {
+    ptr[0]= 255;
+    ptr++;
+
+    drizzle_set_byte2(ptr, result->error_code);
+    ptr+= 2;
+
+    ptr[0]= '#';
+    ptr++;
+
+    memcpy(ptr, result->sqlstate, DRIZZLE_MAX_SQLSTATE_SIZE);
+    ptr+= DRIZZLE_MAX_SQLSTATE_SIZE;
+
+    memcpy(ptr, result->info, strlen(result->info));
+    ptr+= strlen(result->info);
+  }
+  else if (result->column_count == 0)
+  {
+    ptr[0]= 0;
+    ptr++;
+
+    ptr= drizzle_pack_length(result->affected_rows, ptr);
+    ptr= drizzle_pack_length(result->insert_id, ptr);
+
+    drizzle_set_byte2(ptr, con->status);
+    ptr+= 2;
+
+    drizzle_set_byte2(ptr, result->warning_count);
+    ptr+= 2;
+
+    memcpy(ptr, result->info, strlen(result->info));
+    ptr+= strlen(result->info);
+  }
+  else
+    ptr= drizzle_pack_length(result->column_count, ptr);
+
+  con->packet_size= ((size_t)(ptr - start) - 4);
+  con->buffer_size+= (4 + con->packet_size);
+
+  /* Store packet size now. */
+  drizzle_set_byte3(start, con->packet_size);
+
+  drizzle_state_pop(con);
+  return DRIZZLE_RETURN_OK;
+}

=== added file 'libdrizzle/result.h'
--- libdrizzle/result.h	1970-01-01 00:00:00 +0000
+++ libdrizzle/result.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,131 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Result Declarations
+ */
+
+#ifndef __DRIZZLE_RESULT_H
+#define __DRIZZLE_RESULT_H
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/**
+ * @addtogroup drizzle_result Result Declarations
+ * @ingroup drizzle_client_interface
+ * @ingroup drizzle_server_interface
+ *
+ * These are core result functions used by both clients and servers.
+ * @{
+ */
+
+/**
+ * Initialize a result structure.
+ */
+DRIZZLE_API
+drizzle_result_st *drizzle_result_create(drizzle_con_st *con,
+                                         drizzle_result_st *result);
+
+/**
+ * Clone a connection structure.
+ */
+DRIZZLE_API
+drizzle_result_st *drizzle_result_clone(drizzle_con_st *con,
+                                        drizzle_result_st *result,
+                                        drizzle_result_st *from);
+
+/**
+ * Free a result structure.
+ */
+DRIZZLE_API
+void drizzle_result_free(drizzle_result_st *result);
+
+/**
+ * Free all result structures.
+ */
+DRIZZLE_API
+void drizzle_result_free_all(drizzle_con_st *con);
+
+/**
+ * Get the drizzle_con_st struct that the result belongs to.
+ */
+DRIZZLE_API
+drizzle_con_st *drizzle_result_drizzle_con(drizzle_result_st *result);
+
+/**
+ * Get EOF flag for a result.
+ */
+DRIZZLE_API
+bool drizzle_result_eof(drizzle_result_st *result);
+
+/**
+ * Get information string for a result.
+ */
+DRIZZLE_API
+const char *drizzle_result_info(drizzle_result_st *result);
+
+/**
+ * Get error string for a result.
+ */
+DRIZZLE_API
+const char *drizzle_result_error(drizzle_result_st *result);
+
+/**
+ * Get server defined error code for a result.
+ */
+DRIZZLE_API
+uint16_t drizzle_result_error_code(drizzle_result_st *result);
+
+/**
+ * Get SQL state code for a result.
+ */
+DRIZZLE_API
+const char *drizzle_result_sqlstate(drizzle_result_st *result);
+
+/**
+ * Get the number of warnings encounted during a command.
+ */
+DRIZZLE_API
+uint16_t drizzle_result_warning_count(drizzle_result_st *result);
+
+/**
+ * Get inet ID of the last command, if any.
+ */
+DRIZZLE_API
+uint64_t drizzle_result_insert_id(drizzle_result_st *result);
+
+/**
+ * Get the number of affected rows during the command.
+ */
+DRIZZLE_API
+uint64_t drizzle_result_affected_rows(drizzle_result_st *result);
+
+/**
+ * Get the number of columns in a result set.
+ */
+DRIZZLE_API
+uint16_t drizzle_result_column_count(drizzle_result_st *result);
+
+/**
+ * Get the number of rows returned for the command.
+ */
+DRIZZLE_API
+uint64_t drizzle_result_row_count(drizzle_result_st *result);
+
+/** @} */
+
+#ifdef __cplusplus
+}
+#endif
+
+#endif /* __DRIZZLE_RESULT_H */

=== added file 'libdrizzle/result_client.h'
--- libdrizzle/result_client.h	1970-01-01 00:00:00 +0000
+++ libdrizzle/result_client.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,57 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Result Declarations for Clients
+ */
+
+#ifndef __DRIZZLE_RESULT_CLIENT_H
+#define __DRIZZLE_RESULT_CLIENT_H
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/**
+ * @addtogroup drizzle_result_client Result Declarations for Clients
+ * @ingroup drizzle_client_interface
+ *
+ * These functions read or buffer the result for a client command.
+ * @{
+ */
+
+/**
+ * Read result packet.
+ */
+DRIZZLE_API
+drizzle_result_st *drizzle_result_read(drizzle_con_st *con,
+                                       drizzle_result_st *result,
+                                       drizzle_return_t *ret_ptr);
+
+/**
+ * Buffer all data for a result.
+ */
+DRIZZLE_API
+drizzle_return_t drizzle_result_buffer(drizzle_result_st *result);
+
+/**
+ * Get result row packet size.
+ */
+DRIZZLE_API
+size_t drizzle_result_row_size(drizzle_result_st *result);
+
+/** @} */
+
+#ifdef __cplusplus
+}
+#endif
+
+#endif /* __DRIZZLE_RESULT_CLIENT_H */

=== added file 'libdrizzle/result_server.h'
--- libdrizzle/result_server.h	1970-01-01 00:00:00 +0000
+++ libdrizzle/result_server.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,118 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Result Declarations for Servers
+ */
+
+#ifndef __DRIZZLE_RESULT_SERVER_H
+#define __DRIZZLE_RESULT_SERVER_H
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/**
+ * @addtogroup drizzle_result_server Result Declarations for Servers
+ * @ingroup drizzle_server_interface
+ *
+ * These functions allow you to send result packets over a connection.
+ * @{
+ */
+
+/**
+ * Write result packet.
+ */
+DRIZZLE_API
+drizzle_return_t drizzle_result_write(drizzle_con_st *con,
+                                      drizzle_result_st *result, bool flush);
+
+/**
+ * Set result row packet size.
+ */
+DRIZZLE_API
+void drizzle_result_set_row_size(drizzle_result_st *result, size_t size);
+
+/**
+ * Set result row packet size from field and size arrays.
+ */
+DRIZZLE_API
+void drizzle_result_calc_row_size(drizzle_result_st *result,
+                                  const drizzle_field_t *field,
+                                  const size_t *size);
+
+/**
+ * Set information string for a result.
+ */
+DRIZZLE_API
+void drizzle_result_set_eof(drizzle_result_st *result, bool eof);
+
+/**
+ * Set information string for a result.
+ */
+DRIZZLE_API
+void drizzle_result_set_info(drizzle_result_st *result, const char *info);
+
+/**
+ * Set error string for a result.
+ */
+DRIZZLE_API
+void drizzle_result_set_error(drizzle_result_st *result, const char *error);
+
+/**
+ * Set server defined error code for a result.
+ */
+DRIZZLE_API
+void drizzle_result_set_error_code(drizzle_result_st *result,
+                                   uint16_t error_code);
+
+/**
+ * Set SQL state code for a result.
+ */
+DRIZZLE_API
+void drizzle_result_set_sqlstate(drizzle_result_st *result,
+                                 const char *sqlstate);
+
+/**
+ * Set the number of warnings encounted during a command.
+ */
+DRIZZLE_API
+void drizzle_result_set_warning_count(drizzle_result_st *result,
+                                      uint16_t warning_count);
+
+/**
+ * Set inet ID of the last command, if any.
+ */
+DRIZZLE_API
+void drizzle_result_set_insert_id(drizzle_result_st *result,
+                                  uint64_t insert_id);
+
+/**
+ * Set the number of affected rows during the command.
+ */
+DRIZZLE_API
+void drizzle_result_set_affected_rows(drizzle_result_st *result,
+                                      uint64_t affected_rows);
+
+/**
+ * Set the number of fields in a result set.
+ */
+DRIZZLE_API
+void drizzle_result_set_column_count(drizzle_result_st *result,
+                                     uint16_t column_count);
+
+/** @} */
+
+#ifdef __cplusplus
+}
+#endif
+
+#endif /* __DRIZZLE_RESULT_SERVER_H */

=== added file 'libdrizzle/row.c'
--- libdrizzle/row.c	1970-01-01 00:00:00 +0000
+++ libdrizzle/row.c	2010-08-12 20:27:32 +0000
@@ -0,0 +1,216 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Row definitions
+ */
+
+#include "common.h"
+
+/*
+ * Client definitions
+ */
+
+uint64_t drizzle_row_read(drizzle_result_st *result, drizzle_return_t *ret_ptr)
+{
+  if (drizzle_state_none(result->con))
+  {
+    drizzle_state_push(result->con, drizzle_state_row_read);
+    drizzle_state_push(result->con, drizzle_state_packet_read);
+  }
+
+  *ret_ptr= drizzle_state_loop(result->con);
+
+  return result->row_current;
+}
+
+drizzle_row_t drizzle_row_buffer(drizzle_result_st *result,
+                                 drizzle_return_t *ret_ptr)
+{
+  size_t total;
+  drizzle_field_t field;
+  drizzle_row_t row;
+
+  if (result->row == NULL)
+  {
+    if (drizzle_row_read(result, ret_ptr) == 0 || *ret_ptr != DRIZZLE_RETURN_OK)
+      return NULL;
+
+    result->row= malloc((sizeof(drizzle_field_t) + sizeof(size_t)) *
+                        result->column_count);
+    if (result->row == NULL)
+    {
+      drizzle_set_error(result->con->drizzle, "drizzle_row_buffer", "malloc");
+      *ret_ptr= DRIZZLE_RETURN_MEMORY;
+      return NULL;
+    }
+
+    result->field_sizes= (size_t *)(result->row + result->column_count);
+  }
+
+  while (1)
+  {
+    field= drizzle_field_buffer(result, &total, ret_ptr);
+    if (*ret_ptr == DRIZZLE_RETURN_ROW_END)
+      break;
+    if (*ret_ptr != DRIZZLE_RETURN_OK)
+    {
+      if (*ret_ptr != DRIZZLE_RETURN_IO_WAIT)
+      {
+        free(result->row);
+        result->row= NULL;
+        result->field_sizes= NULL;
+      }
+
+      return NULL;
+    }
+
+    result->row[result->field_current - 1]= field;
+    result->field_sizes[result->field_current - 1]= total;
+  }
+
+  *ret_ptr= DRIZZLE_RETURN_OK;
+  row= result->row;
+  result->row= NULL;
+
+  return row;
+}
+
+void drizzle_row_free(drizzle_result_st *result, drizzle_row_t row)
+{
+  uint16_t x;
+
+  for (x= 0; x < result->column_count; x++)
+      drizzle_field_free(row[x]);
+
+  free(row);
+}
+
+size_t *drizzle_row_field_sizes(drizzle_result_st *result)
+{
+  return result->field_sizes;
+}
+
+drizzle_row_t drizzle_row_next(drizzle_result_st *result)
+{
+  if (result->row_current == result->row_count)
+    return NULL;
+
+  result->field_sizes= result->field_sizes_list[result->row_current];
+  result->row_current++;
+  return result->row_list[result->row_current - 1];
+}
+
+drizzle_row_t drizzle_row_prev(drizzle_result_st *result)
+{
+  if (result->row_current == 0)
+    return NULL;
+
+  result->row_current--;
+  result->field_sizes= result->field_sizes_list[result->row_current];
+  return result->row_list[result->row_current];
+}
+
+void drizzle_row_seek(drizzle_result_st *result, uint64_t row)
+{
+  if (row <= result->row_count)
+    result->row_current= row;
+}
+
+drizzle_row_t drizzle_row_index(drizzle_result_st *result, uint64_t row)
+{
+  if (row >= result->row_count)
+    return NULL;
+
+  return result->row_list[row];
+}
+
+uint64_t drizzle_row_current(drizzle_result_st *result)
+{
+  return result->row_current;
+}
+
+/*
+ * Server definitions
+ */
+
+drizzle_return_t drizzle_row_write(drizzle_result_st *result)
+{
+  if (drizzle_state_none(result->con))
+    drizzle_state_push(result->con, drizzle_state_row_write);
+
+  return drizzle_state_loop(result->con);
+}
+
+/*
+ * Internal state functions.
+ */
+
+drizzle_return_t drizzle_state_row_read(drizzle_con_st *con)
+{
+  drizzle_log_debug(con->drizzle, "drizzle_state_row_read");
+
+  if (con->packet_size != 0 && con->buffer_size < con->packet_size)
+  {
+    drizzle_state_push(con, drizzle_state_read);
+    return DRIZZLE_RETURN_OK;
+  }
+
+  if (con->packet_size == 5 && con->buffer_ptr[0] == 254)
+  {
+    /* Got EOF packet, no more rows. */
+    con->result->row_current= 0;
+    con->result->warning_count= drizzle_get_byte2(con->buffer_ptr + 1);
+    con->status= drizzle_get_byte2(con->buffer_ptr + 3);
+    con->buffer_ptr+= 5;
+    con->buffer_size-= 5;
+  }
+  else if (con->buffer_ptr[0] == 255)
+  {
+    drizzle_state_pop(con);
+    drizzle_state_push(con, drizzle_state_result_read);
+    return DRIZZLE_RETURN_OK;
+  }
+  else if (con->result->options & DRIZZLE_RESULT_ROW_BREAK)
+    con->result->options&= (drizzle_result_options_t)~DRIZZLE_RESULT_ROW_BREAK;
+  else
+  {
+    con->result->row_count++;
+    con->result->row_current++;
+    con->result->field_current= 0;
+  }
+
+  drizzle_state_pop(con);
+  return DRIZZLE_RETURN_OK;
+}
+
+drizzle_return_t drizzle_state_row_write(drizzle_con_st *con)
+{
+  uint8_t *start= con->buffer_ptr + con->buffer_size;
+
+  drizzle_log_debug(con->drizzle, "drizzle_state_row_write");
+
+  /* Flush buffer if there is not enough room. */
+  if (((size_t)DRIZZLE_MAX_BUFFER_SIZE - (size_t)(start - con->buffer)) < 4)
+  {
+    drizzle_state_push(con, drizzle_state_write);
+    return DRIZZLE_RETURN_OK;
+  }
+
+  drizzle_set_byte3(start, con->packet_size);
+  start[3]= con->packet_number;
+  con->packet_number++;
+
+  con->buffer_size+= 4;
+
+  drizzle_state_pop(con);
+  return DRIZZLE_RETURN_OK;
+}

=== added file 'libdrizzle/row_client.h'
--- libdrizzle/row_client.h	1970-01-01 00:00:00 +0000
+++ libdrizzle/row_client.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,101 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Row Declarations for Clients
+ */
+
+#ifndef __DRIZZLE_ROW_CLIENT_H
+#define __DRIZZLE_ROW_CLIENT_H
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/**
+ * @addtogroup drizzle_row_client Row Declarations for Clients
+ * @ingroup drizzle_client_interface
+ *
+ * These functions allow you to access rows in a result set. If the result is
+ * unbuffered, you can read and buffer rows one at a time. If the rows are
+ * buffered in the result, the drizzle_row_next() and related functions can be
+ * used.
+ * @{
+ */
+
+/**
+ * Get next row number for unbuffered results. Use the drizzle_field* functions
+ * to read individual fields after this function succeeds.
+ */
+DRIZZLE_API
+uint64_t drizzle_row_read(drizzle_result_st *result, drizzle_return_t *ret_ptr);
+
+/**
+ * Read and buffer one row. The returned row must be freed by the caller with
+ * drizzle_row_free().
+ *
+ * @param[in,out] result pointer to the result structure to read from.
+ * @param[out] ret_pointer Standard drizzle return value.
+ * @return the row that was read, or NULL if there are no more rows.
+ */
+DRIZZLE_API
+drizzle_row_t drizzle_row_buffer(drizzle_result_st *result,
+                                 drizzle_return_t *ret_ptr);
+
+/**
+ * Free a row that was buffered with drizzle_row_buffer().
+ */
+DRIZZLE_API
+void drizzle_row_free(drizzle_result_st *result, drizzle_row_t row);
+
+/**
+ * Get an array of all field sizes for buffered rows.
+ */
+DRIZZLE_API
+size_t *drizzle_row_field_sizes(drizzle_result_st *result);
+
+/**
+ * Get next buffered row from a fully buffered result.
+ */
+DRIZZLE_API
+drizzle_row_t drizzle_row_next(drizzle_result_st *result);
+
+/**
+ * Get previous buffered row from a fully buffered result.
+ */
+DRIZZLE_API
+drizzle_row_t drizzle_row_prev(drizzle_result_st *result);
+
+/**
+ * Seek to the given buffered row in a fully buffered result.
+ */
+DRIZZLE_API
+void drizzle_row_seek(drizzle_result_st *result, uint64_t row);
+
+/**
+ * Get the given buffered row from a fully buffered result.
+ */
+DRIZZLE_API
+drizzle_row_t drizzle_row_index(drizzle_result_st *result, uint64_t row);
+
+/**
+ * Get current row number.
+ */
+DRIZZLE_API
+uint64_t drizzle_row_current(drizzle_result_st *result);
+
+/** @} */
+
+#ifdef __cplusplus
+}
+#endif
+
+#endif /* __DRIZZLE_ROW_CLIENT_H */

=== added file 'libdrizzle/row_server.h'
--- libdrizzle/row_server.h	1970-01-01 00:00:00 +0000
+++ libdrizzle/row_server.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,43 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Row Declarations for Servers
+ */
+
+#ifndef __DRIZZLE_ROW_SERVER_H
+#define __DRIZZLE_ROW_SERVER_H
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/**
+ * @addtogroup drizzle_row_server Row Declarations for Servers
+ * @ingroup drizzle_server_interface
+ *
+ * These functions allow you to send row information over a connection.
+ * @{
+ */
+
+/**
+ * Write next row.
+ */
+DRIZZLE_API
+drizzle_return_t drizzle_row_write(drizzle_result_st *result);
+
+/** @} */
+
+#ifdef __cplusplus
+}
+#endif
+
+#endif /* __DRIZZLE_ROW_SERVER_H */

=== added file 'libdrizzle/sha1.c'
--- libdrizzle/sha1.c	1970-01-01 00:00:00 +0000
+++ libdrizzle/sha1.c	2010-08-12 20:27:32 +0000
@@ -0,0 +1,175 @@
+/**
+ * @file
+ * @brief SHA1 Definitions
+ */
+
+/*
+ * SHA-1 in C
+ * By Steve Reid <steve@edmweb.com>
+ * 100% Public Domain
+ *
+ * Test Vectors (from FIPS PUB 180-1)
+ * "abc"
+ *   A9993E36 4706816A BA3E2571 7850C26C 9CD0D89D
+ * "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq"
+ *   84983E44 1C3BD26E BAAE4AA1 F95129E5 E54670F1
+ * A million repetitions of "a"
+ *   34AA973C D4C4DAA4 F61EEB2B DBAD2731 6534016F
+ */
+
+#include "common.h"
+
+#define rol(value, bits) (((value) << (bits)) | ((value) >> (32 - (bits))))
+
+/*
+ * blk0() and blk() perform the initial expand.
+ * I got the idea of expanding during the round function from SSLeay
+ */
+#ifndef WORDS_BIGENDIAN
+# define blk0(i) (block->l[i] = (rol(block->l[i],24)&0xFF00FF00) \
+    |(rol(block->l[i],8)&0x00FF00FF))
+#else
+# define blk0(i) block->l[i]
+#endif
+#define blk(i) (block->l[i&15] = rol(block->l[(i+13)&15]^block->l[(i+8)&15] \
+    ^block->l[(i+2)&15]^block->l[i&15],1))
+
+/*
+ * (R0+R1), R2, R3, R4 are the different operations (rounds) used in SHA1
+ */
+#define R0(v,w,x,y,z,i) z+=((w&(x^y))^y)+blk0(i)+0x5A827999+rol(v,5);w=rol(w,30);
+#define R1(v,w,x,y,z,i) z+=((w&(x^y))^y)+blk(i)+0x5A827999+rol(v,5);w=rol(w,30);
+#define R2(v,w,x,y,z,i) z+=(w^x^y)+blk(i)+0x6ED9EBA1+rol(v,5);w=rol(w,30);
+#define R3(v,w,x,y,z,i) z+=(((w|x)&y)|(w&x))+blk(i)+0x8F1BBCDC+rol(v,5);w=rol(w,30);
+#define R4(v,w,x,y,z,i) z+=(w^x^y)+blk(i)+0xCA62C1D6+rol(v,5);w=rol(w,30);
+
+/*
+ * Hash a single 512-bit block. This is the core of the algorithm.
+ */
+void
+SHA1Transform(uint32_t state[5], const uint8_t buffer[SHA1_BLOCK_LENGTH])
+{
+	uint32_t a, b, c, d, e;
+	typedef union {
+		uint8_t c[64];
+		uint32_t l[16];
+	} CHAR64LONG16;
+	CHAR64LONG16 realBlock;
+	CHAR64LONG16 *block= &realBlock;
+
+	(void)memcpy(block, buffer, SHA1_BLOCK_LENGTH);
+
+	/* Copy context->state[] to working vars */
+	a = state[0];
+	b = state[1];
+	c = state[2];
+	d = state[3];
+	e = state[4];
+
+	/* 4 rounds of 20 operations each. Loop unrolled. */
+	R0(a,b,c,d,e, 0); R0(e,a,b,c,d, 1); R0(d,e,a,b,c, 2); R0(c,d,e,a,b, 3);
+	R0(b,c,d,e,a, 4); R0(a,b,c,d,e, 5); R0(e,a,b,c,d, 6); R0(d,e,a,b,c, 7);
+	R0(c,d,e,a,b, 8); R0(b,c,d,e,a, 9); R0(a,b,c,d,e,10); R0(e,a,b,c,d,11);
+	R0(d,e,a,b,c,12); R0(c,d,e,a,b,13); R0(b,c,d,e,a,14); R0(a,b,c,d,e,15);
+	R1(e,a,b,c,d,16); R1(d,e,a,b,c,17); R1(c,d,e,a,b,18); R1(b,c,d,e,a,19);
+	R2(a,b,c,d,e,20); R2(e,a,b,c,d,21); R2(d,e,a,b,c,22); R2(c,d,e,a,b,23);
+	R2(b,c,d,e,a,24); R2(a,b,c,d,e,25); R2(e,a,b,c,d,26); R2(d,e,a,b,c,27);
+	R2(c,d,e,a,b,28); R2(b,c,d,e,a,29); R2(a,b,c,d,e,30); R2(e,a,b,c,d,31);
+	R2(d,e,a,b,c,32); R2(c,d,e,a,b,33); R2(b,c,d,e,a,34); R2(a,b,c,d,e,35);
+	R2(e,a,b,c,d,36); R2(d,e,a,b,c,37); R2(c,d,e,a,b,38); R2(b,c,d,e,a,39);
+	R3(a,b,c,d,e,40); R3(e,a,b,c,d,41); R3(d,e,a,b,c,42); R3(c,d,e,a,b,43);
+	R3(b,c,d,e,a,44); R3(a,b,c,d,e,45); R3(e,a,b,c,d,46); R3(d,e,a,b,c,47);
+	R3(c,d,e,a,b,48); R3(b,c,d,e,a,49); R3(a,b,c,d,e,50); R3(e,a,b,c,d,51);
+	R3(d,e,a,b,c,52); R3(c,d,e,a,b,53); R3(b,c,d,e,a,54); R3(a,b,c,d,e,55);
+	R3(e,a,b,c,d,56); R3(d,e,a,b,c,57); R3(c,d,e,a,b,58); R3(b,c,d,e,a,59);
+	R4(a,b,c,d,e,60); R4(e,a,b,c,d,61); R4(d,e,a,b,c,62); R4(c,d,e,a,b,63);
+	R4(b,c,d,e,a,64); R4(a,b,c,d,e,65); R4(e,a,b,c,d,66); R4(d,e,a,b,c,67);
+	R4(c,d,e,a,b,68); R4(b,c,d,e,a,69); R4(a,b,c,d,e,70); R4(e,a,b,c,d,71);
+	R4(d,e,a,b,c,72); R4(c,d,e,a,b,73); R4(b,c,d,e,a,74); R4(a,b,c,d,e,75);
+	R4(e,a,b,c,d,76); R4(d,e,a,b,c,77); R4(c,d,e,a,b,78); R4(b,c,d,e,a,79);
+
+	/* Add the working vars back into context.state[] */
+	state[0] += a;
+	state[1] += b;
+	state[2] += c;
+	state[3] += d;
+	state[4] += e;
+
+	/* Wipe variables */
+	a = b = c = d = e = 0;
+}
+
+
+/*
+ * SHA1Init - Initialize new context
+ */
+void
+SHA1Init(SHA1_CTX *context)
+{
+
+	/* SHA1 initialization constants */
+	context->count = 0;
+	context->state[0] = 0x67452301;
+	context->state[1] = 0xEFCDAB89;
+	context->state[2] = 0x98BADCFE;
+	context->state[3] = 0x10325476;
+	context->state[4] = 0xC3D2E1F0;
+}
+
+
+/*
+ * Run your data through this.
+ */
+void
+SHA1Update(SHA1_CTX *context, const uint8_t *data, size_t len)
+{
+	size_t i, j;
+
+	j = (size_t)((context->count >> 3) & 63);
+	context->count += (len << 3);
+	if ((j + len) > 63) {
+		(void)memcpy(&context->buffer[j], data, (i = 64-j));
+		SHA1Transform(context->state, context->buffer);
+		for ( ; i + 63 < len; i += 64)
+			SHA1Transform(context->state, (uint8_t *)&data[i]);
+		j = 0;
+	} else {
+		i = 0;
+	}
+	(void)memcpy(&context->buffer[j], &data[i], len - i);
+}
+
+
+/*
+ * Add padding and return the message digest.
+ */
+void
+SHA1Pad(SHA1_CTX *context)
+{
+	uint8_t finalcount[8];
+	u_int i;
+
+	for (i = 0; i < 8; i++) {
+		finalcount[i] = (uint8_t)((context->count >>
+		    ((7 - (i & 7)) * 8)) & 255);	/* Endian independent */
+	}
+	SHA1Update(context, (uint8_t *)"\200", 1);
+	while ((context->count & 504) != 448)
+		SHA1Update(context, (uint8_t *)"\0", 1);
+	SHA1Update(context, finalcount, 8); /* Should cause a SHA1Transform() */
+}
+
+void
+SHA1Final(uint8_t digest[SHA1_DIGEST_LENGTH], SHA1_CTX *context)
+{
+	u_int i;
+
+	SHA1Pad(context);
+	if (digest) {
+		for (i = 0; i < SHA1_DIGEST_LENGTH; i++) {
+			digest[i] = (uint8_t)
+			   ((context->state[i>>2] >> ((3-(i & 3)) * 8) ) & 255);
+		}
+		memset(context, 0, sizeof(*context));
+	}
+}

=== added file 'libdrizzle/sha1.h'
--- libdrizzle/sha1.h	1970-01-01 00:00:00 +0000
+++ libdrizzle/sha1.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,42 @@
+/**
+ * @file
+ * @brief SHA1 Declarations
+ */
+
+#ifndef _SHA1_H
+#define _SHA1_H
+
+#ifdef  __cplusplus
+extern "C" {
+#endif
+
+/**
+ * @addtogroup sha1 SHA-1 in C
+ * By Steve Reid <steve@edmweb.com>
+ * 100% Public Domain
+ * @{
+ */
+
+#define	SHA1_BLOCK_LENGTH		64
+#define	SHA1_DIGEST_LENGTH		20
+#define	SHA1_DIGEST_STRING_LENGTH	(SHA1_DIGEST_LENGTH * 2 + 1)
+
+typedef struct {
+    uint32_t state[5];
+    uint64_t count;
+    uint8_t buffer[SHA1_BLOCK_LENGTH];
+} SHA1_CTX;
+
+void SHA1Init(SHA1_CTX *);
+void SHA1Pad(SHA1_CTX *);
+void SHA1Transform(uint32_t [5], const uint8_t [SHA1_BLOCK_LENGTH]);
+void SHA1Update(SHA1_CTX *, const uint8_t *, size_t);
+void SHA1Final(uint8_t [SHA1_DIGEST_LENGTH], SHA1_CTX *);
+
+/** @} */
+
+#ifdef  __cplusplus
+}
+#endif
+
+#endif /* _SHA1_H */

=== added file 'libdrizzle/state.c'
--- libdrizzle/state.c	1970-01-01 00:00:00 +0000
+++ libdrizzle/state.c	2010-08-12 20:27:32 +0000
@@ -0,0 +1,70 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief State machine definitions
+ */
+
+#include "common.h"
+
+drizzle_return_t drizzle_state_loop(drizzle_con_st *con)
+{
+  drizzle_return_t ret;
+
+  while (!drizzle_state_none(con))
+  {
+    ret= con->state_stack[con->state_current - 1](con);
+    if (ret != DRIZZLE_RETURN_OK)
+    {
+      if (ret != DRIZZLE_RETURN_IO_WAIT && ret != DRIZZLE_RETURN_PAUSE &&
+          ret != DRIZZLE_RETURN_ERROR_CODE)
+      {
+        drizzle_con_close(con);
+      }
+
+      return ret;
+    }
+  }
+
+  return DRIZZLE_RETURN_OK;
+}
+
+drizzle_return_t drizzle_state_packet_read(drizzle_con_st *con)
+{
+  drizzle_log_debug(con->drizzle, "drizzle_state_packet_read");
+
+  if (con->buffer_size < 4)
+  {
+    drizzle_state_push(con, drizzle_state_read);
+    return DRIZZLE_RETURN_OK;
+  }
+
+  con->packet_size= drizzle_get_byte3(con->buffer_ptr);
+
+  if (con->packet_number != con->buffer_ptr[3])
+  {
+    drizzle_set_error(con->drizzle, "drizzle_state_packet_read",
+                      "bad packet number:%u:%u", con->packet_number,
+                      con->buffer_ptr[3]);
+    return DRIZZLE_RETURN_BAD_PACKET_NUMBER;
+  }
+
+  drizzle_log_debug(con->drizzle, "packet_size= %zu, packet_number= %u",
+                    con->packet_size, con->packet_number);
+
+  con->packet_number++;
+
+  con->buffer_ptr+= 4;
+  con->buffer_size-= 4;
+
+  drizzle_state_pop(con);
+  return DRIZZLE_RETURN_OK;
+}

=== added file 'libdrizzle/state.h'
--- libdrizzle/state.h	1970-01-01 00:00:00 +0000
+++ libdrizzle/state.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,85 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief State Machine Declarations
+ */
+
+#ifndef __DRIZZLE_STATE_H
+#define __DRIZZLE_STATE_H
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/**
+ * @addtogroup drizzle_state State Machine Declarations
+ *
+ * These functions are used in the protocol parsing state machine. Not all
+ * functions are defined in state.c, they are in the most appropriate source
+ * file (for example, handshake.c for drizzle_state_handshake_server_read).
+ * @{
+ */
+
+/**
+ * Main state loop for connections.
+ *
+ * @param[in] con Connection structure previously initialized with
+ *  drizzle_con_create(), drizzle_con_clone(), or related functions.
+ * @return Standard drizzle return value.
+ */
+drizzle_return_t drizzle_state_loop(drizzle_con_st *con);
+
+/* Functions in state.c */
+drizzle_return_t drizzle_state_packet_read(drizzle_con_st *con);
+
+/* Functions in conn.c */
+drizzle_return_t drizzle_state_addrinfo(drizzle_con_st *con);
+drizzle_return_t drizzle_state_connect(drizzle_con_st *con);
+drizzle_return_t drizzle_state_connecting(drizzle_con_st *con);
+drizzle_return_t drizzle_state_read(drizzle_con_st *con);
+drizzle_return_t drizzle_state_write(drizzle_con_st *con);
+drizzle_return_t drizzle_state_listen(drizzle_con_st *con);
+
+/* Functions in handshake.c */
+drizzle_return_t drizzle_state_handshake_server_read(drizzle_con_st *con);
+drizzle_return_t drizzle_state_handshake_server_write(drizzle_con_st *con);
+drizzle_return_t drizzle_state_handshake_client_read(drizzle_con_st *con);
+drizzle_return_t drizzle_state_handshake_client_write(drizzle_con_st *con);
+drizzle_return_t drizzle_state_handshake_result_read(drizzle_con_st *con);
+
+/* Functions in command.c */
+drizzle_return_t drizzle_state_command_read(drizzle_con_st *con);
+drizzle_return_t drizzle_state_command_write(drizzle_con_st *con);
+
+/* Functions in result.c */
+drizzle_return_t drizzle_state_result_read(drizzle_con_st *con);
+drizzle_return_t drizzle_state_result_write(drizzle_con_st *con);
+
+/* Functions in column.c */
+drizzle_return_t drizzle_state_column_read(drizzle_con_st *con);
+drizzle_return_t drizzle_state_column_write(drizzle_con_st *con);
+
+/* Functions in row.c */
+drizzle_return_t drizzle_state_row_read(drizzle_con_st *con);
+drizzle_return_t drizzle_state_row_write(drizzle_con_st *con);
+
+/* Functions in field.c */
+drizzle_return_t drizzle_state_field_read(drizzle_con_st *con);
+drizzle_return_t drizzle_state_field_write(drizzle_con_st *con);
+
+/** @} */
+
+#ifdef __cplusplus
+}
+#endif
+
+#endif /* __DRIZZLE_STATE_H */

=== added file 'libdrizzle/structs.h'
--- libdrizzle/structs.h	1970-01-01 00:00:00 +0000
+++ libdrizzle/structs.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,211 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Struct Definitions
+ */
+
+#ifndef __DRIZZLE_STRUCTS_H
+#define __DRIZZLE_STRUCTS_H
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/**
+ * @ingroup drizzle
+ */
+struct drizzle_st
+{
+  uint16_t error_code;
+  drizzle_options_t options;
+  drizzle_verbose_t verbose;
+  uint32_t con_count;
+  uint32_t pfds_size;
+  uint32_t query_count;
+  uint32_t query_new;
+  uint32_t query_running;
+  int last_errno;
+  int timeout;
+  drizzle_con_st *con_list;
+  void *context;
+  drizzle_context_free_fn *context_free_fn;
+  drizzle_event_watch_fn *event_watch_fn;
+  void *event_watch_context;
+  drizzle_log_fn *log_fn;
+  void *log_context;
+  struct pollfd *pfds;
+  drizzle_query_st *query_list;
+  char sqlstate[DRIZZLE_MAX_SQLSTATE_SIZE + 1];
+  char last_error[DRIZZLE_MAX_ERROR_SIZE];
+};
+
+/**
+ * @ingroup drizzle_con
+ */
+struct drizzle_con_tcp_st
+{
+  in_port_t port;
+  struct addrinfo *addrinfo;
+  char *host;
+  char host_buffer[NI_MAXHOST];
+};
+
+/**
+ * @ingroup drizzle_con
+ */
+struct drizzle_con_uds_st
+{
+  struct addrinfo addrinfo;
+  struct sockaddr_un sockaddr;
+};
+
+/**
+ * @ingroup drizzle_con
+ */
+struct drizzle_con_st
+{
+  uint8_t packet_number;
+  uint8_t protocol_version;
+  uint8_t state_current;
+  short events;
+  short revents;
+  drizzle_capabilities_t capabilities;
+  drizzle_charset_t charset;
+  drizzle_command_t command;
+  drizzle_con_options_t options;
+  drizzle_con_socket_t socket_type;
+  drizzle_con_status_t status;
+  uint32_t max_packet_size;
+  uint32_t result_count;
+  uint32_t thread_id;
+  int backlog;
+  int fd;
+  size_t buffer_size;
+  size_t command_offset;
+  size_t command_size;
+  size_t command_total;
+  size_t packet_size;
+  struct addrinfo *addrinfo_next;
+  uint8_t *buffer_ptr;
+  uint8_t *command_buffer;
+  uint8_t *command_data;
+  void *context;
+  drizzle_con_context_free_fn *context_free_fn;
+  drizzle_st *drizzle;
+  drizzle_con_st *next;
+  drizzle_con_st *prev;
+  drizzle_query_st *query;
+  drizzle_result_st *result;
+  drizzle_result_st *result_list;
+  uint8_t *scramble;
+  union
+  {
+    drizzle_con_tcp_st tcp;
+    drizzle_con_uds_st uds;
+  } socket;
+  uint8_t buffer[DRIZZLE_MAX_BUFFER_SIZE];
+  char db[DRIZZLE_MAX_DB_SIZE];
+  char password[DRIZZLE_MAX_PASSWORD_SIZE];
+  uint8_t scramble_buffer[DRIZZLE_MAX_SCRAMBLE_SIZE];
+  char server_version[DRIZZLE_MAX_SERVER_VERSION_SIZE];
+  drizzle_state_fn *state_stack[DRIZZLE_STATE_STACK_SIZE];
+  char user[DRIZZLE_MAX_USER_SIZE];
+};
+
+/**
+ * @ingroup drizzle_query
+ */
+struct drizzle_query_st
+{
+  drizzle_st *drizzle;
+  drizzle_query_st *next;
+  drizzle_query_st *prev;
+  drizzle_query_options_t options;
+  drizzle_query_state_t state;
+  drizzle_con_st *con;
+  drizzle_result_st *result;
+  const char *string;
+  size_t size;
+  void *context;
+  drizzle_query_context_free_fn *context_free_fn;
+};
+
+/**
+ * @ingroup drizzle_result
+ */
+struct drizzle_result_st
+{
+  drizzle_con_st *con;
+  drizzle_result_st *next;
+  drizzle_result_st *prev;
+  drizzle_result_options_t options;
+
+  char info[DRIZZLE_MAX_INFO_SIZE];
+  uint16_t error_code;
+  char sqlstate[DRIZZLE_MAX_SQLSTATE_SIZE + 1];
+  uint64_t insert_id;
+  uint16_t warning_count;
+  uint64_t affected_rows;
+
+  uint16_t column_count;
+  uint16_t column_current;
+  drizzle_column_st *column_list;
+  drizzle_column_st *column;
+  drizzle_column_st *column_buffer;
+
+  uint64_t row_count;
+  uint64_t row_current;
+
+  uint16_t field_current;
+  size_t field_total;
+  size_t field_offset;
+  size_t field_size;
+  drizzle_field_t field;
+  drizzle_field_t field_buffer;
+
+  uint64_t row_list_size;
+  drizzle_row_t row;
+  drizzle_row_t *row_list;
+  size_t *field_sizes;
+  size_t **field_sizes_list;
+};
+
+/**
+ * @ingroup drizzle_column
+ */
+struct drizzle_column_st
+{
+  drizzle_result_st *result;
+  drizzle_column_st *next;
+  drizzle_column_st *prev;
+  drizzle_column_options_t options;
+  char catalog[DRIZZLE_MAX_CATALOG_SIZE];
+  char db[DRIZZLE_MAX_DB_SIZE];
+  char table[DRIZZLE_MAX_TABLE_SIZE];
+  char orig_table[DRIZZLE_MAX_TABLE_SIZE];
+  char name[DRIZZLE_MAX_COLUMN_NAME_SIZE];
+  char orig_name[DRIZZLE_MAX_COLUMN_NAME_SIZE];
+  drizzle_charset_t charset;
+  uint32_t size;
+  size_t max_size;
+  drizzle_column_type_t type;
+  drizzle_column_flags_t flags;
+  uint8_t decimals;
+  uint8_t default_value[DRIZZLE_MAX_DEFAULT_VALUE_SIZE];
+  size_t default_value_size;
+};
+
+#ifdef __cplusplus
+}
+#endif
+
+#endif /* __DRIZZLE_STRUCTS_H */

=== added file 'libdrizzle/visibility.h'
--- libdrizzle/visibility.h	1970-01-01 00:00:00 +0000
+++ libdrizzle/visibility.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,55 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ * 
+ * Implementation drawn from visibility.texi in gnulib.
+ */
+
+/**
+ * @file
+ * @brief Visibility Control Macros
+ */
+
+#ifndef __DRIZZLE_VISIBILITY_H
+#define __DRIZZLE_VISIBILITY_H
+
+/**
+ *
+ * DRIZZLE_API is used for the public API symbols. It either DLL imports or
+ * DLL exports (or does nothing for static build).
+ *
+ * DRIZZLE_LOCAL is used for non-api symbols.
+ */
+
+#if defined(_WIN32)
+# define DRIZZLE_API
+# define DRIZZLE_LOCAL
+#else
+#if defined(BUILDING_LIBDRIZZLE)
+# if defined(HAVE_VISIBILITY)
+#  define DRIZZLE_API __attribute__ ((visibility("default")))
+#  define DRIZZLE_LOCAL  __attribute__ ((visibility("hidden")))
+# elif defined (__SUNPRO_C) && (__SUNPRO_C >= 0x550)
+#  define DRIZZLE_API __global
+#  define DRIZZLE_API __hidden
+# elif defined(_MSC_VER)
+#  define DRIZZLE_API extern __declspec(dllexport) 
+#  define DRIZZLE_LOCAL
+# endif /* defined(HAVE_VISIBILITY) */
+#else  /* defined(BUILDING_LIBDRIZZLE) */
+# if defined(_MSC_VER)
+#  define DRIZZLE_API extern __declspec(dllimport) 
+#  define DRIZZLE_LOCAL
+# else
+#  define DRIZZLE_API
+#  define DRIZZLE_LOCAL
+# endif /* defined(_MSC_VER) */
+#endif /* defined(BUILDING_LIBDRIZZLE) */
+#endif /* _WIN32 */
+
+#endif /* __DRIZZLE_VISIBILITY_H */

=== modified file 'm4/pandora_drizzle_build.m4'
--- m4/pandora_drizzle_build.m4	2010-08-06 23:07:46 +0000
+++ m4/pandora_drizzle_build.m4	2010-08-12 20:27:32 +0000
@@ -50,6 +50,7 @@
   # and defines HAVE_LIBM etc
   #--------------------------------------------------------------------
   
+  AC_CHECK_FUNC(inet_ntoa, [], [AC_CHECK_LIB(nsl, inet_ntoa)])
     # For the sched_yield() function on Solaris
   AC_CHECK_FUNC(sched_yield, [],
     [AC_CHECK_LIB(posix4, [sched_yield],

=== modified file 'support-files/include.am'
--- support-files/include.am	2010-02-10 09:07:38 +0000
+++ support-files/include.am	2010-08-12 20:27:32 +0000
@@ -18,7 +18,8 @@
 # Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 # MA 02111-1307, USA
 
-pkgconfigdir = $(libdir)/pkgconfig
+pkgconfigdir= $(libdir)/pkgconfig
+pkgconfig_DATA= support-files/libdrizzle.pc
 
 # Default same as 'pkgdatadir', but we can override it
 pkgsuppdir =		$(datadir)/@PACKAGE@

=== added file 'support-files/libdrizzle.pc.in'
--- support-files/libdrizzle.pc.in	1970-01-01 00:00:00 +0000
+++ support-files/libdrizzle.pc.in	2010-08-12 20:27:32 +0000
@@ -0,0 +1,10 @@
+prefix=@prefix@
+exec_prefix=@exec_prefix@
+libdir=@libdir@
+includedir=@includedir@
+
+Name: libdrizzle
+Description: Drizzle Client & Protocol Library
+Version: @VERSION@
+Libs: -L${libdir} -ldrizzle
+Cflags: -I${includedir}

=== added file 'unittests/client_server.c'
--- unittests/client_server.c	1970-01-01 00:00:00 +0000
+++ unittests/client_server.c	2010-08-12 20:27:32 +0000
@@ -0,0 +1,541 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Tests for drizzle_st Structures
+ */
+
+#include <assert.h>
+
+#include "tests/common.h"
+
+#define DRIZZLE_TEST_HOST "127.0.0.1"
+#define DRIZZLE_TEST_PORT 12399
+
+typedef enum
+{
+  SERVER_STATE_START,
+  SERVER_STATE_HANDSHAKE_WRITE,
+  SERVER_STATE_HANDSHAKE_READ,
+  SERVER_STATE_HANDSHAKE_RESULT,
+  SERVER_STATE_COMMAND_BUFFER,
+  SERVER_STATE_RESULT_HEADER,
+  SERVER_STATE_RESULT_COLUMN_1,
+  SERVER_STATE_RESULT_COLUMN_2,
+  SERVER_STATE_RESULT_COLUMN_EOF,
+  SERVER_STATE_RESULT_ROW_1,
+  SERVER_STATE_RESULT_ROW_1_FIELD_1,
+  SERVER_STATE_RESULT_ROW_1_FIELD_2,
+  SERVER_STATE_RESULT_ROW_EOF,
+  SERVER_STATE_DONE
+} server_state_t;
+
+typedef struct
+{
+  server_state_t state;
+  drizzle_result_st result;
+  drizzle_column_st column;
+  drizzle_command_t command;
+  char *data;
+  size_t total;
+} server_state_st;
+
+typedef enum
+{
+  CLIENT_STATE_START,
+  CLIENT_STATE_RESULT,
+  CLIENT_STATE_DONE
+} client_state_t;
+
+typedef struct
+{
+  client_state_t state;
+  drizzle_result_st result;
+} client_state_st;
+
+static void _server(drizzle_con_st *con, server_state_st *state);
+static void _client(drizzle_con_st *con, client_state_st *state);
+
+int main(void)
+{
+  drizzle_st drizzle;
+  drizzle_con_st listen_con;
+  drizzle_con_st client;
+  drizzle_con_st server;
+  drizzle_return_t ret;
+  bool server_accepted = false;
+  server_state_st server_state;
+  client_state_st client_state;
+
+  drizzle_test("drizzle_create");
+  if (drizzle_create(&drizzle) == NULL)
+    drizzle_test_error("returned NULL");
+
+  drizzle_test("drizzle_con_add_tcp_listen");
+  if (drizzle_con_add_tcp_listen(&drizzle, &listen_con, DRIZZLE_TEST_HOST,
+                                 DRIZZLE_TEST_PORT, 1,
+                                 DRIZZLE_CON_NONE) == NULL)
+  {
+    drizzle_test_error("returned NULL");
+  }
+
+  drizzle_test("drizzle_con_listen");
+  ret= drizzle_con_listen(&listen_con);
+  if (ret != DRIZZLE_RETURN_OK)
+    drizzle_test_error("returned %s (%d)", drizzle_error(&drizzle), ret);
+
+  drizzle_test("drizzle_con_add_tcp");
+  if (drizzle_con_add_tcp(&drizzle, &client, DRIZZLE_TEST_HOST,
+                          DRIZZLE_TEST_PORT, NULL, NULL, NULL,
+                          DRIZZLE_CON_NONE) == NULL)
+  {
+    drizzle_test_error("returned NULL");
+  }
+
+  drizzle_test("drizzle_add_options");
+  drizzle_add_options(&drizzle, DRIZZLE_NON_BLOCKING);
+
+  server_state.state= SERVER_STATE_START;
+  client_state.state= CLIENT_STATE_START;
+
+  while (true)
+  {
+    if (!server_accepted)
+    {
+      drizzle_test("drizzle_con_accept");
+      (void)drizzle_con_accept(&drizzle, &server, &ret);
+      if (ret == DRIZZLE_RETURN_OK)
+        server_accepted = true;
+      else if (ret != DRIZZLE_RETURN_IO_WAIT)
+        drizzle_test_error("returned %s (%d)", drizzle_error(&drizzle), ret);
+    }
+
+    if (server_accepted)
+      _server(&server, &server_state);
+
+    _client(&client, &client_state);
+
+    if (server_state.state == SERVER_STATE_DONE &&
+        client_state.state == CLIENT_STATE_DONE)
+    {
+      break;
+    }
+
+    drizzle_test("drizzle_con_wait");
+    ret= drizzle_con_wait(&drizzle);
+    if (ret != DRIZZLE_RETURN_OK)
+      drizzle_test_error("returned %s (%d)", drizzle_error(&drizzle), ret);
+  }
+
+  if (server_accepted)
+  {
+    drizzle_test("drizzle_con_free");
+    drizzle_con_free(&server);
+  }
+
+  drizzle_test("drizzle_con_free");
+  drizzle_con_free(&client);
+
+  drizzle_test("drizzle_con_free");
+  drizzle_con_free(&listen_con);
+
+  drizzle_test("drizzle_free");
+  drizzle_free(&drizzle);
+
+  return 0;
+}
+
+static void _server(drizzle_con_st *con, server_state_st *state)
+{
+  drizzle_return_t ret;
+  const drizzle_field_t const fields[2]=
+  {
+    (const drizzle_field_t)"test_field_1",
+    (const drizzle_field_t)"test_field_2"
+  };
+  const size_t field_sizes[2]= { 12, 12 };
+
+  switch(state->state)
+  {
+  case SERVER_STATE_START:
+    drizzle_con_set_protocol_version(con, 10);
+    drizzle_con_set_server_version(con, "test_version");
+    drizzle_con_set_thread_id(con, 1);
+    drizzle_con_set_scramble(con, (const uint8_t *)"ABCDEFGHIJKLMNOPQRST");
+    drizzle_con_set_capabilities(con, DRIZZLE_CAPABILITIES_NONE);
+    drizzle_con_set_charset(con, 8);
+    drizzle_con_set_status(con, DRIZZLE_CON_STATUS_NONE);
+    drizzle_con_set_max_packet_size(con, DRIZZLE_MAX_PACKET_SIZE);
+
+  case SERVER_STATE_HANDSHAKE_WRITE:
+    drizzle_test("drizzle_handshake_server_write");
+    ret= drizzle_handshake_server_write(con);
+    if (ret == DRIZZLE_RETURN_IO_WAIT)
+    {
+      state->state = SERVER_STATE_HANDSHAKE_WRITE;
+      return;
+    }
+    else if (ret != DRIZZLE_RETURN_OK)
+      drizzle_test_error("returned %s (%d)", drizzle_con_error(con), ret);
+
+  case SERVER_STATE_HANDSHAKE_READ:
+    drizzle_test("drizzle_handshake_client_read");
+    ret= drizzle_handshake_client_read(con);
+    if (ret == DRIZZLE_RETURN_IO_WAIT)
+    {
+      state->state = SERVER_STATE_HANDSHAKE_READ;
+      return;
+    }
+    else if (ret != DRIZZLE_RETURN_OK)
+      drizzle_test_error("returned %s (%d)", drizzle_con_error(con), ret);
+
+    drizzle_test("drizzle_result_create");
+    if (drizzle_result_create(con, &state->result) == NULL)
+      drizzle_test_error("returned %s", drizzle_con_error(con));
+
+  case SERVER_STATE_HANDSHAKE_RESULT:
+    drizzle_test("drizzle_handshake_result_write");
+    ret= drizzle_result_write(con, &state->result, true);
+    if (ret == DRIZZLE_RETURN_IO_WAIT)
+    {
+      state->state = SERVER_STATE_HANDSHAKE_RESULT;
+      return;
+    }
+    else if (ret != DRIZZLE_RETURN_OK)
+      drizzle_test_error("returned %s (%d)", drizzle_con_error(con), ret);
+
+    drizzle_result_free(&state->result);
+
+  case SERVER_STATE_COMMAND_BUFFER:
+    drizzle_test("drizzle_con_command_buffer");
+    state->data= drizzle_con_command_buffer(con, &state->command, &state->total,
+                                            &ret);
+    if (ret == DRIZZLE_RETURN_IO_WAIT)
+    {
+      state->state = SERVER_STATE_COMMAND_BUFFER;
+      return;
+    }
+    else if (ret != DRIZZLE_RETURN_OK)
+      drizzle_test_error("returned %s (%d)", drizzle_con_error(con), ret);
+
+    if (state->command != DRIZZLE_COMMAND_QUERY)
+      drizzle_test_error("command was not a query");
+
+    if (state->total != 6 && !memcmp(state->data, "SELECT", 6))
+      drizzle_test_error("query doesn't match");
+
+    if (state->data != NULL)
+    {
+      free(state->data);
+      state->data= NULL;
+    }
+
+    drizzle_test("drizzle_result_create");
+    if (drizzle_result_create(con, &state->result) == NULL)
+      drizzle_test_error("returned %s", drizzle_con_error(con));
+
+    drizzle_result_set_column_count(&state->result, 2);
+
+  case SERVER_STATE_RESULT_HEADER:
+    drizzle_test("drizzle_handshake_result_write");
+    ret= drizzle_result_write(con, &state->result, false);
+    if (ret == DRIZZLE_RETURN_IO_WAIT)
+    {
+      state->state = SERVER_STATE_RESULT_HEADER;
+      return;
+    }
+    else if (ret != DRIZZLE_RETURN_OK)
+      drizzle_test_error("returned %s (%d)", drizzle_con_error(con), ret);
+
+    drizzle_test("drizzle_column_create");
+    if (drizzle_column_create(&state->result, &state->column) == NULL)
+      drizzle_test_error("returned %s", drizzle_con_error(con));
+
+    drizzle_column_set_catalog(&state->column, "test_catalog");
+    drizzle_column_set_db(&state->column, "test_db");
+    drizzle_column_set_table(&state->column, "test_table");
+    drizzle_column_set_orig_table(&state->column, "test_orig_table");
+    drizzle_column_set_name(&state->column, "test_column_1");
+    drizzle_column_set_orig_name(&state->column, "test_orig_column_1");
+    drizzle_column_set_charset(&state->column, 8);
+    drizzle_column_set_size(&state->column, 32);
+    drizzle_column_set_type(&state->column, DRIZZLE_COLUMN_TYPE_VARCHAR);
+    drizzle_column_set_flags(&state->column, DRIZZLE_COLUMN_FLAGS_NONE);
+
+  case SERVER_STATE_RESULT_COLUMN_1:
+    drizzle_test("drizzle_column_write");
+    ret= drizzle_column_write(&state->result, &state->column);
+    if (ret == DRIZZLE_RETURN_IO_WAIT)
+    {
+      state->state = SERVER_STATE_RESULT_COLUMN_1;
+      return;
+    }
+    else if (ret != DRIZZLE_RETURN_OK)
+      drizzle_test_error("returned %s (%d)", drizzle_con_error(con), ret);
+
+    drizzle_column_set_name(&state->column, "test_column_2");
+    drizzle_column_set_orig_name(&state->column, "test_orig_column_2");
+
+  case SERVER_STATE_RESULT_COLUMN_2:
+    drizzle_test("drizzle_column_write");
+    ret= drizzle_column_write(&state->result, &state->column);
+    if (ret == DRIZZLE_RETURN_IO_WAIT)
+    {
+      state->state = SERVER_STATE_RESULT_COLUMN_2;
+      return;
+    }
+    else if (ret != DRIZZLE_RETURN_OK)
+      drizzle_test_error("returned %s (%d)", drizzle_con_error(con), ret);
+
+    drizzle_test("drizzle_column_free");
+    drizzle_column_free(&state->column);
+
+    drizzle_result_set_eof(&state->result, true);
+
+  case SERVER_STATE_RESULT_COLUMN_EOF:
+    drizzle_test("drizzle_handshake_result_write");
+    ret= drizzle_result_write(con, &state->result, false);
+    if (ret == DRIZZLE_RETURN_IO_WAIT)
+    {
+      state->state = SERVER_STATE_RESULT_COLUMN_EOF;
+      return;
+    }
+    else if (ret != DRIZZLE_RETURN_OK)
+      drizzle_test_error("returned %s (%d)", drizzle_con_error(con), ret);
+
+    drizzle_result_calc_row_size(&state->result, fields, field_sizes);
+
+  case SERVER_STATE_RESULT_ROW_1:
+    ret= drizzle_row_write(&state->result);
+
+  case SERVER_STATE_RESULT_ROW_1_FIELD_1:
+    ret= drizzle_field_write(&state->result, fields[0], field_sizes[0],
+                             field_sizes[0]);
+
+  case SERVER_STATE_RESULT_ROW_1_FIELD_2:
+    ret= drizzle_field_write(&state->result, fields[1], field_sizes[1],
+                             field_sizes[1]);
+
+  case SERVER_STATE_RESULT_ROW_EOF:
+    drizzle_test("drizzle_handshake_result_write");
+    ret= drizzle_result_write(con, &state->result, true);
+    if (ret == DRIZZLE_RETURN_IO_WAIT)
+    {
+      state->state = SERVER_STATE_RESULT_ROW_EOF;
+      return;
+    }
+    else if (ret != DRIZZLE_RETURN_OK)
+      drizzle_test_error("returned %s (%d)", drizzle_con_error(con), ret);
+
+    drizzle_result_free(&state->result);
+
+    state->state = SERVER_STATE_DONE;
+
+  case SERVER_STATE_DONE:
+    return;
+
+  default:
+    drizzle_test_error("invalid server state");
+  }
+}
+
+static void _client(drizzle_con_st *con, client_state_st *state)
+{
+  drizzle_return_t ret;
+  drizzle_column_st *column;
+  drizzle_row_t row;
+  size_t *field_sizes;
+
+  switch(state->state)
+  {
+  case CLIENT_STATE_START:
+    drizzle_test("drizzle_query_str");
+    (void)drizzle_query_str(con, &state->result, "SELECT", &ret);
+    if (ret == DRIZZLE_RETURN_IO_WAIT)
+    {
+      state->state = CLIENT_STATE_START;
+      return;
+    }
+    else if (ret != DRIZZLE_RETURN_OK)
+      drizzle_test_error("returned %s (%d)", drizzle_con_error(con), ret);
+
+  case CLIENT_STATE_RESULT:
+    drizzle_test("drizzle_result_buffer");
+    ret = drizzle_result_buffer(&state->result);
+    if (ret == DRIZZLE_RETURN_IO_WAIT)
+    {
+      state->state = CLIENT_STATE_RESULT;
+      return;
+    }
+    else if (ret != DRIZZLE_RETURN_OK)
+      drizzle_test_error("returned %s (%d)", drizzle_con_error(con), ret);
+
+    drizzle_test("drizzle_con_protocol_version");
+    if (drizzle_con_protocol_version(con) != 10)
+      drizzle_test_error("no match");
+
+    drizzle_test("drizzle_con_server_version");
+    if (strcmp(drizzle_con_server_version(con), "test_version"))
+      drizzle_test_error("no match");
+
+    drizzle_test("drizzle_con_thread_id");
+    if (drizzle_con_thread_id(con) != 1)
+      drizzle_test_error("no match");
+
+    drizzle_test("drizzle_con_scramble");
+    if (memcmp(drizzle_con_scramble(con), "ABCDEFGHIJKLMNOPQRST",
+               DRIZZLE_MAX_SCRAMBLE_SIZE))
+    {
+      drizzle_test_error("no match");
+    }
+
+    /* We default to MySQL protocol right now, which sets this flag. */
+    drizzle_test("drizzle_con_capabilities");
+    if (drizzle_con_capabilities(con) != DRIZZLE_CAPABILITIES_PROTOCOL_41)
+      drizzle_test_error("no match");
+
+    drizzle_test("drizzle_con_charset");
+    if (drizzle_con_charset(con) != 8)
+      drizzle_test_error("no match");
+
+    drizzle_test("drizzle_con_status");
+    if (drizzle_con_status(con) != DRIZZLE_CON_STATUS_NONE)
+      drizzle_test_error("no match");
+
+    drizzle_test("drizzle_con_packet_size");
+    if (drizzle_con_max_packet_size(con) != DRIZZLE_MAX_PACKET_SIZE)
+      drizzle_test_error("no match");
+
+    drizzle_test("drizzle_column_next");
+    column= drizzle_column_next(&state->result);
+    if (column == NULL)
+      drizzle_test_error("column not found");
+
+    drizzle_test("drizzle_column_drizzle_result");
+    if (drizzle_column_drizzle_result(column) != &state->result)
+      drizzle_test_error("no match");
+
+    drizzle_test("drizzle_column_catalog");
+    if (strcmp(drizzle_column_catalog(column), "test_catalog"))
+      drizzle_test_error("no match");
+
+    drizzle_test("drizzle_column_db");
+    if (strcmp(drizzle_column_db(column), "test_db"))
+      drizzle_test_error("no match");
+
+    drizzle_test("drizzle_column_table");
+    if (strcmp(drizzle_column_table(column), "test_table"))
+      drizzle_test_error("no match");
+
+    drizzle_test("drizzle_column_orig_table");
+    if (strcmp(drizzle_column_orig_table(column), "test_orig_table"))
+      drizzle_test_error("no match");
+
+    drizzle_test("drizzle_column_name");
+    if (strcmp(drizzle_column_name(column), "test_column_1"))
+      drizzle_test_error("no match");
+
+    drizzle_test("drizzle_column_orig_name");
+    if (strcmp(drizzle_column_orig_name(column), "test_orig_column_1"))
+      drizzle_test_error("no match");
+
+    drizzle_test("drizzle_column_charset");
+    if (drizzle_column_charset(column) != 8)
+      drizzle_test_error("no match");
+
+    drizzle_test("drizzle_column_size");
+    if (drizzle_column_size(column) != 32)
+      drizzle_test_error("no match");
+
+    drizzle_test("drizzle_column_max_size");
+    if (drizzle_column_max_size(column) != 12)
+      drizzle_test_error("no match");
+
+    drizzle_test("drizzle_column_type_drizzle");
+    if (drizzle_column_type_drizzle(column) != DRIZZLE_COLUMN_TYPE_DRIZZLE_VARCHAR)
+      drizzle_test_error("no match");
+
+    drizzle_test("drizzle_column_flags");
+    if (drizzle_column_flags(column) != DRIZZLE_COLUMN_FLAGS_NONE)
+      drizzle_test_error("no match");
+
+    drizzle_test("drizzle_column_next");
+    column= drizzle_column_next(&state->result);
+    if (column == NULL)
+      drizzle_test_error("column not found");
+
+    drizzle_test("drizzle_column_name");
+    if (strcmp(drizzle_column_name(column), "test_column_2"))
+      drizzle_test_error("no match");
+
+    drizzle_test("drizzle_column_next");
+    column= drizzle_column_next(&state->result);
+    if (column != NULL)
+      drizzle_test_error("column found");
+
+    drizzle_test("drizzle_column_prev");
+    column= drizzle_column_prev(&state->result);
+    if (column == NULL)
+      drizzle_test_error("column not found");
+
+    drizzle_test("drizzle_column_name");
+    if (strcmp(drizzle_column_name(column), "test_column_2"))
+      drizzle_test_error("no match");
+
+    drizzle_test("drizzle_row_next");
+    row= drizzle_row_next(&state->result);
+    if (row == NULL)
+      drizzle_test_error("row not found");
+
+    if (strcmp(row[0], "test_field_1") || strcmp(row[1], "test_field_2"))
+      drizzle_test_error("no match");
+
+    drizzle_test("drizzle_row_field_sizes");
+    field_sizes= drizzle_row_field_sizes(&state->result);
+    if (field_sizes[0] != 12 || field_sizes[1] != 12)
+      drizzle_test_error("no match");
+
+    drizzle_test("drizzle_row_prev");
+    row = drizzle_row_prev(&state->result);
+    if (row == NULL)
+      drizzle_test_error("row not found");
+
+    if (strcmp(row[0], "test_field_1") || strcmp(row[1], "test_field_2"))
+      drizzle_test_error("no match");
+
+    drizzle_test("drizzle_row_next");
+    row = drizzle_row_next(&state->result);
+
+    drizzle_test("drizzle_row_index");
+    row = drizzle_row_index(&state->result, 0);
+    if (row == NULL)
+      drizzle_test_error("row not found");
+
+    if (strcmp(row[0], "test_field_1") || strcmp(row[1], "test_field_2"))
+      drizzle_test_error("no match");
+
+    drizzle_test("drizzle_row_index");
+    row = drizzle_row_index(&state->result, 1);
+    if (row != NULL)
+      drizzle_test_error("row found");
+
+    drizzle_test("drizzle_result_free");
+    drizzle_result_free(&state->result);
+
+    state->state = CLIENT_STATE_DONE;
+
+  case CLIENT_STATE_DONE:
+    return;
+
+  default:
+    drizzle_test_error("invalid client state");
+  }
+}

=== added file 'unittests/common.h'
--- unittests/common.h	1970-01-01 00:00:00 +0000
+++ unittests/common.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,35 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Includes and macros for tests
+ */
+
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include <libdrizzle/drizzle_client.h>
+#include <libdrizzle/drizzle_server.h>
+
+#define drizzle_test(...) do \
+{ \
+  printf(__VA_ARGS__); \
+  printf("\n"); \
+} while (0);
+
+#define drizzle_test_error(...) do \
+{ \
+  printf("*** %s:%d *** ", __FILE__, __LINE__); \
+  printf(__VA_ARGS__); \
+  printf("\n"); \
+  exit(1); \
+} while (0);

=== added file 'unittests/drizzle_column_st.c'
--- unittests/drizzle_column_st.c	1970-01-01 00:00:00 +0000
+++ unittests/drizzle_column_st.c	2010-08-12 20:27:32 +0000
@@ -0,0 +1,69 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief drizzle_column_st tests
+ */
+
+#include "tests/common.h"
+
+int main(void)
+{
+  drizzle_st *drizzle;
+  drizzle_con_st *con;
+  drizzle_result_st *result;
+  drizzle_column_st *column;
+  drizzle_column_st column_buffer;
+
+  printf("sizeof(drizzle_column_st) = %zu\n", sizeof(drizzle_column_st));
+
+  drizzle_test("drizzle_create");
+  if ((drizzle= drizzle_create(NULL)) == NULL)
+    drizzle_test_error("returned NULL");
+
+  drizzle_test("drizzle_con_create");
+  if ((con= drizzle_con_create(drizzle, NULL)) == NULL)
+    drizzle_test_error("returned NULL");
+
+  drizzle_test("drizzle_result_create");
+  if ((result= drizzle_result_create(con, NULL)) == NULL)
+    drizzle_test_error("returned NULL");
+
+  drizzle_test("drizzle_column_create buffer");
+  if ((column= drizzle_column_create(result, &column_buffer)) == NULL)
+    drizzle_test_error("returned NULL");
+  drizzle_column_free(column);
+
+  drizzle_test("drizzle_column_create");
+  if ((column= drizzle_column_create(result, NULL)) == NULL)
+    drizzle_test_error("returned NULL");
+
+  drizzle_test("drizzle_column_set_catalog");
+  drizzle_column_set_catalog(column, "simple test");
+
+  drizzle_test("drizzle_column_catalog");
+  if (strcmp(drizzle_column_catalog(column), "simple test"))
+    drizzle_test_error("does not match what was set");
+
+  drizzle_test("drizzle_column_free");
+  drizzle_column_free(column);
+
+  drizzle_test("drizzle_result_free");
+  drizzle_result_free(result);
+
+  drizzle_test("drizzle_con_free");
+  drizzle_con_free(con);
+
+  drizzle_test("drizzle_free");
+  drizzle_free(drizzle);
+
+  return 0;
+}

=== added file 'unittests/drizzle_con_st.c'
--- unittests/drizzle_con_st.c	1970-01-01 00:00:00 +0000
+++ unittests/drizzle_con_st.c	2010-08-12 20:27:32 +0000
@@ -0,0 +1,116 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief drizzle_con_st tests
+ */
+
+#include <assert.h>
+
+#include "tests/common.h"
+
+#define DRIZZLE_TEST_PORT 12399
+
+int main(void)
+{
+  drizzle_st *drizzle;
+  drizzle_con_st *con;
+  drizzle_con_st con_buffer;
+  drizzle_con_st *clone;
+  drizzle_return_t ret;
+
+  printf("sizeof(drizzle_con_st) = %zu\n", sizeof(drizzle_con_st));
+  printf("sizeof(drizzle_con_tcp_st) = %zu\n", sizeof(drizzle_con_tcp_st));
+  printf("sizeof(drizzle_con_uds_st) = %zu\n", sizeof(drizzle_con_uds_st));
+  printf("sizeof(drizzle_con_options_t) = %zu\n",
+         sizeof(drizzle_con_options_t));
+  printf("sizeof(drizzle_con_socket_t) = %zu\n", sizeof(drizzle_con_socket_t));
+  printf("sizeof(drizzle_con_status_t) = %zu\n", sizeof(drizzle_con_status_t));
+  printf("sizeof(drizzle_capabilities_t) = %zu\n",
+         sizeof(drizzle_capabilities_t));
+
+  drizzle_test("drizzle_create");
+  if ((drizzle= drizzle_create(NULL)) == NULL)
+    drizzle_test_error("returned NULL");
+
+  drizzle_test("drizzle_con_create buffer");
+  if ((con= drizzle_con_create(drizzle, &con_buffer)) == NULL)
+    drizzle_test_error("returned NULL");
+  drizzle_con_free(con);
+
+  drizzle_test("drizzle_con_create");
+  if ((con= drizzle_con_create(drizzle, NULL)) == NULL)
+    drizzle_test_error("returned NULL");
+
+  drizzle_test("drizzle_con_clone");
+  if ((clone= drizzle_con_clone(drizzle, NULL, con)) == NULL)
+    drizzle_test_error("returned NULL");
+  drizzle_con_free(clone);
+
+  drizzle_test("drizzle_con_options");
+  if (drizzle_con_options(con) != (DRIZZLE_CON_ALLOCATED | DRIZZLE_CON_MYSQL))
+    drizzle_test_error("drizzle_con_options");
+
+  drizzle_test("drizzle_con_add_options");
+  drizzle_con_add_options(con, DRIZZLE_CON_EXPERIMENTAL);
+  if (drizzle_con_options(con) != (DRIZZLE_CON_ALLOCATED | DRIZZLE_CON_EXPERIMENTAL))
+    drizzle_test_error("drizzle_con_options");
+
+  drizzle_test("drizzle_con_set_tcp");
+  drizzle_con_set_tcp(con, "127.0.0.1", 1);
+
+  drizzle_test("drizzle_con_host");
+  if (strcmp(drizzle_con_host(con), "127.0.0.1"))
+    drizzle_test_error("expected host not set");
+
+  drizzle_test("drizzle_con_port");
+  if (drizzle_con_port(con) != 1)
+    drizzle_test_error("expected port not set");
+
+  drizzle_test("drizzle_con_fd");
+  if (drizzle_con_fd(con) != -1) 
+    drizzle_test_error("drizzle_con_fd != -1 for a new connection");
+
+  drizzle_test("drizzle_con_connect");
+  ret= drizzle_con_connect(con);
+  if (ret != DRIZZLE_RETURN_COULD_NOT_CONNECT) 
+    drizzle_test_error("expected COULD_NOT_CONNECT, got: %s\n", drizzle_error(drizzle));
+
+  if (drizzle_con_fd(con) != -1)
+    drizzle_test_error("drizzle_con_fd != -1 for unconnected connection");
+
+  drizzle_test("drizzle_con_listen");
+  drizzle_con_set_tcp(con, "127.0.0.1", DRIZZLE_TEST_PORT);
+  ret = drizzle_con_listen(con);
+  if (ret != DRIZZLE_RETURN_OK) 
+    drizzle_test_error("drizzle_con_listen: %s", drizzle_error(drizzle));
+
+  if (drizzle_con_fd(con) <= 0)
+    drizzle_test_error("drizzle_con_fd <= 0 for a listening connection");
+
+  drizzle_test("drizzle_con_free");
+  drizzle_con_free(con);
+
+  drizzle_test("drizzle_con_clone with uds");
+  con= drizzle_con_create(drizzle, NULL);
+  assert(con != NULL);
+  drizzle_con_set_uds(con, "/dev/null");
+  clone= drizzle_con_clone(drizzle, NULL, con);
+  if (clone == NULL) 
+    drizzle_test_error("drizzle_con_clone uds: %s", drizzle_error(drizzle));
+  drizzle_con_free(clone);
+  drizzle_con_free(con);
+  
+  drizzle_test("drizzle_free");
+  drizzle_free(drizzle);
+
+  return 0;
+}

=== added file 'unittests/drizzle_query_st.c'
--- unittests/drizzle_query_st.c	1970-01-01 00:00:00 +0000
+++ unittests/drizzle_query_st.c	2010-08-12 20:27:32 +0000
@@ -0,0 +1,52 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief drizzle_query_st tests
+ */
+
+#include "tests/common.h"
+
+int main(void)
+{
+  drizzle_st *drizzle;
+  drizzle_query_st *query;
+  drizzle_query_st query_buffer;
+  size_t size;
+
+  printf("sizeof(drizzle_query_st) = %zu\n", sizeof(drizzle_query_st));
+
+  if ((drizzle= drizzle_create(NULL)) == NULL)
+    drizzle_test_error("drizzle_create");
+
+  if ((query= drizzle_query_create(drizzle, &query_buffer)) == NULL)
+    drizzle_test_error("drizzle_query_create");
+  drizzle_query_free(query);
+
+  if ((query= drizzle_query_create(drizzle, NULL)) == NULL)
+    drizzle_test_error("drizzle_query_create");
+
+  if (drizzle_query_options(query) != DRIZZLE_QUERY_ALLOCATED)
+    drizzle_test_error("drizzle_query_options");
+
+  drizzle_query_set_string(query, "SELECT 1+1", 10);
+
+  if (strncmp(drizzle_query_string(query, &size), "SELECT 1+1", 10) ||
+      size != 10)
+  {
+    drizzle_test_error("drizzle_query_string");
+  }
+
+  drizzle_query_free(query);
+  drizzle_free(drizzle);
+
+  return 0;
+}

=== added file 'unittests/drizzle_result_st.c'
--- unittests/drizzle_result_st.c	1970-01-01 00:00:00 +0000
+++ unittests/drizzle_result_st.c	2010-08-12 20:27:32 +0000
@@ -0,0 +1,55 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief drizzle_result_st tests
+ */
+
+#include "tests/common.h"
+
+int main(void)
+{
+  drizzle_st *drizzle;
+  drizzle_con_st *con;
+  drizzle_result_st *result;
+  drizzle_result_st result_buffer;
+  drizzle_result_st *clone;
+
+  printf("sizeof(drizzle_result_st) = %zu\n", sizeof(drizzle_result_st));
+
+  if ((drizzle= drizzle_create(NULL)) == NULL)
+    drizzle_test_error("drizzle_create");
+
+  if ((con= drizzle_con_create(drizzle, NULL)) == NULL)
+    drizzle_test_error("drizzle_con_create");
+
+  if ((result= drizzle_result_create(con, &result_buffer)) == NULL)
+    drizzle_test_error("drizzle_result_create");
+  drizzle_result_free(result);
+
+  if ((result= drizzle_result_create(con, NULL)) == NULL)
+    drizzle_test_error("drizzle_result_create");
+
+  if ((clone= drizzle_result_clone(con, NULL, result)) == NULL)
+    drizzle_test_error("drizzle_result_clone");
+  drizzle_result_free(clone);
+
+  drizzle_result_set_info(result, "simple test");
+
+  if (strcmp(drizzle_result_info(result), "simple test"))
+    drizzle_test_error("drizzle_result_info");
+
+  drizzle_result_free(result);
+  drizzle_con_free(con);
+  drizzle_free(drizzle);
+
+  return 0;
+}

=== added file 'unittests/drizzle_st.c'
--- unittests/drizzle_st.c	1970-01-01 00:00:00 +0000
+++ unittests/drizzle_st.c	2010-08-12 20:27:32 +0000
@@ -0,0 +1,216 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief Tests for drizzle_st Structures
+ */
+
+#include <assert.h>
+
+#include "tests/common.h"
+
+#define DRIZZLE_TEST_PORT 12399
+
+static void _log(const char *line, drizzle_verbose_t verbose,
+                 void *context);
+static drizzle_return_t _event_watch(drizzle_con_st *con, short events,
+                                     void *context);
+
+static int _event_watch_read_bits= 0;
+static int _event_watch_write_bits= 0;
+
+int main(void)
+{
+  drizzle_verbose_t verbose;
+  drizzle_st *drizzle;
+  drizzle_st drizzle_buffer;
+  drizzle_st *clone;
+  drizzle_st clone_buffer;
+  drizzle_con_st *con;
+  drizzle_con_st *listen_con;
+  drizzle_return_t ret;
+
+  printf("sizeof(drizzle_st) = %zu\n", sizeof(drizzle_st));
+  printf("sizeof(drizzle_return_t) = %zu\n", sizeof(drizzle_return_t));
+  printf("sizeof(drizzle_verbose_t) = %zu\n", sizeof(drizzle_verbose_t));
+  printf("sizeof(drizzle_options_t) = %zu\n", sizeof(drizzle_options_t));
+
+  drizzle_test("drizzle_version");
+  printf("  %s\n", drizzle_version());
+
+  drizzle_test("drizzle_bugreport");
+  printf("  %s\n", drizzle_bugreport());
+
+  drizzle_test("drizzle_verbose_name");
+  for (verbose= DRIZZLE_VERBOSE_NEVER; verbose <= DRIZZLE_VERBOSE_MAX;
+       verbose++)
+  {
+    printf("  %s\n", drizzle_verbose_name(verbose));
+  }
+
+  drizzle_test("drizzle_create buffer");
+  if ((drizzle= drizzle_create(&drizzle_buffer)) == NULL)
+    drizzle_test_error("returned NULL");
+
+  drizzle_test("drizzle_free buffer");
+  drizzle_free(drizzle);
+
+  drizzle_test("drizzle_create");
+  if ((drizzle= drizzle_create(NULL)) == NULL)
+    drizzle_test_error("returned NULL");
+
+  drizzle_test("drizzle_clone");
+  if ((clone= drizzle_clone(NULL, drizzle)) == NULL)
+    drizzle_test_error("drizzle_clone");
+
+  drizzle_test("drizzle_free");
+  drizzle_free(clone);
+
+  drizzle_test("drizzle_clone buffer");
+  if ((clone= drizzle_clone(&clone_buffer, drizzle)) == NULL)
+    drizzle_test_error("returned NULL");
+
+  drizzle_test("drizzle_free buffer");
+  drizzle_free(clone);
+
+  drizzle_test("drizzle_error");
+  if (strcmp(drizzle_error(drizzle), ""))
+    drizzle_test_error("error not empty");
+
+  drizzle_test("drizzle_errno");
+  if (drizzle_errno(drizzle) != 0)
+    drizzle_test_error("errno not 0");
+
+  drizzle_test("drizzle_error_code");
+  if (drizzle_error_code(drizzle) != 0)
+    drizzle_test_error("error_code not 0");
+
+  drizzle_test("drizzle_sqlstate");
+  if (strcmp(drizzle_sqlstate(drizzle), ""))
+    drizzle_test_error("sqlstate not empty");
+
+  /* @todo remove this option with new API. */
+  drizzle_remove_options(drizzle, DRIZZLE_FREE_OBJECTS);
+
+  drizzle_test("drizzle_options");
+  if (drizzle_options(drizzle) != DRIZZLE_ALLOCATED)
+    drizzle_test_error("expected options not set");
+
+  drizzle_test("drizzle_add_options");
+  drizzle_add_options(drizzle, DRIZZLE_NON_BLOCKING);
+
+  drizzle_test("drizzle_options");
+  if (drizzle_options(drizzle) != (DRIZZLE_ALLOCATED | DRIZZLE_NON_BLOCKING))
+    drizzle_test_error("expected options not set");
+
+  drizzle_test("drizzle_remove_options");
+  drizzle_remove_options(drizzle, DRIZZLE_NON_BLOCKING);
+
+  drizzle_test("drizzle_options");
+  if (drizzle_options(drizzle) != DRIZZLE_ALLOCATED)
+    drizzle_test_error("expected options not set");
+
+  drizzle_test("drizzle_set_options");
+  drizzle_set_options(drizzle, DRIZZLE_ALLOCATED | DRIZZLE_NON_BLOCKING);
+
+  drizzle_test("drizzle_options");
+  if (drizzle_options(drizzle) != (DRIZZLE_ALLOCATED | DRIZZLE_NON_BLOCKING))
+    drizzle_test_error("expected options not set");
+
+  drizzle_test("drizzle_set_options");
+  drizzle_set_options(drizzle, DRIZZLE_ALLOCATED);
+
+  drizzle_test("drizzle_options");
+  if (drizzle_options(drizzle) != DRIZZLE_ALLOCATED)
+    drizzle_test_error("expected options not set");
+
+  drizzle_test("drizzle_set_timeout");
+  drizzle_set_timeout(drizzle, 1234);
+
+  drizzle_test("drizzle_timeout");
+  if (drizzle_timeout(drizzle) != 1234)
+    drizzle_test_error("expected timeout not set");
+
+  drizzle_test("drizzle_set_verbose");
+  drizzle_set_verbose(drizzle, DRIZZLE_VERBOSE_CRAZY);
+
+  drizzle_test("drizzle_verbose");
+  if (drizzle_verbose(drizzle) != DRIZZLE_VERBOSE_CRAZY)
+    drizzle_test_error("expected verbose not set");
+
+  drizzle_test("drizzle_set_log_fn");
+  drizzle_set_log_fn(drizzle, _log, NULL);
+
+  drizzle_test("drizzle_set_event_watch_fn");
+  drizzle_set_event_watch_fn(drizzle, _event_watch, NULL);
+
+  /* Create a listening connection to verify that event_watch_fn gets called. */
+  listen_con= drizzle_con_create(drizzle, NULL);
+  assert(listen_con != NULL);
+  drizzle_con_set_tcp(listen_con, "127.0.0.1", DRIZZLE_TEST_PORT);
+  ret= drizzle_con_listen(listen_con);
+  assert(ret == DRIZZLE_RETURN_OK);  
+  if (_event_watch_read_bits == 0)
+    drizzle_test_error("event_watch_fn not called to wait for connections");
+  _event_watch_read_bits= 0;
+
+  /* Attempt a non-blocking connection. */
+  drizzle_add_options(drizzle, DRIZZLE_NON_BLOCKING);
+  con= drizzle_con_add_tcp(drizzle, NULL, "127.0.0.1", DRIZZLE_TEST_PORT, "user", "pw", "db",
+                           DRIZZLE_CON_NONE);
+  assert(con != NULL);
+  ret= drizzle_con_connect(con);
+  assert(ret == DRIZZLE_RETURN_IO_WAIT);
+  if (_event_watch_read_bits == 0 && _event_watch_write_bits == 0)
+    drizzle_test_error("event_watch_fn not called to wait for I/O");
+  drizzle_con_free(con);
+  _event_watch_read_bits= 0;
+  _event_watch_write_bits= 0;
+
+  drizzle_con_free(listen_con);
+
+  drizzle_test("drizzle_free");
+  drizzle_free(drizzle);
+
+  return 0;
+}
+
+static void _log(const char *line, drizzle_verbose_t verbose,
+                 void *context)
+{
+  (void) line;
+  (void) verbose;
+  (void) context;
+}
+
+static drizzle_return_t _event_watch(drizzle_con_st *con, short events,
+                                     void *context)
+{
+  (void) con;
+  (void) events;
+  (void) context;
+
+  /* fake register the file descriptor */
+  int fd= drizzle_con_fd(con);
+  assert(0 <= fd && fd < (int) sizeof(_event_watch_read_bits) * 8);
+  if (events & POLLIN) {
+    _event_watch_read_bits|= 1 << fd;
+  } else {
+    _event_watch_read_bits&= ~(1 << fd);
+  }
+  if (events & POLLOUT) {
+    _event_watch_write_bits|= 1 << fd;
+  } else {
+    _event_watch_write_bits&= ~(1 << fd);
+  }
+
+  return DRIZZLE_RETURN_OK;
+}

=== modified file 'unittests/include.am'
--- unittests/include.am	2010-07-07 06:52:54 +0000
+++ unittests/include.am	2010-08-12 20:27:32 +0000
@@ -3,10 +3,17 @@
 # All paths should be given relative to the root
 
 if HAVE_LIBGTEST
-check_PROGRAMS += unittests/unittests
+check_PROGRAMS += unittests/unittests \
+		  unittests/drizzle_st \
+		  unittests/drizzle_con_st \
+		  unittests/drizzle_query_st \
+		  unittests/drizzle_result_st \
+		  unittests/drizzle_column_st \
+		  unittests/client_server
 endif
 
 noinst_HEADERS += \
+		  unittests/common.h \
 		  unittests/plugin/plugin_stubs.h \
 		  unittests/temporal_generator.h
 

=== added directory 'win32'
=== added file 'win32/Makefile.am'
--- win32/Makefile.am	1970-01-01 00:00:00 +0000
+++ win32/Makefile.am	2010-08-12 20:27:32 +0000
@@ -0,0 +1,227 @@
+## DO NOT EDIT! GENERATED AUTOMATICALLY!
+## Process this file with automake to produce Makefile.in.
+# Copyright (C) 2002-2008 Free Software Foundation, Inc.
+#
+# This file is free software, distributed under the terms of the GNU
+# General Public License.  As a special exception to the GNU General
+# Public License, this file may be distributed as part of a program
+# that contains a configuration script generated by Autoconf, under
+# the same distribution terms as the rest of that program.
+#
+# Generated by gnulib-tool.
+# Reproduce by: gnulib-tool --import --dir=. --lib=libgnu --source-base=gnulib --m4-base=m4 --doc-base=doc --tests-base=tests --aux-dir=config --lgpl --libtool --macro-prefix=gl poll
+
+AUTOMAKE_OPTIONS = 1.5 gnits
+
+SUBDIRS =
+noinst_HEADERS =
+noinst_LIBRARIES =
+noinst_LTLIBRARIES =
+EXTRA_DIST =
+BUILT_SOURCES =
+SUFFIXES =
+MOSTLYCLEANFILES = core *.stackdump
+MOSTLYCLEANDIRS =
+CLEANFILES =
+DISTCLEANFILES =
+MAINTAINERCLEANFILES =
+
+AM_CPPFLAGS =
+
+noinst_LTLIBRARIES += libgnu.la
+
+libgnu_la_SOURCES =
+libgnu_la_LIBADD = $(gl_LTLIBOBJS)
+libgnu_la_DEPENDENCIES = $(gl_LTLIBOBJS)
+EXTRA_libgnu_la_SOURCES =
+libgnu_la_LDFLAGS = $(AM_LDFLAGS)
+
+## begin gnulib module alloca
+
+
+EXTRA_DIST += alloca.c
+
+EXTRA_libgnu_la_SOURCES += alloca.c
+
+libgnu_la_LIBADD += @LTALLOCA@
+libgnu_la_DEPENDENCIES += @LTALLOCA@
+## end   gnulib module alloca
+
+## begin gnulib module alloca-opt
+
+BUILT_SOURCES += $(ALLOCA_H)
+
+# We need the following in order to create <alloca.h> when the system
+# doesn't have one that works with the given compiler.
+alloca.h: alloca.in.h
+	{ echo '/* DO NOT EDIT! GENERATED AUTOMATICALLY! */'; \
+	  cat $(srcdir)/alloca.in.h; \
+	} > $@-t
+	mv -f $@-t $@
+MOSTLYCLEANFILES += alloca.h alloca.h-t
+
+EXTRA_DIST += alloca.in.h
+
+## end   gnulib module alloca-opt
+
+## begin gnulib module errno
+
+BUILT_SOURCES += $(ERRNO_H)
+
+# We need the following in order to create <errno.h> when the system
+# doesn't have one that is POSIX compliant.
+errno.h: errno.in.h
+	rm -f $@-t $@
+	{ echo '/* DO NOT EDIT! GENERATED AUTOMATICALLY! */' && \
+	  sed -e 's|@''INCLUDE_NEXT''@|$(INCLUDE_NEXT)|g' \
+	      -e 's|@''PRAGMA_SYSTEM_HEADER''@|@PRAGMA_SYSTEM_HEADER@|g' \
+	      -e 's|@''NEXT_ERRNO_H''@|$(NEXT_ERRNO_H)|g' \
+	      -e 's|@''EMULTIHOP_HIDDEN''@|$(EMULTIHOP_HIDDEN)|g' \
+	      -e 's|@''EMULTIHOP_VALUE''@|$(EMULTIHOP_VALUE)|g' \
+	      -e 's|@''ENOLINK_HIDDEN''@|$(ENOLINK_HIDDEN)|g' \
+	      -e 's|@''ENOLINK_VALUE''@|$(ENOLINK_VALUE)|g' \
+	      -e 's|@''EOVERFLOW_HIDDEN''@|$(EOVERFLOW_HIDDEN)|g' \
+	      -e 's|@''EOVERFLOW_VALUE''@|$(EOVERFLOW_VALUE)|g' \
+	      < $(srcdir)/errno.in.h; \
+	} > $@-t
+	mv $@-t $@
+MOSTLYCLEANFILES += errno.h errno.h-t
+
+EXTRA_DIST += errno.in.h
+
+## end   gnulib module errno
+
+## begin gnulib module link-warning
+
+LINK_WARNING_H=$(top_srcdir)/config/link-warning.h
+
+EXTRA_DIST += $(top_srcdir)/config/link-warning.h
+
+## end   gnulib module link-warning
+
+## begin gnulib module poll
+
+BUILT_SOURCES += $(POLL_H)
+
+# We need the following in order to create <poll.h> when the system
+# doesn't have one.
+poll.h: poll.in.h
+	{ echo '/* DO NOT EDIT! GENERATED AUTOMATICALLY! */'; \
+	  cat $(srcdir)/poll.in.h; \
+	} > $@-t
+	mv -f $@-t $@
+MOSTLYCLEANFILES += poll.h poll.h-t
+
+EXTRA_DIST += poll.c poll.in.h
+
+EXTRA_libgnu_la_SOURCES += poll.c
+
+## end   gnulib module poll
+
+## begin gnulib module sys_select
+
+BUILT_SOURCES += $(SYS_SELECT_H)
+
+# We need the following in order to create <sys/select.h> when the system
+# doesn't have one that works with the given compiler.
+sys/select.h: sys_select.in.h
+	@MKDIR_P@ sys
+	rm -f $@-t $@
+	{ echo '/* DO NOT EDIT! GENERATED AUTOMATICALLY! */'; \
+	  sed -e 's|@''INCLUDE_NEXT''@|$(INCLUDE_NEXT)|g' \
+	      -e 's|@''PRAGMA_SYSTEM_HEADER''@|@PRAGMA_SYSTEM_HEADER@|g' \
+	      -e 's|@''NEXT_SYS_SELECT_H''@|$(NEXT_SYS_SELECT_H)|g' \
+	      -e 's|@''HAVE_SYS_SELECT_H''@|$(HAVE_SYS_SELECT_H)|g' \
+	      -e 's|@''GNULIB_SELECT''@|$(GNULIB_SELECT)|g' \
+	      -e 's|@''HAVE_WINSOCK2_H''@|$(HAVE_WINSOCK2_H)|g' \
+	      -e '/definition of GL_LINK_WARNING/r $(LINK_WARNING_H)' \
+	      < $(srcdir)/sys_select.in.h; \
+	} > $@-t
+	mv $@-t $@
+MOSTLYCLEANFILES += sys/select.h sys/select.h-t
+MOSTLYCLEANDIRS += sys
+
+EXTRA_DIST += sys_select.in.h
+
+## end   gnulib module sys_select
+
+## begin gnulib module sys_socket
+
+BUILT_SOURCES += $(SYS_SOCKET_H)
+
+# We need the following in order to create <sys/socket.h> when the system
+# doesn't have one that works with the given compiler.
+sys/socket.h: sys_socket.in.h
+	@MKDIR_P@ sys
+	{ echo '/* DO NOT EDIT! GENERATED AUTOMATICALLY! */'; \
+	  sed -e 's|@''INCLUDE_NEXT''@|$(INCLUDE_NEXT)|g' \
+	      -e 's|@''PRAGMA_SYSTEM_HEADER''@|@PRAGMA_SYSTEM_HEADER@|g' \
+	      -e 's|@''NEXT_SYS_SOCKET_H''@|$(NEXT_SYS_SOCKET_H)|g' \
+	      -e 's|@''HAVE_SYS_SOCKET_H''@|$(HAVE_SYS_SOCKET_H)|g' \
+	      -e 's|@''GNULIB_CLOSE''@|$(GNULIB_CLOSE)|g' \
+	      -e 's|@''GNULIB_SOCKET''@|$(GNULIB_SOCKET)|g' \
+	      -e 's|@''GNULIB_CONNECT''@|$(GNULIB_CONNECT)|g' \
+	      -e 's|@''GNULIB_ACCEPT''@|$(GNULIB_ACCEPT)|g' \
+	      -e 's|@''GNULIB_BIND''@|$(GNULIB_BIND)|g' \
+	      -e 's|@''GNULIB_GETPEERNAME''@|$(GNULIB_GETPEERNAME)|g' \
+	      -e 's|@''GNULIB_GETSOCKNAME''@|$(GNULIB_GETSOCKNAME)|g' \
+	      -e 's|@''GNULIB_GETSOCKOPT''@|$(GNULIB_GETSOCKOPT)|g' \
+	      -e 's|@''GNULIB_LISTEN''@|$(GNULIB_LISTEN)|g' \
+	      -e 's|@''GNULIB_RECV''@|$(GNULIB_RECV)|g' \
+	      -e 's|@''GNULIB_SEND''@|$(GNULIB_SEND)|g' \
+	      -e 's|@''GNULIB_RECVFROM''@|$(GNULIB_RECVFROM)|g' \
+	      -e 's|@''GNULIB_SENDTO''@|$(GNULIB_SENDTO)|g' \
+	      -e 's|@''GNULIB_SETSOCKOPT''@|$(GNULIB_SETSOCKOPT)|g' \
+	      -e 's|@''GNULIB_SHUTDOWN''@|$(GNULIB_SHUTDOWN)|g' \
+	      -e 's|@''HAVE_WINSOCK2_H''@|$(HAVE_WINSOCK2_H)|g' \
+	      -e 's|@''HAVE_WS2TCPIP_H''@|$(HAVE_WS2TCPIP_H)|g' \
+	      -e '/definition of GL_LINK_WARNING/r $(LINK_WARNING_H)' \
+	      < $(srcdir)/sys_socket.in.h; \
+	} > $@-t
+	mv -f $@-t $@
+MOSTLYCLEANFILES += sys/socket.h sys/socket.h-t
+MOSTLYCLEANDIRS += sys
+
+EXTRA_DIST += sys_socket.in.h
+
+## end   gnulib module sys_socket
+
+## begin gnulib module sys_time
+
+BUILT_SOURCES += $(SYS_TIME_H)
+
+# We need the following in order to create <sys/time.h> when the system
+# doesn't have one that works with the given compiler.
+sys/time.h: sys_time.in.h
+	@MKDIR_P@ sys
+	rm -f $@-t $@
+	{ echo '/* DO NOT EDIT! GENERATED AUTOMATICALLY! */'; \
+	  sed -e 's/@''HAVE_SYS_TIME_H''@/$(HAVE_SYS_TIME_H)/g' \
+	      -e 's|@''INCLUDE_NEXT''@|$(INCLUDE_NEXT)|g' \
+	      -e 's|@''PRAGMA_SYSTEM_HEADER''@|@PRAGMA_SYSTEM_HEADER@|g' \
+	      -e 's|@''NEXT_SYS_TIME_H''@|$(NEXT_SYS_TIME_H)|g' \
+	      -e 's/@''REPLACE_GETTIMEOFDAY''@/$(REPLACE_GETTIMEOFDAY)/g' \
+	      -e 's/@''HAVE_STRUCT_TIMEVAL''@/$(HAVE_STRUCT_TIMEVAL)/g' \
+	      < $(srcdir)/sys_time.in.h; \
+	} > $@-t
+	mv $@-t $@
+MOSTLYCLEANFILES += sys/time.h sys/time.h-t
+
+EXTRA_DIST += sys_time.in.h
+
+## end   gnulib module sys_time
+
+## begin gnulib module dummy
+
+libgnu_la_SOURCES += dummy.c
+
+## end   gnulib module dummy
+
+
+mostlyclean-local: mostlyclean-generic
+	@for dir in '' $(MOSTLYCLEANDIRS); do \
+	  if test -n "$$dir" && test -d $$dir; then \
+	    echo "rmdir $$dir"; rmdir $$dir; \
+	  fi; \
+	done; \
+	:

=== added file 'win32/alloca.c'
--- win32/alloca.c	1970-01-01 00:00:00 +0000
+++ win32/alloca.c	2010-08-12 20:27:32 +0000
@@ -0,0 +1,489 @@
+/* alloca.c -- allocate automatically reclaimed memory
+   (Mostly) portable public-domain implementation -- D A Gwyn
+
+   This implementation of the PWB library alloca function,
+   which is used to allocate space off the run-time stack so
+   that it is automatically reclaimed upon procedure exit,
+   was inspired by discussions with J. Q. Johnson of Cornell.
+   J.Otto Tennant <jot@cray.com> contributed the Cray support.
+
+   There are some preprocessor constants that can
+   be defined when compiling for your specific system, for
+   improved efficiency; however, the defaults should be okay.
+
+   The general concept of this implementation is to keep
+   track of all alloca-allocated blocks, and reclaim any
+   that are found to be deeper in the stack than the current
+   invocation.  This heuristic does not reclaim storage as
+   soon as it becomes invalid, but it will do so eventually.
+
+   As a special case, alloca(0) reclaims storage without
+   allocating any.  It is a good idea to use alloca(0) in
+   your main control loop, etc. to force garbage collection.  */
+
+#include <config.h>
+
+#include <alloca.h>
+
+#include <string.h>
+#include <stdlib.h>
+
+#ifdef emacs
+# include "lisp.h"
+# include "blockinput.h"
+# ifdef EMACS_FREE
+#  undef free
+#  define free EMACS_FREE
+# endif
+#else
+# define memory_full() abort ()
+#endif
+
+/* If compiling with GCC 2, this file's not needed.  */
+#if !defined (__GNUC__) || __GNUC__ < 2
+
+/* If someone has defined alloca as a macro,
+   there must be some other way alloca is supposed to work.  */
+# ifndef alloca
+
+#  ifdef emacs
+#   ifdef static
+/* actually, only want this if static is defined as ""
+   -- this is for usg, in which emacs must undefine static
+   in order to make unexec workable
+   */
+#    ifndef STACK_DIRECTION
+you
+lose
+-- must know STACK_DIRECTION at compile-time
+/* Using #error here is not wise since this file should work for
+   old and obscure compilers.  */
+#    endif /* STACK_DIRECTION undefined */
+#   endif /* static */
+#  endif /* emacs */
+
+/* If your stack is a linked list of frames, you have to
+   provide an "address metric" ADDRESS_FUNCTION macro.  */
+
+#  if defined (CRAY) && defined (CRAY_STACKSEG_END)
+long i00afunc ();
+#   define ADDRESS_FUNCTION(arg) (char *) i00afunc (&(arg))
+#  else
+#   define ADDRESS_FUNCTION(arg) &(arg)
+#  endif
+
+/* Define STACK_DIRECTION if you know the direction of stack
+   growth for your system; otherwise it will be automatically
+   deduced at run-time.
+
+   STACK_DIRECTION > 0 => grows toward higher addresses
+   STACK_DIRECTION < 0 => grows toward lower addresses
+   STACK_DIRECTION = 0 => direction of growth unknown  */
+
+#  ifndef STACK_DIRECTION
+#   define STACK_DIRECTION	0	/* Direction unknown.  */
+#  endif
+
+#  if STACK_DIRECTION != 0
+
+#   define STACK_DIR	STACK_DIRECTION	/* Known at compile-time.  */
+
+#  else /* STACK_DIRECTION == 0; need run-time code.  */
+
+static int stack_dir;		/* 1 or -1 once known.  */
+#   define STACK_DIR	stack_dir
+
+static void
+find_stack_direction (void)
+{
+  static char *addr = NULL;	/* Address of first `dummy', once known.  */
+  auto char dummy;		/* To get stack address.  */
+
+  if (addr == NULL)
+    {				/* Initial entry.  */
+      addr = ADDRESS_FUNCTION (dummy);
+
+      find_stack_direction ();	/* Recurse once.  */
+    }
+  else
+    {
+      /* Second entry.  */
+      if (ADDRESS_FUNCTION (dummy) > addr)
+	stack_dir = 1;		/* Stack grew upward.  */
+      else
+	stack_dir = -1;		/* Stack grew downward.  */
+    }
+}
+
+#  endif /* STACK_DIRECTION == 0 */
+
+/* An "alloca header" is used to:
+   (a) chain together all alloca'ed blocks;
+   (b) keep track of stack depth.
+
+   It is very important that sizeof(header) agree with malloc
+   alignment chunk size.  The following default should work okay.  */
+
+#  ifndef	ALIGN_SIZE
+#   define ALIGN_SIZE	sizeof(double)
+#  endif
+
+typedef union hdr
+{
+  char align[ALIGN_SIZE];	/* To force sizeof(header).  */
+  struct
+    {
+      union hdr *next;		/* For chaining headers.  */
+      char *deep;		/* For stack depth measure.  */
+    } h;
+} header;
+
+static header *last_alloca_header = NULL;	/* -> last alloca header.  */
+
+/* Return a pointer to at least SIZE bytes of storage,
+   which will be automatically reclaimed upon exit from
+   the procedure that called alloca.  Originally, this space
+   was supposed to be taken from the current stack frame of the
+   caller, but that method cannot be made to work for some
+   implementations of C, for example under Gould's UTX/32.  */
+
+void *
+alloca (size_t size)
+{
+  auto char probe;		/* Probes stack depth: */
+  register char *depth = ADDRESS_FUNCTION (probe);
+
+#  if STACK_DIRECTION == 0
+  if (STACK_DIR == 0)		/* Unknown growth direction.  */
+    find_stack_direction ();
+#  endif
+
+  /* Reclaim garbage, defined as all alloca'd storage that
+     was allocated from deeper in the stack than currently.  */
+
+  {
+    register header *hp;	/* Traverses linked list.  */
+
+#  ifdef emacs
+    BLOCK_INPUT;
+#  endif
+
+    for (hp = last_alloca_header; hp != NULL;)
+      if ((STACK_DIR > 0 && hp->h.deep > depth)
+	  || (STACK_DIR < 0 && hp->h.deep < depth))
+	{
+	  register header *np = hp->h.next;
+
+	  free (hp);		/* Collect garbage.  */
+
+	  hp = np;		/* -> next header.  */
+	}
+      else
+	break;			/* Rest are not deeper.  */
+
+    last_alloca_header = hp;	/* -> last valid storage.  */
+
+#  ifdef emacs
+    UNBLOCK_INPUT;
+#  endif
+  }
+
+  if (size == 0)
+    return NULL;		/* No allocation required.  */
+
+  /* Allocate combined header + user data storage.  */
+
+  {
+    /* Address of header.  */
+    register header *new;
+
+    size_t combined_size = sizeof (header) + size;
+    if (combined_size < sizeof (header))
+      memory_full ();
+
+    new = malloc (combined_size);
+
+    if (! new)
+      memory_full ();
+
+    new->h.next = last_alloca_header;
+    new->h.deep = depth;
+
+    last_alloca_header = new;
+
+    /* User storage begins just after header.  */
+
+    return (void *) (new + 1);
+  }
+}
+
+#  if defined (CRAY) && defined (CRAY_STACKSEG_END)
+
+#   ifdef DEBUG_I00AFUNC
+#    include <stdio.h>
+#   endif
+
+#   ifndef CRAY_STACK
+#    define CRAY_STACK
+#    ifndef CRAY2
+/* Stack structures for CRAY-1, CRAY X-MP, and CRAY Y-MP */
+struct stack_control_header
+  {
+    long shgrow:32;		/* Number of times stack has grown.  */
+    long shaseg:32;		/* Size of increments to stack.  */
+    long shhwm:32;		/* High water mark of stack.  */
+    long shsize:32;		/* Current size of stack (all segments).  */
+  };
+
+/* The stack segment linkage control information occurs at
+   the high-address end of a stack segment.  (The stack
+   grows from low addresses to high addresses.)  The initial
+   part of the stack segment linkage control information is
+   0200 (octal) words.  This provides for register storage
+   for the routine which overflows the stack.  */
+
+struct stack_segment_linkage
+  {
+    long ss[0200];		/* 0200 overflow words.  */
+    long sssize:32;		/* Number of words in this segment.  */
+    long ssbase:32;		/* Offset to stack base.  */
+    long:32;
+    long sspseg:32;		/* Offset to linkage control of previous
+				   segment of stack.  */
+    long:32;
+    long sstcpt:32;		/* Pointer to task common address block.  */
+    long sscsnm;		/* Private control structure number for
+				   microtasking.  */
+    long ssusr1;		/* Reserved for user.  */
+    long ssusr2;		/* Reserved for user.  */
+    long sstpid;		/* Process ID for pid based multi-tasking.  */
+    long ssgvup;		/* Pointer to multitasking thread giveup.  */
+    long sscray[7];		/* Reserved for Cray Research.  */
+    long ssa0;
+    long ssa1;
+    long ssa2;
+    long ssa3;
+    long ssa4;
+    long ssa5;
+    long ssa6;
+    long ssa7;
+    long sss0;
+    long sss1;
+    long sss2;
+    long sss3;
+    long sss4;
+    long sss5;
+    long sss6;
+    long sss7;
+  };
+
+#    else /* CRAY2 */
+/* The following structure defines the vector of words
+   returned by the STKSTAT library routine.  */
+struct stk_stat
+  {
+    long now;			/* Current total stack size.  */
+    long maxc;			/* Amount of contiguous space which would
+				   be required to satisfy the maximum
+				   stack demand to date.  */
+    long high_water;		/* Stack high-water mark.  */
+    long overflows;		/* Number of stack overflow ($STKOFEN) calls.  */
+    long hits;			/* Number of internal buffer hits.  */
+    long extends;		/* Number of block extensions.  */
+    long stko_mallocs;		/* Block allocations by $STKOFEN.  */
+    long underflows;		/* Number of stack underflow calls ($STKRETN).  */
+    long stko_free;		/* Number of deallocations by $STKRETN.  */
+    long stkm_free;		/* Number of deallocations by $STKMRET.  */
+    long segments;		/* Current number of stack segments.  */
+    long maxs;			/* Maximum number of stack segments so far.  */
+    long pad_size;		/* Stack pad size.  */
+    long current_address;	/* Current stack segment address.  */
+    long current_size;		/* Current stack segment size.  This
+				   number is actually corrupted by STKSTAT to
+				   include the fifteen word trailer area.  */
+    long initial_address;	/* Address of initial segment.  */
+    long initial_size;		/* Size of initial segment.  */
+  };
+
+/* The following structure describes the data structure which trails
+   any stack segment.  I think that the description in 'asdef' is
+   out of date.  I only describe the parts that I am sure about.  */
+
+struct stk_trailer
+  {
+    long this_address;		/* Address of this block.  */
+    long this_size;		/* Size of this block (does not include
+				   this trailer).  */
+    long unknown2;
+    long unknown3;
+    long link;			/* Address of trailer block of previous
+				   segment.  */
+    long unknown5;
+    long unknown6;
+    long unknown7;
+    long unknown8;
+    long unknown9;
+    long unknown10;
+    long unknown11;
+    long unknown12;
+    long unknown13;
+    long unknown14;
+  };
+
+#    endif /* CRAY2 */
+#   endif /* not CRAY_STACK */
+
+#   ifdef CRAY2
+/* Determine a "stack measure" for an arbitrary ADDRESS.
+   I doubt that "lint" will like this much.  */
+
+static long
+i00afunc (long *address)
+{
+  struct stk_stat status;
+  struct stk_trailer *trailer;
+  long *block, size;
+  long result = 0;
+
+  /* We want to iterate through all of the segments.  The first
+     step is to get the stack status structure.  We could do this
+     more quickly and more directly, perhaps, by referencing the
+     $LM00 common block, but I know that this works.  */
+
+  STKSTAT (&status);
+
+  /* Set up the iteration.  */
+
+  trailer = (struct stk_trailer *) (status.current_address
+				    + status.current_size
+				    - 15);
+
+  /* There must be at least one stack segment.  Therefore it is
+     a fatal error if "trailer" is null.  */
+
+  if (trailer == 0)
+    abort ();
+
+  /* Discard segments that do not contain our argument address.  */
+
+  while (trailer != 0)
+    {
+      block = (long *) trailer->this_address;
+      size = trailer->this_size;
+      if (block == 0 || size == 0)
+	abort ();
+      trailer = (struct stk_trailer *) trailer->link;
+      if ((block <= address) && (address < (block + size)))
+	break;
+    }
+
+  /* Set the result to the offset in this segment and add the sizes
+     of all predecessor segments.  */
+
+  result = address - block;
+
+  if (trailer == 0)
+    {
+      return result;
+    }
+
+  do
+    {
+      if (trailer->this_size <= 0)
+	abort ();
+      result += trailer->this_size;
+      trailer = (struct stk_trailer *) trailer->link;
+    }
+  while (trailer != 0);
+
+  /* We are done.  Note that if you present a bogus address (one
+     not in any segment), you will get a different number back, formed
+     from subtracting the address of the first block.  This is probably
+     not what you want.  */
+
+  return (result);
+}
+
+#   else /* not CRAY2 */
+/* Stack address function for a CRAY-1, CRAY X-MP, or CRAY Y-MP.
+   Determine the number of the cell within the stack,
+   given the address of the cell.  The purpose of this
+   routine is to linearize, in some sense, stack addresses
+   for alloca.  */
+
+static long
+i00afunc (long address)
+{
+  long stkl = 0;
+
+  long size, pseg, this_segment, stack;
+  long result = 0;
+
+  struct stack_segment_linkage *ssptr;
+
+  /* Register B67 contains the address of the end of the
+     current stack segment.  If you (as a subprogram) store
+     your registers on the stack and find that you are past
+     the contents of B67, you have overflowed the segment.
+
+     B67 also points to the stack segment linkage control
+     area, which is what we are really interested in.  */
+
+  stkl = CRAY_STACKSEG_END ();
+  ssptr = (struct stack_segment_linkage *) stkl;
+
+  /* If one subtracts 'size' from the end of the segment,
+     one has the address of the first word of the segment.
+
+     If this is not the first segment, 'pseg' will be
+     nonzero.  */
+
+  pseg = ssptr->sspseg;
+  size = ssptr->sssize;
+
+  this_segment = stkl - size;
+
+  /* It is possible that calling this routine itself caused
+     a stack overflow.  Discard stack segments which do not
+     contain the target address.  */
+
+  while (!(this_segment <= address && address <= stkl))
+    {
+#    ifdef DEBUG_I00AFUNC
+      fprintf (stderr, "%011o %011o %011o\n", this_segment, address, stkl);
+#    endif
+      if (pseg == 0)
+	break;
+      stkl = stkl - pseg;
+      ssptr = (struct stack_segment_linkage *) stkl;
+      size = ssptr->sssize;
+      pseg = ssptr->sspseg;
+      this_segment = stkl - size;
+    }
+
+  result = address - this_segment;
+
+  /* If you subtract pseg from the current end of the stack,
+     you get the address of the previous stack segment's end.
+     This seems a little convoluted to me, but I'll bet you save
+     a cycle somewhere.  */
+
+  while (pseg != 0)
+    {
+#    ifdef DEBUG_I00AFUNC
+      fprintf (stderr, "%011o %011o\n", pseg, size);
+#    endif
+      stkl = stkl - pseg;
+      ssptr = (struct stack_segment_linkage *) stkl;
+      size = ssptr->sssize;
+      pseg = ssptr->sspseg;
+      result += size;
+    }
+  return (result);
+}
+
+#   endif /* not CRAY2 */
+#  endif /* CRAY */
+
+# endif /* no alloca */
+#endif /* not GCC version 2 */

=== added file 'win32/alloca.h'
--- win32/alloca.h	1970-01-01 00:00:00 +0000
+++ win32/alloca.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,56 @@
+/* Memory allocation on the stack.
+
+   Copyright (C) 1995, 1999, 2001-2004, 2006-2008 Free Software
+   Foundation, Inc.
+
+   This program is free software; you can redistribute it and/or modify it
+   under the terms of the GNU Lesser General Public License as published
+   by the Free Software Foundation; either version 2, or (at your option)
+   any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   General Public License for more details.
+
+   You should have received a copy of the GNU Lesser General Public
+   License along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+   USA.  */
+
+/* Avoid using the symbol _ALLOCA_H here, as Bison assumes _ALLOCA_H
+   means there is a real alloca function.  */
+#ifndef _GL_ALLOCA_H
+#define _GL_ALLOCA_H
+
+/* alloca (N) returns a pointer to N bytes of memory
+   allocated on the stack, which will last until the function returns.
+   Use of alloca should be avoided:
+     - inside arguments of function calls - undefined behaviour,
+     - in inline functions - the allocation may actually last until the
+       calling function returns,
+     - for huge N (say, N >= 65536) - you never know how large (or small)
+       the stack is, and when the stack cannot fulfill the memory allocation
+       request, the program just crashes.
+ */
+
+#ifndef alloca
+# ifdef __GNUC__
+#  define alloca __builtin_alloca
+# elif defined _AIX
+#  define alloca __alloca
+# elif defined _MSC_VER
+#  include <malloc.h>
+#  define alloca _alloca
+# elif defined __DECC && defined __VMS
+#  define alloca __ALLOCA
+# else
+#  include <stddef.h>
+#  ifdef  __cplusplus
+extern "C"
+#  endif
+void *alloca (size_t);
+# endif
+#endif
+
+#endif /* _GL_ALLOCA_H */

=== added file 'win32/conn_uds.c'
--- win32/conn_uds.c	1970-01-01 00:00:00 +0000
+++ win32/conn_uds.c	2010-08-12 20:27:32 +0000
@@ -0,0 +1,49 @@
+/*
+ * Drizzle Client & Protocol Library
+ *
+ * Copyright (C) 2008 Eric Day (eday@oddments.org)
+ * All rights reserved.
+ *
+ * Use and distribution licensed under the BSD license.  See
+ * the COPYING file in this directory for full text.
+ */
+
+/**
+ * @file
+ * @brief UDS Connection stubs for Windows
+ */
+
+#include "libdrizzle/common.h"
+#include "libdrizzle/conn_private.h"
+
+/*
+ * Common definitions
+ */
+
+const char *drizzle_con_uds(drizzle_con_st *con)
+{
+  (void)con;
+  return (const char *)NULL;
+}
+
+void drizzle_con_set_uds(drizzle_con_st *con, const char *uds)
+{
+  (void)con;
+  (void)uds;
+}
+
+/*
+ * Private definitions
+ */
+
+bool drizzle_con_uses_uds(drizzle_con_st *con)
+{
+  (void)con;
+  return false;
+}
+
+void drizzle_con_clone_uds(drizzle_con_st *con, drizzle_con_st *from)
+{
+  (void)con;
+  (void)from;
+}

=== added directory 'win32/gnulib'
=== added file 'win32/gnulib/errno.h'
--- win32/gnulib/errno.h	1970-01-01 00:00:00 +0000
+++ win32/gnulib/errno.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,135 @@
+/* A POSIX-like <errno.h>.
+
+   Copyright (C) 2008 Free Software Foundation, Inc.
+
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU Lesser General Public License as published by
+   the Free Software Foundation; either version 2, or (at your option)
+   any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU Lesser General Public License for more details.
+
+   You should have received a copy of the GNU Lesser General Public License
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+
+#ifndef _GL_ERRNO_H
+
+
+#include <errno.h>
+
+#ifndef _GL_ERRNO_H
+#define _GL_ERRNO_H
+
+
+/* On native Windows platforms, many macros are not defined.  */
+# if (defined _WIN32 || defined __WIN32__) && ! defined __CYGWIN__
+
+/* POSIX says that EAGAIN and EWOULDBLOCK may have the same value.  */
+#  define EWOULDBLOCK     EAGAIN
+
+/* Values >= 100 seem safe to use.  */
+#  define ETXTBSY   100
+#  define GNULIB_defined_ETXTBSY 1
+
+/* These are intentionally the same values as the WSA* error numbers, defined
+   in <winsock2.h>.  */
+#  define EINPROGRESS     10036
+#  define EALREADY        10037
+#  define ENOTSOCK        10038
+#  define EDESTADDRREQ    10039
+#  define EMSGSIZE        10040
+#  define EPROTOTYPE      10041
+#  define ENOPROTOOPT     10042
+#  define EPROTONOSUPPORT 10043
+#  define ESOCKTNOSUPPORT 10044  /* not required by POSIX */
+#  define EOPNOTSUPP      10045
+#  define EPFNOSUPPORT    10046  /* not required by POSIX */
+#  define EAFNOSUPPORT    10047
+#  define EADDRINUSE      10048
+#  define EADDRNOTAVAIL   10049
+#  define ENETDOWN        10050
+#  define ENETUNREACH     10051
+#  define ENETRESET       10052
+#  define ECONNABORTED    10053
+#  define ECONNRESET      10054
+#  define ENOBUFS         10055
+#  define EISCONN         10056
+#  define ENOTCONN        10057
+#  define ESHUTDOWN       10058  /* not required by POSIX */
+#  define ETOOMANYREFS    10059  /* not required by POSIX */
+#  define ETIMEDOUT       10060
+#  define ECONNREFUSED    10061
+#  define ELOOP           10062
+#  define EHOSTDOWN       10064  /* not required by POSIX */
+#  define EHOSTUNREACH    10065
+#  define EPROCLIM        10067  /* not required by POSIX */
+#  define EUSERS          10068  /* not required by POSIX */
+#  define EDQUOT          10069
+#  define ESTALE          10070
+#  define EREMOTE         10071  /* not required by POSIX */
+#  define GNULIB_defined_ESOCK 1
+
+# endif
+
+
+/* On OpenBSD 4.0 and on native Windows, the macros ENOMSG, EIDRM, ENOLINK,
+   EPROTO, EMULTIHOP, EBADMSG, EOVERFLOW, ENOTSUP, ECANCELED are not defined.
+   Define them here.  Values >= 2000 seem safe to use: Solaris ESTALE = 151,
+   HP-UX EWOULDBLOCK = 246, IRIX EDQUOT = 1133.
+
+   Note: When one of these systems defines some of these macros some day,
+   binaries will have to be recompiled so that they recognizes the new
+   errno values from the system.  */
+
+# ifndef ENOMSG
+#  define ENOMSG    2000
+#  define GNULIB_defined_ENOMSG 1
+# endif
+
+# ifndef EIDRM
+#  define EIDRM     2001
+#  define GNULIB_defined_EIDRM 1
+# endif
+
+# ifndef ENOLINK
+#  define ENOLINK   2002
+#  define GNULIB_defined_ENOLINK 1
+# endif
+
+# ifndef EPROTO
+#  define EPROTO    2003
+#  define GNULIB_defined_EPROTO 1
+# endif
+
+# ifndef EMULTIHOP
+#  define EMULTIHOP 2004
+#  define GNULIB_defined_EMULTIHOP 1
+# endif
+
+# ifndef EBADMSG
+#  define EBADMSG   2005
+#  define GNULIB_defined_EBADMSG 1
+# endif
+
+# ifndef EOVERFLOW
+#  define EOVERFLOW 2006
+#  define GNULIB_defined_EOVERFLOW 1
+# endif
+
+# ifndef ENOTSUP
+#  define ENOTSUP   2007
+#  define GNULIB_defined_ENOTSUP 1
+# endif
+
+# ifndef ECANCELED
+#  define ECANCELED 2008
+#  define GNULIB_defined_ECANCELED 1
+# endif
+
+
+#endif /* _GL_ERRNO_H */
+#endif /* _GL_ERRNO_H */

=== added file 'win32/gnulib/inttypes.h'
--- win32/gnulib/inttypes.h	1970-01-01 00:00:00 +0000
+++ win32/gnulib/inttypes.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,305 @@
+// ISO C9x  compliant inttypes.h for Microsoft Visual Studio
+// Based on ISO/IEC 9899:TC2 Committee draft (May 6, 2005) WG14/N1124 
+// 
+//  Copyright (c) 2006 Alexander Chemeris
+// 
+// Redistribution and use in source and binary forms, with or without
+// modification, are permitted provided that the following conditions are met:
+// 
+//   1. Redistributions of source code must retain the above copyright notice,
+//      this list of conditions and the following disclaimer.
+// 
+//   2. Redistributions in binary form must reproduce the above copyright
+//      notice, this list of conditions and the following disclaimer in the
+//      documentation and/or other materials provided with the distribution.
+// 
+//   3. The name of the author may be used to endorse or promote products
+//      derived from this software without specific prior written permission.
+// 
+// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
+// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
+// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
+// EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
+// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
+// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
+// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+// 
+///////////////////////////////////////////////////////////////////////////////
+
+#ifndef _MSC_VER // [
+#error "Use this header only with Microsoft Visual C++ compilers!"
+#endif // _MSC_VER ]
+
+#ifndef _MSC_INTTYPES_H_ // [
+#define _MSC_INTTYPES_H_
+
+#if _MSC_VER > 1000
+#pragma once
+#endif
+
+#include <stdint.h>
+
+// 7.8 Format conversion of integer types
+
+typedef struct {
+   intmax_t quot;
+   intmax_t rem;
+} imaxdiv_t;
+
+// 7.8.1 Macros for format specifiers
+
+#if !defined(__cplusplus) || defined(__STDC_FORMAT_MACROS) // [   See footnote 185 at page 198
+
+// The fprintf macros for signed integers are:
+#define PRId8       "d"
+#define PRIi8       "i"
+#define PRIdLEAST8  "d"
+#define PRIiLEAST8  "i"
+#define PRIdFAST8   "d"
+#define PRIiFAST8   "i"
+
+#define PRId16       "hd"
+#define PRIi16       "hi"
+#define PRIdLEAST16  "hd"
+#define PRIiLEAST16  "hi"
+#define PRIdFAST16   "hd"
+#define PRIiFAST16   "hi"
+
+#define PRId32       "I32d"
+#define PRIi32       "I32i"
+#define PRIdLEAST32  "I32d"
+#define PRIiLEAST32  "I32i"
+#define PRIdFAST32   "I32d"
+#define PRIiFAST32   "I32i"
+
+#define PRId64       "I64d"
+#define PRIi64       "I64i"
+#define PRIdLEAST64  "I64d"
+#define PRIiLEAST64  "I64i"
+#define PRIdFAST64   "I64d"
+#define PRIiFAST64   "I64i"
+
+#define PRIdMAX     "I64d"
+#define PRIiMAX     "I64i"
+
+#define PRIdPTR     "Id"
+#define PRIiPTR     "Ii"
+
+// The fprintf macros for unsigned integers are:
+#define PRIo8       "o"
+#define PRIu8       "u"
+#define PRIx8       "x"
+#define PRIX8       "X"
+#define PRIoLEAST8  "o"
+#define PRIuLEAST8  "u"
+#define PRIxLEAST8  "x"
+#define PRIXLEAST8  "X"
+#define PRIoFAST8   "o"
+#define PRIuFAST8   "u"
+#define PRIxFAST8   "x"
+#define PRIXFAST8   "X"
+
+#define PRIo16       "ho"
+#define PRIu16       "hu"
+#define PRIx16       "hx"
+#define PRIX16       "hX"
+#define PRIoLEAST16  "ho"
+#define PRIuLEAST16  "hu"
+#define PRIxLEAST16  "hx"
+#define PRIXLEAST16  "hX"
+#define PRIoFAST16   "ho"
+#define PRIuFAST16   "hu"
+#define PRIxFAST16   "hx"
+#define PRIXFAST16   "hX"
+
+#define PRIo32       "I32o"
+#define PRIu32       "I32u"
+#define PRIx32       "I32x"
+#define PRIX32       "I32X"
+#define PRIoLEAST32  "I32o"
+#define PRIuLEAST32  "I32u"
+#define PRIxLEAST32  "I32x"
+#define PRIXLEAST32  "I32X"
+#define PRIoFAST32   "I32o"
+#define PRIuFAST32   "I32u"
+#define PRIxFAST32   "I32x"
+#define PRIXFAST32   "I32X"
+
+#define PRIo64       "I64o"
+#define PRIu64       "I64u"
+#define PRIx64       "I64x"
+#define PRIX64       "I64X"
+#define PRIoLEAST64  "I64o"
+#define PRIuLEAST64  "I64u"
+#define PRIxLEAST64  "I64x"
+#define PRIXLEAST64  "I64X"
+#define PRIoFAST64   "I64o"
+#define PRIuFAST64   "I64u"
+#define PRIxFAST64   "I64x"
+#define PRIXFAST64   "I64X"
+
+#define PRIoMAX     "I64o"
+#define PRIuMAX     "I64u"
+#define PRIxMAX     "I64x"
+#define PRIXMAX     "I64X"
+
+#define PRIoPTR     "Io"
+#define PRIuPTR     "Iu"
+#define PRIxPTR     "Ix"
+#define PRIXPTR     "IX"
+
+// The fscanf macros for signed integers are:
+#define SCNd8       "d"
+#define SCNi8       "i"
+#define SCNdLEAST8  "d"
+#define SCNiLEAST8  "i"
+#define SCNdFAST8   "d"
+#define SCNiFAST8   "i"
+
+#define SCNd16       "hd"
+#define SCNi16       "hi"
+#define SCNdLEAST16  "hd"
+#define SCNiLEAST16  "hi"
+#define SCNdFAST16   "hd"
+#define SCNiFAST16   "hi"
+
+#define SCNd32       "ld"
+#define SCNi32       "li"
+#define SCNdLEAST32  "ld"
+#define SCNiLEAST32  "li"
+#define SCNdFAST32   "ld"
+#define SCNiFAST32   "li"
+
+#define SCNd64       "I64d"
+#define SCNi64       "I64i"
+#define SCNdLEAST64  "I64d"
+#define SCNiLEAST64  "I64i"
+#define SCNdFAST64   "I64d"
+#define SCNiFAST64   "I64i"
+
+#define SCNdMAX     "I64d"
+#define SCNiMAX     "I64i"
+
+#ifdef _WIN64 // [
+#  define SCNdPTR     "I64d"
+#  define SCNiPTR     "I64i"
+#else  // _WIN64 ][
+#  define SCNdPTR     "ld"
+#  define SCNiPTR     "li"
+#endif  // _WIN64 ]
+
+// The fscanf macros for unsigned integers are:
+#define SCNo8       "o"
+#define SCNu8       "u"
+#define SCNx8       "x"
+#define SCNX8       "X"
+#define SCNoLEAST8  "o"
+#define SCNuLEAST8  "u"
+#define SCNxLEAST8  "x"
+#define SCNXLEAST8  "X"
+#define SCNoFAST8   "o"
+#define SCNuFAST8   "u"
+#define SCNxFAST8   "x"
+#define SCNXFAST8   "X"
+
+#define SCNo16       "ho"
+#define SCNu16       "hu"
+#define SCNx16       "hx"
+#define SCNX16       "hX"
+#define SCNoLEAST16  "ho"
+#define SCNuLEAST16  "hu"
+#define SCNxLEAST16  "hx"
+#define SCNXLEAST16  "hX"
+#define SCNoFAST16   "ho"
+#define SCNuFAST16   "hu"
+#define SCNxFAST16   "hx"
+#define SCNXFAST16   "hX"
+
+#define SCNo32       "lo"
+#define SCNu32       "lu"
+#define SCNx32       "lx"
+#define SCNX32       "lX"
+#define SCNoLEAST32  "lo"
+#define SCNuLEAST32  "lu"
+#define SCNxLEAST32  "lx"
+#define SCNXLEAST32  "lX"
+#define SCNoFAST32   "lo"
+#define SCNuFAST32   "lu"
+#define SCNxFAST32   "lx"
+#define SCNXFAST32   "lX"
+
+#define SCNo64       "I64o"
+#define SCNu64       "I64u"
+#define SCNx64       "I64x"
+#define SCNX64       "I64X"
+#define SCNoLEAST64  "I64o"
+#define SCNuLEAST64  "I64u"
+#define SCNxLEAST64  "I64x"
+#define SCNXLEAST64  "I64X"
+#define SCNoFAST64   "I64o"
+#define SCNuFAST64   "I64u"
+#define SCNxFAST64   "I64x"
+#define SCNXFAST64   "I64X"
+
+#define SCNoMAX     "I64o"
+#define SCNuMAX     "I64u"
+#define SCNxMAX     "I64x"
+#define SCNXMAX     "I64X"
+
+#ifdef _WIN64 // [
+#  define SCNoPTR     "I64o"
+#  define SCNuPTR     "I64u"
+#  define SCNxPTR     "I64x"
+#  define SCNXPTR     "I64X"
+#else  // _WIN64 ][
+#  define SCNoPTR     "lo"
+#  define SCNuPTR     "lu"
+#  define SCNxPTR     "lx"
+#  define SCNXPTR     "lX"
+#endif  // _WIN64 ]
+
+#endif // __STDC_FORMAT_MACROS ]
+
+// 7.8.2 Functions for greatest-width integer types
+
+// 7.8.2.1 The imaxabs function
+#define imaxabs _abs64
+
+// 7.8.2.2 The imaxdiv function
+
+// This is modified version of div() function from Microsoft's div.c found
+// in %MSVC.NET%\crt\src\div.c
+#ifdef STATIC_IMAXDIV // [
+static
+#else // STATIC_IMAXDIV ][
+_inline
+#endif // STATIC_IMAXDIV ]
+imaxdiv_t __cdecl imaxdiv(intmax_t numer, intmax_t denom)
+{
+   imaxdiv_t result;
+
+   result.quot = numer / denom;
+   result.rem = numer % denom;
+
+   if (numer < 0 && result.rem > 0) {
+      // did division wrong; must fix up
+      ++result.quot;
+      result.rem -= denom;
+   }
+
+   return result;
+}
+
+// 7.8.2.3 The strtoimax and strtoumax functions
+#define strtoimax _strtoi64
+#define strtoumax _strtoui64
+
+// 7.8.2.4 The wcstoimax and wcstoumax functions
+#define wcstoimax _wcstoi64
+#define wcstoumax _wcstoui64
+
+
+#endif // _MSC_INTTYPES_H_ ]

=== added file 'win32/gnulib/stdint.h'
--- win32/gnulib/stdint.h	1970-01-01 00:00:00 +0000
+++ win32/gnulib/stdint.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,247 @@
+// ISO C9x  compliant stdint.h for Microsoft Visual Studio
+// Based on ISO/IEC 9899:TC2 Committee draft (May 6, 2005) WG14/N1124 
+// 
+//  Copyright (c) 2006-2008 Alexander Chemeris
+// 
+// Redistribution and use in source and binary forms, with or without
+// modification, are permitted provided that the following conditions are met:
+// 
+//   1. Redistributions of source code must retain the above copyright notice,
+//      this list of conditions and the following disclaimer.
+// 
+//   2. Redistributions in binary form must reproduce the above copyright
+//      notice, this list of conditions and the following disclaimer in the
+//      documentation and/or other materials provided with the distribution.
+// 
+//   3. The name of the author may be used to endorse or promote products
+//      derived from this software without specific prior written permission.
+// 
+// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
+// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
+// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
+// EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
+// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
+// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
+// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+// 
+///////////////////////////////////////////////////////////////////////////////
+
+#ifndef _MSC_VER // [
+#error "Use this header only with Microsoft Visual C++ compilers!"
+#endif // _MSC_VER ]
+
+#ifndef _MSC_STDINT_H_ // [
+#define _MSC_STDINT_H_
+
+#if _MSC_VER > 1000
+#pragma once
+#endif
+
+#include <limits.h>
+
+// For Visual Studio 6 in C++ mode and for many Visual Studio versions when
+// compiling for ARM we should wrap <wchar.h> include with 'extern "C++" {}'
+// or compiler give many errors like this:
+//   error C2733: second C linkage of overloaded function 'wmemchr' not allowed
+#ifdef __cplusplus
+extern "C" {
+#endif
+#  include <wchar.h>
+#ifdef __cplusplus
+}
+#endif
+
+// Define _W64 macros to mark types changing their size, like intptr_t.
+#ifndef _W64
+#  if !defined(__midl) && (defined(_X86_) || defined(_M_IX86)) && _MSC_VER >= 1300
+#     define _W64 __w64
+#  else
+#     define _W64
+#  endif
+#endif
+
+
+// 7.18.1 Integer types
+
+// 7.18.1.1 Exact-width integer types
+
+// Visual Studio 6 and Embedded Visual C++ 4 doesn't
+// realize that, e.g. char has the same size as __int8
+// so we give up on __intX for them.
+#if (_MSC_VER < 1300)
+   typedef char              int8_t;
+   typedef short             int16_t;
+   typedef int               int32_t;
+   typedef unsigned char     uint8_t;
+   typedef unsigned short    uint16_t;
+   typedef unsigned int      uint32_t;
+#else
+   typedef __int8            int8_t;
+   typedef __int16           int16_t;
+   typedef __int32           int32_t;
+   typedef unsigned __int8   uint8_t;
+   typedef unsigned __int16  uint16_t;
+   typedef unsigned __int32  uint32_t;
+#endif
+typedef __int64              int64_t;
+typedef unsigned __int64     uint64_t;
+
+
+// 7.18.1.2 Minimum-width integer types
+typedef int8_t    int_least8_t;
+typedef int16_t   int_least16_t;
+typedef int32_t   int_least32_t;
+typedef int64_t   int_least64_t;
+typedef uint8_t   uint_least8_t;
+typedef uint16_t  uint_least16_t;
+typedef uint32_t  uint_least32_t;
+typedef uint64_t  uint_least64_t;
+
+// 7.18.1.3 Fastest minimum-width integer types
+typedef int8_t    int_fast8_t;
+typedef int16_t   int_fast16_t;
+typedef int32_t   int_fast32_t;
+typedef int64_t   int_fast64_t;
+typedef uint8_t   uint_fast8_t;
+typedef uint16_t  uint_fast16_t;
+typedef uint32_t  uint_fast32_t;
+typedef uint64_t  uint_fast64_t;
+
+// 7.18.1.4 Integer types capable of holding object pointers
+#ifdef _WIN64 // [
+   typedef __int64           intptr_t;
+   typedef unsigned __int64  uintptr_t;
+#else // _WIN64 ][
+   typedef _W64 int               intptr_t;
+   typedef _W64 unsigned int      uintptr_t;
+#endif // _WIN64 ]
+
+// 7.18.1.5 Greatest-width integer types
+typedef int64_t   intmax_t;
+typedef uint64_t  uintmax_t;
+
+
+// 7.18.2 Limits of specified-width integer types
+
+#if !defined(__cplusplus) || defined(__STDC_LIMIT_MACROS) // [   See footnote 220 at page 257 and footnote 221 at page 259
+
+// 7.18.2.1 Limits of exact-width integer types
+#define INT8_MIN     ((int8_t)_I8_MIN)
+#define INT8_MAX     _I8_MAX
+#define INT16_MIN    ((int16_t)_I16_MIN)
+#define INT16_MAX    _I16_MAX
+#define INT32_MIN    ((int32_t)_I32_MIN)
+#define INT32_MAX    _I32_MAX
+#define INT64_MIN    ((int64_t)_I64_MIN)
+#define INT64_MAX    _I64_MAX
+#define UINT8_MAX    _UI8_MAX
+#define UINT16_MAX   _UI16_MAX
+#define UINT32_MAX   _UI32_MAX
+#define UINT64_MAX   _UI64_MAX
+
+// 7.18.2.2 Limits of minimum-width integer types
+#define INT_LEAST8_MIN    INT8_MIN
+#define INT_LEAST8_MAX    INT8_MAX
+#define INT_LEAST16_MIN   INT16_MIN
+#define INT_LEAST16_MAX   INT16_MAX
+#define INT_LEAST32_MIN   INT32_MIN
+#define INT_LEAST32_MAX   INT32_MAX
+#define INT_LEAST64_MIN   INT64_MIN
+#define INT_LEAST64_MAX   INT64_MAX
+#define UINT_LEAST8_MAX   UINT8_MAX
+#define UINT_LEAST16_MAX  UINT16_MAX
+#define UINT_LEAST32_MAX  UINT32_MAX
+#define UINT_LEAST64_MAX  UINT64_MAX
+
+// 7.18.2.3 Limits of fastest minimum-width integer types
+#define INT_FAST8_MIN    INT8_MIN
+#define INT_FAST8_MAX    INT8_MAX
+#define INT_FAST16_MIN   INT16_MIN
+#define INT_FAST16_MAX   INT16_MAX
+#define INT_FAST32_MIN   INT32_MIN
+#define INT_FAST32_MAX   INT32_MAX
+#define INT_FAST64_MIN   INT64_MIN
+#define INT_FAST64_MAX   INT64_MAX
+#define UINT_FAST8_MAX   UINT8_MAX
+#define UINT_FAST16_MAX  UINT16_MAX
+#define UINT_FAST32_MAX  UINT32_MAX
+#define UINT_FAST64_MAX  UINT64_MAX
+
+// 7.18.2.4 Limits of integer types capable of holding object pointers
+#ifdef _WIN64 // [
+#  define INTPTR_MIN   INT64_MIN
+#  define INTPTR_MAX   INT64_MAX
+#  define UINTPTR_MAX  UINT64_MAX
+#else // _WIN64 ][
+#  define INTPTR_MIN   INT32_MIN
+#  define INTPTR_MAX   INT32_MAX
+#  define UINTPTR_MAX  UINT32_MAX
+#endif // _WIN64 ]
+
+// 7.18.2.5 Limits of greatest-width integer types
+#define INTMAX_MIN   INT64_MIN
+#define INTMAX_MAX   INT64_MAX
+#define UINTMAX_MAX  UINT64_MAX
+
+// 7.18.3 Limits of other integer types
+
+#ifdef _WIN64 // [
+#  define PTRDIFF_MIN  _I64_MIN
+#  define PTRDIFF_MAX  _I64_MAX
+#else  // _WIN64 ][
+#  define PTRDIFF_MIN  _I32_MIN
+#  define PTRDIFF_MAX  _I32_MAX
+#endif  // _WIN64 ]
+
+#define SIG_ATOMIC_MIN  INT_MIN
+#define SIG_ATOMIC_MAX  INT_MAX
+
+#ifndef SIZE_MAX // [
+#  ifdef _WIN64 // [
+#     define SIZE_MAX  _UI64_MAX
+#  else // _WIN64 ][
+#     define SIZE_MAX  _UI32_MAX
+#  endif // _WIN64 ]
+#endif // SIZE_MAX ]
+
+// WCHAR_MIN and WCHAR_MAX are also defined in <wchar.h>
+#ifndef WCHAR_MIN // [
+#  define WCHAR_MIN  0
+#endif  // WCHAR_MIN ]
+#ifndef WCHAR_MAX // [
+#  define WCHAR_MAX  _UI16_MAX
+#endif  // WCHAR_MAX ]
+
+#define WINT_MIN  0
+#define WINT_MAX  _UI16_MAX
+
+#endif // __STDC_LIMIT_MACROS ]
+
+
+// 7.18.4 Limits of other integer types
+
+#if !defined(__cplusplus) || defined(__STDC_CONSTANT_MACROS) // [   See footnote 224 at page 260
+
+// 7.18.4.1 Macros for minimum-width integer constants
+
+#define INT8_C(val)  val##i8
+#define INT16_C(val) val##i16
+#define INT32_C(val) val##i32
+#define INT64_C(val) val##i64
+
+#define UINT8_C(val)  val##ui8
+#define UINT16_C(val) val##ui16
+#define UINT32_C(val) val##ui32
+#define UINT64_C(val) val##ui64
+
+// 7.18.4.2 Macros for greatest-width integer constants
+#define INTMAX_C   INT64_C
+#define UINTMAX_C  UINT64_C
+
+#endif // __STDC_CONSTANT_MACROS ]
+
+
+#endif // _MSC_STDINT_H_ ]

=== added file 'win32/poll.c'
--- win32/poll.c	1970-01-01 00:00:00 +0000
+++ win32/poll.c	2010-08-12 20:27:32 +0000
@@ -0,0 +1,562 @@
+/* Emulation for poll(2)
+   Contributed by Paolo Bonzini.
+
+   Copyright 2001, 2002, 2003, 2006, 2007, 2008 Free Software Foundation, Inc.
+
+   This file is part of gnulib.
+
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU Lesser General Public License as published by
+   the Free Software Foundation; either version 2, or (at your option)
+   any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU Lesser General Public License for more details.
+
+   You should have received a copy of the GNU Lesser General Public License along
+   with this program; if not, write to the Free Software Foundation,
+   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+
+#include <config.h>
+#include <alloca.h>
+
+#include <sys/types.h>
+#include "poll.h"
+#include <errno.h>
+#include <limits.h>
+#include <assert.h>
+
+#if (defined _WIN32 || defined __WIN32__) && ! defined __CYGWIN__
+#define WIN32_NATIVE
+#include <winsock2.h>
+#include <windows.h>
+#include <io.h>
+#include <stdio.h>
+#include <conio.h>
+#else
+#include <sys/time.h>
+#include <sys/socket.h>
+#include <sys/select.h>
+#include <unistd.h>
+#endif
+
+#ifdef HAVE_SYS_IOCTL_H
+#include <sys/ioctl.h>
+#endif
+#ifdef HAVE_SYS_FILIO_H
+#include <sys/filio.h>
+#endif
+
+#include <time.h>
+
+#ifndef INFTIM
+#define INFTIM (-1)
+#endif
+
+/* BeOS does not have MSG_PEEK.  */
+#ifndef MSG_PEEK
+#define MSG_PEEK 0
+#endif
+
+#ifdef WIN32_NATIVE
+
+/* Declare data structures for ntdll functions.  */
+typedef struct _FILE_PIPE_LOCAL_INFORMATION {
+  ULONG NamedPipeType;
+  ULONG NamedPipeConfiguration;
+  ULONG MaximumInstances;
+  ULONG CurrentInstances;
+  ULONG InboundQuota;
+  ULONG ReadDataAvailable;
+  ULONG OutboundQuota;
+  ULONG WriteQuotaAvailable;
+  ULONG NamedPipeState;
+  ULONG NamedPipeEnd;
+} FILE_PIPE_LOCAL_INFORMATION, *PFILE_PIPE_LOCAL_INFORMATION;
+
+typedef struct _IO_STATUS_BLOCK
+{
+  union {
+    DWORD Status;
+    PVOID Pointer;
+  } u;
+  ULONG_PTR Information;
+} IO_STATUS_BLOCK, *PIO_STATUS_BLOCK;
+
+typedef enum _FILE_INFORMATION_CLASS {
+  FilePipeLocalInformation = 24
+} FILE_INFORMATION_CLASS, *PFILE_INFORMATION_CLASS;
+
+typedef DWORD (WINAPI *PNtQueryInformationFile)
+	 (HANDLE, IO_STATUS_BLOCK *, VOID *, ULONG, FILE_INFORMATION_CLASS);
+
+#ifndef PIPE_BUF
+#define PIPE_BUF	512
+#endif
+
+/* Compute revents values for file handle H.  */
+
+static int
+win32_compute_revents (HANDLE h, int sought)
+{
+  int ret, happened;
+  unsigned i;
+  INPUT_RECORD *irbuffer;
+  DWORD avail, nbuffer;
+  BOOL bRet;
+  IO_STATUS_BLOCK iosb;
+  FILE_PIPE_LOCAL_INFORMATION fpli;
+  static PNtQueryInformationFile NtQueryInformationFile;
+  static BOOL once_only;
+
+  switch (GetFileType (h))
+    {
+    case FILE_TYPE_PIPE:
+      if (!once_only)
+	{
+	  NtQueryInformationFile = (PNtQueryInformationFile)
+	    GetProcAddress (GetModuleHandle ("ntdll.dll"),
+			    "NtQueryInformationFile");
+	  once_only = TRUE;
+	}
+
+      happened = 0;
+      if (PeekNamedPipe (h, NULL, 0, NULL, &avail, NULL) != 0)
+	{
+	  if (avail)
+	    happened |= sought & (POLLIN | POLLRDNORM);
+	}
+
+      else
+	{
+	  /* It was the write-end of the pipe.  Check if it is writable.
+	     If NtQueryInformationFile fails, optimistically assume the pipe is
+	     writable.  This could happen on Win9x, where NtQueryInformationFile
+	     is not available, or if we inherit a pipe that doesn't permit
+	     FILE_READ_ATTRIBUTES access on the write end (I think this should
+	     not happen since WinXP SP2; WINE seems fine too).  Otherwise,
+	     ensure that enough space is available for atomic writes.  */
+          memset (&iosb, 0, sizeof (iosb));
+          memset (&fpli, 0, sizeof (fpli));
+
+          if (!NtQueryInformationFile
+              || NtQueryInformationFile (h, &iosb, &fpli, sizeof (fpli),
+				         FilePipeLocalInformation)
+	      || fpli.WriteQuotaAvailable >= PIPE_BUF
+	      || (fpli.OutboundQuota < PIPE_BUF &&
+	          fpli.WriteQuotaAvailable == fpli.OutboundQuota))
+	    happened |= sought & (POLLOUT | POLLWRNORM | POLLWRBAND);
+	}
+      return happened;
+
+    case FILE_TYPE_CHAR:
+      ret = WaitForSingleObject (h, 0);
+      if (ret == WAIT_OBJECT_0)
+        {
+	  nbuffer = avail = 0;
+	  bRet = GetNumberOfConsoleInputEvents (h, &nbuffer);
+	  if (!bRet || nbuffer == 0)
+	    return POLLHUP;
+
+	  irbuffer = (INPUT_RECORD *) alloca (nbuffer * sizeof (INPUT_RECORD));
+	  bRet = PeekConsoleInput (h, irbuffer, nbuffer, &avail);
+	  if (!bRet || avail == 0)
+	    return POLLHUP;
+
+	  for (i = 0; i < avail; i++)
+	    if (irbuffer[i].EventType == KEY_EVENT)
+	      return sought & ~(POLLPRI | POLLRDBAND);
+	}
+      break;
+
+    default:
+      ret = WaitForSingleObject (h, 0);
+      if (ret == WAIT_OBJECT_0)
+        return sought & ~(POLLPRI | POLLRDBAND);
+
+      break;
+    }
+
+  return sought & (POLLOUT | POLLWRNORM | POLLWRBAND);
+}
+
+/* Convert fd_sets returned by select into revents values.  */
+
+static int
+win32_compute_revents_socket (SOCKET h, int sought, long lNetworkEvents)
+{
+  int happened = 0;
+
+  if ((lNetworkEvents & (FD_READ | FD_ACCEPT | FD_CLOSE)) == FD_ACCEPT)
+    happened |= (POLLIN | POLLRDNORM) & sought;
+
+  else if (lNetworkEvents & (FD_READ | FD_ACCEPT | FD_CLOSE))
+    {
+      int r, error;
+
+      char data[64];
+      WSASetLastError (0);
+      r = recv (h, data, sizeof (data), MSG_PEEK);
+      error = WSAGetLastError ();
+      WSASetLastError (0);
+
+      if (r > 0 || error == WSAENOTCONN)
+        happened |= (POLLIN | POLLRDNORM) & sought;
+
+      /* Distinguish hung-up sockets from other errors.  */
+      else if (r == 0 || error == WSAESHUTDOWN || error == WSAECONNRESET
+	       || error == WSAECONNABORTED || error == WSAENETRESET)
+        happened |= POLLHUP;
+
+      else
+        happened |= POLLERR;
+    }
+
+  if (lNetworkEvents & (FD_WRITE | FD_CONNECT))
+    happened |= (POLLOUT | POLLWRNORM | POLLWRBAND) & sought;
+
+  if (lNetworkEvents & FD_OOB)
+    happened |= (POLLPRI | POLLRDBAND) & sought;
+
+  return happened;
+}
+
+#else /* !MinGW */
+
+/* Convert select(2) returned fd_sets into poll(2) revents values.  */
+static int
+compute_revents (int fd, int sought, fd_set *rfds, fd_set *wfds, fd_set *efds)
+{
+  int happened = 0;
+  if (FD_ISSET (fd, rfds))
+    {
+      int r;
+      int socket_errno;
+
+#if defined __MACH__ && defined __APPLE__
+      /* There is a bug in Mac OS X that causes it to ignore MSG_PEEK
+         for some kinds of descriptors.  Detect if this descriptor is a
+         connected socket, a server socket, or something else using a
+         0-byte recv, and use ioctl(2) to detect POLLHUP.  */
+      r = recv (fd, NULL, 0, MSG_PEEK);
+      socket_errno = (r < 0) ? errno : 0;
+      if (r == 0 || socket_errno == ENOTSOCK)
+	ioctl (fd, FIONREAD, &r);
+#else
+      char data[64];
+      r = recv (fd, data, sizeof (data), MSG_PEEK);
+      socket_errno = (r < 0) ? errno : 0;
+#endif
+      if (r == 0)
+	happened |= POLLHUP;
+
+      /* If the event happened on an unconnected server socket,
+         that's fine. */
+      else if (r > 0 || ( /* (r == -1) && */ socket_errno == ENOTCONN))
+	happened |= (POLLIN | POLLRDNORM) & sought;
+
+      /* Distinguish hung-up sockets from other errors.  */
+      else if (socket_errno == ESHUTDOWN || socket_errno == ECONNRESET
+	       || socket_errno == ECONNABORTED || socket_errno == ENETRESET)
+	happened |= POLLHUP;
+
+      else
+	happened |= POLLERR;
+    }
+
+  if (FD_ISSET (fd, wfds))
+    happened |= (POLLOUT | POLLWRNORM | POLLWRBAND) & sought;
+
+  if (FD_ISSET (fd, efds))
+    happened |= (POLLPRI | POLLRDBAND) & sought;
+
+  return happened;
+}
+#endif /* !MinGW */
+
+int
+poll (pfd, nfd, timeout)
+     struct pollfd *pfd;
+     nfds_t nfd;
+     int timeout;
+{
+#ifndef WIN32_NATIVE
+  fd_set rfds, wfds, efds;
+  struct timeval tv;
+  struct timeval *ptv;
+  int maxfd, rc;
+  nfds_t i;
+
+#ifdef _SC_OPEN_MAX
+  static int sc_open_max = -1;
+
+  if (nfd < 0
+      || (nfd > sc_open_max
+          && (sc_open_max != -1
+	      || nfd > (sc_open_max = sysconf (_SC_OPEN_MAX)))))
+    {
+      errno = EINVAL;
+      return -1;
+    }
+#else /* !_SC_OPEN_MAX */
+#ifdef OPEN_MAX
+  if (nfd < 0 || nfd > OPEN_MAX)
+    {
+      errno = EINVAL;
+      return -1;
+    }
+#endif /* OPEN_MAX -- else, no check is needed */
+#endif /* !_SC_OPEN_MAX */
+
+  /* EFAULT is not necessary to implement, but let's do it in the
+     simplest case. */
+  if (!pfd)
+    {
+      errno = EFAULT;
+      return -1;
+    }
+
+  /* convert timeout number into a timeval structure */
+  if (timeout == 0)
+    {
+      ptv = &tv;
+      ptv->tv_sec = 0;
+      ptv->tv_usec = 0;
+    }
+  else if (timeout > 0)
+    {
+      ptv = &tv;
+      ptv->tv_sec = timeout / 1000;
+      ptv->tv_usec = (timeout % 1000) * 1000;
+    }
+  else if (timeout == INFTIM)
+    /* wait forever */
+    ptv = NULL;
+  else
+    {
+      errno = EINVAL;
+      return -1;
+    }
+
+  /* create fd sets and determine max fd */
+  maxfd = -1;
+  FD_ZERO (&rfds);
+  FD_ZERO (&wfds);
+  FD_ZERO (&efds);
+  for (i = 0; i < nfd; i++)
+    {
+      if (pfd[i].fd < 0)
+	continue;
+
+      if (pfd[i].events & (POLLIN | POLLRDNORM))
+	FD_SET (pfd[i].fd, &rfds);
+
+      /* see select(2): "the only exceptional condition detectable
+         is out-of-band data received on a socket", hence we push
+         POLLWRBAND events onto wfds instead of efds. */
+      if (pfd[i].events & (POLLOUT | POLLWRNORM | POLLWRBAND))
+	FD_SET (pfd[i].fd, &wfds);
+      if (pfd[i].events & (POLLPRI | POLLRDBAND))
+	FD_SET (pfd[i].fd, &efds);
+      if (pfd[i].fd >= maxfd
+	  && (pfd[i].events & (POLLIN | POLLOUT | POLLPRI
+			       | POLLRDNORM | POLLRDBAND
+			       | POLLWRNORM | POLLWRBAND)))
+	{
+	  maxfd = pfd[i].fd;
+	  if (maxfd > FD_SETSIZE)
+	    {
+	      errno = EOVERFLOW;
+	      return -1;
+	    }
+	}
+    }
+
+  /* examine fd sets */
+  rc = select (maxfd + 1, &rfds, &wfds, &efds, ptv);
+  if (rc < 0)
+    return rc;
+
+  /* establish results */
+  rc = 0;
+  for (i = 0; i < nfd; i++)
+    if (pfd[i].fd < 0)
+      pfd[i].revents = 0;
+    else
+      {
+        int happened = compute_revents (pfd[i].fd, pfd[i].events,
+                                        &rfds, &wfds, &efds);
+	if (happened)
+	  {
+	    pfd[i].revents = happened;
+	    rc++;
+	  }
+      }
+
+  return rc;
+#else
+  static struct timeval tv0;
+  static HANDLE hEvent;
+  WSANETWORKEVENTS ev;
+  HANDLE h, handle_array[FD_SETSIZE + 2];
+  DWORD ret, wait_timeout, nhandles;
+  fd_set rfds, wfds, xfds;
+  BOOL poll_again;
+  MSG msg;
+  int rc= 0;
+  nfds_t i;
+
+  if (timeout < -1)
+    {
+      errno = EINVAL;
+      return -1;
+    }
+
+  if (!hEvent)
+    hEvent = CreateEvent (NULL, FALSE, FALSE, NULL);
+
+  handle_array[0] = hEvent;
+  nhandles = 1;
+  FD_ZERO (&rfds);
+  FD_ZERO (&wfds);
+  FD_ZERO (&xfds);
+
+  /* Classify socket handles and create fd sets. */
+  for (i = 0; i < nfd; i++)
+    {
+      pfd[i].revents = 0;
+      if (pfd[i].fd < 0)
+        continue;
+      if (!(pfd[i].events & (POLLIN | POLLRDNORM |
+                             POLLOUT | POLLWRNORM | POLLWRBAND)))
+	continue;
+
+      h = (HANDLE) _get_osfhandle (pfd[i].fd);
+      assert (h != NULL);
+
+      /* Under Wine, it seems that getsockopt returns 0 for pipes too.
+	 WSAEnumNetworkEvents instead distinguishes the two correctly.  */
+      ev.lNetworkEvents = 0xDEADBEEF;
+      WSAEnumNetworkEvents ((SOCKET) h, NULL, &ev);
+      if (ev.lNetworkEvents != (long)0xDEADBEEF)
+        {
+          int requested = FD_CLOSE;
+
+          /* see above; socket handles are mapped onto select.  */
+          if (pfd[i].events & (POLLIN | POLLRDNORM))
+	    {
+              requested |= FD_READ | FD_ACCEPT;
+	      FD_SET ((SOCKET) h, &rfds);
+	    }
+          if (pfd[i].events & (POLLOUT | POLLWRNORM | POLLWRBAND))
+	    {
+              requested |= FD_WRITE | FD_CONNECT;
+	      FD_SET ((SOCKET) h, &wfds);
+	    }
+          if (pfd[i].events & (POLLPRI | POLLRDBAND))
+	    {
+              requested |= FD_OOB;
+	      FD_SET ((SOCKET) h, &xfds);
+	    }
+
+          if (requested)
+            WSAEventSelect ((SOCKET) h, hEvent, requested);
+        }
+      else
+        {
+          handle_array[nhandles++] = h;
+
+	  /* Poll now.  If we get an event, do not poll again.  */
+          pfd[i].revents = win32_compute_revents (h, pfd[i].events);
+          if (pfd[i].revents)
+	    wait_timeout = 0;
+        }
+    }
+
+  if (select (0, &rfds, &wfds, &xfds, &tv0) > 0)
+    {
+      /* Do MsgWaitForMultipleObjects anyway to dispatch messages, but
+	 no need to call select again.  */
+      poll_again = FALSE;
+      wait_timeout = 0;
+    }
+  else
+    {
+      poll_again = TRUE;
+      if (timeout == INFTIM)
+        wait_timeout = INFINITE;
+      else
+        wait_timeout = timeout;
+    }
+
+  for (;;)
+    {
+      ret = MsgWaitForMultipleObjects (nhandles, handle_array, FALSE,
+				       wait_timeout, QS_ALLINPUT);
+
+      if (ret == WAIT_OBJECT_0 + nhandles)
+	{
+          /* new input of some other kind */
+	  BOOL bRet;
+          while ((bRet = PeekMessage (&msg, NULL, 0, 0, PM_REMOVE)) != 0)
+            {
+              TranslateMessage (&msg);
+              DispatchMessage (&msg);
+            }
+	}
+      else
+	break;
+    }
+
+  if (poll_again)
+    select (0, &rfds, &wfds, &xfds, &tv0);
+
+  /* Place a sentinel at the end of the array.  */
+  handle_array[nhandles] = NULL;
+  nhandles = 1;
+  for (i = 0; i < nfd; i++)
+    {
+      int happened;
+
+      if (pfd[i].fd < 0)
+        continue;
+      if (!(pfd[i].events & (POLLIN | POLLRDNORM |
+                             POLLOUT | POLLWRNORM | POLLWRBAND)))
+	continue;
+
+      h = (HANDLE) _get_osfhandle (pfd[i].fd);
+      if (h != handle_array[nhandles])
+        {
+          /* It's a socket.  */
+          WSAEnumNetworkEvents ((SOCKET) h, NULL, &ev);
+	  WSAEventSelect ((SOCKET) h, 0, 0);
+
+	  /* If we're lucky, WSAEnumNetworkEvents already provided a way
+	     to distinguish FD_READ and FD_ACCEPT; this saves a recv later.  */
+	  if (FD_ISSET ((SOCKET) h, &rfds)
+	      && !(ev.lNetworkEvents & (FD_READ | FD_ACCEPT)))
+	    ev.lNetworkEvents |= FD_READ | FD_ACCEPT;
+	  if (FD_ISSET ((SOCKET) h, &wfds))
+	    ev.lNetworkEvents |= FD_WRITE | FD_CONNECT;
+	  if (FD_ISSET ((SOCKET) h, &xfds))
+	    ev.lNetworkEvents |= FD_OOB;
+
+          happened = win32_compute_revents_socket ((SOCKET) h, pfd[i].events,
+						   ev.lNetworkEvents);
+        }
+      else
+        {
+          /* Not a socket.  */
+          nhandles++;
+          happened = win32_compute_revents (h, pfd[i].events);
+        }
+
+       if ((pfd[i].revents |= happened) != 0)
+        rc++;
+    }
+
+  return rc;
+#endif
+}

=== added file 'win32/poll.h'
--- win32/poll.h	1970-01-01 00:00:00 +0000
+++ win32/poll.h	2010-08-12 20:27:32 +0000
@@ -0,0 +1,53 @@
+/* Header for poll(2) emulation
+   Contributed by Paolo Bonzini.
+
+   Copyright 2001, 2002, 2003, 2007 Free Software Foundation, Inc.
+
+   This file is part of gnulib.
+
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU Lesser General Public License as published by
+   the Free Software Foundation; either version 2, or (at your option)
+   any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU Lesser General Public License for more details.
+
+   You should have received a copy of the GNU Lesser General Public License along
+   with this program; if not, write to the Free Software Foundation,
+   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+
+#ifndef _GL_POLL_H
+#define _GL_POLL_H
+
+/* fake a poll(2) environment */
+#define POLLIN      0x0001	/* any readable data available   */
+#define POLLPRI     0x0002	/* OOB/Urgent readable data      */
+#define POLLOUT     0x0004	/* file descriptor is writeable  */
+#define POLLERR     0x0008	/* some poll error occurred      */
+#define POLLHUP     0x0010	/* file descriptor was "hung up" */
+#define POLLNVAL    0x0020	/* requested events "invalid"    */
+#define POLLRDNORM  0x0040
+#define POLLRDBAND  0x0080
+#define POLLWRNORM  0x0100
+#define POLLWRBAND  0x0200
+
+struct pollfd
+{
+  int fd;			/* which file descriptor to poll */
+  short events;			/* events we are interested in   */
+  short revents;		/* events found on return        */
+};
+
+typedef unsigned long nfds_t;
+
+extern int poll (struct pollfd *pfd, nfds_t nfd, int timeout);
+
+/* Define INFTIM only if doing so conforms to POSIX.  */
+#if !defined (_POSIX_C_SOURCE) && !defined (_XOPEN_SOURCE)
+#define INFTIM (-1)
+#endif
+
+#endif /* _GL_POLL_H */

